// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_BROWSER_UI_WEBUI_APP_MANAGEMENT_APP_MANAGEMENT_MOJOM_H_
#define CHROME_BROWSER_UI_WEBUI_APP_MANAGEMENT_APP_MANAGEMENT_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/browser/ui/webui/app_management/app_management.mojom-shared.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace app_management {
namespace mojom {
class PageHandlerFactory;
using PageHandlerFactoryPtr = mojo::InterfacePtr<PageHandlerFactory>;
using PageHandlerFactoryPtrInfo = mojo::InterfacePtrInfo<PageHandlerFactory>;
using ThreadSafePageHandlerFactoryPtr =
    mojo::ThreadSafeInterfacePtr<PageHandlerFactory>;
using PageHandlerFactoryRequest = mojo::InterfaceRequest<PageHandlerFactory>;
using PageHandlerFactoryAssociatedPtr =
    mojo::AssociatedInterfacePtr<PageHandlerFactory>;
using ThreadSafePageHandlerFactoryAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<PageHandlerFactory>;
using PageHandlerFactoryAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<PageHandlerFactory>;
using PageHandlerFactoryAssociatedRequest =
    mojo::AssociatedInterfaceRequest<PageHandlerFactory>;

class PageHandler;
using PageHandlerPtr = mojo::InterfacePtr<PageHandler>;
using PageHandlerPtrInfo = mojo::InterfacePtrInfo<PageHandler>;
using ThreadSafePageHandlerPtr =
    mojo::ThreadSafeInterfacePtr<PageHandler>;
using PageHandlerRequest = mojo::InterfaceRequest<PageHandler>;
using PageHandlerAssociatedPtr =
    mojo::AssociatedInterfacePtr<PageHandler>;
using ThreadSafePageHandlerAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<PageHandler>;
using PageHandlerAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<PageHandler>;
using PageHandlerAssociatedRequest =
    mojo::AssociatedInterfaceRequest<PageHandler>;

class Page;
using PagePtr = mojo::InterfacePtr<Page>;
using PagePtrInfo = mojo::InterfacePtrInfo<Page>;
using ThreadSafePagePtr =
    mojo::ThreadSafeInterfacePtr<Page>;
using PageRequest = mojo::InterfaceRequest<Page>;
using PageAssociatedPtr =
    mojo::AssociatedInterfacePtr<Page>;
using ThreadSafePageAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Page>;
using PageAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Page>;
using PageAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Page>;


class PageHandlerFactoryProxy;

template <typename ImplRefTraits>
class PageHandlerFactoryStub;

class PageHandlerFactoryRequestValidator;

class  PageHandlerFactory
    : public PageHandlerFactoryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = PageHandlerFactoryProxy;

  template <typename ImplRefTraits>
  using Stub_ = PageHandlerFactoryStub<ImplRefTraits>;

  using RequestValidator_ = PageHandlerFactoryRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kCreatePageHandlerMinVersion = 0,
  };
  virtual ~PageHandlerFactory() {}

  virtual void CreatePageHandler(PagePtr page, PageHandlerRequest handler) = 0;
};
class  PageHandlerFactoryInterceptorForTesting : public PageHandlerFactory {
  virtual PageHandlerFactory* GetForwardingInterface() = 0;
  void CreatePageHandler(PagePtr page, PageHandlerRequest handler) override;
};
class  PageHandlerFactoryAsyncWaiter {
 public:
  explicit PageHandlerFactoryAsyncWaiter(PageHandlerFactory* proxy);
  ~PageHandlerFactoryAsyncWaiter();

 private:
  PageHandlerFactory* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PageHandlerFactoryAsyncWaiter);
};

class PageHandlerProxy;

template <typename ImplRefTraits>
class PageHandlerStub;

class PageHandlerRequestValidator;

class  PageHandler
    : public PageHandlerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = PageHandlerProxy;

  template <typename ImplRefTraits>
  using Stub_ = PageHandlerStub<ImplRefTraits>;

  using RequestValidator_ = PageHandlerRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kGetAppsMinVersion = 0,
  };
  virtual ~PageHandler() {}

  virtual void GetApps() = 0;
};
class  PageHandlerInterceptorForTesting : public PageHandler {
  virtual PageHandler* GetForwardingInterface() = 0;
  void GetApps() override;
};
class  PageHandlerAsyncWaiter {
 public:
  explicit PageHandlerAsyncWaiter(PageHandler* proxy);
  ~PageHandlerAsyncWaiter();

 private:
  PageHandler* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PageHandlerAsyncWaiter);
};

class PageProxy;

template <typename ImplRefTraits>
class PageStub;

class PageRequestValidator;

class  Page
    : public PageInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = PageProxy;

  template <typename ImplRefTraits>
  using Stub_ = PageStub<ImplRefTraits>;

  using RequestValidator_ = PageRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnAppsAddedMinVersion = 0,
  };
  virtual ~Page() {}

  virtual void OnAppsAdded(const std::vector<std::string>& ids) = 0;
};
class  PageInterceptorForTesting : public Page {
  virtual Page* GetForwardingInterface() = 0;
  void OnAppsAdded(const std::vector<std::string>& ids) override;
};
class  PageAsyncWaiter {
 public:
  explicit PageAsyncWaiter(Page* proxy);
  ~PageAsyncWaiter();

 private:
  Page* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PageAsyncWaiter);
};

class  PageHandlerFactoryProxy
    : public PageHandlerFactory {
 public:
  explicit PageHandlerFactoryProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreatePageHandler(PagePtr page, PageHandlerRequest handler) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  PageHandlerProxy
    : public PageHandler {
 public:
  explicit PageHandlerProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetApps() final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  PageProxy
    : public Page {
 public:
  explicit PageProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnAppsAdded(const std::vector<std::string>& ids) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  PageHandlerFactoryStubDispatch {
 public:
  static bool Accept(PageHandlerFactory* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      PageHandlerFactory* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PageHandlerFactory>>
class PageHandlerFactoryStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PageHandlerFactoryStub() {}
  ~PageHandlerFactoryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageHandlerFactoryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageHandlerFactoryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  PageHandlerStubDispatch {
 public:
  static bool Accept(PageHandler* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      PageHandler* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PageHandler>>
class PageHandlerStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PageHandlerStub() {}
  ~PageHandlerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageHandlerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageHandlerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  PageStubDispatch {
 public:
  static bool Accept(Page* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Page* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Page>>
class PageStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PageStub() {}
  ~PageStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  PageHandlerFactoryRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  PageHandlerRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  PageRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace app_management

namespace mojo {

}  // namespace mojo

#endif  // CHROME_BROWSER_UI_WEBUI_APP_MANAGEMENT_APP_MANAGEMENT_MOJOM_H_