// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_COMMON_AVAILABLE_OFFLINE_CONTENT_MOJOM_SHARED_H_
#define CHROME_COMMON_AVAILABLE_OFFLINE_CONTENT_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "chrome/common/available_offline_content.mojom-shared-internal.h"
#include "url/mojom/url.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace chrome {
namespace mojom {
class AvailableOfflineContentDataView;

class AvailableOfflineContentSummaryDataView;



}  // namespace mojom
}  // namespace chrome

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::chrome::mojom::AvailableOfflineContentDataView> {
  using Data = ::chrome::mojom::internal::AvailableOfflineContent_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::chrome::mojom::AvailableOfflineContentSummaryDataView> {
  using Data = ::chrome::mojom::internal::AvailableOfflineContentSummary_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace chrome {
namespace mojom {

enum class AvailableContentType : int32_t {
  kPrefetchedPage = 0,
  kVideo = 1,
  kAudio = 2,
  kOtherPage = 3,
  kUninteresting = 4,
  kMinValue = 0,
  kMaxValue = 4,
};

inline std::ostream& operator<<(std::ostream& os, AvailableContentType value) {
  switch(value) {
    case AvailableContentType::kPrefetchedPage:
      return os << "AvailableContentType::kPrefetchedPage";
    case AvailableContentType::kVideo:
      return os << "AvailableContentType::kVideo";
    case AvailableContentType::kAudio:
      return os << "AvailableContentType::kAudio";
    case AvailableContentType::kOtherPage:
      return os << "AvailableContentType::kOtherPage";
    case AvailableContentType::kUninteresting:
      return os << "AvailableContentType::kUninteresting";
    default:
      return os << "Unknown AvailableContentType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(AvailableContentType value) {
  return internal::AvailableContentType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class AvailableOfflineContentProviderInterfaceBase {};

using AvailableOfflineContentProviderPtrDataView =
    mojo::InterfacePtrDataView<AvailableOfflineContentProviderInterfaceBase>;
using AvailableOfflineContentProviderRequestDataView =
    mojo::InterfaceRequestDataView<AvailableOfflineContentProviderInterfaceBase>;
using AvailableOfflineContentProviderAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<AvailableOfflineContentProviderInterfaceBase>;
using AvailableOfflineContentProviderAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<AvailableOfflineContentProviderInterfaceBase>;
class AvailableOfflineContentDataView {
 public:
  AvailableOfflineContentDataView() {}

  AvailableOfflineContentDataView(
      internal::AvailableOfflineContent_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadId(UserType* output) {
    auto* pointer = data_->id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetNameSpaceDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadNameSpace(UserType* output) {
    auto* pointer = data_->name_space.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetTitleDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTitle(UserType* output) {
    auto* pointer = data_->title.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetSnippetDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSnippet(UserType* output) {
    auto* pointer = data_->snippet.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetDateModifiedDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDateModified(UserType* output) {
    auto* pointer = data_->date_modified.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetAttributionDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAttribution(UserType* output) {
    auto* pointer = data_->attribution.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetThumbnailDataUriDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadThumbnailDataUri(UserType* output) {
    auto* pointer = data_->thumbnail_data_uri.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadContentType(UserType* output) const {
    auto data_value = data_->content_type;
    return mojo::internal::Deserialize<::chrome::mojom::AvailableContentType>(
        data_value, output);
  }

  AvailableContentType content_type() const {
    return static_cast<AvailableContentType>(data_->content_type);
  }
 private:
  internal::AvailableOfflineContent_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AvailableOfflineContentSummaryDataView {
 public:
  AvailableOfflineContentSummaryDataView() {}

  AvailableOfflineContentSummaryDataView(
      internal::AvailableOfflineContentSummary_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint32_t total_items() const {
    return data_->total_items;
  }
  bool has_prefetched_page() const {
    return data_->has_prefetched_page;
  }
  bool has_offline_page() const {
    return data_->has_offline_page;
  }
  bool has_video() const {
    return data_->has_video;
  }
  bool has_audio() const {
    return data_->has_audio;
  }
 private:
  internal::AvailableOfflineContentSummary_Data* data_ = nullptr;
};

class AvailableOfflineContentProvider_List_ParamsDataView {
 public:
  AvailableOfflineContentProvider_List_ParamsDataView() {}

  AvailableOfflineContentProvider_List_ParamsDataView(
      internal::AvailableOfflineContentProvider_List_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::AvailableOfflineContentProvider_List_Params_Data* data_ = nullptr;
};

class AvailableOfflineContentProvider_List_ResponseParamsDataView {
 public:
  AvailableOfflineContentProvider_List_ResponseParamsDataView() {}

  AvailableOfflineContentProvider_List_ResponseParamsDataView(
      internal::AvailableOfflineContentProvider_List_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  bool list_visible_by_prefs() const {
    return data_->list_visible_by_prefs;
  }
  inline void GetOutDataView(
      mojo::ArrayDataView<AvailableOfflineContentDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOut(UserType* output) {
    auto* pointer = data_->out.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::chrome::mojom::AvailableOfflineContentDataView>>(
        pointer, output, context_);
  }
 private:
  internal::AvailableOfflineContentProvider_List_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AvailableOfflineContentProvider_Summarize_ParamsDataView {
 public:
  AvailableOfflineContentProvider_Summarize_ParamsDataView() {}

  AvailableOfflineContentProvider_Summarize_ParamsDataView(
      internal::AvailableOfflineContentProvider_Summarize_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::AvailableOfflineContentProvider_Summarize_Params_Data* data_ = nullptr;
};

class AvailableOfflineContentProvider_Summarize_ResponseParamsDataView {
 public:
  AvailableOfflineContentProvider_Summarize_ResponseParamsDataView() {}

  AvailableOfflineContentProvider_Summarize_ResponseParamsDataView(
      internal::AvailableOfflineContentProvider_Summarize_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetOutDataView(
      AvailableOfflineContentSummaryDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOut(UserType* output) {
    auto* pointer = data_->out.Get();
    return mojo::internal::Deserialize<::chrome::mojom::AvailableOfflineContentSummaryDataView>(
        pointer, output, context_);
  }
 private:
  internal::AvailableOfflineContentProvider_Summarize_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AvailableOfflineContentProvider_LaunchItem_ParamsDataView {
 public:
  AvailableOfflineContentProvider_LaunchItem_ParamsDataView() {}

  AvailableOfflineContentProvider_LaunchItem_ParamsDataView(
      internal::AvailableOfflineContentProvider_LaunchItem_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetItemIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadItemId(UserType* output) {
    auto* pointer = data_->item_id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetNameSpaceDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadNameSpace(UserType* output) {
    auto* pointer = data_->name_space.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::AvailableOfflineContentProvider_LaunchItem_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AvailableOfflineContentProvider_LaunchDownloadsPage_ParamsDataView {
 public:
  AvailableOfflineContentProvider_LaunchDownloadsPage_ParamsDataView() {}

  AvailableOfflineContentProvider_LaunchDownloadsPage_ParamsDataView(
      internal::AvailableOfflineContentProvider_LaunchDownloadsPage_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool open_prefetched_articles_tab() const {
    return data_->open_prefetched_articles_tab;
  }
 private:
  internal::AvailableOfflineContentProvider_LaunchDownloadsPage_Params_Data* data_ = nullptr;
};

class AvailableOfflineContentProvider_ListVisibilityChanged_ParamsDataView {
 public:
  AvailableOfflineContentProvider_ListVisibilityChanged_ParamsDataView() {}

  AvailableOfflineContentProvider_ListVisibilityChanged_ParamsDataView(
      internal::AvailableOfflineContentProvider_ListVisibilityChanged_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool is_visible() const {
    return data_->is_visible;
  }
 private:
  internal::AvailableOfflineContentProvider_ListVisibilityChanged_Params_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace chrome

namespace std {

template <>
struct hash<::chrome::mojom::AvailableContentType>
    : public mojo::internal::EnumHashImpl<::chrome::mojom::AvailableContentType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::chrome::mojom::AvailableContentType, ::chrome::mojom::AvailableContentType> {
  static ::chrome::mojom::AvailableContentType ToMojom(::chrome::mojom::AvailableContentType input) { return input; }
  static bool FromMojom(::chrome::mojom::AvailableContentType input, ::chrome::mojom::AvailableContentType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::chrome::mojom::AvailableContentType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::chrome::mojom::AvailableContentType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::chrome::mojom::AvailableContentType>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::chrome::mojom::AvailableOfflineContentDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::chrome::mojom::AvailableOfflineContentDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::chrome::mojom::internal::AvailableOfflineContent_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::id, input, custom_context)) in_id = CallWithContext(Traits::id, input, custom_context);
    typename decltype((*output)->id)::BaseType::BufferWriter
        id_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_id, buffer, &id_writer, context);
    (*output)->id.Set(
        id_writer.is_null() ? nullptr : id_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null id in AvailableOfflineContent struct");
    decltype(CallWithContext(Traits::name_space, input, custom_context)) in_name_space = CallWithContext(Traits::name_space, input, custom_context);
    typename decltype((*output)->name_space)::BaseType::BufferWriter
        name_space_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_name_space, buffer, &name_space_writer, context);
    (*output)->name_space.Set(
        name_space_writer.is_null() ? nullptr : name_space_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->name_space.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null name_space in AvailableOfflineContent struct");
    decltype(CallWithContext(Traits::title, input, custom_context)) in_title = CallWithContext(Traits::title, input, custom_context);
    typename decltype((*output)->title)::BaseType::BufferWriter
        title_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_title, buffer, &title_writer, context);
    (*output)->title.Set(
        title_writer.is_null() ? nullptr : title_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->title.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null title in AvailableOfflineContent struct");
    decltype(CallWithContext(Traits::snippet, input, custom_context)) in_snippet = CallWithContext(Traits::snippet, input, custom_context);
    typename decltype((*output)->snippet)::BaseType::BufferWriter
        snippet_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_snippet, buffer, &snippet_writer, context);
    (*output)->snippet.Set(
        snippet_writer.is_null() ? nullptr : snippet_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->snippet.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null snippet in AvailableOfflineContent struct");
    decltype(CallWithContext(Traits::date_modified, input, custom_context)) in_date_modified = CallWithContext(Traits::date_modified, input, custom_context);
    typename decltype((*output)->date_modified)::BaseType::BufferWriter
        date_modified_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_date_modified, buffer, &date_modified_writer, context);
    (*output)->date_modified.Set(
        date_modified_writer.is_null() ? nullptr : date_modified_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->date_modified.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null date_modified in AvailableOfflineContent struct");
    decltype(CallWithContext(Traits::attribution, input, custom_context)) in_attribution = CallWithContext(Traits::attribution, input, custom_context);
    typename decltype((*output)->attribution)::BaseType::BufferWriter
        attribution_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_attribution, buffer, &attribution_writer, context);
    (*output)->attribution.Set(
        attribution_writer.is_null() ? nullptr : attribution_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->attribution.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null attribution in AvailableOfflineContent struct");
    decltype(CallWithContext(Traits::thumbnail_data_uri, input, custom_context)) in_thumbnail_data_uri = CallWithContext(Traits::thumbnail_data_uri, input, custom_context);
    typename decltype((*output)->thumbnail_data_uri)::BaseType::BufferWriter
        thumbnail_data_uri_writer;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_thumbnail_data_uri, buffer, &thumbnail_data_uri_writer, context);
    (*output)->thumbnail_data_uri.Set(
        thumbnail_data_uri_writer.is_null() ? nullptr : thumbnail_data_uri_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->thumbnail_data_uri.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null thumbnail_data_uri in AvailableOfflineContent struct");
    mojo::internal::Serialize<::chrome::mojom::AvailableContentType>(
        CallWithContext(Traits::content_type, input, custom_context), &(*output)->content_type);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::chrome::mojom::internal::AvailableOfflineContent_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::chrome::mojom::AvailableOfflineContentDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::chrome::mojom::AvailableOfflineContentSummaryDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::chrome::mojom::AvailableOfflineContentSummaryDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::chrome::mojom::internal::AvailableOfflineContentSummary_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->total_items = CallWithContext(Traits::total_items, input, custom_context);
    (*output)->has_prefetched_page = CallWithContext(Traits::has_prefetched_page, input, custom_context);
    (*output)->has_offline_page = CallWithContext(Traits::has_offline_page, input, custom_context);
    (*output)->has_video = CallWithContext(Traits::has_video, input, custom_context);
    (*output)->has_audio = CallWithContext(Traits::has_audio, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::chrome::mojom::internal::AvailableOfflineContentSummary_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::chrome::mojom::AvailableOfflineContentSummaryDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace chrome {
namespace mojom {

inline void AvailableOfflineContentDataView::GetIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AvailableOfflineContentDataView::GetNameSpaceDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->name_space.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AvailableOfflineContentDataView::GetTitleDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->title.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AvailableOfflineContentDataView::GetSnippetDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->snippet.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AvailableOfflineContentDataView::GetDateModifiedDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->date_modified.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AvailableOfflineContentDataView::GetAttributionDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->attribution.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AvailableOfflineContentDataView::GetThumbnailDataUriDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->thumbnail_data_uri.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}






inline void AvailableOfflineContentProvider_List_ResponseParamsDataView::GetOutDataView(
    mojo::ArrayDataView<AvailableOfflineContentDataView>* output) {
  auto pointer = data_->out.Get();
  *output = mojo::ArrayDataView<AvailableOfflineContentDataView>(pointer, context_);
}




inline void AvailableOfflineContentProvider_Summarize_ResponseParamsDataView::GetOutDataView(
    AvailableOfflineContentSummaryDataView* output) {
  auto pointer = data_->out.Get();
  *output = AvailableOfflineContentSummaryDataView(pointer, context_);
}


inline void AvailableOfflineContentProvider_LaunchItem_ParamsDataView::GetItemIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->item_id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AvailableOfflineContentProvider_LaunchItem_ParamsDataView::GetNameSpaceDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->name_space.Get();
  *output = mojo::StringDataView(pointer, context_);
}







}  // namespace mojom
}  // namespace chrome

#endif  // CHROME_COMMON_AVAILABLE_OFFLINE_CONTENT_MOJOM_SHARED_H_
