// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_COMMON_NETWORK_EASTER_EGG_MOJOM_H_
#define CHROME_COMMON_NETWORK_EASTER_EGG_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/common/network_easter_egg.mojom-shared.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace chrome {
namespace mojom {
class NetworkEasterEgg;
using NetworkEasterEggPtr = mojo::InterfacePtr<NetworkEasterEgg>;
using NetworkEasterEggPtrInfo = mojo::InterfacePtrInfo<NetworkEasterEgg>;
using ThreadSafeNetworkEasterEggPtr =
    mojo::ThreadSafeInterfacePtr<NetworkEasterEgg>;
using NetworkEasterEggRequest = mojo::InterfaceRequest<NetworkEasterEgg>;
using NetworkEasterEggAssociatedPtr =
    mojo::AssociatedInterfacePtr<NetworkEasterEgg>;
using ThreadSafeNetworkEasterEggAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<NetworkEasterEgg>;
using NetworkEasterEggAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<NetworkEasterEgg>;
using NetworkEasterEggAssociatedRequest =
    mojo::AssociatedInterfaceRequest<NetworkEasterEgg>;


class NetworkEasterEggProxy;

template <typename ImplRefTraits>
class NetworkEasterEggStub;

class NetworkEasterEggRequestValidator;
class NetworkEasterEggResponseValidator;

class  NetworkEasterEgg
    : public NetworkEasterEggInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = NetworkEasterEggProxy;

  template <typename ImplRefTraits>
  using Stub_ = NetworkEasterEggStub<ImplRefTraits>;

  using RequestValidator_ = NetworkEasterEggRequestValidator;
  using ResponseValidator_ = NetworkEasterEggResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetHighScoreMinVersion = 0,
    kUpdateHighScoreMinVersion = 0,
  };
  virtual ~NetworkEasterEgg() {}


  using GetHighScoreCallback = base::OnceCallback<void(uint32_t)>;
  virtual void GetHighScore(GetHighScoreCallback callback) = 0;

  virtual void UpdateHighScore(uint32_t high_score) = 0;
};
class  NetworkEasterEggInterceptorForTesting : public NetworkEasterEgg {
  virtual NetworkEasterEgg* GetForwardingInterface() = 0;
  void GetHighScore(GetHighScoreCallback callback) override;
  void UpdateHighScore(uint32_t high_score) override;
};
class  NetworkEasterEggAsyncWaiter {
 public:
  explicit NetworkEasterEggAsyncWaiter(NetworkEasterEgg* proxy);
  ~NetworkEasterEggAsyncWaiter();
  void GetHighScore(
      uint32_t* out_high_score);

 private:
  NetworkEasterEgg* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(NetworkEasterEggAsyncWaiter);
};

class  NetworkEasterEggProxy
    : public NetworkEasterEgg {
 public:
  explicit NetworkEasterEggProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetHighScore(GetHighScoreCallback callback) final;
  void UpdateHighScore(uint32_t high_score) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  NetworkEasterEggStubDispatch {
 public:
  static bool Accept(NetworkEasterEgg* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      NetworkEasterEgg* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<NetworkEasterEgg>>
class NetworkEasterEggStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  NetworkEasterEggStub() {}
  ~NetworkEasterEggStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NetworkEasterEggStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NetworkEasterEggStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  NetworkEasterEggRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  NetworkEasterEggResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace chrome

namespace mojo {

}  // namespace mojo

#endif  // CHROME_COMMON_NETWORK_EASTER_EGG_MOJOM_H_