// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_COMMON_RENDERER_CONFIGURATION_MOJOM_SHARED_H_
#define CHROME_COMMON_RENDERER_CONFIGURATION_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "chrome/common/renderer_configuration.mojom-shared-internal.h"
#include "components/content_settings/core/common/content_settings.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace chrome {
namespace mojom {
class DynamicParamsDataView;



}  // namespace mojom
}  // namespace chrome

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::chrome::mojom::DynamicParamsDataView> {
  using Data = ::chrome::mojom::internal::DynamicParams_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace chrome {
namespace mojom {
// Interface base classes. They are used for type safety check.
class RendererConfigurationInterfaceBase {};

using RendererConfigurationPtrDataView =
    mojo::InterfacePtrDataView<RendererConfigurationInterfaceBase>;
using RendererConfigurationRequestDataView =
    mojo::InterfaceRequestDataView<RendererConfigurationInterfaceBase>;
using RendererConfigurationAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<RendererConfigurationInterfaceBase>;
using RendererConfigurationAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<RendererConfigurationInterfaceBase>;
class DynamicParamsDataView {
 public:
  DynamicParamsDataView() {}

  DynamicParamsDataView(
      internal::DynamicParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  bool force_safe_search() const {
    return data_->force_safe_search;
  }
  int32_t youtube_restrict() const {
    return data_->youtube_restrict;
  }
  inline void GetAllowedDomainsForAppsDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAllowedDomainsForApps(UserType* output) {
    auto* pointer = data_->allowed_domains_for_apps.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetVariationIdsHeaderDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadVariationIdsHeader(UserType* output) {
    auto* pointer = data_->variation_ids_header.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::DynamicParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class RendererConfiguration_SetInitialConfiguration_ParamsDataView {
 public:
  RendererConfiguration_SetInitialConfiguration_ParamsDataView() {}

  RendererConfiguration_SetInitialConfiguration_ParamsDataView(
      internal::RendererConfiguration_SetInitialConfiguration_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool is_incognito_process() const {
    return data_->is_incognito_process;
  }
 private:
  internal::RendererConfiguration_SetInitialConfiguration_Params_Data* data_ = nullptr;
};

class RendererConfiguration_SetConfiguration_ParamsDataView {
 public:
  RendererConfiguration_SetConfiguration_ParamsDataView() {}

  RendererConfiguration_SetConfiguration_ParamsDataView(
      internal::RendererConfiguration_SetConfiguration_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetParamsDataView(
      DynamicParamsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadParams(UserType* output) {
    auto* pointer = data_->params.Get();
    return mojo::internal::Deserialize<::chrome::mojom::DynamicParamsDataView>(
        pointer, output, context_);
  }
 private:
  internal::RendererConfiguration_SetConfiguration_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class RendererConfiguration_SetContentSettingRules_ParamsDataView {
 public:
  RendererConfiguration_SetContentSettingRules_ParamsDataView() {}

  RendererConfiguration_SetContentSettingRules_ParamsDataView(
      internal::RendererConfiguration_SetContentSettingRules_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRulesDataView(
      ::content_settings::mojom::RendererContentSettingRulesDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRules(UserType* output) {
    auto* pointer = data_->rules.Get();
    return mojo::internal::Deserialize<::content_settings::mojom::RendererContentSettingRulesDataView>(
        pointer, output, context_);
  }
 private:
  internal::RendererConfiguration_SetContentSettingRules_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class RendererConfiguration_SetFieldTrialGroup_ParamsDataView {
 public:
  RendererConfiguration_SetFieldTrialGroup_ParamsDataView() {}

  RendererConfiguration_SetFieldTrialGroup_ParamsDataView(
      internal::RendererConfiguration_SetFieldTrialGroup_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetTrialNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTrialName(UserType* output) {
    auto* pointer = data_->trial_name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetGroupNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadGroupName(UserType* output) {
    auto* pointer = data_->group_name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::RendererConfiguration_SetFieldTrialGroup_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace chrome

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::chrome::mojom::DynamicParamsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::chrome::mojom::DynamicParamsDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::chrome::mojom::internal::DynamicParams_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->force_safe_search = CallWithContext(Traits::force_safe_search, input, custom_context);
    (*output)->youtube_restrict = CallWithContext(Traits::youtube_restrict, input, custom_context);
    decltype(CallWithContext(Traits::allowed_domains_for_apps, input, custom_context)) in_allowed_domains_for_apps = CallWithContext(Traits::allowed_domains_for_apps, input, custom_context);
    typename decltype((*output)->allowed_domains_for_apps)::BaseType::BufferWriter
        allowed_domains_for_apps_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_allowed_domains_for_apps, buffer, &allowed_domains_for_apps_writer, context);
    (*output)->allowed_domains_for_apps.Set(
        allowed_domains_for_apps_writer.is_null() ? nullptr : allowed_domains_for_apps_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->allowed_domains_for_apps.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null allowed_domains_for_apps in DynamicParams struct");
    decltype(CallWithContext(Traits::variation_ids_header, input, custom_context)) in_variation_ids_header = CallWithContext(Traits::variation_ids_header, input, custom_context);
    typename decltype((*output)->variation_ids_header)::BaseType::BufferWriter
        variation_ids_header_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_variation_ids_header, buffer, &variation_ids_header_writer, context);
    (*output)->variation_ids_header.Set(
        variation_ids_header_writer.is_null() ? nullptr : variation_ids_header_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->variation_ids_header.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null variation_ids_header in DynamicParams struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::chrome::mojom::internal::DynamicParams_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::chrome::mojom::DynamicParamsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace chrome {
namespace mojom {

inline void DynamicParamsDataView::GetAllowedDomainsForAppsDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->allowed_domains_for_apps.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void DynamicParamsDataView::GetVariationIdsHeaderDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->variation_ids_header.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void RendererConfiguration_SetConfiguration_ParamsDataView::GetParamsDataView(
    DynamicParamsDataView* output) {
  auto pointer = data_->params.Get();
  *output = DynamicParamsDataView(pointer, context_);
}


inline void RendererConfiguration_SetContentSettingRules_ParamsDataView::GetRulesDataView(
    ::content_settings::mojom::RendererContentSettingRulesDataView* output) {
  auto pointer = data_->rules.Get();
  *output = ::content_settings::mojom::RendererContentSettingRulesDataView(pointer, context_);
}


inline void RendererConfiguration_SetFieldTrialGroup_ParamsDataView::GetTrialNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->trial_name.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void RendererConfiguration_SetFieldTrialGroup_ParamsDataView::GetGroupNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->group_name.Get();
  *output = mojo::StringDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace chrome

#endif  // CHROME_COMMON_RENDERER_CONFIGURATION_MOJOM_SHARED_H_
