// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_APP_SERVICE_MOJOM_SHARED_H_
#define CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_APP_SERVICE_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "chrome/services/app_service/public/mojom/app_service.mojom-shared-internal.h"
#include "chrome/services/app_service/public/mojom/types.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace apps {
namespace mojom {
class ConnectOptionsDataView;



}  // namespace mojom
}  // namespace apps

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::apps::mojom::ConnectOptionsDataView> {
  using Data = ::apps::mojom::internal::ConnectOptions_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace apps {
namespace mojom {
// Interface base classes. They are used for type safety check.
class AppServiceInterfaceBase {};

using AppServicePtrDataView =
    mojo::InterfacePtrDataView<AppServiceInterfaceBase>;
using AppServiceRequestDataView =
    mojo::InterfaceRequestDataView<AppServiceInterfaceBase>;
using AppServiceAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<AppServiceInterfaceBase>;
using AppServiceAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<AppServiceInterfaceBase>;
class PublisherInterfaceBase {};

using PublisherPtrDataView =
    mojo::InterfacePtrDataView<PublisherInterfaceBase>;
using PublisherRequestDataView =
    mojo::InterfaceRequestDataView<PublisherInterfaceBase>;
using PublisherAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<PublisherInterfaceBase>;
using PublisherAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<PublisherInterfaceBase>;
class SubscriberInterfaceBase {};

using SubscriberPtrDataView =
    mojo::InterfacePtrDataView<SubscriberInterfaceBase>;
using SubscriberRequestDataView =
    mojo::InterfaceRequestDataView<SubscriberInterfaceBase>;
using SubscriberAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<SubscriberInterfaceBase>;
using SubscriberAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<SubscriberInterfaceBase>;
class ConnectOptionsDataView {
 public:
  ConnectOptionsDataView() {}

  ConnectOptionsDataView(
      internal::ConnectOptions_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::ConnectOptions_Data* data_ = nullptr;
};

class AppService_RegisterPublisher_ParamsDataView {
 public:
  AppService_RegisterPublisher_ParamsDataView() {}

  AppService_RegisterPublisher_ParamsDataView(
      internal::AppService_RegisterPublisher_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakePublisher() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::apps::mojom::PublisherPtrDataView>(
            &data_->publisher, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAppType(UserType* output) const {
    auto data_value = data_->app_type;
    return mojo::internal::Deserialize<::apps::mojom::AppType>(
        data_value, output);
  }

  ::apps::mojom::AppType app_type() const {
    return static_cast<::apps::mojom::AppType>(data_->app_type);
  }
 private:
  internal::AppService_RegisterPublisher_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AppService_RegisterSubscriber_ParamsDataView {
 public:
  AppService_RegisterSubscriber_ParamsDataView() {}

  AppService_RegisterSubscriber_ParamsDataView(
      internal::AppService_RegisterSubscriber_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeSubscriber() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::apps::mojom::SubscriberPtrDataView>(
            &data_->subscriber, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetOptsDataView(
      ConnectOptionsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOpts(UserType* output) {
    auto* pointer = data_->opts.Get();
    return mojo::internal::Deserialize<::apps::mojom::ConnectOptionsDataView>(
        pointer, output, context_);
  }
 private:
  internal::AppService_RegisterSubscriber_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AppService_LoadIcon_ParamsDataView {
 public:
  AppService_LoadIcon_ParamsDataView() {}

  AppService_LoadIcon_ParamsDataView(
      internal::AppService_LoadIcon_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAppType(UserType* output) const {
    auto data_value = data_->app_type;
    return mojo::internal::Deserialize<::apps::mojom::AppType>(
        data_value, output);
  }

  ::apps::mojom::AppType app_type() const {
    return static_cast<::apps::mojom::AppType>(data_->app_type);
  }
  inline void GetAppIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAppId(UserType* output) {
    auto* pointer = data_->app_id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetIconKeyDataView(
      ::apps::mojom::IconKeyDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIconKey(UserType* output) {
    auto* pointer = data_->icon_key.Get();
    return mojo::internal::Deserialize<::apps::mojom::IconKeyDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIconCompression(UserType* output) const {
    auto data_value = data_->icon_compression;
    return mojo::internal::Deserialize<::apps::mojom::IconCompression>(
        data_value, output);
  }

  ::apps::mojom::IconCompression icon_compression() const {
    return static_cast<::apps::mojom::IconCompression>(data_->icon_compression);
  }
  int32_t size_hint_in_dip() const {
    return data_->size_hint_in_dip;
  }
 private:
  internal::AppService_LoadIcon_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AppService_LoadIcon_ResponseParamsDataView {
 public:
  AppService_LoadIcon_ResponseParamsDataView() {}

  AppService_LoadIcon_ResponseParamsDataView(
      internal::AppService_LoadIcon_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetIconValueDataView(
      ::apps::mojom::IconValueDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIconValue(UserType* output) {
    auto* pointer = data_->icon_value.Get();
    return mojo::internal::Deserialize<::apps::mojom::IconValueDataView>(
        pointer, output, context_);
  }
 private:
  internal::AppService_LoadIcon_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AppService_Launch_ParamsDataView {
 public:
  AppService_Launch_ParamsDataView() {}

  AppService_Launch_ParamsDataView(
      internal::AppService_Launch_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAppType(UserType* output) const {
    auto data_value = data_->app_type;
    return mojo::internal::Deserialize<::apps::mojom::AppType>(
        data_value, output);
  }

  ::apps::mojom::AppType app_type() const {
    return static_cast<::apps::mojom::AppType>(data_->app_type);
  }
  inline void GetAppIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAppId(UserType* output) {
    auto* pointer = data_->app_id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  int32_t event_flags() const {
    return data_->event_flags;
  }
 private:
  internal::AppService_Launch_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Publisher_Connect_ParamsDataView {
 public:
  Publisher_Connect_ParamsDataView() {}

  Publisher_Connect_ParamsDataView(
      internal::Publisher_Connect_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeSubscriber() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::apps::mojom::SubscriberPtrDataView>(
            &data_->subscriber, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetOptsDataView(
      ConnectOptionsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOpts(UserType* output) {
    auto* pointer = data_->opts.Get();
    return mojo::internal::Deserialize<::apps::mojom::ConnectOptionsDataView>(
        pointer, output, context_);
  }
 private:
  internal::Publisher_Connect_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Publisher_LoadIcon_ParamsDataView {
 public:
  Publisher_LoadIcon_ParamsDataView() {}

  Publisher_LoadIcon_ParamsDataView(
      internal::Publisher_LoadIcon_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetAppIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAppId(UserType* output) {
    auto* pointer = data_->app_id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetIconKeyDataView(
      ::apps::mojom::IconKeyDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIconKey(UserType* output) {
    auto* pointer = data_->icon_key.Get();
    return mojo::internal::Deserialize<::apps::mojom::IconKeyDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIconCompression(UserType* output) const {
    auto data_value = data_->icon_compression;
    return mojo::internal::Deserialize<::apps::mojom::IconCompression>(
        data_value, output);
  }

  ::apps::mojom::IconCompression icon_compression() const {
    return static_cast<::apps::mojom::IconCompression>(data_->icon_compression);
  }
  int32_t size_hint_in_dip() const {
    return data_->size_hint_in_dip;
  }
 private:
  internal::Publisher_LoadIcon_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Publisher_LoadIcon_ResponseParamsDataView {
 public:
  Publisher_LoadIcon_ResponseParamsDataView() {}

  Publisher_LoadIcon_ResponseParamsDataView(
      internal::Publisher_LoadIcon_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetIconValueDataView(
      ::apps::mojom::IconValueDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIconValue(UserType* output) {
    auto* pointer = data_->icon_value.Get();
    return mojo::internal::Deserialize<::apps::mojom::IconValueDataView>(
        pointer, output, context_);
  }
 private:
  internal::Publisher_LoadIcon_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Publisher_Launch_ParamsDataView {
 public:
  Publisher_Launch_ParamsDataView() {}

  Publisher_Launch_ParamsDataView(
      internal::Publisher_Launch_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetAppIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAppId(UserType* output) {
    auto* pointer = data_->app_id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  int32_t event_flags() const {
    return data_->event_flags;
  }
 private:
  internal::Publisher_Launch_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Subscriber_OnApps_ParamsDataView {
 public:
  Subscriber_OnApps_ParamsDataView() {}

  Subscriber_OnApps_ParamsDataView(
      internal::Subscriber_OnApps_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetDeltasDataView(
      mojo::ArrayDataView<::apps::mojom::AppDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDeltas(UserType* output) {
    auto* pointer = data_->deltas.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::apps::mojom::AppDataView>>(
        pointer, output, context_);
  }
 private:
  internal::Subscriber_OnApps_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Subscriber_Clone_ParamsDataView {
 public:
  Subscriber_Clone_ParamsDataView() {}

  Subscriber_Clone_ParamsDataView(
      internal::Subscriber_Clone_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::apps::mojom::SubscriberRequestDataView>(
            &data_->request, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::Subscriber_Clone_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace apps

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::ConnectOptionsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::apps::mojom::ConnectOptionsDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::apps::mojom::internal::ConnectOptions_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::apps::mojom::internal::ConnectOptions_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::apps::mojom::ConnectOptionsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace apps {
namespace mojom {





inline void AppService_RegisterSubscriber_ParamsDataView::GetOptsDataView(
    ConnectOptionsDataView* output) {
  auto pointer = data_->opts.Get();
  *output = ConnectOptionsDataView(pointer, context_);
}


inline void AppService_LoadIcon_ParamsDataView::GetAppIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->app_id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AppService_LoadIcon_ParamsDataView::GetIconKeyDataView(
    ::apps::mojom::IconKeyDataView* output) {
  auto pointer = data_->icon_key.Get();
  *output = ::apps::mojom::IconKeyDataView(pointer, context_);
}


inline void AppService_LoadIcon_ResponseParamsDataView::GetIconValueDataView(
    ::apps::mojom::IconValueDataView* output) {
  auto pointer = data_->icon_value.Get();
  *output = ::apps::mojom::IconValueDataView(pointer, context_);
}


inline void AppService_Launch_ParamsDataView::GetAppIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->app_id.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void Publisher_Connect_ParamsDataView::GetOptsDataView(
    ConnectOptionsDataView* output) {
  auto pointer = data_->opts.Get();
  *output = ConnectOptionsDataView(pointer, context_);
}


inline void Publisher_LoadIcon_ParamsDataView::GetAppIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->app_id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void Publisher_LoadIcon_ParamsDataView::GetIconKeyDataView(
    ::apps::mojom::IconKeyDataView* output) {
  auto pointer = data_->icon_key.Get();
  *output = ::apps::mojom::IconKeyDataView(pointer, context_);
}


inline void Publisher_LoadIcon_ResponseParamsDataView::GetIconValueDataView(
    ::apps::mojom::IconValueDataView* output) {
  auto pointer = data_->icon_value.Get();
  *output = ::apps::mojom::IconValueDataView(pointer, context_);
}


inline void Publisher_Launch_ParamsDataView::GetAppIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->app_id.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void Subscriber_OnApps_ParamsDataView::GetDeltasDataView(
    mojo::ArrayDataView<::apps::mojom::AppDataView>* output) {
  auto pointer = data_->deltas.Get();
  *output = mojo::ArrayDataView<::apps::mojom::AppDataView>(pointer, context_);
}





}  // namespace mojom
}  // namespace apps

#endif  // CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_APP_SERVICE_MOJOM_SHARED_H_
