// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_SHARED_H_
#define CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "chrome/services/app_service/public/mojom/types.mojom-shared-internal.h"
#include "ui/gfx/image/mojo/image.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace apps {
namespace mojom {
class AppDataView;

class IconKeyDataView;

class IconValueDataView;



}  // namespace mojom
}  // namespace apps

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::apps::mojom::AppDataView> {
  using Data = ::apps::mojom::internal::App_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::apps::mojom::IconKeyDataView> {
  using Data = ::apps::mojom::internal::IconKey_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::apps::mojom::IconValueDataView> {
  using Data = ::apps::mojom::internal::IconValue_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace apps {
namespace mojom {

enum class AppType : int32_t {
  kUnknown = 0,
  kArc,
  kBuiltIn,
  kCrostini,
  kWeb,
  kMinValue = 0,
  kMaxValue = 4,
};

inline std::ostream& operator<<(std::ostream& os, AppType value) {
  switch(value) {
    case AppType::kUnknown:
      return os << "AppType::kUnknown";
    case AppType::kArc:
      return os << "AppType::kArc";
    case AppType::kBuiltIn:
      return os << "AppType::kBuiltIn";
    case AppType::kCrostini:
      return os << "AppType::kCrostini";
    case AppType::kWeb:
      return os << "AppType::kWeb";
    default:
      return os << "Unknown AppType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(AppType value) {
  return internal::AppType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class Readiness : int32_t {
  kUnknown = 0,
  kReady,
  kDisabledByBlacklist,
  kDisabledByPolicy,
  kDisabledByUser,
  kUninstalledByUser,
  kMinValue = 0,
  kMaxValue = 5,
};

inline std::ostream& operator<<(std::ostream& os, Readiness value) {
  switch(value) {
    case Readiness::kUnknown:
      return os << "Readiness::kUnknown";
    case Readiness::kReady:
      return os << "Readiness::kReady";
    case Readiness::kDisabledByBlacklist:
      return os << "Readiness::kDisabledByBlacklist";
    case Readiness::kDisabledByPolicy:
      return os << "Readiness::kDisabledByPolicy";
    case Readiness::kDisabledByUser:
      return os << "Readiness::kDisabledByUser";
    case Readiness::kUninstalledByUser:
      return os << "Readiness::kUninstalledByUser";
    default:
      return os << "Unknown Readiness value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(Readiness value) {
  return internal::Readiness_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class OptionalBool : int32_t {
  kUnknown = 0,
  kFalse,
  kTrue,
  kMinValue = 0,
  kMaxValue = 2,
};

inline std::ostream& operator<<(std::ostream& os, OptionalBool value) {
  switch(value) {
    case OptionalBool::kUnknown:
      return os << "OptionalBool::kUnknown";
    case OptionalBool::kFalse:
      return os << "OptionalBool::kFalse";
    case OptionalBool::kTrue:
      return os << "OptionalBool::kTrue";
    default:
      return os << "Unknown OptionalBool value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(OptionalBool value) {
  return internal::OptionalBool_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class IconType : int32_t {
  kUnknown,
  kExtension,
  kResource,
  kMinValue = 0,
  kMaxValue = 2,
};

inline std::ostream& operator<<(std::ostream& os, IconType value) {
  switch(value) {
    case IconType::kUnknown:
      return os << "IconType::kUnknown";
    case IconType::kExtension:
      return os << "IconType::kExtension";
    case IconType::kResource:
      return os << "IconType::kResource";
    default:
      return os << "Unknown IconType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(IconType value) {
  return internal::IconType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class IconCompression : int32_t {
  kUnknown,
  kUncompressed,
  kCompressed,
  kMinValue = 0,
  kMaxValue = 2,
};

inline std::ostream& operator<<(std::ostream& os, IconCompression value) {
  switch(value) {
    case IconCompression::kUnknown:
      return os << "IconCompression::kUnknown";
    case IconCompression::kUncompressed:
      return os << "IconCompression::kUncompressed";
    case IconCompression::kCompressed:
      return os << "IconCompression::kCompressed";
    default:
      return os << "Unknown IconCompression value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(IconCompression value) {
  return internal::IconCompression_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
class AppDataView {
 public:
  AppDataView() {}

  AppDataView(
      internal::App_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAppType(UserType* output) const {
    auto data_value = data_->app_type;
    return mojo::internal::Deserialize<::apps::mojom::AppType>(
        data_value, output);
  }

  AppType app_type() const {
    return static_cast<AppType>(data_->app_type);
  }
  inline void GetAppIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAppId(UserType* output) {
    auto* pointer = data_->app_id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadReadiness(UserType* output) const {
    auto data_value = data_->readiness;
    return mojo::internal::Deserialize<::apps::mojom::Readiness>(
        data_value, output);
  }

  Readiness readiness() const {
    return static_cast<Readiness>(data_->readiness);
  }
  inline void GetNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadName(UserType* output) {
    auto* pointer = data_->name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetIconKeyDataView(
      IconKeyDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIconKey(UserType* output) {
    auto* pointer = data_->icon_key.Get();
    return mojo::internal::Deserialize<::apps::mojom::IconKeyDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadShowInLauncher(UserType* output) const {
    auto data_value = data_->show_in_launcher;
    return mojo::internal::Deserialize<::apps::mojom::OptionalBool>(
        data_value, output);
  }

  OptionalBool show_in_launcher() const {
    return static_cast<OptionalBool>(data_->show_in_launcher);
  }
 private:
  internal::App_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class IconKeyDataView {
 public:
  IconKeyDataView() {}

  IconKeyDataView(
      internal::IconKey_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIconType(UserType* output) const {
    auto data_value = data_->icon_type;
    return mojo::internal::Deserialize<::apps::mojom::IconType>(
        data_value, output);
  }

  IconType icon_type() const {
    return static_cast<IconType>(data_->icon_type);
  }
  uint64_t u_key() const {
    return data_->u_key;
  }
  inline void GetSKeyDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSKey(UserType* output) {
    auto* pointer = data_->s_key.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::IconKey_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class IconValueDataView {
 public:
  IconValueDataView() {}

  IconValueDataView(
      internal::IconValue_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIconCompression(UserType* output) const {
    auto data_value = data_->icon_compression;
    return mojo::internal::Deserialize<::apps::mojom::IconCompression>(
        data_value, output);
  }

  IconCompression icon_compression() const {
    return static_cast<IconCompression>(data_->icon_compression);
  }
  inline void GetUncompressedDataView(
      ::gfx::mojom::ImageSkiaDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUncompressed(UserType* output) {
    auto* pointer = data_->uncompressed.Get();
    return mojo::internal::Deserialize<::gfx::mojom::ImageSkiaDataView>(
        pointer, output, context_);
  }
  inline void GetCompressedDataView(
      mojo::ArrayDataView<uint8_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCompressed(UserType* output) {
    auto* pointer = data_->compressed.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint8_t>>(
        pointer, output, context_);
  }
 private:
  internal::IconValue_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace apps

namespace std {

template <>
struct hash<::apps::mojom::AppType>
    : public mojo::internal::EnumHashImpl<::apps::mojom::AppType> {};

template <>
struct hash<::apps::mojom::Readiness>
    : public mojo::internal::EnumHashImpl<::apps::mojom::Readiness> {};

template <>
struct hash<::apps::mojom::OptionalBool>
    : public mojo::internal::EnumHashImpl<::apps::mojom::OptionalBool> {};

template <>
struct hash<::apps::mojom::IconType>
    : public mojo::internal::EnumHashImpl<::apps::mojom::IconType> {};

template <>
struct hash<::apps::mojom::IconCompression>
    : public mojo::internal::EnumHashImpl<::apps::mojom::IconCompression> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::apps::mojom::AppType, ::apps::mojom::AppType> {
  static ::apps::mojom::AppType ToMojom(::apps::mojom::AppType input) { return input; }
  static bool FromMojom(::apps::mojom::AppType input, ::apps::mojom::AppType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::AppType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::AppType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::AppType>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::apps::mojom::Readiness, ::apps::mojom::Readiness> {
  static ::apps::mojom::Readiness ToMojom(::apps::mojom::Readiness input) { return input; }
  static bool FromMojom(::apps::mojom::Readiness input, ::apps::mojom::Readiness* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::Readiness, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::Readiness, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::Readiness>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::apps::mojom::OptionalBool, ::apps::mojom::OptionalBool> {
  static ::apps::mojom::OptionalBool ToMojom(::apps::mojom::OptionalBool input) { return input; }
  static bool FromMojom(::apps::mojom::OptionalBool input, ::apps::mojom::OptionalBool* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::OptionalBool, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::OptionalBool, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::OptionalBool>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::apps::mojom::IconType, ::apps::mojom::IconType> {
  static ::apps::mojom::IconType ToMojom(::apps::mojom::IconType input) { return input; }
  static bool FromMojom(::apps::mojom::IconType input, ::apps::mojom::IconType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::IconType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::IconType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::IconType>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::apps::mojom::IconCompression, ::apps::mojom::IconCompression> {
  static ::apps::mojom::IconCompression ToMojom(::apps::mojom::IconCompression input) { return input; }
  static bool FromMojom(::apps::mojom::IconCompression input, ::apps::mojom::IconCompression* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::IconCompression, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::apps::mojom::IconCompression, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::apps::mojom::IconCompression>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::AppDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::apps::mojom::AppDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::apps::mojom::internal::App_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    mojo::internal::Serialize<::apps::mojom::AppType>(
        CallWithContext(Traits::app_type, input, custom_context), &(*output)->app_type);
    decltype(CallWithContext(Traits::app_id, input, custom_context)) in_app_id = CallWithContext(Traits::app_id, input, custom_context);
    typename decltype((*output)->app_id)::BaseType::BufferWriter
        app_id_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_app_id, buffer, &app_id_writer, context);
    (*output)->app_id.Set(
        app_id_writer.is_null() ? nullptr : app_id_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->app_id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null app_id in App struct");
    mojo::internal::Serialize<::apps::mojom::Readiness>(
        CallWithContext(Traits::readiness, input, custom_context), &(*output)->readiness);
    decltype(CallWithContext(Traits::name, input, custom_context)) in_name = CallWithContext(Traits::name, input, custom_context);
    typename decltype((*output)->name)::BaseType::BufferWriter
        name_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_name, buffer, &name_writer, context);
    (*output)->name.Set(
        name_writer.is_null() ? nullptr : name_writer.data());
    decltype(CallWithContext(Traits::icon_key, input, custom_context)) in_icon_key = CallWithContext(Traits::icon_key, input, custom_context);
    typename decltype((*output)->icon_key)::BaseType::BufferWriter
        icon_key_writer;
    mojo::internal::Serialize<::apps::mojom::IconKeyDataView>(
        in_icon_key, buffer, &icon_key_writer, context);
    (*output)->icon_key.Set(
        icon_key_writer.is_null() ? nullptr : icon_key_writer.data());
    mojo::internal::Serialize<::apps::mojom::OptionalBool>(
        CallWithContext(Traits::show_in_launcher, input, custom_context), &(*output)->show_in_launcher);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::apps::mojom::internal::App_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::apps::mojom::AppDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::IconKeyDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::apps::mojom::IconKeyDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::apps::mojom::internal::IconKey_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    mojo::internal::Serialize<::apps::mojom::IconType>(
        CallWithContext(Traits::icon_type, input, custom_context), &(*output)->icon_type);
    (*output)->u_key = CallWithContext(Traits::u_key, input, custom_context);
    decltype(CallWithContext(Traits::s_key, input, custom_context)) in_s_key = CallWithContext(Traits::s_key, input, custom_context);
    typename decltype((*output)->s_key)::BaseType::BufferWriter
        s_key_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_s_key, buffer, &s_key_writer, context);
    (*output)->s_key.Set(
        s_key_writer.is_null() ? nullptr : s_key_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->s_key.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null s_key in IconKey struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::apps::mojom::internal::IconKey_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::apps::mojom::IconKeyDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::apps::mojom::IconValueDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::apps::mojom::IconValueDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::apps::mojom::internal::IconValue_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    mojo::internal::Serialize<::apps::mojom::IconCompression>(
        CallWithContext(Traits::icon_compression, input, custom_context), &(*output)->icon_compression);
    decltype(CallWithContext(Traits::uncompressed, input, custom_context)) in_uncompressed = CallWithContext(Traits::uncompressed, input, custom_context);
    typename decltype((*output)->uncompressed)::BaseType::BufferWriter
        uncompressed_writer;
    mojo::internal::Serialize<::gfx::mojom::ImageSkiaDataView>(
        in_uncompressed, buffer, &uncompressed_writer, context);
    (*output)->uncompressed.Set(
        uncompressed_writer.is_null() ? nullptr : uncompressed_writer.data());
    decltype(CallWithContext(Traits::compressed, input, custom_context)) in_compressed = CallWithContext(Traits::compressed, input, custom_context);
    typename decltype((*output)->compressed)::BaseType::BufferWriter
        compressed_writer;
    const mojo::internal::ContainerValidateParams compressed_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
        in_compressed, buffer, &compressed_writer, &compressed_validate_params,
        context);
    (*output)->compressed.Set(
        compressed_writer.is_null() ? nullptr : compressed_writer.data());
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::apps::mojom::internal::IconValue_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::apps::mojom::IconValueDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace apps {
namespace mojom {

inline void AppDataView::GetAppIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->app_id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AppDataView::GetNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->name.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AppDataView::GetIconKeyDataView(
    IconKeyDataView* output) {
  auto pointer = data_->icon_key.Get();
  *output = IconKeyDataView(pointer, context_);
}


inline void IconKeyDataView::GetSKeyDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->s_key.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void IconValueDataView::GetUncompressedDataView(
    ::gfx::mojom::ImageSkiaDataView* output) {
  auto pointer = data_->uncompressed.Get();
  *output = ::gfx::mojom::ImageSkiaDataView(pointer, context_);
}
inline void IconValueDataView::GetCompressedDataView(
    mojo::ArrayDataView<uint8_t>* output) {
  auto pointer = data_->compressed.Get();
  *output = mojo::ArrayDataView<uint8_t>(pointer, context_);
}



}  // namespace mojom
}  // namespace apps

#endif  // CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_SHARED_H_
