// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "chrome/services/app_service/public/mojom/types.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "chrome/services/app_service/public/mojom/types.mojom-shared-message-ids.h"
#include "mojo/public/cpp/base/big_buffer_mojom_traits.h"
#include "skia/public/interfaces/bitmap_skbitmap_struct_traits.h"
#include "skia/public/interfaces/image_info_struct_traits.h"
#include "ui/gfx/image/mojo/image_skia_struct_traits.h"
namespace apps {
namespace mojom {
App::App()
    : app_type(),
      app_id(),
      readiness(),
      name(),
      icon_key(),
      show_in_launcher() {}

App::App(
    AppType app_type_in,
    const std::string& app_id_in,
    Readiness readiness_in,
    const base::Optional<std::string>& name_in,
    IconKeyPtr icon_key_in,
    OptionalBool show_in_launcher_in)
    : app_type(std::move(app_type_in)),
      app_id(std::move(app_id_in)),
      readiness(std::move(readiness_in)),
      name(std::move(name_in)),
      icon_key(std::move(icon_key_in)),
      show_in_launcher(std::move(show_in_launcher_in)) {}

App::~App() = default;

bool App::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
IconKey::IconKey()
    : icon_type(),
      u_key(),
      s_key() {}

IconKey::IconKey(
    IconType icon_type_in,
    uint64_t u_key_in,
    const std::string& s_key_in)
    : icon_type(std::move(icon_type_in)),
      u_key(std::move(u_key_in)),
      s_key(std::move(s_key_in)) {}

IconKey::~IconKey() = default;
size_t IconKey::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->icon_type);
  seed = mojo::internal::Hash(seed, this->u_key);
  seed = mojo::internal::Hash(seed, this->s_key);
  return seed;
}

bool IconKey::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
IconValue::IconValue()
    : icon_compression(),
      uncompressed(),
      compressed() {}

IconValue::IconValue(
    IconCompression icon_compression_in,
    const gfx::ImageSkia& uncompressed_in,
    const base::Optional<std::vector<uint8_t>>& compressed_in)
    : icon_compression(std::move(icon_compression_in)),
      uncompressed(std::move(uncompressed_in)),
      compressed(std::move(compressed_in)) {}

IconValue::~IconValue() = default;

bool IconValue::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
}  // namespace mojom
}  // namespace apps

namespace mojo {


// static
bool StructTraits<::apps::mojom::App::DataView, ::apps::mojom::AppPtr>::Read(
    ::apps::mojom::App::DataView input,
    ::apps::mojom::AppPtr* output) {
  bool success = true;
  ::apps::mojom::AppPtr result(::apps::mojom::App::New());
  
      if (!input.ReadAppType(&result->app_type))
        success = false;
      if (!input.ReadAppId(&result->app_id))
        success = false;
      if (!input.ReadReadiness(&result->readiness))
        success = false;
      if (!input.ReadName(&result->name))
        success = false;
      if (!input.ReadIconKey(&result->icon_key))
        success = false;
      if (!input.ReadShowInLauncher(&result->show_in_launcher))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::apps::mojom::IconKey::DataView, ::apps::mojom::IconKeyPtr>::Read(
    ::apps::mojom::IconKey::DataView input,
    ::apps::mojom::IconKeyPtr* output) {
  bool success = true;
  ::apps::mojom::IconKeyPtr result(::apps::mojom::IconKey::New());
  
      if (!input.ReadIconType(&result->icon_type))
        success = false;
      result->u_key = input.u_key();
      if (!input.ReadSKey(&result->s_key))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::apps::mojom::IconValue::DataView, ::apps::mojom::IconValuePtr>::Read(
    ::apps::mojom::IconValue::DataView input,
    ::apps::mojom::IconValuePtr* output) {
  bool success = true;
  ::apps::mojom::IconValuePtr result(::apps::mojom::IconValue::New());
  
      if (!input.ReadIconCompression(&result->icon_compression))
        success = false;
      if (!input.ReadUncompressed(&result->uncompressed))
        success = false;
      if (!input.ReadCompressed(&result->compressed))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif