// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_SERVICES_PRINTING_PUBLIC_MOJOM_PDF_NUP_CONVERTER_MOJOM_SHARED_H_
#define CHROME_SERVICES_PRINTING_PUBLIC_MOJOM_PDF_NUP_CONVERTER_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "chrome/services/printing/public/mojom/pdf_nup_converter.mojom-shared-internal.h"
#include "mojo/public/mojom/base/shared_memory.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"
#include "url/mojom/url.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace printing {
namespace mojom {


}  // namespace mojom
}  // namespace printing

namespace mojo {
namespace internal {

}  // namespace internal
}  // namespace mojo


namespace printing {
namespace mojom {

enum class PdfNupConverter_Status : int32_t {
  SUCCESS = 0,
  CONVERSION_FAILURE = 1,
  HANDLE_MAP_ERROR = 2,
  kMinValue = 0,
  kMaxValue = 2,
};

inline std::ostream& operator<<(std::ostream& os, PdfNupConverter_Status value) {
  switch(value) {
    case PdfNupConverter_Status::SUCCESS:
      return os << "PdfNupConverter_Status::SUCCESS";
    case PdfNupConverter_Status::CONVERSION_FAILURE:
      return os << "PdfNupConverter_Status::CONVERSION_FAILURE";
    case PdfNupConverter_Status::HANDLE_MAP_ERROR:
      return os << "PdfNupConverter_Status::HANDLE_MAP_ERROR";
    default:
      return os << "Unknown PdfNupConverter_Status value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(PdfNupConverter_Status value) {
  return internal::PdfNupConverter_Status_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class PdfNupConverterInterfaceBase {};

using PdfNupConverterPtrDataView =
    mojo::InterfacePtrDataView<PdfNupConverterInterfaceBase>;
using PdfNupConverterRequestDataView =
    mojo::InterfaceRequestDataView<PdfNupConverterInterfaceBase>;
using PdfNupConverterAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<PdfNupConverterInterfaceBase>;
using PdfNupConverterAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<PdfNupConverterInterfaceBase>;
class PdfNupConverter_NupPageConvert_ParamsDataView {
 public:
  PdfNupConverter_NupPageConvert_ParamsDataView() {}

  PdfNupConverter_NupPageConvert_ParamsDataView(
      internal::PdfNupConverter_NupPageConvert_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  uint32_t pages_per_sheet() const {
    return data_->pages_per_sheet;
  }
  inline void GetPageSizeDataView(
      ::gfx::mojom::SizeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPageSize(UserType* output) {
    auto* pointer = data_->page_size.Get();
    return mojo::internal::Deserialize<::gfx::mojom::SizeDataView>(
        pointer, output, context_);
  }
  inline void GetPrintableAreaDataView(
      ::gfx::mojom::RectDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPrintableArea(UserType* output) {
    auto* pointer = data_->printable_area.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectDataView>(
        pointer, output, context_);
  }
  inline void GetPdfPageRegionsDataView(
      mojo::ArrayDataView<::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPdfPageRegions(UserType* output) {
    auto* pointer = data_->pdf_page_regions.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView>>(
        pointer, output, context_);
  }
 private:
  internal::PdfNupConverter_NupPageConvert_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PdfNupConverter_NupPageConvert_ResponseParamsDataView {
 public:
  PdfNupConverter_NupPageConvert_ResponseParamsDataView() {}

  PdfNupConverter_NupPageConvert_ResponseParamsDataView(
      internal::PdfNupConverter_NupPageConvert_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStatus(UserType* output) const {
    auto data_value = data_->status;
    return mojo::internal::Deserialize<::printing::mojom::PdfNupConverter_Status>(
        data_value, output);
  }

  PdfNupConverter_Status status() const {
    return static_cast<PdfNupConverter_Status>(data_->status);
  }
  inline void GetPdfRegionDataView(
      ::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPdfRegion(UserType* output) {
    auto* pointer = data_->pdf_region.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView>(
        pointer, output, context_);
  }
 private:
  internal::PdfNupConverter_NupPageConvert_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PdfNupConverter_NupDocumentConvert_ParamsDataView {
 public:
  PdfNupConverter_NupDocumentConvert_ParamsDataView() {}

  PdfNupConverter_NupDocumentConvert_ParamsDataView(
      internal::PdfNupConverter_NupDocumentConvert_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  uint32_t pages_per_sheet() const {
    return data_->pages_per_sheet;
  }
  inline void GetPageSizeDataView(
      ::gfx::mojom::SizeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPageSize(UserType* output) {
    auto* pointer = data_->page_size.Get();
    return mojo::internal::Deserialize<::gfx::mojom::SizeDataView>(
        pointer, output, context_);
  }
  inline void GetPrintableAreaDataView(
      ::gfx::mojom::RectDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPrintableArea(UserType* output) {
    auto* pointer = data_->printable_area.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectDataView>(
        pointer, output, context_);
  }
  inline void GetSrcPdfRegionDataView(
      ::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSrcPdfRegion(UserType* output) {
    auto* pointer = data_->src_pdf_region.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView>(
        pointer, output, context_);
  }
 private:
  internal::PdfNupConverter_NupDocumentConvert_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PdfNupConverter_NupDocumentConvert_ResponseParamsDataView {
 public:
  PdfNupConverter_NupDocumentConvert_ResponseParamsDataView() {}

  PdfNupConverter_NupDocumentConvert_ResponseParamsDataView(
      internal::PdfNupConverter_NupDocumentConvert_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStatus(UserType* output) const {
    auto data_value = data_->status;
    return mojo::internal::Deserialize<::printing::mojom::PdfNupConverter_Status>(
        data_value, output);
  }

  PdfNupConverter_Status status() const {
    return static_cast<PdfNupConverter_Status>(data_->status);
  }
  inline void GetPdfRegionDataView(
      ::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPdfRegion(UserType* output) {
    auto* pointer = data_->pdf_region.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView>(
        pointer, output, context_);
  }
 private:
  internal::PdfNupConverter_NupDocumentConvert_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PdfNupConverter_SetWebContentsURL_ParamsDataView {
 public:
  PdfNupConverter_SetWebContentsURL_ParamsDataView() {}

  PdfNupConverter_SetWebContentsURL_ParamsDataView(
      internal::PdfNupConverter_SetWebContentsURL_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::PdfNupConverter_SetWebContentsURL_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace printing

namespace std {

template <>
struct hash<::printing::mojom::PdfNupConverter_Status>
    : public mojo::internal::EnumHashImpl<::printing::mojom::PdfNupConverter_Status> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::printing::mojom::PdfNupConverter_Status, ::printing::mojom::PdfNupConverter_Status> {
  static ::printing::mojom::PdfNupConverter_Status ToMojom(::printing::mojom::PdfNupConverter_Status input) { return input; }
  static bool FromMojom(::printing::mojom::PdfNupConverter_Status input, ::printing::mojom::PdfNupConverter_Status* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::printing::mojom::PdfNupConverter_Status, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::printing::mojom::PdfNupConverter_Status, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::printing::mojom::PdfNupConverter_Status>(input), output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace printing {
namespace mojom {

inline void PdfNupConverter_NupPageConvert_ParamsDataView::GetPageSizeDataView(
    ::gfx::mojom::SizeDataView* output) {
  auto pointer = data_->page_size.Get();
  *output = ::gfx::mojom::SizeDataView(pointer, context_);
}
inline void PdfNupConverter_NupPageConvert_ParamsDataView::GetPrintableAreaDataView(
    ::gfx::mojom::RectDataView* output) {
  auto pointer = data_->printable_area.Get();
  *output = ::gfx::mojom::RectDataView(pointer, context_);
}
inline void PdfNupConverter_NupPageConvert_ParamsDataView::GetPdfPageRegionsDataView(
    mojo::ArrayDataView<::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView>* output) {
  auto pointer = data_->pdf_page_regions.Get();
  *output = mojo::ArrayDataView<::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView>(pointer, context_);
}


inline void PdfNupConverter_NupPageConvert_ResponseParamsDataView::GetPdfRegionDataView(
    ::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView* output) {
  auto pointer = data_->pdf_region.Get();
  *output = ::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView(pointer, context_);
}


inline void PdfNupConverter_NupDocumentConvert_ParamsDataView::GetPageSizeDataView(
    ::gfx::mojom::SizeDataView* output) {
  auto pointer = data_->page_size.Get();
  *output = ::gfx::mojom::SizeDataView(pointer, context_);
}
inline void PdfNupConverter_NupDocumentConvert_ParamsDataView::GetPrintableAreaDataView(
    ::gfx::mojom::RectDataView* output) {
  auto pointer = data_->printable_area.Get();
  *output = ::gfx::mojom::RectDataView(pointer, context_);
}
inline void PdfNupConverter_NupDocumentConvert_ParamsDataView::GetSrcPdfRegionDataView(
    ::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView* output) {
  auto pointer = data_->src_pdf_region.Get();
  *output = ::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView(pointer, context_);
}


inline void PdfNupConverter_NupDocumentConvert_ResponseParamsDataView::GetPdfRegionDataView(
    ::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView* output) {
  auto pointer = data_->pdf_region.Get();
  *output = ::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView(pointer, context_);
}


inline void PdfNupConverter_SetWebContentsURL_ParamsDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace printing

#endif  // CHROME_SERVICES_PRINTING_PUBLIC_MOJOM_PDF_NUP_CONVERTER_MOJOM_SHARED_H_
