// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef COMPONENTS_MIRRORING_MOJOM_MIRRORING_SERVICE_MOJOM_H_
#define COMPONENTS_MIRRORING_MOJOM_MIRRORING_SERVICE_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "components/mirroring/mojom/mirroring_service.mojom-shared.h"
#include "components/mirroring/mojom/cast_message_channel.mojom.h"
#include "components/mirroring/mojom/resource_provider.mojom.h"
#include "components/mirroring/mojom/session_observer.mojom.h"
#include "components/mirroring/mojom/session_parameters.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace mirroring {
namespace mojom {
class MirroringService;
using MirroringServicePtr = mojo::InterfacePtr<MirroringService>;
using MirroringServicePtrInfo = mojo::InterfacePtrInfo<MirroringService>;
using ThreadSafeMirroringServicePtr =
    mojo::ThreadSafeInterfacePtr<MirroringService>;
using MirroringServiceRequest = mojo::InterfaceRequest<MirroringService>;
using MirroringServiceAssociatedPtr =
    mojo::AssociatedInterfacePtr<MirroringService>;
using ThreadSafeMirroringServiceAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<MirroringService>;
using MirroringServiceAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<MirroringService>;
using MirroringServiceAssociatedRequest =
    mojo::AssociatedInterfaceRequest<MirroringService>;


class MirroringServiceProxy;

template <typename ImplRefTraits>
class MirroringServiceStub;

class MirroringServiceRequestValidator;

class  MirroringService
    : public MirroringServiceInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = MirroringServiceProxy;

  template <typename ImplRefTraits>
  using Stub_ = MirroringServiceStub<ImplRefTraits>;

  using RequestValidator_ = MirroringServiceRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kStartMinVersion = 0,
  };
  virtual ~MirroringService() {}

  virtual void Start(::mirroring::mojom::SessionParametersPtr params, const gfx::Size& max_resolution, ::mirroring::mojom::SessionObserverPtr observer, ::mirroring::mojom::ResourceProviderPtr resource_provider, ::mirroring::mojom::CastMessageChannelPtr outbound_channel, ::mirroring::mojom::CastMessageChannelRequest inbound_channel) = 0;
};
class  MirroringServiceInterceptorForTesting : public MirroringService {
  virtual MirroringService* GetForwardingInterface() = 0;
  void Start(::mirroring::mojom::SessionParametersPtr params, const gfx::Size& max_resolution, ::mirroring::mojom::SessionObserverPtr observer, ::mirroring::mojom::ResourceProviderPtr resource_provider, ::mirroring::mojom::CastMessageChannelPtr outbound_channel, ::mirroring::mojom::CastMessageChannelRequest inbound_channel) override;
};
class  MirroringServiceAsyncWaiter {
 public:
  explicit MirroringServiceAsyncWaiter(MirroringService* proxy);
  ~MirroringServiceAsyncWaiter();

 private:
  MirroringService* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(MirroringServiceAsyncWaiter);
};

class  MirroringServiceProxy
    : public MirroringService {
 public:
  explicit MirroringServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void Start(::mirroring::mojom::SessionParametersPtr params, const gfx::Size& max_resolution, ::mirroring::mojom::SessionObserverPtr observer, ::mirroring::mojom::ResourceProviderPtr resource_provider, ::mirroring::mojom::CastMessageChannelPtr outbound_channel, ::mirroring::mojom::CastMessageChannelRequest inbound_channel) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  MirroringServiceStubDispatch {
 public:
  static bool Accept(MirroringService* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      MirroringService* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<MirroringService>>
class MirroringServiceStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  MirroringServiceStub() {}
  ~MirroringServiceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MirroringServiceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MirroringServiceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  MirroringServiceRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace mirroring

namespace mojo {

}  // namespace mojo

#endif  // COMPONENTS_MIRRORING_MOJOM_MIRRORING_SERVICE_MOJOM_H_