// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef COMPONENTS_MIRRORING_MOJOM_RESOURCE_PROVIDER_MOJOM_SHARED_H_
#define COMPONENTS_MIRRORING_MOJOM_RESOURCE_PROVIDER_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "components/mirroring/mojom/resource_provider.mojom-shared-internal.h"
#include "media/capture/mojom/video_capture.mojom-shared.h"
#include "services/network/public/mojom/network_context.mojom-shared.h"
#include "media/mojo/interfaces/remoting.mojom-shared.h"
#include "media/mojo/interfaces/audio_data_pipe.mojom-shared.h"
#include "media/mojo/interfaces/audio_input_stream.mojom-shared.h"
#include "media/mojo/interfaces/audio_parameters.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace mirroring {
namespace mojom {


}  // namespace mojom
}  // namespace mirroring

namespace mojo {
namespace internal {

}  // namespace internal
}  // namespace mojo


namespace mirroring {
namespace mojom {
// Interface base classes. They are used for type safety check.
class AudioStreamCreatorClientInterfaceBase {};

using AudioStreamCreatorClientPtrDataView =
    mojo::InterfacePtrDataView<AudioStreamCreatorClientInterfaceBase>;
using AudioStreamCreatorClientRequestDataView =
    mojo::InterfaceRequestDataView<AudioStreamCreatorClientInterfaceBase>;
using AudioStreamCreatorClientAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<AudioStreamCreatorClientInterfaceBase>;
using AudioStreamCreatorClientAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<AudioStreamCreatorClientInterfaceBase>;
class ResourceProviderInterfaceBase {};

using ResourceProviderPtrDataView =
    mojo::InterfacePtrDataView<ResourceProviderInterfaceBase>;
using ResourceProviderRequestDataView =
    mojo::InterfaceRequestDataView<ResourceProviderInterfaceBase>;
using ResourceProviderAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<ResourceProviderInterfaceBase>;
using ResourceProviderAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<ResourceProviderInterfaceBase>;
class AudioStreamCreatorClient_StreamCreated_ParamsDataView {
 public:
  AudioStreamCreatorClient_StreamCreated_ParamsDataView() {}

  AudioStreamCreatorClient_StreamCreated_ParamsDataView(
      internal::AudioStreamCreatorClient_StreamCreated_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeStream() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::media::mojom::AudioInputStreamPtrDataView>(
            &data_->stream, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeClientRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::media::mojom::AudioInputStreamClientRequestDataView>(
            &data_->client_request, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetDataPipeDataView(
      ::media::mojom::ReadOnlyAudioDataPipeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDataPipe(UserType* output) {
    auto* pointer = data_->data_pipe.Get();
    return mojo::internal::Deserialize<::media::mojom::ReadOnlyAudioDataPipeDataView>(
        pointer, output, context_);
  }
  bool initially_muted() const {
    return data_->initially_muted;
  }
 private:
  internal::AudioStreamCreatorClient_StreamCreated_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ResourceProvider_GetVideoCaptureHost_ParamsDataView {
 public:
  ResourceProvider_GetVideoCaptureHost_ParamsDataView() {}

  ResourceProvider_GetVideoCaptureHost_ParamsDataView(
      internal::ResourceProvider_GetVideoCaptureHost_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::media::mojom::VideoCaptureHostRequestDataView>(
            &data_->request, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::ResourceProvider_GetVideoCaptureHost_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ResourceProvider_GetNetworkContext_ParamsDataView {
 public:
  ResourceProvider_GetNetworkContext_ParamsDataView() {}

  ResourceProvider_GetNetworkContext_ParamsDataView(
      internal::ResourceProvider_GetNetworkContext_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::NetworkContextRequestDataView>(
            &data_->request, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::ResourceProvider_GetNetworkContext_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ResourceProvider_CreateAudioStream_ParamsDataView {
 public:
  ResourceProvider_CreateAudioStream_ParamsDataView() {}

  ResourceProvider_CreateAudioStream_ParamsDataView(
      internal::ResourceProvider_CreateAudioStream_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeClient() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::mirroring::mojom::AudioStreamCreatorClientPtrDataView>(
            &data_->client, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetParamDataView(
      ::media::mojom::AudioParametersDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadParam(UserType* output) {
    auto* pointer = data_->param.Get();
    return mojo::internal::Deserialize<::media::mojom::AudioParametersDataView>(
        pointer, output, context_);
  }
  uint32_t shared_memory_count() const {
    return data_->shared_memory_count;
  }
 private:
  internal::ResourceProvider_CreateAudioStream_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ResourceProvider_ConnectToRemotingSource_ParamsDataView {
 public:
  ResourceProvider_ConnectToRemotingSource_ParamsDataView() {}

  ResourceProvider_ConnectToRemotingSource_ParamsDataView(
      internal::ResourceProvider_ConnectToRemotingSource_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeRemoter() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::media::mojom::RemoterPtrDataView>(
            &data_->remoter, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::media::mojom::RemotingSourceRequestDataView>(
            &data_->request, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::ResourceProvider_ConnectToRemotingSource_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace mirroring

namespace std {

}  // namespace std

namespace mojo {

}  // namespace mojo


namespace mirroring {
namespace mojom {

inline void AudioStreamCreatorClient_StreamCreated_ParamsDataView::GetDataPipeDataView(
    ::media::mojom::ReadOnlyAudioDataPipeDataView* output) {
  auto pointer = data_->data_pipe.Get();
  *output = ::media::mojom::ReadOnlyAudioDataPipeDataView(pointer, context_);
}






inline void ResourceProvider_CreateAudioStream_ParamsDataView::GetParamDataView(
    ::media::mojom::AudioParametersDataView* output) {
  auto pointer = data_->param.Get();
  *output = ::media::mojom::AudioParametersDataView(pointer, context_);
}





}  // namespace mojom
}  // namespace mirroring

#endif  // COMPONENTS_MIRRORING_MOJOM_RESOURCE_PROVIDER_MOJOM_SHARED_H_
