// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef COMPONENTS_MIRRORING_MOJOM_RESOURCE_PROVIDER_MOJOM_H_
#define COMPONENTS_MIRRORING_MOJOM_RESOURCE_PROVIDER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "components/mirroring/mojom/resource_provider.mojom-shared.h"
#include "media/capture/mojom/video_capture.mojom.h"
#include "services/network/public/mojom/network_context.mojom.h"
#include "media/mojo/interfaces/remoting.mojom.h"
#include "media/mojo/interfaces/audio_data_pipe.mojom.h"
#include "media/mojo/interfaces/audio_input_stream.mojom.h"
#include "media/mojo/interfaces/audio_parameters.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace mirroring {
namespace mojom {
class AudioStreamCreatorClient;
using AudioStreamCreatorClientPtr = mojo::InterfacePtr<AudioStreamCreatorClient>;
using AudioStreamCreatorClientPtrInfo = mojo::InterfacePtrInfo<AudioStreamCreatorClient>;
using ThreadSafeAudioStreamCreatorClientPtr =
    mojo::ThreadSafeInterfacePtr<AudioStreamCreatorClient>;
using AudioStreamCreatorClientRequest = mojo::InterfaceRequest<AudioStreamCreatorClient>;
using AudioStreamCreatorClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<AudioStreamCreatorClient>;
using ThreadSafeAudioStreamCreatorClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<AudioStreamCreatorClient>;
using AudioStreamCreatorClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<AudioStreamCreatorClient>;
using AudioStreamCreatorClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<AudioStreamCreatorClient>;

class ResourceProvider;
using ResourceProviderPtr = mojo::InterfacePtr<ResourceProvider>;
using ResourceProviderPtrInfo = mojo::InterfacePtrInfo<ResourceProvider>;
using ThreadSafeResourceProviderPtr =
    mojo::ThreadSafeInterfacePtr<ResourceProvider>;
using ResourceProviderRequest = mojo::InterfaceRequest<ResourceProvider>;
using ResourceProviderAssociatedPtr =
    mojo::AssociatedInterfacePtr<ResourceProvider>;
using ThreadSafeResourceProviderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ResourceProvider>;
using ResourceProviderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ResourceProvider>;
using ResourceProviderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ResourceProvider>;


class AudioStreamCreatorClientProxy;

template <typename ImplRefTraits>
class AudioStreamCreatorClientStub;

class AudioStreamCreatorClientRequestValidator;

class  AudioStreamCreatorClient
    : public AudioStreamCreatorClientInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = AudioStreamCreatorClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = AudioStreamCreatorClientStub<ImplRefTraits>;

  using RequestValidator_ = AudioStreamCreatorClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kStreamCreatedMinVersion = 0,
  };
  virtual ~AudioStreamCreatorClient() {}

  virtual void StreamCreated(::media::mojom::AudioInputStreamPtr stream, ::media::mojom::AudioInputStreamClientRequest client_request, ::media::mojom::ReadOnlyAudioDataPipePtr data_pipe, bool initially_muted) = 0;
};
class  AudioStreamCreatorClientInterceptorForTesting : public AudioStreamCreatorClient {
  virtual AudioStreamCreatorClient* GetForwardingInterface() = 0;
  void StreamCreated(::media::mojom::AudioInputStreamPtr stream, ::media::mojom::AudioInputStreamClientRequest client_request, ::media::mojom::ReadOnlyAudioDataPipePtr data_pipe, bool initially_muted) override;
};
class  AudioStreamCreatorClientAsyncWaiter {
 public:
  explicit AudioStreamCreatorClientAsyncWaiter(AudioStreamCreatorClient* proxy);
  ~AudioStreamCreatorClientAsyncWaiter();

 private:
  AudioStreamCreatorClient* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(AudioStreamCreatorClientAsyncWaiter);
};

class ResourceProviderProxy;

template <typename ImplRefTraits>
class ResourceProviderStub;

class ResourceProviderRequestValidator;

class  ResourceProvider
    : public ResourceProviderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ResourceProviderProxy;

  template <typename ImplRefTraits>
  using Stub_ = ResourceProviderStub<ImplRefTraits>;

  using RequestValidator_ = ResourceProviderRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kGetVideoCaptureHostMinVersion = 0,
    kGetNetworkContextMinVersion = 0,
    kCreateAudioStreamMinVersion = 0,
    kConnectToRemotingSourceMinVersion = 0,
  };
  virtual ~ResourceProvider() {}

  virtual void GetVideoCaptureHost(::media::mojom::VideoCaptureHostRequest request) = 0;

  virtual void GetNetworkContext(::network::mojom::NetworkContextRequest request) = 0;

  virtual void CreateAudioStream(AudioStreamCreatorClientPtr client, const media::AudioParameters& param, uint32_t shared_memory_count) = 0;

  virtual void ConnectToRemotingSource(::media::mojom::RemoterPtr remoter, ::media::mojom::RemotingSourceRequest request) = 0;
};
class  ResourceProviderInterceptorForTesting : public ResourceProvider {
  virtual ResourceProvider* GetForwardingInterface() = 0;
  void GetVideoCaptureHost(::media::mojom::VideoCaptureHostRequest request) override;
  void GetNetworkContext(::network::mojom::NetworkContextRequest request) override;
  void CreateAudioStream(AudioStreamCreatorClientPtr client, const media::AudioParameters& param, uint32_t shared_memory_count) override;
  void ConnectToRemotingSource(::media::mojom::RemoterPtr remoter, ::media::mojom::RemotingSourceRequest request) override;
};
class  ResourceProviderAsyncWaiter {
 public:
  explicit ResourceProviderAsyncWaiter(ResourceProvider* proxy);
  ~ResourceProviderAsyncWaiter();

 private:
  ResourceProvider* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(ResourceProviderAsyncWaiter);
};

class  AudioStreamCreatorClientProxy
    : public AudioStreamCreatorClient {
 public:
  explicit AudioStreamCreatorClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void StreamCreated(::media::mojom::AudioInputStreamPtr stream, ::media::mojom::AudioInputStreamClientRequest client_request, ::media::mojom::ReadOnlyAudioDataPipePtr data_pipe, bool initially_muted) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  ResourceProviderProxy
    : public ResourceProvider {
 public:
  explicit ResourceProviderProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetVideoCaptureHost(::media::mojom::VideoCaptureHostRequest request) final;
  void GetNetworkContext(::network::mojom::NetworkContextRequest request) final;
  void CreateAudioStream(AudioStreamCreatorClientPtr client, const media::AudioParameters& param, uint32_t shared_memory_count) final;
  void ConnectToRemotingSource(::media::mojom::RemoterPtr remoter, ::media::mojom::RemotingSourceRequest request) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  AudioStreamCreatorClientStubDispatch {
 public:
  static bool Accept(AudioStreamCreatorClient* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      AudioStreamCreatorClient* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<AudioStreamCreatorClient>>
class AudioStreamCreatorClientStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  AudioStreamCreatorClientStub() {}
  ~AudioStreamCreatorClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AudioStreamCreatorClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AudioStreamCreatorClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  ResourceProviderStubDispatch {
 public:
  static bool Accept(ResourceProvider* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ResourceProvider* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ResourceProvider>>
class ResourceProviderStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ResourceProviderStub() {}
  ~ResourceProviderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ResourceProviderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ResourceProviderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  AudioStreamCreatorClientRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  ResourceProviderRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace mirroring

namespace mojo {

}  // namespace mojo

#endif  // COMPONENTS_MIRRORING_MOJOM_RESOURCE_PROVIDER_MOJOM_H_