// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef COMPONENTS_MIRRORING_MOJOM_SESSION_OBSERVER_MOJOM_SHARED_H_
#define COMPONENTS_MIRRORING_MOJOM_SESSION_OBSERVER_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "components/mirroring/mojom/session_observer.mojom-shared-internal.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace mirroring {
namespace mojom {


}  // namespace mojom
}  // namespace mirroring

namespace mojo {
namespace internal {

}  // namespace internal
}  // namespace mojo


namespace mirroring {
namespace mojom {

enum class SessionError : int32_t {
  ANSWER_TIME_OUT,
  ANSWER_NOT_OK,
  ANSWER_MISMATCHED_CAST_MODE,
  ANSWER_MISMATCHED_SSRC_LENGTH,
  ANSWER_SELECT_MULTIPLE_AUDIO,
  ANSWER_SELECT_MULTIPLE_VIDEO,
  ANSWER_SELECT_INVALID_INDEX,
  ANSWER_NO_AUDIO_OR_VIDEO,
  AUDIO_CAPTURE_ERROR,
  VIDEO_CAPTURE_ERROR,
  RTP_STREAM_ERROR,
  ENCODING_ERROR,
  CAST_TRANSPORT_ERROR,
  kMinValue = 0,
  kMaxValue = 12,
};

inline std::ostream& operator<<(std::ostream& os, SessionError value) {
  switch(value) {
    case SessionError::ANSWER_TIME_OUT:
      return os << "SessionError::ANSWER_TIME_OUT";
    case SessionError::ANSWER_NOT_OK:
      return os << "SessionError::ANSWER_NOT_OK";
    case SessionError::ANSWER_MISMATCHED_CAST_MODE:
      return os << "SessionError::ANSWER_MISMATCHED_CAST_MODE";
    case SessionError::ANSWER_MISMATCHED_SSRC_LENGTH:
      return os << "SessionError::ANSWER_MISMATCHED_SSRC_LENGTH";
    case SessionError::ANSWER_SELECT_MULTIPLE_AUDIO:
      return os << "SessionError::ANSWER_SELECT_MULTIPLE_AUDIO";
    case SessionError::ANSWER_SELECT_MULTIPLE_VIDEO:
      return os << "SessionError::ANSWER_SELECT_MULTIPLE_VIDEO";
    case SessionError::ANSWER_SELECT_INVALID_INDEX:
      return os << "SessionError::ANSWER_SELECT_INVALID_INDEX";
    case SessionError::ANSWER_NO_AUDIO_OR_VIDEO:
      return os << "SessionError::ANSWER_NO_AUDIO_OR_VIDEO";
    case SessionError::AUDIO_CAPTURE_ERROR:
      return os << "SessionError::AUDIO_CAPTURE_ERROR";
    case SessionError::VIDEO_CAPTURE_ERROR:
      return os << "SessionError::VIDEO_CAPTURE_ERROR";
    case SessionError::RTP_STREAM_ERROR:
      return os << "SessionError::RTP_STREAM_ERROR";
    case SessionError::ENCODING_ERROR:
      return os << "SessionError::ENCODING_ERROR";
    case SessionError::CAST_TRANSPORT_ERROR:
      return os << "SessionError::CAST_TRANSPORT_ERROR";
    default:
      return os << "Unknown SessionError value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(SessionError value) {
  return internal::SessionError_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class SessionObserverInterfaceBase {};

using SessionObserverPtrDataView =
    mojo::InterfacePtrDataView<SessionObserverInterfaceBase>;
using SessionObserverRequestDataView =
    mojo::InterfaceRequestDataView<SessionObserverInterfaceBase>;
using SessionObserverAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<SessionObserverInterfaceBase>;
using SessionObserverAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<SessionObserverInterfaceBase>;
class SessionObserver_OnError_ParamsDataView {
 public:
  SessionObserver_OnError_ParamsDataView() {}

  SessionObserver_OnError_ParamsDataView(
      internal::SessionObserver_OnError_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadError(UserType* output) const {
    auto data_value = data_->error;
    return mojo::internal::Deserialize<::mirroring::mojom::SessionError>(
        data_value, output);
  }

  SessionError error() const {
    return static_cast<SessionError>(data_->error);
  }
 private:
  internal::SessionObserver_OnError_Params_Data* data_ = nullptr;
};

class SessionObserver_DidStart_ParamsDataView {
 public:
  SessionObserver_DidStart_ParamsDataView() {}

  SessionObserver_DidStart_ParamsDataView(
      internal::SessionObserver_DidStart_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::SessionObserver_DidStart_Params_Data* data_ = nullptr;
};

class SessionObserver_DidStop_ParamsDataView {
 public:
  SessionObserver_DidStop_ParamsDataView() {}

  SessionObserver_DidStop_ParamsDataView(
      internal::SessionObserver_DidStop_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::SessionObserver_DidStop_Params_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace mirroring

namespace std {

template <>
struct hash<::mirroring::mojom::SessionError>
    : public mojo::internal::EnumHashImpl<::mirroring::mojom::SessionError> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::mirroring::mojom::SessionError, ::mirroring::mojom::SessionError> {
  static ::mirroring::mojom::SessionError ToMojom(::mirroring::mojom::SessionError input) { return input; }
  static bool FromMojom(::mirroring::mojom::SessionError input, ::mirroring::mojom::SessionError* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::mirroring::mojom::SessionError, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::mirroring::mojom::SessionError, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::mirroring::mojom::SessionError>(input), output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace mirroring {
namespace mojom {








}  // namespace mojom
}  // namespace mirroring

#endif  // COMPONENTS_MIRRORING_MOJOM_SESSION_OBSERVER_MOJOM_SHARED_H_
