// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "components/subresource_filter/mojom/subresource_filter.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "components/subresource_filter/mojom/subresource_filter.mojom-shared-message-ids.h"
#include "mojo/public/cpp/base/time_mojom_traits.h"
namespace subresource_filter {
namespace mojom {
ActivationState::ActivationState()
    : activation_level(ActivationLevel::kDisabled),
      filtering_disabled_for_document(false),
      generic_blocking_rules_disabled(false),
      measure_performance(false),
      enable_logging(false) {}

ActivationState::ActivationState(
    ActivationLevel activation_level_in,
    bool filtering_disabled_for_document_in,
    bool generic_blocking_rules_disabled_in,
    bool measure_performance_in,
    bool enable_logging_in)
    : activation_level(std::move(activation_level_in)),
      filtering_disabled_for_document(std::move(filtering_disabled_for_document_in)),
      generic_blocking_rules_disabled(std::move(generic_blocking_rules_disabled_in)),
      measure_performance(std::move(measure_performance_in)),
      enable_logging(std::move(enable_logging_in)) {}

ActivationState::~ActivationState() = default;
size_t ActivationState::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->activation_level);
  seed = mojo::internal::Hash(seed, this->filtering_disabled_for_document);
  seed = mojo::internal::Hash(seed, this->generic_blocking_rules_disabled);
  seed = mojo::internal::Hash(seed, this->measure_performance);
  seed = mojo::internal::Hash(seed, this->enable_logging);
  return seed;
}

bool ActivationState::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
DocumentLoadStatistics::DocumentLoadStatistics()
    : num_loads_total(0),
      num_loads_evaluated(0),
      num_loads_matching_rules(0),
      num_loads_disallowed(0),
      evaluation_total_wall_duration(),
      evaluation_total_cpu_duration() {}

DocumentLoadStatistics::DocumentLoadStatistics(
    int32_t num_loads_total_in,
    int32_t num_loads_evaluated_in,
    int32_t num_loads_matching_rules_in,
    int32_t num_loads_disallowed_in,
    base::TimeDelta evaluation_total_wall_duration_in,
    base::TimeDelta evaluation_total_cpu_duration_in)
    : num_loads_total(std::move(num_loads_total_in)),
      num_loads_evaluated(std::move(num_loads_evaluated_in)),
      num_loads_matching_rules(std::move(num_loads_matching_rules_in)),
      num_loads_disallowed(std::move(num_loads_disallowed_in)),
      evaluation_total_wall_duration(std::move(evaluation_total_wall_duration_in)),
      evaluation_total_cpu_duration(std::move(evaluation_total_cpu_duration_in)) {}

DocumentLoadStatistics::~DocumentLoadStatistics() = default;

bool DocumentLoadStatistics::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char SubresourceFilterHost::Name_[] = "subresource_filter.mojom.SubresourceFilterHost";

SubresourceFilterHostProxy::SubresourceFilterHostProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void SubresourceFilterHostProxy::DidDisallowFirstSubresource(
    ) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "subresource_filter::mojom::SubresourceFilterHost::DidDisallowFirstSubresource");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kSubresourceFilterHost_DidDisallowFirstSubresource_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::subresource_filter::mojom::internal::SubresourceFilterHost_DidDisallowFirstSubresource_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(SubresourceFilterHost::Name_);
  message.set_method_name("DidDisallowFirstSubresource");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void SubresourceFilterHostProxy::FrameIsAdSubframe(
    ) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "subresource_filter::mojom::SubresourceFilterHost::FrameIsAdSubframe");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kSubresourceFilterHost_FrameIsAdSubframe_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::subresource_filter::mojom::internal::SubresourceFilterHost_FrameIsAdSubframe_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(SubresourceFilterHost::Name_);
  message.set_method_name("FrameIsAdSubframe");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void SubresourceFilterHostProxy::SetDocumentLoadStatistics(
    DocumentLoadStatisticsPtr in_statistics) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "subresource_filter::mojom::SubresourceFilterHost::SetDocumentLoadStatistics");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kSubresourceFilterHost_SetDocumentLoadStatistics_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::subresource_filter::mojom::internal::SubresourceFilterHost_SetDocumentLoadStatistics_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->statistics)::BaseType::BufferWriter
      statistics_writer;
  mojo::internal::Serialize<::subresource_filter::mojom::DocumentLoadStatisticsDataView>(
      in_statistics, buffer, &statistics_writer, &serialization_context);
  params->statistics.Set(
      statistics_writer.is_null() ? nullptr : statistics_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->statistics.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null statistics in SubresourceFilterHost.SetDocumentLoadStatistics request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(SubresourceFilterHost::Name_);
  message.set_method_name("SetDocumentLoadStatistics");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool SubresourceFilterHostStubDispatch::Accept(
    SubresourceFilterHost* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kSubresourceFilterHost_DidDisallowFirstSubresource_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)subresource_filter::mojom::SubresourceFilterHost::DidDisallowFirstSubresource",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::SubresourceFilterHost_DidDisallowFirstSubresource_Params_Data* params =
          reinterpret_cast<internal::SubresourceFilterHost_DidDisallowFirstSubresource_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      SubresourceFilterHost_DidDisallowFirstSubresource_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "SubresourceFilterHost::DidDisallowFirstSubresource deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->DidDisallowFirstSubresource();
      return true;
    }
    case internal::kSubresourceFilterHost_FrameIsAdSubframe_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)subresource_filter::mojom::SubresourceFilterHost::FrameIsAdSubframe",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::SubresourceFilterHost_FrameIsAdSubframe_Params_Data* params =
          reinterpret_cast<internal::SubresourceFilterHost_FrameIsAdSubframe_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      SubresourceFilterHost_FrameIsAdSubframe_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "SubresourceFilterHost::FrameIsAdSubframe deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->FrameIsAdSubframe();
      return true;
    }
    case internal::kSubresourceFilterHost_SetDocumentLoadStatistics_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)subresource_filter::mojom::SubresourceFilterHost::SetDocumentLoadStatistics",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::SubresourceFilterHost_SetDocumentLoadStatistics_Params_Data* params =
          reinterpret_cast<internal::SubresourceFilterHost_SetDocumentLoadStatistics_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      DocumentLoadStatisticsPtr p_statistics{};
      SubresourceFilterHost_SetDocumentLoadStatistics_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadStatistics(&p_statistics))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "SubresourceFilterHost::SetDocumentLoadStatistics deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetDocumentLoadStatistics(
std::move(p_statistics));
      return true;
    }
  }
  return false;
}

// static
bool SubresourceFilterHostStubDispatch::AcceptWithResponder(
    SubresourceFilterHost* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kSubresourceFilterHost_DidDisallowFirstSubresource_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)subresource_filter::mojom::SubresourceFilterHost::DidDisallowFirstSubresource",
               "message", message->name());
#endif
      break;
    }
    case internal::kSubresourceFilterHost_FrameIsAdSubframe_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)subresource_filter::mojom::SubresourceFilterHost::FrameIsAdSubframe",
               "message", message->name());
#endif
      break;
    }
    case internal::kSubresourceFilterHost_SetDocumentLoadStatistics_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)subresource_filter::mojom::SubresourceFilterHost::SetDocumentLoadStatistics",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool SubresourceFilterHostRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "SubresourceFilterHost RequestValidator");

  switch (message->header()->name) {
    case internal::kSubresourceFilterHost_DidDisallowFirstSubresource_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::SubresourceFilterHost_DidDisallowFirstSubresource_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kSubresourceFilterHost_FrameIsAdSubframe_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::SubresourceFilterHost_FrameIsAdSubframe_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kSubresourceFilterHost_SetDocumentLoadStatistics_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::SubresourceFilterHost_SetDocumentLoadStatistics_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void SubresourceFilterHostInterceptorForTesting::DidDisallowFirstSubresource() {
  GetForwardingInterface()->DidDisallowFirstSubresource();
}
void SubresourceFilterHostInterceptorForTesting::FrameIsAdSubframe() {
  GetForwardingInterface()->FrameIsAdSubframe();
}
void SubresourceFilterHostInterceptorForTesting::SetDocumentLoadStatistics(DocumentLoadStatisticsPtr statistics) {
  GetForwardingInterface()->SetDocumentLoadStatistics(std::move(statistics));
}
SubresourceFilterHostAsyncWaiter::SubresourceFilterHostAsyncWaiter(
    SubresourceFilterHost* proxy) : proxy_(proxy) {}

SubresourceFilterHostAsyncWaiter::~SubresourceFilterHostAsyncWaiter() = default;


const char SubresourceFilterAgent::Name_[] = "subresource_filter.mojom.SubresourceFilterAgent";

SubresourceFilterAgentProxy::SubresourceFilterAgentProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void SubresourceFilterAgentProxy::ActivateForNextCommittedLoad(
    ActivationStatePtr in_activation_state, bool in_is_ad_subframe) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "subresource_filter::mojom::SubresourceFilterAgent::ActivateForNextCommittedLoad");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kSubresourceFilterAgent_ActivateForNextCommittedLoad_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::subresource_filter::mojom::internal::SubresourceFilterAgent_ActivateForNextCommittedLoad_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->activation_state)::BaseType::BufferWriter
      activation_state_writer;
  mojo::internal::Serialize<::subresource_filter::mojom::ActivationStateDataView>(
      in_activation_state, buffer, &activation_state_writer, &serialization_context);
  params->activation_state.Set(
      activation_state_writer.is_null() ? nullptr : activation_state_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->activation_state.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null activation_state in SubresourceFilterAgent.ActivateForNextCommittedLoad request");
  params->is_ad_subframe = in_is_ad_subframe;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(SubresourceFilterAgent::Name_);
  message.set_method_name("ActivateForNextCommittedLoad");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool SubresourceFilterAgentStubDispatch::Accept(
    SubresourceFilterAgent* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kSubresourceFilterAgent_ActivateForNextCommittedLoad_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)subresource_filter::mojom::SubresourceFilterAgent::ActivateForNextCommittedLoad",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::SubresourceFilterAgent_ActivateForNextCommittedLoad_Params_Data* params =
          reinterpret_cast<internal::SubresourceFilterAgent_ActivateForNextCommittedLoad_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ActivationStatePtr p_activation_state{};
      bool p_is_ad_subframe{};
      SubresourceFilterAgent_ActivateForNextCommittedLoad_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadActivationState(&p_activation_state))
        success = false;
      p_is_ad_subframe = input_data_view.is_ad_subframe();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "SubresourceFilterAgent::ActivateForNextCommittedLoad deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ActivateForNextCommittedLoad(
std::move(p_activation_state), 
std::move(p_is_ad_subframe));
      return true;
    }
  }
  return false;
}

// static
bool SubresourceFilterAgentStubDispatch::AcceptWithResponder(
    SubresourceFilterAgent* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kSubresourceFilterAgent_ActivateForNextCommittedLoad_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)subresource_filter::mojom::SubresourceFilterAgent::ActivateForNextCommittedLoad",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool SubresourceFilterAgentRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "SubresourceFilterAgent RequestValidator");

  switch (message->header()->name) {
    case internal::kSubresourceFilterAgent_ActivateForNextCommittedLoad_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::SubresourceFilterAgent_ActivateForNextCommittedLoad_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void SubresourceFilterAgentInterceptorForTesting::ActivateForNextCommittedLoad(ActivationStatePtr activation_state, bool is_ad_subframe) {
  GetForwardingInterface()->ActivateForNextCommittedLoad(std::move(activation_state), std::move(is_ad_subframe));
}
SubresourceFilterAgentAsyncWaiter::SubresourceFilterAgentAsyncWaiter(
    SubresourceFilterAgent* proxy) : proxy_(proxy) {}

SubresourceFilterAgentAsyncWaiter::~SubresourceFilterAgentAsyncWaiter() = default;


}  // namespace mojom
}  // namespace subresource_filter

namespace mojo {


// static
bool StructTraits<::subresource_filter::mojom::ActivationState::DataView, ::subresource_filter::mojom::ActivationStatePtr>::Read(
    ::subresource_filter::mojom::ActivationState::DataView input,
    ::subresource_filter::mojom::ActivationStatePtr* output) {
  bool success = true;
  ::subresource_filter::mojom::ActivationStatePtr result(::subresource_filter::mojom::ActivationState::New());
  
      if (!input.ReadActivationLevel(&result->activation_level))
        success = false;
      result->filtering_disabled_for_document = input.filtering_disabled_for_document();
      result->generic_blocking_rules_disabled = input.generic_blocking_rules_disabled();
      result->measure_performance = input.measure_performance();
      result->enable_logging = input.enable_logging();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::subresource_filter::mojom::DocumentLoadStatistics::DataView, ::subresource_filter::mojom::DocumentLoadStatisticsPtr>::Read(
    ::subresource_filter::mojom::DocumentLoadStatistics::DataView input,
    ::subresource_filter::mojom::DocumentLoadStatisticsPtr* output) {
  bool success = true;
  ::subresource_filter::mojom::DocumentLoadStatisticsPtr result(::subresource_filter::mojom::DocumentLoadStatistics::New());
  
      result->num_loads_total = input.num_loads_total();
      result->num_loads_evaluated = input.num_loads_evaluated();
      result->num_loads_matching_rules = input.num_loads_matching_rules();
      result->num_loads_disallowed = input.num_loads_disallowed();
      if (!input.ReadEvaluationTotalWallDuration(&result->evaluation_total_wall_duration))
        success = false;
      if (!input.ReadEvaluationTotalCpuDuration(&result->evaluation_total_cpu_duration))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif