// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_CONTROLLER_MOJOM_BLINK_H_
#define SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_CONTROLLER_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/media_session/public/mojom/media_controller.mojom-shared.h"
#include "mojo/public/mojom/base/time.mojom-blink.h"
#include "services/media_session/public/mojom/media_session.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_clone_equals_util.h"
#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/blink/renderer/platform/mojo/revocable_interface_ptr.h"
#include "third_party/blink/renderer/platform/wtf/hash_functions.h"
#include "third_party/blink/renderer/platform/wtf/text/wtf_string.h"

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "third_party/blink/renderer/platform/mojo/revocable_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "third_party/blink/public/platform/web_common.h"


namespace media_session {
namespace mojom {
namespace blink {
class MediaController;
using MediaControllerPtr = mojo::InterfacePtr<MediaController>;
using RevocableMediaControllerPtr = ::blink::RevocableInterfacePtr<MediaController>;
using MediaControllerPtrInfo = mojo::InterfacePtrInfo<MediaController>;
using ThreadSafeMediaControllerPtr =
    mojo::ThreadSafeInterfacePtr<MediaController>;
using MediaControllerRequest = mojo::InterfaceRequest<MediaController>;
using MediaControllerAssociatedPtr =
    mojo::AssociatedInterfacePtr<MediaController>;
using ThreadSafeMediaControllerAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<MediaController>;
using MediaControllerAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<MediaController>;
using MediaControllerAssociatedRequest =
    mojo::AssociatedInterfaceRequest<MediaController>;


class MediaControllerProxy;

template <typename ImplRefTraits>
class MediaControllerStub;

class MediaControllerRequestValidator;

class BLINK_PLATFORM_EXPORT MediaController
    : public MediaControllerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = MediaControllerProxy;

  template <typename ImplRefTraits>
  using Stub_ = MediaControllerStub<ImplRefTraits>;

  using RequestValidator_ = MediaControllerRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSuspendMinVersion = 0,
    kResumeMinVersion = 0,
    kToggleSuspendResumeMinVersion = 0,
    kAddObserverMinVersion = 0,
    kPreviousTrackMinVersion = 0,
    kNextTrackMinVersion = 0,
    kSeekMinVersion = 0,
  };
  virtual ~MediaController() {}

  virtual void Suspend() = 0;

  virtual void Resume() = 0;

  virtual void ToggleSuspendResume() = 0;

  virtual void AddObserver(::media_session::mojom::blink::MediaSessionObserverPtr observer) = 0;

  virtual void PreviousTrack() = 0;

  virtual void NextTrack() = 0;

  virtual void Seek(WTF::TimeDelta seek_time) = 0;
};
class BLINK_PLATFORM_EXPORT MediaControllerInterceptorForTesting : public MediaController {
  virtual MediaController* GetForwardingInterface() = 0;
  void Suspend() override;
  void Resume() override;
  void ToggleSuspendResume() override;
  void AddObserver(::media_session::mojom::blink::MediaSessionObserverPtr observer) override;
  void PreviousTrack() override;
  void NextTrack() override;
  void Seek(WTF::TimeDelta seek_time) override;
};
class BLINK_PLATFORM_EXPORT MediaControllerAsyncWaiter {
 public:
  explicit MediaControllerAsyncWaiter(MediaController* proxy);
  ~MediaControllerAsyncWaiter();

 private:
  MediaController* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(MediaControllerAsyncWaiter);
};

class BLINK_PLATFORM_EXPORT MediaControllerProxy
    : public MediaController {
 public:
  explicit MediaControllerProxy(mojo::MessageReceiverWithResponder* receiver);
  void Suspend() final;
  void Resume() final;
  void ToggleSuspendResume() final;
  void AddObserver(::media_session::mojom::blink::MediaSessionObserverPtr observer) final;
  void PreviousTrack() final;
  void NextTrack() final;
  void Seek(WTF::TimeDelta seek_time) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class BLINK_PLATFORM_EXPORT MediaControllerStubDispatch {
 public:
  static bool Accept(MediaController* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      MediaController* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<MediaController>>
class MediaControllerStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  MediaControllerStub() {}
  ~MediaControllerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MediaControllerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MediaControllerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_PLATFORM_EXPORT MediaControllerRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace blink
}  // namespace mojom
}  // namespace media_session

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_CONTROLLER_MOJOM_BLINK_H_