// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "services/network/public/mojom/host_resolver.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "services/network/public/mojom/host_resolver.mojom-shared-message-ids.h"
#include "mojo/public/cpp/base/time_mojom_traits.h"
#include "services/network/public/cpp/address_list_mojom_traits.h"
#include "services/network/public/cpp/host_resolver_mojom_traits.h"
#include "services/network/public/cpp/ip_address_mojom_traits.h"
#include "services/network/public/cpp/ip_endpoint_mojom_traits.h"
#include "services/network/public/cpp/net_ipc_param_traits.h"
#include "services/network/public/cpp/network_ipc_param_traits.h"
#include "services/network/public/cpp/network_param_mojom_traits.h"
#include "services/network/public/cpp/url_request_mojom_traits.h"
namespace network {
namespace mojom {
DnsHost::DnsHost()
    : hostname(),
      address() {}

DnsHost::DnsHost(
    const std::string& hostname_in,
    const net::IPAddress& address_in)
    : hostname(std::move(hostname_in)),
      address(std::move(address_in)) {}

DnsHost::~DnsHost() = default;

bool DnsHost::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
DnsOverHttpsServer::DnsOverHttpsServer()
    : server_template(),
      use_post(false) {}

DnsOverHttpsServer::DnsOverHttpsServer(
    const std::string& server_template_in,
    bool use_post_in)
    : server_template(std::move(server_template_in)),
      use_post(std::move(use_post_in)) {}

DnsOverHttpsServer::~DnsOverHttpsServer() = default;
size_t DnsOverHttpsServer::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->server_template);
  seed = mojo::internal::Hash(seed, this->use_post);
  return seed;
}

bool DnsOverHttpsServer::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
DnsConfigOverrides::DnsConfigOverrides()
    : nameservers(),
      search(),
      hosts(),
      append_to_multi_label_name(DnsConfigOverrides_Tristate::NO_OVERRIDE),
      randomize_ports(DnsConfigOverrides_Tristate::NO_OVERRIDE),
      ndots(-1),
      timeout(),
      attempts(-1),
      rotate(DnsConfigOverrides_Tristate::NO_OVERRIDE),
      use_local_ipv6(DnsConfigOverrides_Tristate::NO_OVERRIDE),
      dns_over_https_servers() {}

DnsConfigOverrides::DnsConfigOverrides(
    const base::Optional<std::vector<net::IPEndPoint>>& nameservers_in,
    const base::Optional<std::vector<std::string>>& search_in,
    base::Optional<std::vector<DnsHostPtr>> hosts_in,
    DnsConfigOverrides::Tristate append_to_multi_label_name_in,
    DnsConfigOverrides::Tristate randomize_ports_in,
    int8_t ndots_in,
    base::Optional<base::TimeDelta> timeout_in,
    int32_t attempts_in,
    DnsConfigOverrides::Tristate rotate_in,
    DnsConfigOverrides::Tristate use_local_ipv6_in,
    base::Optional<std::vector<DnsOverHttpsServerPtr>> dns_over_https_servers_in)
    : nameservers(std::move(nameservers_in)),
      search(std::move(search_in)),
      hosts(std::move(hosts_in)),
      append_to_multi_label_name(std::move(append_to_multi_label_name_in)),
      randomize_ports(std::move(randomize_ports_in)),
      ndots(std::move(ndots_in)),
      timeout(std::move(timeout_in)),
      attempts(std::move(attempts_in)),
      rotate(std::move(rotate_in)),
      use_local_ipv6(std::move(use_local_ipv6_in)),
      dns_over_https_servers(std::move(dns_over_https_servers_in)) {}

DnsConfigOverrides::~DnsConfigOverrides() = default;

bool DnsConfigOverrides::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
ResolveHostParameters::ResolveHostParameters()
    : dns_query_type(mojo::internal::ConvertEnumValue<ResolveHostParameters::DnsQueryType, net::DnsQueryType>(ResolveHostParameters_DnsQueryType::UNSPECIFIED)),
      initial_priority(mojo::internal::ConvertEnumValue<::network::mojom::RequestPriority, net::RequestPriority>(::network::mojom::RequestPriority::kLowest)),
      source(mojo::internal::ConvertEnumValue<ResolveHostParameters::Source, net::HostResolverSource>(ResolveHostParameters_Source::ANY)),
      allow_cached_response(true),
      control_handle(),
      include_canonical_name(false),
      loopback_only(false),
      is_speculative(false) {}

ResolveHostParameters::ResolveHostParameters(
    net::DnsQueryType dns_query_type_in,
    net::RequestPriority initial_priority_in,
    net::HostResolverSource source_in,
    bool allow_cached_response_in,
    ResolveHostHandleRequest control_handle_in,
    bool include_canonical_name_in,
    bool loopback_only_in,
    bool is_speculative_in)
    : dns_query_type(std::move(dns_query_type_in)),
      initial_priority(std::move(initial_priority_in)),
      source(std::move(source_in)),
      allow_cached_response(std::move(allow_cached_response_in)),
      control_handle(std::move(control_handle_in)),
      include_canonical_name(std::move(include_canonical_name_in)),
      loopback_only(std::move(loopback_only_in)),
      is_speculative(std::move(is_speculative_in)) {}

ResolveHostParameters::~ResolveHostParameters() = default;

bool ResolveHostParameters::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char ResolveHostHandle::Name_[] = "network.mojom.ResolveHostHandle";

ResolveHostHandleProxy::ResolveHostHandleProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class ResolveHostHandleProxy_Cancel_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit ResolveHostHandleProxy_Cancel_Message(
      uint32_t message_flags
      , int32_t param_result
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kResolveHostHandle_Cancel_Name,
          message_flags)
      , param_result_(std::move(param_result)){}
  ~ResolveHostHandleProxy_Cancel_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      int32_t param_result) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<ResolveHostHandleProxy_Cancel_Message>(
          kFlags
          , std::move(param_result)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kResolveHostHandle_Cancel_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::ResolveHostHandle_Cancel_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->result = param_result;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(ResolveHostHandle* impl) {
    impl->Cancel(
        std::move(param_result_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::ResolveHostHandle_Cancel_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->result = param_result_;
  }
  int32_t param_result_;

  DISALLOW_COPY_AND_ASSIGN(ResolveHostHandleProxy_Cancel_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
ResolveHostHandleProxy_Cancel_Message::kMessageTag = {};

void ResolveHostHandleProxy::Cancel(
    int32_t in_result) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::ResolveHostHandle::Cancel");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = ResolveHostHandleProxy_Cancel_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_result));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(ResolveHostHandle::Name_);
  message.set_method_name("Cancel");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool ResolveHostHandleStubDispatch::Accept(
    ResolveHostHandle* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kResolveHostHandle_Cancel_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::ResolveHostHandle::Cancel",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            ResolveHostHandleProxy_Cancel_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::ResolveHostHandle_Cancel_Params_Data* params =
          reinterpret_cast<internal::ResolveHostHandle_Cancel_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_result{};
      ResolveHostHandle_Cancel_ParamsDataView input_data_view(params, &serialization_context);
      
      p_result = input_data_view.result();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "ResolveHostHandle::Cancel deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->Cancel(
std::move(p_result));
      return true;
    }
  }
  return false;
}

// static
bool ResolveHostHandleStubDispatch::AcceptWithResponder(
    ResolveHostHandle* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kResolveHostHandle_Cancel_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::ResolveHostHandle::Cancel",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool ResolveHostHandleRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "ResolveHostHandle RequestValidator");

  switch (message->header()->name) {
    case internal::kResolveHostHandle_Cancel_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::ResolveHostHandle_Cancel_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void ResolveHostHandleInterceptorForTesting::Cancel(int32_t result) {
  GetForwardingInterface()->Cancel(std::move(result));
}
ResolveHostHandleAsyncWaiter::ResolveHostHandleAsyncWaiter(
    ResolveHostHandle* proxy) : proxy_(proxy) {}

ResolveHostHandleAsyncWaiter::~ResolveHostHandleAsyncWaiter() = default;


const char ResolveHostClient::Name_[] = "network.mojom.ResolveHostClient";

ResolveHostClientProxy::ResolveHostClientProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class ResolveHostClientProxy_OnComplete_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit ResolveHostClientProxy_OnComplete_Message(
      uint32_t message_flags
      , int32_t param_result
      , const base::Optional<net::AddressList>& param_resolved_addresses
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kResolveHostClient_OnComplete_Name,
          message_flags)
      , param_result_(std::move(param_result))
      , param_resolved_addresses_(std::move(param_resolved_addresses)){}
  ~ResolveHostClientProxy_OnComplete_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      int32_t param_result, const base::Optional<net::AddressList>& param_resolved_addresses) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<ResolveHostClientProxy_OnComplete_Message>(
          kFlags
          , std::move(param_result)
          , std::move(param_resolved_addresses)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kResolveHostClient_OnComplete_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::ResolveHostClient_OnComplete_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->result = param_result;
  typename decltype(params->resolved_addresses)::BaseType::BufferWriter
      resolved_addresses_writer;
  mojo::internal::Serialize<::network::mojom::AddressListDataView>(
      param_resolved_addresses, buffer, &resolved_addresses_writer, &serialization_context);
  params->resolved_addresses.Set(
      resolved_addresses_writer.is_null() ? nullptr : resolved_addresses_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(ResolveHostClient* impl) {
    impl->OnComplete(
        std::move(param_result_),
        std::move(param_resolved_addresses_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::ResolveHostClient_OnComplete_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->result = param_result_;
  typename decltype(params->resolved_addresses)::BaseType::BufferWriter
      resolved_addresses_writer;
  mojo::internal::Serialize<::network::mojom::AddressListDataView>(
      param_resolved_addresses_, buffer, &resolved_addresses_writer, serialization_context);
  params->resolved_addresses.Set(
      resolved_addresses_writer.is_null() ? nullptr : resolved_addresses_writer.data());
  }
  int32_t param_result_;
  base::Optional<net::AddressList> param_resolved_addresses_;

  DISALLOW_COPY_AND_ASSIGN(ResolveHostClientProxy_OnComplete_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
ResolveHostClientProxy_OnComplete_Message::kMessageTag = {};

void ResolveHostClientProxy::OnComplete(
    int32_t in_result, const base::Optional<net::AddressList>& in_resolved_addresses) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::ResolveHostClient::OnComplete");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = ResolveHostClientProxy_OnComplete_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_result), std::move(in_resolved_addresses));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(ResolveHostClient::Name_);
  message.set_method_name("OnComplete");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool ResolveHostClientStubDispatch::Accept(
    ResolveHostClient* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kResolveHostClient_OnComplete_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::ResolveHostClient::OnComplete",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            ResolveHostClientProxy_OnComplete_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::ResolveHostClient_OnComplete_Params_Data* params =
          reinterpret_cast<internal::ResolveHostClient_OnComplete_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_result{};
      base::Optional<net::AddressList> p_resolved_addresses{};
      ResolveHostClient_OnComplete_ParamsDataView input_data_view(params, &serialization_context);
      
      p_result = input_data_view.result();
      if (!input_data_view.ReadResolvedAddresses(&p_resolved_addresses))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "ResolveHostClient::OnComplete deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnComplete(
std::move(p_result), 
std::move(p_resolved_addresses));
      return true;
    }
  }
  return false;
}

// static
bool ResolveHostClientStubDispatch::AcceptWithResponder(
    ResolveHostClient* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kResolveHostClient_OnComplete_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::ResolveHostClient::OnComplete",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool ResolveHostClientRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "ResolveHostClient RequestValidator");

  switch (message->header()->name) {
    case internal::kResolveHostClient_OnComplete_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::ResolveHostClient_OnComplete_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void ResolveHostClientInterceptorForTesting::OnComplete(int32_t result, const base::Optional<net::AddressList>& resolved_addresses) {
  GetForwardingInterface()->OnComplete(std::move(result), std::move(resolved_addresses));
}
ResolveHostClientAsyncWaiter::ResolveHostClientAsyncWaiter(
    ResolveHostClient* proxy) : proxy_(proxy) {}

ResolveHostClientAsyncWaiter::~ResolveHostClientAsyncWaiter() = default;


const char HostResolver::Name_[] = "network.mojom.HostResolver";

HostResolverProxy::HostResolverProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class HostResolverProxy_ResolveHost_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit HostResolverProxy_ResolveHost_Message(
      uint32_t message_flags
      , const net::HostPortPair& param_host
      , ResolveHostParametersPtr param_optional_parameters
      , ResolveHostClientPtr param_response_client
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kHostResolver_ResolveHost_Name,
          message_flags)
      , param_host_(std::move(param_host))
      , param_optional_parameters_(std::move(param_optional_parameters))
      , param_response_client_(param_response_client.PassInterface()){}
  ~HostResolverProxy_ResolveHost_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const net::HostPortPair& param_host, ResolveHostParametersPtr param_optional_parameters, ResolveHostClientPtr param_response_client) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<HostResolverProxy_ResolveHost_Message>(
          kFlags
          , std::move(param_host)
          , std::move(param_optional_parameters)
          , std::move(param_response_client)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kHostResolver_ResolveHost_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::HostResolver_ResolveHost_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->host)::BaseType::BufferWriter
      host_writer;
  mojo::internal::Serialize<::network::mojom::HostPortPairDataView>(
      param_host, buffer, &host_writer, &serialization_context);
  params->host.Set(
      host_writer.is_null() ? nullptr : host_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->host.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null host in HostResolver.ResolveHost request");
  typename decltype(params->optional_parameters)::BaseType::BufferWriter
      optional_parameters_writer;
  mojo::internal::Serialize<::network::mojom::ResolveHostParametersDataView>(
      param_optional_parameters, buffer, &optional_parameters_writer, &serialization_context);
  params->optional_parameters.Set(
      optional_parameters_writer.is_null() ? nullptr : optional_parameters_writer.data());
  mojo::internal::Serialize<::network::mojom::ResolveHostClientPtrDataView>(
      param_response_client, &params->response_client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->response_client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid response_client in HostResolver.ResolveHost request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(HostResolver* impl) {
    impl->ResolveHost(
        std::move(param_host_),
        std::move(param_optional_parameters_),
        ResolveHostClientPtr(std::move(param_response_client_)));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::HostResolver_ResolveHost_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->host)::BaseType::BufferWriter
      host_writer;
  mojo::internal::Serialize<::network::mojom::HostPortPairDataView>(
      param_host_, buffer, &host_writer, serialization_context);
  params->host.Set(
      host_writer.is_null() ? nullptr : host_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->host.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null host in HostResolver.ResolveHost request");
  typename decltype(params->optional_parameters)::BaseType::BufferWriter
      optional_parameters_writer;
  mojo::internal::Serialize<::network::mojom::ResolveHostParametersDataView>(
      param_optional_parameters_, buffer, &optional_parameters_writer, serialization_context);
  params->optional_parameters.Set(
      optional_parameters_writer.is_null() ? nullptr : optional_parameters_writer.data());
  mojo::internal::Serialize<::network::mojom::ResolveHostClientPtrDataView>(
      param_response_client_, &params->response_client, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->response_client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid response_client in HostResolver.ResolveHost request");
  }
  net::HostPortPair param_host_;
  ResolveHostParametersPtr param_optional_parameters_;
  ResolveHostClientPtrInfo param_response_client_;

  DISALLOW_COPY_AND_ASSIGN(HostResolverProxy_ResolveHost_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
HostResolverProxy_ResolveHost_Message::kMessageTag = {};

void HostResolverProxy::ResolveHost(
    const net::HostPortPair& in_host, ResolveHostParametersPtr in_optional_parameters, ResolveHostClientPtr in_response_client) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::HostResolver::ResolveHost");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = HostResolverProxy_ResolveHost_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_host), std::move(in_optional_parameters), std::move(in_response_client));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(HostResolver::Name_);
  message.set_method_name("ResolveHost");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool HostResolverStubDispatch::Accept(
    HostResolver* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kHostResolver_ResolveHost_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::HostResolver::ResolveHost",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            HostResolverProxy_ResolveHost_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::HostResolver_ResolveHost_Params_Data* params =
          reinterpret_cast<internal::HostResolver_ResolveHost_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      net::HostPortPair p_host{};
      ResolveHostParametersPtr p_optional_parameters{};
      ResolveHostClientPtr p_response_client{};
      HostResolver_ResolveHost_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadHost(&p_host))
        success = false;
      if (!input_data_view.ReadOptionalParameters(&p_optional_parameters))
        success = false;
      p_response_client =
          input_data_view.TakeResponseClient<decltype(p_response_client)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "HostResolver::ResolveHost deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ResolveHost(
std::move(p_host), 
std::move(p_optional_parameters), 
std::move(p_response_client));
      return true;
    }
  }
  return false;
}

// static
bool HostResolverStubDispatch::AcceptWithResponder(
    HostResolver* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kHostResolver_ResolveHost_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::HostResolver::ResolveHost",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool HostResolverRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "HostResolver RequestValidator");

  switch (message->header()->name) {
    case internal::kHostResolver_ResolveHost_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::HostResolver_ResolveHost_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void HostResolverInterceptorForTesting::ResolveHost(const net::HostPortPair& host, ResolveHostParametersPtr optional_parameters, ResolveHostClientPtr response_client) {
  GetForwardingInterface()->ResolveHost(std::move(host), std::move(optional_parameters), std::move(response_client));
}
HostResolverAsyncWaiter::HostResolverAsyncWaiter(
    HostResolver* proxy) : proxy_(proxy) {}

HostResolverAsyncWaiter::~HostResolverAsyncWaiter() = default;


const char DnsConfigChangeManagerClient::Name_[] = "network.mojom.DnsConfigChangeManagerClient";

DnsConfigChangeManagerClientProxy::DnsConfigChangeManagerClientProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class DnsConfigChangeManagerClientProxy_OnSystemDnsConfigChanged_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit DnsConfigChangeManagerClientProxy_OnSystemDnsConfigChanged_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kDnsConfigChangeManagerClient_OnSystemDnsConfigChanged_Name,
          message_flags){}
  ~DnsConfigChangeManagerClientProxy_OnSystemDnsConfigChanged_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<DnsConfigChangeManagerClientProxy_OnSystemDnsConfigChanged_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kDnsConfigChangeManagerClient_OnSystemDnsConfigChanged_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::DnsConfigChangeManagerClient_OnSystemDnsConfigChanged_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(DnsConfigChangeManagerClient* impl) {
    impl->OnSystemDnsConfigChanged();
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::DnsConfigChangeManagerClient_OnSystemDnsConfigChanged_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(DnsConfigChangeManagerClientProxy_OnSystemDnsConfigChanged_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
DnsConfigChangeManagerClientProxy_OnSystemDnsConfigChanged_Message::kMessageTag = {};

void DnsConfigChangeManagerClientProxy::OnSystemDnsConfigChanged(
    ) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::DnsConfigChangeManagerClient::OnSystemDnsConfigChanged");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = DnsConfigChangeManagerClientProxy_OnSystemDnsConfigChanged_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(DnsConfigChangeManagerClient::Name_);
  message.set_method_name("OnSystemDnsConfigChanged");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool DnsConfigChangeManagerClientStubDispatch::Accept(
    DnsConfigChangeManagerClient* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kDnsConfigChangeManagerClient_OnSystemDnsConfigChanged_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::DnsConfigChangeManagerClient::OnSystemDnsConfigChanged",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            DnsConfigChangeManagerClientProxy_OnSystemDnsConfigChanged_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::DnsConfigChangeManagerClient_OnSystemDnsConfigChanged_Params_Data* params =
          reinterpret_cast<internal::DnsConfigChangeManagerClient_OnSystemDnsConfigChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      DnsConfigChangeManagerClient_OnSystemDnsConfigChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "DnsConfigChangeManagerClient::OnSystemDnsConfigChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnSystemDnsConfigChanged();
      return true;
    }
  }
  return false;
}

// static
bool DnsConfigChangeManagerClientStubDispatch::AcceptWithResponder(
    DnsConfigChangeManagerClient* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kDnsConfigChangeManagerClient_OnSystemDnsConfigChanged_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::DnsConfigChangeManagerClient::OnSystemDnsConfigChanged",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool DnsConfigChangeManagerClientRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "DnsConfigChangeManagerClient RequestValidator");

  switch (message->header()->name) {
    case internal::kDnsConfigChangeManagerClient_OnSystemDnsConfigChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::DnsConfigChangeManagerClient_OnSystemDnsConfigChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void DnsConfigChangeManagerClientInterceptorForTesting::OnSystemDnsConfigChanged() {
  GetForwardingInterface()->OnSystemDnsConfigChanged();
}
DnsConfigChangeManagerClientAsyncWaiter::DnsConfigChangeManagerClientAsyncWaiter(
    DnsConfigChangeManagerClient* proxy) : proxy_(proxy) {}

DnsConfigChangeManagerClientAsyncWaiter::~DnsConfigChangeManagerClientAsyncWaiter() = default;


const char DnsConfigChangeManager::Name_[] = "network.mojom.DnsConfigChangeManager";

DnsConfigChangeManagerProxy::DnsConfigChangeManagerProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class DnsConfigChangeManagerProxy_RequestNotifications_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit DnsConfigChangeManagerProxy_RequestNotifications_Message(
      uint32_t message_flags
      , DnsConfigChangeManagerClientPtr param_client_ptr
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kDnsConfigChangeManager_RequestNotifications_Name,
          message_flags)
      , param_client_ptr_(param_client_ptr.PassInterface()){}
  ~DnsConfigChangeManagerProxy_RequestNotifications_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      DnsConfigChangeManagerClientPtr param_client_ptr) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<DnsConfigChangeManagerProxy_RequestNotifications_Message>(
          kFlags
          , std::move(param_client_ptr)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kDnsConfigChangeManager_RequestNotifications_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::DnsConfigChangeManager_RequestNotifications_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::DnsConfigChangeManagerClientPtrDataView>(
      param_client_ptr, &params->client_ptr, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client_ptr),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client_ptr in DnsConfigChangeManager.RequestNotifications request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(DnsConfigChangeManager* impl) {
    impl->RequestNotifications(
        DnsConfigChangeManagerClientPtr(std::move(param_client_ptr_)));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::DnsConfigChangeManager_RequestNotifications_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::DnsConfigChangeManagerClientPtrDataView>(
      param_client_ptr_, &params->client_ptr, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client_ptr),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client_ptr in DnsConfigChangeManager.RequestNotifications request");
  }
  DnsConfigChangeManagerClientPtrInfo param_client_ptr_;

  DISALLOW_COPY_AND_ASSIGN(DnsConfigChangeManagerProxy_RequestNotifications_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
DnsConfigChangeManagerProxy_RequestNotifications_Message::kMessageTag = {};

void DnsConfigChangeManagerProxy::RequestNotifications(
    DnsConfigChangeManagerClientPtr in_client_ptr) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::DnsConfigChangeManager::RequestNotifications");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = DnsConfigChangeManagerProxy_RequestNotifications_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_client_ptr));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(DnsConfigChangeManager::Name_);
  message.set_method_name("RequestNotifications");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool DnsConfigChangeManagerStubDispatch::Accept(
    DnsConfigChangeManager* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kDnsConfigChangeManager_RequestNotifications_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::DnsConfigChangeManager::RequestNotifications",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            DnsConfigChangeManagerProxy_RequestNotifications_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::DnsConfigChangeManager_RequestNotifications_Params_Data* params =
          reinterpret_cast<internal::DnsConfigChangeManager_RequestNotifications_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      DnsConfigChangeManagerClientPtr p_client_ptr{};
      DnsConfigChangeManager_RequestNotifications_ParamsDataView input_data_view(params, &serialization_context);
      
      p_client_ptr =
          input_data_view.TakeClientPtr<decltype(p_client_ptr)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "DnsConfigChangeManager::RequestNotifications deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->RequestNotifications(
std::move(p_client_ptr));
      return true;
    }
  }
  return false;
}

// static
bool DnsConfigChangeManagerStubDispatch::AcceptWithResponder(
    DnsConfigChangeManager* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kDnsConfigChangeManager_RequestNotifications_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::DnsConfigChangeManager::RequestNotifications",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool DnsConfigChangeManagerRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "DnsConfigChangeManager RequestValidator");

  switch (message->header()->name) {
    case internal::kDnsConfigChangeManager_RequestNotifications_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::DnsConfigChangeManager_RequestNotifications_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void DnsConfigChangeManagerInterceptorForTesting::RequestNotifications(DnsConfigChangeManagerClientPtr client_ptr) {
  GetForwardingInterface()->RequestNotifications(std::move(client_ptr));
}
DnsConfigChangeManagerAsyncWaiter::DnsConfigChangeManagerAsyncWaiter(
    DnsConfigChangeManager* proxy) : proxy_(proxy) {}

DnsConfigChangeManagerAsyncWaiter::~DnsConfigChangeManagerAsyncWaiter() = default;


}  // namespace mojom
}  // namespace network

namespace mojo {


// static
bool StructTraits<::network::mojom::DnsHost::DataView, ::network::mojom::DnsHostPtr>::Read(
    ::network::mojom::DnsHost::DataView input,
    ::network::mojom::DnsHostPtr* output) {
  bool success = true;
  ::network::mojom::DnsHostPtr result(::network::mojom::DnsHost::New());
  
      if (!input.ReadHostname(&result->hostname))
        success = false;
      if (!input.ReadAddress(&result->address))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::network::mojom::DnsOverHttpsServer::DataView, ::network::mojom::DnsOverHttpsServerPtr>::Read(
    ::network::mojom::DnsOverHttpsServer::DataView input,
    ::network::mojom::DnsOverHttpsServerPtr* output) {
  bool success = true;
  ::network::mojom::DnsOverHttpsServerPtr result(::network::mojom::DnsOverHttpsServer::New());
  
      if (!input.ReadServerTemplate(&result->server_template))
        success = false;
      result->use_post = input.use_post();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::network::mojom::DnsConfigOverrides::DataView, ::network::mojom::DnsConfigOverridesPtr>::Read(
    ::network::mojom::DnsConfigOverrides::DataView input,
    ::network::mojom::DnsConfigOverridesPtr* output) {
  bool success = true;
  ::network::mojom::DnsConfigOverridesPtr result(::network::mojom::DnsConfigOverrides::New());
  
      if (!input.ReadNameservers(&result->nameservers))
        success = false;
      if (!input.ReadSearch(&result->search))
        success = false;
      if (!input.ReadHosts(&result->hosts))
        success = false;
      if (!input.ReadAppendToMultiLabelName(&result->append_to_multi_label_name))
        success = false;
      if (!input.ReadRandomizePorts(&result->randomize_ports))
        success = false;
      result->ndots = input.ndots();
      if (!input.ReadTimeout(&result->timeout))
        success = false;
      result->attempts = input.attempts();
      if (!input.ReadRotate(&result->rotate))
        success = false;
      if (!input.ReadUseLocalIpv6(&result->use_local_ipv6))
        success = false;
      if (!input.ReadDnsOverHttpsServers(&result->dns_over_https_servers))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::network::mojom::ResolveHostParameters::DataView, ::network::mojom::ResolveHostParametersPtr>::Read(
    ::network::mojom::ResolveHostParameters::DataView input,
    ::network::mojom::ResolveHostParametersPtr* output) {
  bool success = true;
  ::network::mojom::ResolveHostParametersPtr result(::network::mojom::ResolveHostParameters::New());
  
      if (!input.ReadDnsQueryType(&result->dns_query_type))
        success = false;
      if (!input.ReadInitialPriority(&result->initial_priority))
        success = false;
      if (!input.ReadSource(&result->source))
        success = false;
      result->allow_cached_response = input.allow_cached_response();
      result->control_handle =
          input.TakeControlHandle<decltype(result->control_handle)>();
      result->include_canonical_name = input.include_canonical_name();
      result->loopback_only = input.loopback_only();
      result->is_speculative = input.is_speculative();
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif