// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "services/network/public/mojom/network_context.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "services/network/public/mojom/network_context.mojom-shared-message-ids.h"
#include "components/content_settings/core/common/content_settings_struct_traits.h"
#include "mojo/public/cpp/base/file_path_mojom_traits.h"
#include "mojo/public/cpp/base/time_mojom_traits.h"
#include "mojo/public/cpp/base/unguessable_token_mojom_traits.h"
#include "mojo/public/cpp/base/values_mojom_traits.h"
#include "services/network/public/cpp/address_list_mojom_traits.h"
#include "services/network/public/cpp/host_resolver_mojom_traits.h"
#include "services/network/public/cpp/http_request_headers_mojom_traits.h"
#include "services/network/public/cpp/ip_address_mojom_traits.h"
#include "services/network/public/cpp/ip_endpoint_mojom_traits.h"
#include "services/network/public/cpp/mutable_network_traffic_annotation_tag_mojom_traits.h"
#include "services/network/public/cpp/net_ipc_param_traits.h"
#include "services/network/public/cpp/network_ipc_param_traits.h"
#include "services/network/public/cpp/network_param_mojom_traits.h"
#include "services/network/public/cpp/proxy_config_mojom_traits.h"
#include "services/network/public/cpp/proxy_config_with_annotation_mojom_traits.h"
#include "services/network/public/cpp/url_request_mojom_traits.h"
#include "url/mojom/origin_mojom_traits.h"
#include "url/mojom/url_gurl_mojom_traits.h"
namespace network {
namespace mojom {
CustomProxyConfig::CustomProxyConfig()
    : rules(),
      alternate_proxy_list(),
      pre_cache_headers(),
      post_cache_headers() {}

CustomProxyConfig::CustomProxyConfig(
    const net::ProxyConfig::ProxyRules& rules_in,
    const net::ProxyList& alternate_proxy_list_in,
    const net::HttpRequestHeaders& pre_cache_headers_in,
    const net::HttpRequestHeaders& post_cache_headers_in)
    : rules(std::move(rules_in)),
      alternate_proxy_list(std::move(alternate_proxy_list_in)),
      pre_cache_headers(std::move(pre_cache_headers_in)),
      post_cache_headers(std::move(post_cache_headers_in)) {}

CustomProxyConfig::~CustomProxyConfig() = default;

bool CustomProxyConfig::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
NetworkContextParams::NetworkContextParams()
    : context_name(),
      user_agent(),
      accept_language(),
      enable_brotli(true),
      quic_user_agent_id(),
      enable_referrers(true),
      validate_referrer_policy_on_initial_request(true),
      proxy_resolver_factory(),
      cookie_path(),
      enable_encrypted_cookies(true),
      restore_old_session_cookies(false),
      persist_session_cookies(false),
      channel_id_path(),
      http_cache_enabled(true),
      http_cache_max_size(0),
      http_cache_path(),
      http_server_properties_path(),
      transport_security_persister_path(),
      enable_data_url_support(false),
      enable_file_url_support(false),
      enable_ftp_url_support(false),
      enforce_chrome_ct_policy(false),
      http_09_on_non_default_ports_enabled(false),
      initial_ssl_config(),
      ssl_config_client_request(),
      initial_proxy_config(),
      proxy_config_client_request(),
      initial_custom_proxy_config(),
      custom_proxy_config_client_request(),
      proxy_config_poller_client(),
      proxy_error_client(),
      pac_quick_check_enabled(true),
      dangerously_allow_pac_access_to_secure_urls(false),
      enable_certificate_reporting(false),
      enable_expect_ct_reporting(false),
      ct_logs(),
      cookie_manager_params(),
      enable_domain_reliability(false),
      domain_reliability_upload_reporter(),
      discard_domain_reliablity_uploads(false),
      primary_network_context(false),
      cors_origin_access_list() {}

NetworkContextParams::NetworkContextParams(
    const base::Optional<std::string>& context_name_in,
    const std::string& user_agent_in,
    const std::string& accept_language_in,
    bool enable_brotli_in,
    const std::string& quic_user_agent_id_in,
    bool enable_referrers_in,
    bool validate_referrer_policy_on_initial_request_in,
    ::proxy_resolver::mojom::ProxyResolverFactoryPtrInfo proxy_resolver_factory_in,
    const base::Optional<base::FilePath>& cookie_path_in,
    bool enable_encrypted_cookies_in,
    bool restore_old_session_cookies_in,
    bool persist_session_cookies_in,
    const base::Optional<base::FilePath>& channel_id_path_in,
    bool http_cache_enabled_in,
    int32_t http_cache_max_size_in,
    const base::Optional<base::FilePath>& http_cache_path_in,
    const base::Optional<base::FilePath>& http_server_properties_path_in,
    const base::Optional<base::FilePath>& transport_security_persister_path_in,
    bool enable_data_url_support_in,
    bool enable_file_url_support_in,
    bool enable_ftp_url_support_in,
    bool enforce_chrome_ct_policy_in,
    bool http_09_on_non_default_ports_enabled_in,
    ::network::mojom::SSLConfigPtr initial_ssl_config_in,
    ::network::mojom::SSLConfigClientRequest ssl_config_client_request_in,
    const base::Optional<net::ProxyConfigWithAnnotation>& initial_proxy_config_in,
    ::network::mojom::ProxyConfigClientRequest proxy_config_client_request_in,
    CustomProxyConfigPtr initial_custom_proxy_config_in,
    CustomProxyConfigClientRequest custom_proxy_config_client_request_in,
    ::network::mojom::ProxyConfigPollerClientPtrInfo proxy_config_poller_client_in,
    ::network::mojom::ProxyErrorClientPtrInfo proxy_error_client_in,
    bool pac_quick_check_enabled_in,
    bool dangerously_allow_pac_access_to_secure_urls_in,
    bool enable_certificate_reporting_in,
    bool enable_expect_ct_reporting_in,
    std::vector<::network::mojom::CTLogInfoPtr> ct_logs_in,
    ::network::mojom::CookieManagerParamsPtr cookie_manager_params_in,
    bool enable_domain_reliability_in,
    const std::string& domain_reliability_upload_reporter_in,
    bool discard_domain_reliablity_uploads_in,
    bool primary_network_context_in,
    std::vector<::network::mojom::CorsOriginAccessPatternsPtr> cors_origin_access_list_in)
    : context_name(std::move(context_name_in)),
      user_agent(std::move(user_agent_in)),
      accept_language(std::move(accept_language_in)),
      enable_brotli(std::move(enable_brotli_in)),
      quic_user_agent_id(std::move(quic_user_agent_id_in)),
      enable_referrers(std::move(enable_referrers_in)),
      validate_referrer_policy_on_initial_request(std::move(validate_referrer_policy_on_initial_request_in)),
      proxy_resolver_factory(std::move(proxy_resolver_factory_in)),
      cookie_path(std::move(cookie_path_in)),
      enable_encrypted_cookies(std::move(enable_encrypted_cookies_in)),
      restore_old_session_cookies(std::move(restore_old_session_cookies_in)),
      persist_session_cookies(std::move(persist_session_cookies_in)),
      channel_id_path(std::move(channel_id_path_in)),
      http_cache_enabled(std::move(http_cache_enabled_in)),
      http_cache_max_size(std::move(http_cache_max_size_in)),
      http_cache_path(std::move(http_cache_path_in)),
      http_server_properties_path(std::move(http_server_properties_path_in)),
      transport_security_persister_path(std::move(transport_security_persister_path_in)),
      enable_data_url_support(std::move(enable_data_url_support_in)),
      enable_file_url_support(std::move(enable_file_url_support_in)),
      enable_ftp_url_support(std::move(enable_ftp_url_support_in)),
      enforce_chrome_ct_policy(std::move(enforce_chrome_ct_policy_in)),
      http_09_on_non_default_ports_enabled(std::move(http_09_on_non_default_ports_enabled_in)),
      initial_ssl_config(std::move(initial_ssl_config_in)),
      ssl_config_client_request(std::move(ssl_config_client_request_in)),
      initial_proxy_config(std::move(initial_proxy_config_in)),
      proxy_config_client_request(std::move(proxy_config_client_request_in)),
      initial_custom_proxy_config(std::move(initial_custom_proxy_config_in)),
      custom_proxy_config_client_request(std::move(custom_proxy_config_client_request_in)),
      proxy_config_poller_client(std::move(proxy_config_poller_client_in)),
      proxy_error_client(std::move(proxy_error_client_in)),
      pac_quick_check_enabled(std::move(pac_quick_check_enabled_in)),
      dangerously_allow_pac_access_to_secure_urls(std::move(dangerously_allow_pac_access_to_secure_urls_in)),
      enable_certificate_reporting(std::move(enable_certificate_reporting_in)),
      enable_expect_ct_reporting(std::move(enable_expect_ct_reporting_in)),
      ct_logs(std::move(ct_logs_in)),
      cookie_manager_params(std::move(cookie_manager_params_in)),
      enable_domain_reliability(std::move(enable_domain_reliability_in)),
      domain_reliability_upload_reporter(std::move(domain_reliability_upload_reporter_in)),
      discard_domain_reliablity_uploads(std::move(discard_domain_reliablity_uploads_in)),
      primary_network_context(std::move(primary_network_context_in)),
      cors_origin_access_list(std::move(cors_origin_access_list_in)) {}

NetworkContextParams::~NetworkContextParams() = default;

bool NetworkContextParams::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
NetworkConditions::NetworkConditions()
    : offline(),
      latency(),
      download_throughput(),
      upload_throughput() {}

NetworkConditions::NetworkConditions(
    bool offline_in,
    base::TimeDelta latency_in,
    double download_throughput_in,
    double upload_throughput_in)
    : offline(std::move(offline_in)),
      latency(std::move(latency_in)),
      download_throughput(std::move(download_throughput_in)),
      upload_throughput(std::move(upload_throughput_in)) {}

NetworkConditions::~NetworkConditions() = default;

bool NetworkConditions::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
ClearDataFilter::ClearDataFilter()
    : type(),
      domains(),
      origins() {}

ClearDataFilter::ClearDataFilter(
    ClearDataFilter::Type type_in,
    const std::vector<std::string>& domains_in,
    const std::vector<url::Origin>& origins_in)
    : type(std::move(type_in)),
      domains(std::move(domains_in)),
      origins(std::move(origins_in)) {}

ClearDataFilter::~ClearDataFilter() = default;

bool ClearDataFilter::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
NetworkUsage::NetworkUsage()
    : process_id(),
      routing_id(),
      total_bytes_received(),
      total_bytes_sent() {}

NetworkUsage::NetworkUsage(
    uint32_t process_id_in,
    uint32_t routing_id_in,
    int64_t total_bytes_received_in,
    int64_t total_bytes_sent_in)
    : process_id(std::move(process_id_in)),
      routing_id(std::move(routing_id_in)),
      total_bytes_received(std::move(total_bytes_received_in)),
      total_bytes_sent(std::move(total_bytes_sent_in)) {}

NetworkUsage::~NetworkUsage() = default;
size_t NetworkUsage::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->process_id);
  seed = mojo::internal::Hash(seed, this->routing_id);
  seed = mojo::internal::Hash(seed, this->total_bytes_received);
  seed = mojo::internal::Hash(seed, this->total_bytes_sent);
  return seed;
}

bool NetworkUsage::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
URLLoaderFactoryParams::URLLoaderFactoryParams()
    : process_id(kInvalidProcessId),
      is_corb_enabled(true),
      corb_detachable_resource_type(-1),
      corb_excluded_resource_type(-1),
      disable_web_security(false),
      header_client() {}

URLLoaderFactoryParams::URLLoaderFactoryParams(
    uint32_t process_id_in,
    bool is_corb_enabled_in,
    int32_t corb_detachable_resource_type_in,
    int32_t corb_excluded_resource_type_in,
    bool disable_web_security_in,
    TrustedURLLoaderHeaderClientPtrInfo header_client_in)
    : process_id(std::move(process_id_in)),
      is_corb_enabled(std::move(is_corb_enabled_in)),
      corb_detachable_resource_type(std::move(corb_detachable_resource_type_in)),
      corb_excluded_resource_type(std::move(corb_excluded_resource_type_in)),
      disable_web_security(std::move(disable_web_security_in)),
      header_client(std::move(header_client_in)) {}

URLLoaderFactoryParams::~URLLoaderFactoryParams() = default;

bool URLLoaderFactoryParams::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char CustomProxyConfigClient::Name_[] = "network.mojom.CustomProxyConfigClient";

class CustomProxyConfigClient_MarkProxiesAsBad_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  CustomProxyConfigClient_MarkProxiesAsBad_ForwardToCallback(
      CustomProxyConfigClient::MarkProxiesAsBadCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  CustomProxyConfigClient::MarkProxiesAsBadCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(CustomProxyConfigClient_MarkProxiesAsBad_ForwardToCallback);
};

CustomProxyConfigClientProxy::CustomProxyConfigClientProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class CustomProxyConfigClientProxy_OnCustomProxyConfigUpdated_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit CustomProxyConfigClientProxy_OnCustomProxyConfigUpdated_Message(
      uint32_t message_flags
      , CustomProxyConfigPtr param_proxy_config
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kCustomProxyConfigClient_OnCustomProxyConfigUpdated_Name,
          message_flags)
      , param_proxy_config_(std::move(param_proxy_config)){}
  ~CustomProxyConfigClientProxy_OnCustomProxyConfigUpdated_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      CustomProxyConfigPtr param_proxy_config) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<CustomProxyConfigClientProxy_OnCustomProxyConfigUpdated_Message>(
          kFlags
          , std::move(param_proxy_config)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kCustomProxyConfigClient_OnCustomProxyConfigUpdated_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::CustomProxyConfigClient_OnCustomProxyConfigUpdated_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->proxy_config)::BaseType::BufferWriter
      proxy_config_writer;
  mojo::internal::Serialize<::network::mojom::CustomProxyConfigDataView>(
      param_proxy_config, buffer, &proxy_config_writer, &serialization_context);
  params->proxy_config.Set(
      proxy_config_writer.is_null() ? nullptr : proxy_config_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->proxy_config.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null proxy_config in CustomProxyConfigClient.OnCustomProxyConfigUpdated request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(CustomProxyConfigClient* impl) {
    impl->OnCustomProxyConfigUpdated(
        std::move(param_proxy_config_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::CustomProxyConfigClient_OnCustomProxyConfigUpdated_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->proxy_config)::BaseType::BufferWriter
      proxy_config_writer;
  mojo::internal::Serialize<::network::mojom::CustomProxyConfigDataView>(
      param_proxy_config_, buffer, &proxy_config_writer, serialization_context);
  params->proxy_config.Set(
      proxy_config_writer.is_null() ? nullptr : proxy_config_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->proxy_config.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null proxy_config in CustomProxyConfigClient.OnCustomProxyConfigUpdated request");
  }
  CustomProxyConfigPtr param_proxy_config_;

  DISALLOW_COPY_AND_ASSIGN(CustomProxyConfigClientProxy_OnCustomProxyConfigUpdated_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
CustomProxyConfigClientProxy_OnCustomProxyConfigUpdated_Message::kMessageTag = {};

void CustomProxyConfigClientProxy::OnCustomProxyConfigUpdated(
    CustomProxyConfigPtr in_proxy_config) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::CustomProxyConfigClient::OnCustomProxyConfigUpdated");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = CustomProxyConfigClientProxy_OnCustomProxyConfigUpdated_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_proxy_config));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(CustomProxyConfigClient::Name_);
  message.set_method_name("OnCustomProxyConfigUpdated");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class CustomProxyConfigClientProxy_MarkProxiesAsBad_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit CustomProxyConfigClientProxy_MarkProxiesAsBad_Message(
      uint32_t message_flags
      , base::TimeDelta param_bypass_duration
      , const net::ProxyList& param_bad_proxies
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kCustomProxyConfigClient_MarkProxiesAsBad_Name,
          message_flags)
      , param_bypass_duration_(std::move(param_bypass_duration))
      , param_bad_proxies_(std::move(param_bad_proxies)){}
  ~CustomProxyConfigClientProxy_MarkProxiesAsBad_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      base::TimeDelta param_bypass_duration, const net::ProxyList& param_bad_proxies) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<CustomProxyConfigClientProxy_MarkProxiesAsBad_Message>(
          kFlags
          , std::move(param_bypass_duration)
          , std::move(param_bad_proxies)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kCustomProxyConfigClient_MarkProxiesAsBad_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::CustomProxyConfigClient_MarkProxiesAsBad_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->bypass_duration)::BaseType::BufferWriter
      bypass_duration_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDeltaDataView>(
      param_bypass_duration, buffer, &bypass_duration_writer, &serialization_context);
  params->bypass_duration.Set(
      bypass_duration_writer.is_null() ? nullptr : bypass_duration_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->bypass_duration.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null bypass_duration in CustomProxyConfigClient.MarkProxiesAsBad request");
  typename decltype(params->bad_proxies)::BaseType::BufferWriter
      bad_proxies_writer;
  mojo::internal::Serialize<::network::mojom::ProxyListDataView>(
      param_bad_proxies, buffer, &bad_proxies_writer, &serialization_context);
  params->bad_proxies.Set(
      bad_proxies_writer.is_null() ? nullptr : bad_proxies_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->bad_proxies.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null bad_proxies in CustomProxyConfigClient.MarkProxiesAsBad request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(CustomProxyConfigClient* impl, CustomProxyConfigClient::MarkProxiesAsBadCallback callback) {
    impl->MarkProxiesAsBad(
        std::move(param_bypass_duration_),
        std::move(param_bad_proxies_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::CustomProxyConfigClient_MarkProxiesAsBad_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->bypass_duration)::BaseType::BufferWriter
      bypass_duration_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDeltaDataView>(
      param_bypass_duration_, buffer, &bypass_duration_writer, serialization_context);
  params->bypass_duration.Set(
      bypass_duration_writer.is_null() ? nullptr : bypass_duration_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->bypass_duration.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null bypass_duration in CustomProxyConfigClient.MarkProxiesAsBad request");
  typename decltype(params->bad_proxies)::BaseType::BufferWriter
      bad_proxies_writer;
  mojo::internal::Serialize<::network::mojom::ProxyListDataView>(
      param_bad_proxies_, buffer, &bad_proxies_writer, serialization_context);
  params->bad_proxies.Set(
      bad_proxies_writer.is_null() ? nullptr : bad_proxies_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->bad_proxies.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null bad_proxies in CustomProxyConfigClient.MarkProxiesAsBad request");
  }
  base::TimeDelta param_bypass_duration_;
  net::ProxyList param_bad_proxies_;

  DISALLOW_COPY_AND_ASSIGN(CustomProxyConfigClientProxy_MarkProxiesAsBad_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
CustomProxyConfigClientProxy_MarkProxiesAsBad_Message::kMessageTag = {};

void CustomProxyConfigClientProxy::MarkProxiesAsBad(
    base::TimeDelta in_bypass_duration, const net::ProxyList& in_bad_proxies, MarkProxiesAsBadCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::CustomProxyConfigClient::MarkProxiesAsBad");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = CustomProxyConfigClientProxy_MarkProxiesAsBad_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_bypass_duration), std::move(in_bad_proxies));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(CustomProxyConfigClient::Name_);
  message.set_method_name("MarkProxiesAsBad");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new CustomProxyConfigClient_MarkProxiesAsBad_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class CustomProxyConfigClientProxy_ClearBadProxiesCache_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit CustomProxyConfigClientProxy_ClearBadProxiesCache_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kCustomProxyConfigClient_ClearBadProxiesCache_Name,
          message_flags){}
  ~CustomProxyConfigClientProxy_ClearBadProxiesCache_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<CustomProxyConfigClientProxy_ClearBadProxiesCache_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kCustomProxyConfigClient_ClearBadProxiesCache_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::CustomProxyConfigClient_ClearBadProxiesCache_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(CustomProxyConfigClient* impl) {
    impl->ClearBadProxiesCache();
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::CustomProxyConfigClient_ClearBadProxiesCache_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(CustomProxyConfigClientProxy_ClearBadProxiesCache_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
CustomProxyConfigClientProxy_ClearBadProxiesCache_Message::kMessageTag = {};

void CustomProxyConfigClientProxy::ClearBadProxiesCache(
    ) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::CustomProxyConfigClient::ClearBadProxiesCache");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = CustomProxyConfigClientProxy_ClearBadProxiesCache_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(CustomProxyConfigClient::Name_);
  message.set_method_name("ClearBadProxiesCache");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class CustomProxyConfigClient_MarkProxiesAsBad_ProxyToResponder {
 public:
  static CustomProxyConfigClient::MarkProxiesAsBadCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<CustomProxyConfigClient_MarkProxiesAsBad_ProxyToResponder> proxy(
        new CustomProxyConfigClient_MarkProxiesAsBad_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&CustomProxyConfigClient_MarkProxiesAsBad_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~CustomProxyConfigClient_MarkProxiesAsBad_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  CustomProxyConfigClient_MarkProxiesAsBad_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "CustomProxyConfigClient::MarkProxiesAsBadCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(CustomProxyConfigClient_MarkProxiesAsBad_ProxyToResponder);
};
class CustomProxyConfigClient_MarkProxiesAsBad_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit CustomProxyConfigClient_MarkProxiesAsBad_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kCustomProxyConfigClient_MarkProxiesAsBad_Name,
          message_flags){}
  ~CustomProxyConfigClient_MarkProxiesAsBad_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<CustomProxyConfigClient_MarkProxiesAsBad_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kCustomProxyConfigClient_MarkProxiesAsBad_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::CustomProxyConfigClient_MarkProxiesAsBad_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(CustomProxyConfigClient::MarkProxiesAsBadCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::CustomProxyConfigClient_MarkProxiesAsBad_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(CustomProxyConfigClient_MarkProxiesAsBad_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
CustomProxyConfigClient_MarkProxiesAsBad_Response_Message::kMessageTag = {};

bool CustomProxyConfigClient_MarkProxiesAsBad_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::CustomProxyConfigClient::MarkProxiesAsBadCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<CustomProxyConfigClient_MarkProxiesAsBad_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::CustomProxyConfigClient_MarkProxiesAsBad_ResponseParams_Data* params =
      reinterpret_cast<
          internal::CustomProxyConfigClient_MarkProxiesAsBad_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  CustomProxyConfigClient_MarkProxiesAsBad_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "CustomProxyConfigClient::MarkProxiesAsBad response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void CustomProxyConfigClient_MarkProxiesAsBad_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = CustomProxyConfigClient_MarkProxiesAsBad_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::CustomProxyConfigClient::MarkProxiesAsBadCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(CustomProxyConfigClient::Name_);
  message.set_method_name("MarkProxiesAsBad");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool CustomProxyConfigClientStubDispatch::Accept(
    CustomProxyConfigClient* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kCustomProxyConfigClient_OnCustomProxyConfigUpdated_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::CustomProxyConfigClient::OnCustomProxyConfigUpdated",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            CustomProxyConfigClientProxy_OnCustomProxyConfigUpdated_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::CustomProxyConfigClient_OnCustomProxyConfigUpdated_Params_Data* params =
          reinterpret_cast<internal::CustomProxyConfigClient_OnCustomProxyConfigUpdated_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      CustomProxyConfigPtr p_proxy_config{};
      CustomProxyConfigClient_OnCustomProxyConfigUpdated_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadProxyConfig(&p_proxy_config))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "CustomProxyConfigClient::OnCustomProxyConfigUpdated deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnCustomProxyConfigUpdated(
std::move(p_proxy_config));
      return true;
    }
    case internal::kCustomProxyConfigClient_MarkProxiesAsBad_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::CustomProxyConfigClient::MarkProxiesAsBad",
               "message", message->name());
#endif
      break;
    }
    case internal::kCustomProxyConfigClient_ClearBadProxiesCache_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::CustomProxyConfigClient::ClearBadProxiesCache",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            CustomProxyConfigClientProxy_ClearBadProxiesCache_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::CustomProxyConfigClient_ClearBadProxiesCache_Params_Data* params =
          reinterpret_cast<internal::CustomProxyConfigClient_ClearBadProxiesCache_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      CustomProxyConfigClient_ClearBadProxiesCache_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "CustomProxyConfigClient::ClearBadProxiesCache deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ClearBadProxiesCache();
      return true;
    }
  }
  return false;
}

// static
bool CustomProxyConfigClientStubDispatch::AcceptWithResponder(
    CustomProxyConfigClient* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kCustomProxyConfigClient_OnCustomProxyConfigUpdated_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::CustomProxyConfigClient::OnCustomProxyConfigUpdated",
               "message", message->name());
#endif
      break;
    }
    case internal::kCustomProxyConfigClient_MarkProxiesAsBad_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::CustomProxyConfigClient::MarkProxiesAsBad",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            CustomProxyConfigClientProxy_MarkProxiesAsBad_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          CustomProxyConfigClient::MarkProxiesAsBadCallback callback =
              CustomProxyConfigClient_MarkProxiesAsBad_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::CustomProxyConfigClient_MarkProxiesAsBad_Params_Data* params =
          reinterpret_cast<
              internal::CustomProxyConfigClient_MarkProxiesAsBad_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      base::TimeDelta p_bypass_duration{};
      net::ProxyList p_bad_proxies{};
      CustomProxyConfigClient_MarkProxiesAsBad_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadBypassDuration(&p_bypass_duration))
        success = false;
      if (!input_data_view.ReadBadProxies(&p_bad_proxies))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "CustomProxyConfigClient::MarkProxiesAsBad deserializer");
        return false;
      }
      CustomProxyConfigClient::MarkProxiesAsBadCallback callback =
          CustomProxyConfigClient_MarkProxiesAsBad_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->MarkProxiesAsBad(
std::move(p_bypass_duration), 
std::move(p_bad_proxies), std::move(callback));
      return true;
    }
    case internal::kCustomProxyConfigClient_ClearBadProxiesCache_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::CustomProxyConfigClient::ClearBadProxiesCache",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool CustomProxyConfigClientRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "CustomProxyConfigClient RequestValidator");

  switch (message->header()->name) {
    case internal::kCustomProxyConfigClient_OnCustomProxyConfigUpdated_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::CustomProxyConfigClient_OnCustomProxyConfigUpdated_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kCustomProxyConfigClient_MarkProxiesAsBad_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::CustomProxyConfigClient_MarkProxiesAsBad_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kCustomProxyConfigClient_ClearBadProxiesCache_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::CustomProxyConfigClient_ClearBadProxiesCache_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool CustomProxyConfigClientResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "CustomProxyConfigClient ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kCustomProxyConfigClient_MarkProxiesAsBad_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::CustomProxyConfigClient_MarkProxiesAsBad_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
void CustomProxyConfigClientInterceptorForTesting::OnCustomProxyConfigUpdated(CustomProxyConfigPtr proxy_config) {
  GetForwardingInterface()->OnCustomProxyConfigUpdated(std::move(proxy_config));
}
void CustomProxyConfigClientInterceptorForTesting::MarkProxiesAsBad(base::TimeDelta bypass_duration, const net::ProxyList& bad_proxies, MarkProxiesAsBadCallback callback) {
  GetForwardingInterface()->MarkProxiesAsBad(std::move(bypass_duration), std::move(bad_proxies), std::move(callback));
}
void CustomProxyConfigClientInterceptorForTesting::ClearBadProxiesCache() {
  GetForwardingInterface()->ClearBadProxiesCache();
}
CustomProxyConfigClientAsyncWaiter::CustomProxyConfigClientAsyncWaiter(
    CustomProxyConfigClient* proxy) : proxy_(proxy) {}

CustomProxyConfigClientAsyncWaiter::~CustomProxyConfigClientAsyncWaiter() = default;

void CustomProxyConfigClientAsyncWaiter::MarkProxiesAsBad(
    base::TimeDelta bypass_duration, const net::ProxyList& bad_proxies) {
  base::RunLoop loop;
  proxy_->MarkProxiesAsBad(std::move(bypass_duration),std::move(bad_proxies),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}

const char TrustedURLLoaderHeaderClient::Name_[] = "network.mojom.TrustedURLLoaderHeaderClient";

class TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ForwardToCallback(
      TrustedURLLoaderHeaderClient::OnBeforeSendHeadersCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  TrustedURLLoaderHeaderClient::OnBeforeSendHeadersCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ForwardToCallback);
};

class TrustedURLLoaderHeaderClient_OnHeadersReceived_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  TrustedURLLoaderHeaderClient_OnHeadersReceived_ForwardToCallback(
      TrustedURLLoaderHeaderClient::OnHeadersReceivedCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  TrustedURLLoaderHeaderClient::OnHeadersReceivedCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(TrustedURLLoaderHeaderClient_OnHeadersReceived_ForwardToCallback);
};

TrustedURLLoaderHeaderClientProxy::TrustedURLLoaderHeaderClientProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class TrustedURLLoaderHeaderClientProxy_OnBeforeSendHeaders_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit TrustedURLLoaderHeaderClientProxy_OnBeforeSendHeaders_Message(
      uint32_t message_flags
      , int32_t param_request_id
      , const net::HttpRequestHeaders& param_headers
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kTrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Name,
          message_flags)
      , param_request_id_(std::move(param_request_id))
      , param_headers_(std::move(param_headers)){}
  ~TrustedURLLoaderHeaderClientProxy_OnBeforeSendHeaders_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      int32_t param_request_id, const net::HttpRequestHeaders& param_headers) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<TrustedURLLoaderHeaderClientProxy_OnBeforeSendHeaders_Message>(
          kFlags
          , std::move(param_request_id)
          , std::move(param_headers)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kTrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->request_id = param_request_id;
  typename decltype(params->headers)::BaseType::BufferWriter
      headers_writer;
  mojo::internal::Serialize<::network::mojom::HttpRequestHeadersDataView>(
      param_headers, buffer, &headers_writer, &serialization_context);
  params->headers.Set(
      headers_writer.is_null() ? nullptr : headers_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->headers.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null headers in TrustedURLLoaderHeaderClient.OnBeforeSendHeaders request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(TrustedURLLoaderHeaderClient* impl, TrustedURLLoaderHeaderClient::OnBeforeSendHeadersCallback callback) {
    impl->OnBeforeSendHeaders(
        std::move(param_request_id_),
        std::move(param_headers_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->request_id = param_request_id_;
  typename decltype(params->headers)::BaseType::BufferWriter
      headers_writer;
  mojo::internal::Serialize<::network::mojom::HttpRequestHeadersDataView>(
      param_headers_, buffer, &headers_writer, serialization_context);
  params->headers.Set(
      headers_writer.is_null() ? nullptr : headers_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->headers.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null headers in TrustedURLLoaderHeaderClient.OnBeforeSendHeaders request");
  }
  int32_t param_request_id_;
  net::HttpRequestHeaders param_headers_;

  DISALLOW_COPY_AND_ASSIGN(TrustedURLLoaderHeaderClientProxy_OnBeforeSendHeaders_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
TrustedURLLoaderHeaderClientProxy_OnBeforeSendHeaders_Message::kMessageTag = {};

void TrustedURLLoaderHeaderClientProxy::OnBeforeSendHeaders(
    int32_t in_request_id, const net::HttpRequestHeaders& in_headers, OnBeforeSendHeadersCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::TrustedURLLoaderHeaderClient::OnBeforeSendHeaders");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = TrustedURLLoaderHeaderClientProxy_OnBeforeSendHeaders_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_request_id), std::move(in_headers));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(TrustedURLLoaderHeaderClient::Name_);
  message.set_method_name("OnBeforeSendHeaders");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class TrustedURLLoaderHeaderClientProxy_OnHeadersReceived_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit TrustedURLLoaderHeaderClientProxy_OnHeadersReceived_Message(
      uint32_t message_flags
      , int32_t param_request_id
      , const std::string& param_headers
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kTrustedURLLoaderHeaderClient_OnHeadersReceived_Name,
          message_flags)
      , param_request_id_(std::move(param_request_id))
      , param_headers_(std::move(param_headers)){}
  ~TrustedURLLoaderHeaderClientProxy_OnHeadersReceived_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      int32_t param_request_id, const std::string& param_headers) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<TrustedURLLoaderHeaderClientProxy_OnHeadersReceived_Message>(
          kFlags
          , std::move(param_request_id)
          , std::move(param_headers)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kTrustedURLLoaderHeaderClient_OnHeadersReceived_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::TrustedURLLoaderHeaderClient_OnHeadersReceived_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->request_id = param_request_id;
  typename decltype(params->headers)::BaseType::BufferWriter
      headers_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_headers, buffer, &headers_writer, &serialization_context);
  params->headers.Set(
      headers_writer.is_null() ? nullptr : headers_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->headers.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null headers in TrustedURLLoaderHeaderClient.OnHeadersReceived request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(TrustedURLLoaderHeaderClient* impl, TrustedURLLoaderHeaderClient::OnHeadersReceivedCallback callback) {
    impl->OnHeadersReceived(
        std::move(param_request_id_),
        std::move(param_headers_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::TrustedURLLoaderHeaderClient_OnHeadersReceived_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->request_id = param_request_id_;
  typename decltype(params->headers)::BaseType::BufferWriter
      headers_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_headers_, buffer, &headers_writer, serialization_context);
  params->headers.Set(
      headers_writer.is_null() ? nullptr : headers_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->headers.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null headers in TrustedURLLoaderHeaderClient.OnHeadersReceived request");
  }
  int32_t param_request_id_;
  std::string param_headers_;

  DISALLOW_COPY_AND_ASSIGN(TrustedURLLoaderHeaderClientProxy_OnHeadersReceived_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
TrustedURLLoaderHeaderClientProxy_OnHeadersReceived_Message::kMessageTag = {};

void TrustedURLLoaderHeaderClientProxy::OnHeadersReceived(
    int32_t in_request_id, const std::string& in_headers, OnHeadersReceivedCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::TrustedURLLoaderHeaderClient::OnHeadersReceived");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = TrustedURLLoaderHeaderClientProxy_OnHeadersReceived_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_request_id), std::move(in_headers));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(TrustedURLLoaderHeaderClient::Name_);
  message.set_method_name("OnHeadersReceived");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new TrustedURLLoaderHeaderClient_OnHeadersReceived_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ProxyToResponder {
 public:
  static TrustedURLLoaderHeaderClient::OnBeforeSendHeadersCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ProxyToResponder> proxy(
        new TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "TrustedURLLoaderHeaderClient::OnBeforeSendHeadersCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      int32_t in_result, const base::Optional<net::HttpRequestHeaders>& in_headers);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ProxyToResponder);
};
class TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Response_Message(
      uint32_t message_flags
      , int32_t param_result
      , const base::Optional<net::HttpRequestHeaders>& param_headers
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kTrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Name,
          message_flags)
      , param_result_(std::move(param_result))
      , param_headers_(std::move(param_headers)){}
  ~TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      int32_t param_result, const base::Optional<net::HttpRequestHeaders>& param_headers) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Response_Message>(
          kFlags
          , std::move(param_result)
          , std::move(param_headers)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kTrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->result = param_result;
  typename decltype(params->headers)::BaseType::BufferWriter
      headers_writer;
  mojo::internal::Serialize<::network::mojom::HttpRequestHeadersDataView>(
      param_headers, buffer, &headers_writer, &serialization_context);
  params->headers.Set(
      headers_writer.is_null() ? nullptr : headers_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(TrustedURLLoaderHeaderClient::OnBeforeSendHeadersCallback* callback) {
    std::move(*callback).Run(
        std::move(param_result_), 
        std::move(param_headers_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->result = param_result_;
  typename decltype(params->headers)::BaseType::BufferWriter
      headers_writer;
  mojo::internal::Serialize<::network::mojom::HttpRequestHeadersDataView>(
      param_headers_, buffer, &headers_writer, serialization_context);
  params->headers.Set(
      headers_writer.is_null() ? nullptr : headers_writer.data());
  }
  int32_t param_result_;
  base::Optional<net::HttpRequestHeaders> param_headers_;

  DISALLOW_COPY_AND_ASSIGN(TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Response_Message::kMessageTag = {};

bool TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::TrustedURLLoaderHeaderClient::OnBeforeSendHeadersCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ResponseParams_Data* params =
      reinterpret_cast<
          internal::TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_result{};
  base::Optional<net::HttpRequestHeaders> p_headers{};
  TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_result = input_data_view.result();
  if (!input_data_view.ReadHeaders(&p_headers))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "TrustedURLLoaderHeaderClient::OnBeforeSendHeaders response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_result), 
std::move(p_headers));
  return true;
}

void TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ProxyToResponder::Run(
    int32_t in_result, const base::Optional<net::HttpRequestHeaders>& in_headers) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Response_Message::Build(kSerialize, is_sync_, std::move(in_result), std::move(in_headers));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::TrustedURLLoaderHeaderClient::OnBeforeSendHeadersCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(TrustedURLLoaderHeaderClient::Name_);
  message.set_method_name("OnBeforeSendHeaders");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class TrustedURLLoaderHeaderClient_OnHeadersReceived_ProxyToResponder {
 public:
  static TrustedURLLoaderHeaderClient::OnHeadersReceivedCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<TrustedURLLoaderHeaderClient_OnHeadersReceived_ProxyToResponder> proxy(
        new TrustedURLLoaderHeaderClient_OnHeadersReceived_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&TrustedURLLoaderHeaderClient_OnHeadersReceived_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~TrustedURLLoaderHeaderClient_OnHeadersReceived_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  TrustedURLLoaderHeaderClient_OnHeadersReceived_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "TrustedURLLoaderHeaderClient::OnHeadersReceivedCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      int32_t in_result, const base::Optional<std::string>& in_headers, const GURL& in_allowed_unsafe_redirect_url);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(TrustedURLLoaderHeaderClient_OnHeadersReceived_ProxyToResponder);
};
class TrustedURLLoaderHeaderClient_OnHeadersReceived_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit TrustedURLLoaderHeaderClient_OnHeadersReceived_Response_Message(
      uint32_t message_flags
      , int32_t param_result
      , const base::Optional<std::string>& param_headers
      , const GURL& param_allowed_unsafe_redirect_url
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kTrustedURLLoaderHeaderClient_OnHeadersReceived_Name,
          message_flags)
      , param_result_(std::move(param_result))
      , param_headers_(std::move(param_headers))
      , param_allowed_unsafe_redirect_url_(std::move(param_allowed_unsafe_redirect_url)){}
  ~TrustedURLLoaderHeaderClient_OnHeadersReceived_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      int32_t param_result, const base::Optional<std::string>& param_headers, const GURL& param_allowed_unsafe_redirect_url) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<TrustedURLLoaderHeaderClient_OnHeadersReceived_Response_Message>(
          kFlags
          , std::move(param_result)
          , std::move(param_headers)
          , std::move(param_allowed_unsafe_redirect_url)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kTrustedURLLoaderHeaderClient_OnHeadersReceived_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::TrustedURLLoaderHeaderClient_OnHeadersReceived_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->result = param_result;
  typename decltype(params->headers)::BaseType::BufferWriter
      headers_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_headers, buffer, &headers_writer, &serialization_context);
  params->headers.Set(
      headers_writer.is_null() ? nullptr : headers_writer.data());
  typename decltype(params->allowed_unsafe_redirect_url)::BaseType::BufferWriter
      allowed_unsafe_redirect_url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_allowed_unsafe_redirect_url, buffer, &allowed_unsafe_redirect_url_writer, &serialization_context);
  params->allowed_unsafe_redirect_url.Set(
      allowed_unsafe_redirect_url_writer.is_null() ? nullptr : allowed_unsafe_redirect_url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->allowed_unsafe_redirect_url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null allowed_unsafe_redirect_url in TrustedURLLoaderHeaderClient.OnHeadersReceived response");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(TrustedURLLoaderHeaderClient::OnHeadersReceivedCallback* callback) {
    std::move(*callback).Run(
        std::move(param_result_), 
        std::move(param_headers_), 
        std::move(param_allowed_unsafe_redirect_url_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::TrustedURLLoaderHeaderClient_OnHeadersReceived_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->result = param_result_;
  typename decltype(params->headers)::BaseType::BufferWriter
      headers_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_headers_, buffer, &headers_writer, serialization_context);
  params->headers.Set(
      headers_writer.is_null() ? nullptr : headers_writer.data());
  typename decltype(params->allowed_unsafe_redirect_url)::BaseType::BufferWriter
      allowed_unsafe_redirect_url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_allowed_unsafe_redirect_url_, buffer, &allowed_unsafe_redirect_url_writer, serialization_context);
  params->allowed_unsafe_redirect_url.Set(
      allowed_unsafe_redirect_url_writer.is_null() ? nullptr : allowed_unsafe_redirect_url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->allowed_unsafe_redirect_url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null allowed_unsafe_redirect_url in TrustedURLLoaderHeaderClient.OnHeadersReceived response");
  }
  int32_t param_result_;
  base::Optional<std::string> param_headers_;
  GURL param_allowed_unsafe_redirect_url_;

  DISALLOW_COPY_AND_ASSIGN(TrustedURLLoaderHeaderClient_OnHeadersReceived_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
TrustedURLLoaderHeaderClient_OnHeadersReceived_Response_Message::kMessageTag = {};

bool TrustedURLLoaderHeaderClient_OnHeadersReceived_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::TrustedURLLoaderHeaderClient::OnHeadersReceivedCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<TrustedURLLoaderHeaderClient_OnHeadersReceived_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::TrustedURLLoaderHeaderClient_OnHeadersReceived_ResponseParams_Data* params =
      reinterpret_cast<
          internal::TrustedURLLoaderHeaderClient_OnHeadersReceived_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_result{};
  base::Optional<std::string> p_headers{};
  GURL p_allowed_unsafe_redirect_url{};
  TrustedURLLoaderHeaderClient_OnHeadersReceived_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_result = input_data_view.result();
  if (!input_data_view.ReadHeaders(&p_headers))
    success = false;
  if (!input_data_view.ReadAllowedUnsafeRedirectUrl(&p_allowed_unsafe_redirect_url))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "TrustedURLLoaderHeaderClient::OnHeadersReceived response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_result), 
std::move(p_headers), 
std::move(p_allowed_unsafe_redirect_url));
  return true;
}

void TrustedURLLoaderHeaderClient_OnHeadersReceived_ProxyToResponder::Run(
    int32_t in_result, const base::Optional<std::string>& in_headers, const GURL& in_allowed_unsafe_redirect_url) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = TrustedURLLoaderHeaderClient_OnHeadersReceived_Response_Message::Build(kSerialize, is_sync_, std::move(in_result), std::move(in_headers), std::move(in_allowed_unsafe_redirect_url));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::TrustedURLLoaderHeaderClient::OnHeadersReceivedCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(TrustedURLLoaderHeaderClient::Name_);
  message.set_method_name("OnHeadersReceived");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool TrustedURLLoaderHeaderClientStubDispatch::Accept(
    TrustedURLLoaderHeaderClient* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kTrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::TrustedURLLoaderHeaderClient::OnBeforeSendHeaders",
               "message", message->name());
#endif
      break;
    }
    case internal::kTrustedURLLoaderHeaderClient_OnHeadersReceived_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::TrustedURLLoaderHeaderClient::OnHeadersReceived",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

// static
bool TrustedURLLoaderHeaderClientStubDispatch::AcceptWithResponder(
    TrustedURLLoaderHeaderClient* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kTrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::TrustedURLLoaderHeaderClient::OnBeforeSendHeaders",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            TrustedURLLoaderHeaderClientProxy_OnBeforeSendHeaders_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          TrustedURLLoaderHeaderClient::OnBeforeSendHeadersCallback callback =
              TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Params_Data* params =
          reinterpret_cast<
              internal::TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_request_id{};
      net::HttpRequestHeaders p_headers{};
      TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ParamsDataView input_data_view(params, &serialization_context);
      
      p_request_id = input_data_view.request_id();
      if (!input_data_view.ReadHeaders(&p_headers))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "TrustedURLLoaderHeaderClient::OnBeforeSendHeaders deserializer");
        return false;
      }
      TrustedURLLoaderHeaderClient::OnBeforeSendHeadersCallback callback =
          TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnBeforeSendHeaders(
std::move(p_request_id), 
std::move(p_headers), std::move(callback));
      return true;
    }
    case internal::kTrustedURLLoaderHeaderClient_OnHeadersReceived_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::TrustedURLLoaderHeaderClient::OnHeadersReceived",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            TrustedURLLoaderHeaderClientProxy_OnHeadersReceived_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          TrustedURLLoaderHeaderClient::OnHeadersReceivedCallback callback =
              TrustedURLLoaderHeaderClient_OnHeadersReceived_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::TrustedURLLoaderHeaderClient_OnHeadersReceived_Params_Data* params =
          reinterpret_cast<
              internal::TrustedURLLoaderHeaderClient_OnHeadersReceived_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_request_id{};
      std::string p_headers{};
      TrustedURLLoaderHeaderClient_OnHeadersReceived_ParamsDataView input_data_view(params, &serialization_context);
      
      p_request_id = input_data_view.request_id();
      if (!input_data_view.ReadHeaders(&p_headers))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "TrustedURLLoaderHeaderClient::OnHeadersReceived deserializer");
        return false;
      }
      TrustedURLLoaderHeaderClient::OnHeadersReceivedCallback callback =
          TrustedURLLoaderHeaderClient_OnHeadersReceived_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnHeadersReceived(
std::move(p_request_id), 
std::move(p_headers), std::move(callback));
      return true;
    }
  }
  return false;
}

bool TrustedURLLoaderHeaderClientRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "TrustedURLLoaderHeaderClient RequestValidator");

  switch (message->header()->name) {
    case internal::kTrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kTrustedURLLoaderHeaderClient_OnHeadersReceived_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::TrustedURLLoaderHeaderClient_OnHeadersReceived_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool TrustedURLLoaderHeaderClientResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "TrustedURLLoaderHeaderClient ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kTrustedURLLoaderHeaderClient_OnBeforeSendHeaders_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::TrustedURLLoaderHeaderClient_OnBeforeSendHeaders_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kTrustedURLLoaderHeaderClient_OnHeadersReceived_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::TrustedURLLoaderHeaderClient_OnHeadersReceived_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
void TrustedURLLoaderHeaderClientInterceptorForTesting::OnBeforeSendHeaders(int32_t request_id, const net::HttpRequestHeaders& headers, OnBeforeSendHeadersCallback callback) {
  GetForwardingInterface()->OnBeforeSendHeaders(std::move(request_id), std::move(headers), std::move(callback));
}
void TrustedURLLoaderHeaderClientInterceptorForTesting::OnHeadersReceived(int32_t request_id, const std::string& headers, OnHeadersReceivedCallback callback) {
  GetForwardingInterface()->OnHeadersReceived(std::move(request_id), std::move(headers), std::move(callback));
}
TrustedURLLoaderHeaderClientAsyncWaiter::TrustedURLLoaderHeaderClientAsyncWaiter(
    TrustedURLLoaderHeaderClient* proxy) : proxy_(proxy) {}

TrustedURLLoaderHeaderClientAsyncWaiter::~TrustedURLLoaderHeaderClientAsyncWaiter() = default;

void TrustedURLLoaderHeaderClientAsyncWaiter::OnBeforeSendHeaders(
    int32_t request_id, const net::HttpRequestHeaders& headers, int32_t* out_result, base::Optional<net::HttpRequestHeaders>* out_headers) {
  base::RunLoop loop;
  proxy_->OnBeforeSendHeaders(std::move(request_id),std::move(headers),
      base::BindOnce(
          [](base::RunLoop* loop,
             int32_t* out_result
,
             base::Optional<net::HttpRequestHeaders>* out_headers
,
             int32_t result,
             const base::Optional<net::HttpRequestHeaders>& headers) {*out_result = std::move(result);*out_headers = std::move(headers);
            loop->Quit();
          },
          &loop,
          out_result,
          out_headers));
  loop.Run();
}
void TrustedURLLoaderHeaderClientAsyncWaiter::OnHeadersReceived(
    int32_t request_id, const std::string& headers, int32_t* out_result, base::Optional<std::string>* out_headers, GURL* out_allowed_unsafe_redirect_url) {
  base::RunLoop loop;
  proxy_->OnHeadersReceived(std::move(request_id),std::move(headers),
      base::BindOnce(
          [](base::RunLoop* loop,
             int32_t* out_result
,
             base::Optional<std::string>* out_headers
,
             GURL* out_allowed_unsafe_redirect_url
,
             int32_t result,
             const base::Optional<std::string>& headers,
             const GURL& allowed_unsafe_redirect_url) {*out_result = std::move(result);*out_headers = std::move(headers);*out_allowed_unsafe_redirect_url = std::move(allowed_unsafe_redirect_url);
            loop->Quit();
          },
          &loop,
          out_result,
          out_headers,
          out_allowed_unsafe_redirect_url));
  loop.Run();
}

const char NetworkContextClient::Name_[] = "network.mojom.NetworkContextClient";

class NetworkContextClient_OnCanSendReportingReports_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContextClient_OnCanSendReportingReports_ForwardToCallback(
      NetworkContextClient::OnCanSendReportingReportsCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContextClient::OnCanSendReportingReportsCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContextClient_OnCanSendReportingReports_ForwardToCallback);
};

class NetworkContextClient_OnCanSendDomainReliabilityUpload_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContextClient_OnCanSendDomainReliabilityUpload_ForwardToCallback(
      NetworkContextClient::OnCanSendDomainReliabilityUploadCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContextClient::OnCanSendDomainReliabilityUploadCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContextClient_OnCanSendDomainReliabilityUpload_ForwardToCallback);
};

NetworkContextClientProxy::NetworkContextClientProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class NetworkContextClientProxy_OnCanSendReportingReports_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextClientProxy_OnCanSendReportingReports_Message(
      uint32_t message_flags
      , const std::vector<url::Origin>& param_origins
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContextClient_OnCanSendReportingReports_Name,
          message_flags)
      , param_origins_(std::move(param_origins)){}
  ~NetworkContextClientProxy_OnCanSendReportingReports_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const std::vector<url::Origin>& param_origins) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextClientProxy_OnCanSendReportingReports_Message>(
          kFlags
          , std::move(param_origins)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContextClient_OnCanSendReportingReports_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContextClient_OnCanSendReportingReports_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->origins)::BaseType::BufferWriter
      origins_writer;
  const mojo::internal::ContainerValidateParams origins_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::url::mojom::OriginDataView>>(
      param_origins, buffer, &origins_writer, &origins_validate_params,
      &serialization_context);
  params->origins.Set(
      origins_writer.is_null() ? nullptr : origins_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origins.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origins in NetworkContextClient.OnCanSendReportingReports request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContextClient* impl, NetworkContextClient::OnCanSendReportingReportsCallback callback) {
    impl->OnCanSendReportingReports(
        std::move(param_origins_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContextClient_OnCanSendReportingReports_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->origins)::BaseType::BufferWriter
      origins_writer;
  const mojo::internal::ContainerValidateParams origins_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::url::mojom::OriginDataView>>(
      param_origins_, buffer, &origins_writer, &origins_validate_params,
      serialization_context);
  params->origins.Set(
      origins_writer.is_null() ? nullptr : origins_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origins.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origins in NetworkContextClient.OnCanSendReportingReports request");
  }
  std::vector<url::Origin> param_origins_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextClientProxy_OnCanSendReportingReports_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextClientProxy_OnCanSendReportingReports_Message::kMessageTag = {};

void NetworkContextClientProxy::OnCanSendReportingReports(
    const std::vector<url::Origin>& in_origins, OnCanSendReportingReportsCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContextClient::OnCanSendReportingReports");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextClientProxy_OnCanSendReportingReports_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_origins));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContextClient::Name_);
  message.set_method_name("OnCanSendReportingReports");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContextClient_OnCanSendReportingReports_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextClientProxy_OnCanSendDomainReliabilityUpload_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextClientProxy_OnCanSendDomainReliabilityUpload_Message(
      uint32_t message_flags
      , const GURL& param_origin
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContextClient_OnCanSendDomainReliabilityUpload_Name,
          message_flags)
      , param_origin_(std::move(param_origin)){}
  ~NetworkContextClientProxy_OnCanSendDomainReliabilityUpload_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const GURL& param_origin) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextClientProxy_OnCanSendDomainReliabilityUpload_Message>(
          kFlags
          , std::move(param_origin)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContextClient_OnCanSendDomainReliabilityUpload_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContextClient_OnCanSendDomainReliabilityUpload_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->origin)::BaseType::BufferWriter
      origin_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_origin, buffer, &origin_writer, &serialization_context);
  params->origin.Set(
      origin_writer.is_null() ? nullptr : origin_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in NetworkContextClient.OnCanSendDomainReliabilityUpload request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContextClient* impl, NetworkContextClient::OnCanSendDomainReliabilityUploadCallback callback) {
    impl->OnCanSendDomainReliabilityUpload(
        std::move(param_origin_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContextClient_OnCanSendDomainReliabilityUpload_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->origin)::BaseType::BufferWriter
      origin_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_origin_, buffer, &origin_writer, serialization_context);
  params->origin.Set(
      origin_writer.is_null() ? nullptr : origin_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in NetworkContextClient.OnCanSendDomainReliabilityUpload request");
  }
  GURL param_origin_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextClientProxy_OnCanSendDomainReliabilityUpload_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextClientProxy_OnCanSendDomainReliabilityUpload_Message::kMessageTag = {};

void NetworkContextClientProxy::OnCanSendDomainReliabilityUpload(
    const GURL& in_origin, OnCanSendDomainReliabilityUploadCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContextClient::OnCanSendDomainReliabilityUpload");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextClientProxy_OnCanSendDomainReliabilityUpload_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_origin));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContextClient::Name_);
  message.set_method_name("OnCanSendDomainReliabilityUpload");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContextClient_OnCanSendDomainReliabilityUpload_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextClient_OnCanSendReportingReports_ProxyToResponder {
 public:
  static NetworkContextClient::OnCanSendReportingReportsCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContextClient_OnCanSendReportingReports_ProxyToResponder> proxy(
        new NetworkContextClient_OnCanSendReportingReports_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContextClient_OnCanSendReportingReports_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContextClient_OnCanSendReportingReports_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContextClient_OnCanSendReportingReports_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContextClient::OnCanSendReportingReportsCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      const std::vector<url::Origin>& in_origins);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextClient_OnCanSendReportingReports_ProxyToResponder);
};
class NetworkContextClient_OnCanSendReportingReports_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextClient_OnCanSendReportingReports_Response_Message(
      uint32_t message_flags
      , const std::vector<url::Origin>& param_origins
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContextClient_OnCanSendReportingReports_Name,
          message_flags)
      , param_origins_(std::move(param_origins)){}
  ~NetworkContextClient_OnCanSendReportingReports_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      const std::vector<url::Origin>& param_origins) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextClient_OnCanSendReportingReports_Response_Message>(
          kFlags
          , std::move(param_origins)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContextClient_OnCanSendReportingReports_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContextClient_OnCanSendReportingReports_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->origins)::BaseType::BufferWriter
      origins_writer;
  const mojo::internal::ContainerValidateParams origins_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::url::mojom::OriginDataView>>(
      param_origins, buffer, &origins_writer, &origins_validate_params,
      &serialization_context);
  params->origins.Set(
      origins_writer.is_null() ? nullptr : origins_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origins.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origins in NetworkContextClient.OnCanSendReportingReports response");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContextClient::OnCanSendReportingReportsCallback* callback) {
    std::move(*callback).Run(
        std::move(param_origins_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContextClient_OnCanSendReportingReports_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->origins)::BaseType::BufferWriter
      origins_writer;
  const mojo::internal::ContainerValidateParams origins_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::url::mojom::OriginDataView>>(
      param_origins_, buffer, &origins_writer, &origins_validate_params,
      serialization_context);
  params->origins.Set(
      origins_writer.is_null() ? nullptr : origins_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origins.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origins in NetworkContextClient.OnCanSendReportingReports response");
  }
  std::vector<url::Origin> param_origins_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextClient_OnCanSendReportingReports_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextClient_OnCanSendReportingReports_Response_Message::kMessageTag = {};

bool NetworkContextClient_OnCanSendReportingReports_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContextClient::OnCanSendReportingReportsCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContextClient_OnCanSendReportingReports_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContextClient_OnCanSendReportingReports_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContextClient_OnCanSendReportingReports_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  std::vector<url::Origin> p_origins{};
  NetworkContextClient_OnCanSendReportingReports_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadOrigins(&p_origins))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContextClient::OnCanSendReportingReports response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_origins));
  return true;
}

void NetworkContextClient_OnCanSendReportingReports_ProxyToResponder::Run(
    const std::vector<url::Origin>& in_origins) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContextClient_OnCanSendReportingReports_Response_Message::Build(kSerialize, is_sync_, std::move(in_origins));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContextClient::OnCanSendReportingReportsCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContextClient::Name_);
  message.set_method_name("OnCanSendReportingReports");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContextClient_OnCanSendDomainReliabilityUpload_ProxyToResponder {
 public:
  static NetworkContextClient::OnCanSendDomainReliabilityUploadCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContextClient_OnCanSendDomainReliabilityUpload_ProxyToResponder> proxy(
        new NetworkContextClient_OnCanSendDomainReliabilityUpload_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContextClient_OnCanSendDomainReliabilityUpload_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContextClient_OnCanSendDomainReliabilityUpload_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContextClient_OnCanSendDomainReliabilityUpload_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContextClient::OnCanSendDomainReliabilityUploadCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      bool in_allowed);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextClient_OnCanSendDomainReliabilityUpload_ProxyToResponder);
};
class NetworkContextClient_OnCanSendDomainReliabilityUpload_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextClient_OnCanSendDomainReliabilityUpload_Response_Message(
      uint32_t message_flags
      , bool param_allowed
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContextClient_OnCanSendDomainReliabilityUpload_Name,
          message_flags)
      , param_allowed_(std::move(param_allowed)){}
  ~NetworkContextClient_OnCanSendDomainReliabilityUpload_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      bool param_allowed) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextClient_OnCanSendDomainReliabilityUpload_Response_Message>(
          kFlags
          , std::move(param_allowed)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContextClient_OnCanSendDomainReliabilityUpload_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContextClient_OnCanSendDomainReliabilityUpload_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->allowed = param_allowed;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContextClient::OnCanSendDomainReliabilityUploadCallback* callback) {
    std::move(*callback).Run(
        std::move(param_allowed_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContextClient_OnCanSendDomainReliabilityUpload_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->allowed = param_allowed_;
  }
  bool param_allowed_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextClient_OnCanSendDomainReliabilityUpload_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextClient_OnCanSendDomainReliabilityUpload_Response_Message::kMessageTag = {};

bool NetworkContextClient_OnCanSendDomainReliabilityUpload_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContextClient::OnCanSendDomainReliabilityUploadCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContextClient_OnCanSendDomainReliabilityUpload_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContextClient_OnCanSendDomainReliabilityUpload_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContextClient_OnCanSendDomainReliabilityUpload_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  bool p_allowed{};
  NetworkContextClient_OnCanSendDomainReliabilityUpload_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_allowed = input_data_view.allowed();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContextClient::OnCanSendDomainReliabilityUpload response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_allowed));
  return true;
}

void NetworkContextClient_OnCanSendDomainReliabilityUpload_ProxyToResponder::Run(
    bool in_allowed) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContextClient_OnCanSendDomainReliabilityUpload_Response_Message::Build(kSerialize, is_sync_, std::move(in_allowed));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContextClient::OnCanSendDomainReliabilityUploadCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContextClient::Name_);
  message.set_method_name("OnCanSendDomainReliabilityUpload");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool NetworkContextClientStubDispatch::Accept(
    NetworkContextClient* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kNetworkContextClient_OnCanSendReportingReports_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContextClient::OnCanSendReportingReports",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContextClient_OnCanSendDomainReliabilityUpload_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContextClient::OnCanSendDomainReliabilityUpload",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

// static
bool NetworkContextClientStubDispatch::AcceptWithResponder(
    NetworkContextClient* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kNetworkContextClient_OnCanSendReportingReports_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContextClient::OnCanSendReportingReports",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextClientProxy_OnCanSendReportingReports_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContextClient::OnCanSendReportingReportsCallback callback =
              NetworkContextClient_OnCanSendReportingReports_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContextClient_OnCanSendReportingReports_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContextClient_OnCanSendReportingReports_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::vector<url::Origin> p_origins{};
      NetworkContextClient_OnCanSendReportingReports_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadOrigins(&p_origins))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContextClient::OnCanSendReportingReports deserializer");
        return false;
      }
      NetworkContextClient::OnCanSendReportingReportsCallback callback =
          NetworkContextClient_OnCanSendReportingReports_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnCanSendReportingReports(
std::move(p_origins), std::move(callback));
      return true;
    }
    case internal::kNetworkContextClient_OnCanSendDomainReliabilityUpload_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContextClient::OnCanSendDomainReliabilityUpload",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextClientProxy_OnCanSendDomainReliabilityUpload_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContextClient::OnCanSendDomainReliabilityUploadCallback callback =
              NetworkContextClient_OnCanSendDomainReliabilityUpload_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContextClient_OnCanSendDomainReliabilityUpload_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContextClient_OnCanSendDomainReliabilityUpload_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      GURL p_origin{};
      NetworkContextClient_OnCanSendDomainReliabilityUpload_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadOrigin(&p_origin))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContextClient::OnCanSendDomainReliabilityUpload deserializer");
        return false;
      }
      NetworkContextClient::OnCanSendDomainReliabilityUploadCallback callback =
          NetworkContextClient_OnCanSendDomainReliabilityUpload_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnCanSendDomainReliabilityUpload(
std::move(p_origin), std::move(callback));
      return true;
    }
  }
  return false;
}

bool NetworkContextClientRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "NetworkContextClient RequestValidator");

  switch (message->header()->name) {
    case internal::kNetworkContextClient_OnCanSendReportingReports_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContextClient_OnCanSendReportingReports_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContextClient_OnCanSendDomainReliabilityUpload_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContextClient_OnCanSendDomainReliabilityUpload_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool NetworkContextClientResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "NetworkContextClient ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kNetworkContextClient_OnCanSendReportingReports_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContextClient_OnCanSendReportingReports_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContextClient_OnCanSendDomainReliabilityUpload_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContextClient_OnCanSendDomainReliabilityUpload_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
void NetworkContextClientInterceptorForTesting::OnCanSendReportingReports(const std::vector<url::Origin>& origins, OnCanSendReportingReportsCallback callback) {
  GetForwardingInterface()->OnCanSendReportingReports(std::move(origins), std::move(callback));
}
void NetworkContextClientInterceptorForTesting::OnCanSendDomainReliabilityUpload(const GURL& origin, OnCanSendDomainReliabilityUploadCallback callback) {
  GetForwardingInterface()->OnCanSendDomainReliabilityUpload(std::move(origin), std::move(callback));
}
NetworkContextClientAsyncWaiter::NetworkContextClientAsyncWaiter(
    NetworkContextClient* proxy) : proxy_(proxy) {}

NetworkContextClientAsyncWaiter::~NetworkContextClientAsyncWaiter() = default;

void NetworkContextClientAsyncWaiter::OnCanSendReportingReports(
    const std::vector<url::Origin>& origins, std::vector<url::Origin>* out_origins) {
  base::RunLoop loop;
  proxy_->OnCanSendReportingReports(std::move(origins),
      base::BindOnce(
          [](base::RunLoop* loop,
             std::vector<url::Origin>* out_origins
,
             const std::vector<url::Origin>& origins) {*out_origins = std::move(origins);
            loop->Quit();
          },
          &loop,
          out_origins));
  loop.Run();
}
void NetworkContextClientAsyncWaiter::OnCanSendDomainReliabilityUpload(
    const GURL& origin, bool* out_allowed) {
  base::RunLoop loop;
  proxy_->OnCanSendDomainReliabilityUpload(std::move(origin),
      base::BindOnce(
          [](base::RunLoop* loop,
             bool* out_allowed
,
             bool allowed) {*out_allowed = std::move(allowed);
            loop->Quit();
          },
          &loop,
          out_allowed));
  loop.Run();
}

const char NetworkContext::Name_[] = "network.mojom.NetworkContext";
bool NetworkContext::EnableStaticKeyPinningForTesting() {
  NOTREACHED();
  return false;
}
bool NetworkContext::SetFailingHttpTransactionForTesting(int32_t rv) {
  NOTREACHED();
  return false;
}
bool NetworkContext::VerifyCertificateForTesting(const scoped_refptr<net::X509Certificate>& certificate, const std::string& hostname, const std::string& ocsp_response, int32_t* out_error_code) {
  NOTREACHED();
  return false;
}
bool NetworkContext::AddDomainReliabilityContextForTesting(const GURL& origin, const GURL& upload_url) {
  NOTREACHED();
  return false;
}
bool NetworkContext::ForceDomainReliabilityUploadsForTesting() {
  NOTREACHED();
  return false;
}

class NetworkContext_ClearNetworkingHistorySince_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ClearNetworkingHistorySince_ForwardToCallback(
      NetworkContext::ClearNetworkingHistorySinceCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ClearNetworkingHistorySinceCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearNetworkingHistorySince_ForwardToCallback);
};

class NetworkContext_ClearHttpCache_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ClearHttpCache_ForwardToCallback(
      NetworkContext::ClearHttpCacheCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ClearHttpCacheCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearHttpCache_ForwardToCallback);
};

class NetworkContext_ComputeHttpCacheSize_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ComputeHttpCacheSize_ForwardToCallback(
      NetworkContext::ComputeHttpCacheSizeCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ComputeHttpCacheSizeCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ComputeHttpCacheSize_ForwardToCallback);
};

class NetworkContext_ClearChannelIds_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ClearChannelIds_ForwardToCallback(
      NetworkContext::ClearChannelIdsCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ClearChannelIdsCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearChannelIds_ForwardToCallback);
};

class NetworkContext_ClearHostCache_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ClearHostCache_ForwardToCallback(
      NetworkContext::ClearHostCacheCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ClearHostCacheCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearHostCache_ForwardToCallback);
};

class NetworkContext_ClearHttpAuthCache_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ClearHttpAuthCache_ForwardToCallback(
      NetworkContext::ClearHttpAuthCacheCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ClearHttpAuthCacheCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearHttpAuthCache_ForwardToCallback);
};

class NetworkContext_ClearReportingCacheReports_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ClearReportingCacheReports_ForwardToCallback(
      NetworkContext::ClearReportingCacheReportsCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ClearReportingCacheReportsCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearReportingCacheReports_ForwardToCallback);
};

class NetworkContext_ClearReportingCacheClients_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ClearReportingCacheClients_ForwardToCallback(
      NetworkContext::ClearReportingCacheClientsCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ClearReportingCacheClientsCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearReportingCacheClients_ForwardToCallback);
};

class NetworkContext_ClearNetworkErrorLogging_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ClearNetworkErrorLogging_ForwardToCallback(
      NetworkContext::ClearNetworkErrorLoggingCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ClearNetworkErrorLoggingCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearNetworkErrorLogging_ForwardToCallback);
};

class NetworkContext_ClearDomainReliability_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ClearDomainReliability_ForwardToCallback(
      NetworkContext::ClearDomainReliabilityCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ClearDomainReliabilityCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearDomainReliability_ForwardToCallback);
};

class NetworkContext_GetDomainReliabilityJSON_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_GetDomainReliabilityJSON_ForwardToCallback(
      NetworkContext::GetDomainReliabilityJSONCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::GetDomainReliabilityJSONCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_GetDomainReliabilityJSON_ForwardToCallback);
};

class NetworkContext_CloseAllConnections_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_CloseAllConnections_ForwardToCallback(
      NetworkContext::CloseAllConnectionsCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::CloseAllConnectionsCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CloseAllConnections_ForwardToCallback);
};

class NetworkContext_CloseIdleConnections_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_CloseIdleConnections_ForwardToCallback(
      NetworkContext::CloseIdleConnectionsCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::CloseIdleConnectionsCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CloseIdleConnections_ForwardToCallback);
};

class NetworkContext_AddExpectCT_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_AddExpectCT_ForwardToCallback(
      NetworkContext::AddExpectCTCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::AddExpectCTCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_AddExpectCT_ForwardToCallback);
};

class NetworkContext_SetExpectCTTestReport_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_SetExpectCTTestReport_ForwardToCallback(
      NetworkContext::SetExpectCTTestReportCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::SetExpectCTTestReportCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_SetExpectCTTestReport_ForwardToCallback);
};

class NetworkContext_GetExpectCTState_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_GetExpectCTState_ForwardToCallback(
      NetworkContext::GetExpectCTStateCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::GetExpectCTStateCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_GetExpectCTState_ForwardToCallback);
};

class NetworkContext_CreateTCPServerSocket_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_CreateTCPServerSocket_ForwardToCallback(
      NetworkContext::CreateTCPServerSocketCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::CreateTCPServerSocketCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPServerSocket_ForwardToCallback);
};

class NetworkContext_CreateTCPConnectedSocket_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_CreateTCPConnectedSocket_ForwardToCallback(
      NetworkContext::CreateTCPConnectedSocketCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::CreateTCPConnectedSocketCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPConnectedSocket_ForwardToCallback);
};

class NetworkContext_CreateTCPBoundSocket_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_CreateTCPBoundSocket_ForwardToCallback(
      NetworkContext::CreateTCPBoundSocketCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::CreateTCPBoundSocketCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPBoundSocket_ForwardToCallback);
};

class NetworkContext_ForceReloadProxyConfig_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ForceReloadProxyConfig_ForwardToCallback(
      NetworkContext::ForceReloadProxyConfigCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ForceReloadProxyConfigCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ForceReloadProxyConfig_ForwardToCallback);
};

class NetworkContext_ClearBadProxiesCache_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ClearBadProxiesCache_ForwardToCallback(
      NetworkContext::ClearBadProxiesCacheCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ClearBadProxiesCacheCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearBadProxiesCache_ForwardToCallback);
};

class NetworkContext_VerifyCertForSignedExchange_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_VerifyCertForSignedExchange_ForwardToCallback(
      NetworkContext::VerifyCertForSignedExchangeCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::VerifyCertForSignedExchangeCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_VerifyCertForSignedExchange_ForwardToCallback);
};

class NetworkContext_AddHSTS_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_AddHSTS_ForwardToCallback(
      NetworkContext::AddHSTSCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::AddHSTSCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_AddHSTS_ForwardToCallback);
};

class NetworkContext_IsHSTSActiveForHost_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_IsHSTSActiveForHost_ForwardToCallback(
      NetworkContext::IsHSTSActiveForHostCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::IsHSTSActiveForHostCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_IsHSTSActiveForHost_ForwardToCallback);
};

class NetworkContext_GetHSTSState_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_GetHSTSState_ForwardToCallback(
      NetworkContext::GetHSTSStateCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::GetHSTSStateCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_GetHSTSState_ForwardToCallback);
};

class NetworkContext_SetCorsOriginAccessListsForOrigin_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_SetCorsOriginAccessListsForOrigin_ForwardToCallback(
      NetworkContext::SetCorsOriginAccessListsForOriginCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::SetCorsOriginAccessListsForOriginCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_SetCorsOriginAccessListsForOrigin_ForwardToCallback);
};

class NetworkContext_DeleteDynamicDataForHost_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_DeleteDynamicDataForHost_ForwardToCallback(
      NetworkContext::DeleteDynamicDataForHostCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::DeleteDynamicDataForHostCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_DeleteDynamicDataForHost_ForwardToCallback);
};

class NetworkContext_LookupBasicAuthCredentials_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_LookupBasicAuthCredentials_ForwardToCallback(
      NetworkContext::LookupBasicAuthCredentialsCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::LookupBasicAuthCredentialsCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_LookupBasicAuthCredentials_ForwardToCallback);
};
class NetworkContext_EnableStaticKeyPinningForTesting_HandleSyncResponse
    : public mojo::MessageReceiver {
 public:
  NetworkContext_EnableStaticKeyPinningForTesting_HandleSyncResponse(
      bool* result)
      : result_(result) {
    DCHECK(!*result_);
  }
  bool Accept(mojo::Message* message) override;
 private:
  bool* result_;DISALLOW_COPY_AND_ASSIGN(NetworkContext_EnableStaticKeyPinningForTesting_HandleSyncResponse);
};

class NetworkContext_EnableStaticKeyPinningForTesting_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_EnableStaticKeyPinningForTesting_ForwardToCallback(
      NetworkContext::EnableStaticKeyPinningForTestingCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::EnableStaticKeyPinningForTestingCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_EnableStaticKeyPinningForTesting_ForwardToCallback);
};
class NetworkContext_SetFailingHttpTransactionForTesting_HandleSyncResponse
    : public mojo::MessageReceiver {
 public:
  NetworkContext_SetFailingHttpTransactionForTesting_HandleSyncResponse(
      bool* result)
      : result_(result) {
    DCHECK(!*result_);
  }
  bool Accept(mojo::Message* message) override;
 private:
  bool* result_;DISALLOW_COPY_AND_ASSIGN(NetworkContext_SetFailingHttpTransactionForTesting_HandleSyncResponse);
};

class NetworkContext_SetFailingHttpTransactionForTesting_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_SetFailingHttpTransactionForTesting_ForwardToCallback(
      NetworkContext::SetFailingHttpTransactionForTestingCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::SetFailingHttpTransactionForTestingCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_SetFailingHttpTransactionForTesting_ForwardToCallback);
};
class NetworkContext_VerifyCertificateForTesting_HandleSyncResponse
    : public mojo::MessageReceiver {
 public:
  NetworkContext_VerifyCertificateForTesting_HandleSyncResponse(
      bool* result, int32_t* out_error_code)
      : result_(result), out_error_code_(out_error_code) {
    DCHECK(!*result_);
  }
  bool Accept(mojo::Message* message) override;
 private:
  bool* result_;
  int32_t* out_error_code_;DISALLOW_COPY_AND_ASSIGN(NetworkContext_VerifyCertificateForTesting_HandleSyncResponse);
};

class NetworkContext_VerifyCertificateForTesting_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_VerifyCertificateForTesting_ForwardToCallback(
      NetworkContext::VerifyCertificateForTestingCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::VerifyCertificateForTestingCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_VerifyCertificateForTesting_ForwardToCallback);
};
class NetworkContext_AddDomainReliabilityContextForTesting_HandleSyncResponse
    : public mojo::MessageReceiver {
 public:
  NetworkContext_AddDomainReliabilityContextForTesting_HandleSyncResponse(
      bool* result)
      : result_(result) {
    DCHECK(!*result_);
  }
  bool Accept(mojo::Message* message) override;
 private:
  bool* result_;DISALLOW_COPY_AND_ASSIGN(NetworkContext_AddDomainReliabilityContextForTesting_HandleSyncResponse);
};

class NetworkContext_AddDomainReliabilityContextForTesting_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_AddDomainReliabilityContextForTesting_ForwardToCallback(
      NetworkContext::AddDomainReliabilityContextForTestingCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::AddDomainReliabilityContextForTestingCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_AddDomainReliabilityContextForTesting_ForwardToCallback);
};
class NetworkContext_ForceDomainReliabilityUploadsForTesting_HandleSyncResponse
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ForceDomainReliabilityUploadsForTesting_HandleSyncResponse(
      bool* result)
      : result_(result) {
    DCHECK(!*result_);
  }
  bool Accept(mojo::Message* message) override;
 private:
  bool* result_;DISALLOW_COPY_AND_ASSIGN(NetworkContext_ForceDomainReliabilityUploadsForTesting_HandleSyncResponse);
};

class NetworkContext_ForceDomainReliabilityUploadsForTesting_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ForceDomainReliabilityUploadsForTesting_ForwardToCallback(
      NetworkContext::ForceDomainReliabilityUploadsForTestingCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ForceDomainReliabilityUploadsForTestingCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ForceDomainReliabilityUploadsForTesting_ForwardToCallback);
};

NetworkContextProxy::NetworkContextProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class NetworkContextProxy_SetClient_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_SetClient_Message(
      uint32_t message_flags
      , NetworkContextClientPtr param_client
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_SetClient_Name,
          message_flags)
      , param_client_(param_client.PassInterface()){}
  ~NetworkContextProxy_SetClient_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      NetworkContextClientPtr param_client) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_SetClient_Message>(
          kFlags
          , std::move(param_client)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_SetClient_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_SetClient_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkContextClientPtrDataView>(
      param_client, &params->client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in NetworkContext.SetClient request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->SetClient(
        NetworkContextClientPtr(std::move(param_client_)));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_SetClient_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkContextClientPtrDataView>(
      param_client_, &params->client, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in NetworkContext.SetClient request");
  }
  NetworkContextClientPtrInfo param_client_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_SetClient_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_SetClient_Message::kMessageTag = {};

void NetworkContextProxy::SetClient(
    NetworkContextClientPtr in_client) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::SetClient");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_SetClient_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_client));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("SetClient");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_CreateURLLoaderFactory_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateURLLoaderFactory_Message(
      uint32_t message_flags
      , ::network::mojom::URLLoaderFactoryRequest param_url_loader_factory
      , URLLoaderFactoryParamsPtr param_params
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateURLLoaderFactory_Name,
          message_flags)
      , param_url_loader_factory_(std::move(param_url_loader_factory))
      , param_params_(std::move(param_params)){}
  ~NetworkContextProxy_CreateURLLoaderFactory_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::URLLoaderFactoryRequest param_url_loader_factory, URLLoaderFactoryParamsPtr param_params) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateURLLoaderFactory_Message>(
          kFlags
          , std::move(param_url_loader_factory)
          , std::move(param_params)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateURLLoaderFactory_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateURLLoaderFactory_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::URLLoaderFactoryRequestDataView>(
      param_url_loader_factory, &params->url_loader_factory, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->url_loader_factory),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid url_loader_factory in NetworkContext.CreateURLLoaderFactory request");
  typename decltype(params->params)::BaseType::BufferWriter
      params_writer;
  mojo::internal::Serialize<::network::mojom::URLLoaderFactoryParamsDataView>(
      param_params, buffer, &params_writer, &serialization_context);
  params->params.Set(
      params_writer.is_null() ? nullptr : params_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->params.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null params in NetworkContext.CreateURLLoaderFactory request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->CreateURLLoaderFactory(
        std::move(param_url_loader_factory_),
        std::move(param_params_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateURLLoaderFactory_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::URLLoaderFactoryRequestDataView>(
      param_url_loader_factory_, &params->url_loader_factory, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->url_loader_factory),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid url_loader_factory in NetworkContext.CreateURLLoaderFactory request");
  typename decltype(params->params)::BaseType::BufferWriter
      params_writer;
  mojo::internal::Serialize<::network::mojom::URLLoaderFactoryParamsDataView>(
      param_params_, buffer, &params_writer, serialization_context);
  params->params.Set(
      params_writer.is_null() ? nullptr : params_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->params.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null params in NetworkContext.CreateURLLoaderFactory request");
  }
  ::network::mojom::URLLoaderFactoryRequest param_url_loader_factory_;
  URLLoaderFactoryParamsPtr param_params_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateURLLoaderFactory_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateURLLoaderFactory_Message::kMessageTag = {};

void NetworkContextProxy::CreateURLLoaderFactory(
    ::network::mojom::URLLoaderFactoryRequest in_url_loader_factory, URLLoaderFactoryParamsPtr in_params) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateURLLoaderFactory");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateURLLoaderFactory_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_url_loader_factory), std::move(in_params));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CreateURLLoaderFactory");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_ResetURLLoaderFactories_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ResetURLLoaderFactories_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ResetURLLoaderFactories_Name,
          message_flags){}
  ~NetworkContextProxy_ResetURLLoaderFactories_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ResetURLLoaderFactories_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ResetURLLoaderFactories_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ResetURLLoaderFactories_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->ResetURLLoaderFactories();
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ResetURLLoaderFactories_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ResetURLLoaderFactories_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ResetURLLoaderFactories_Message::kMessageTag = {};

void NetworkContextProxy::ResetURLLoaderFactories(
    ) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ResetURLLoaderFactories");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ResetURLLoaderFactories_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ResetURLLoaderFactories");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_GetCookieManager_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_GetCookieManager_Message(
      uint32_t message_flags
      , ::network::mojom::CookieManagerRequest param_cookie_manager
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_GetCookieManager_Name,
          message_flags)
      , param_cookie_manager_(std::move(param_cookie_manager)){}
  ~NetworkContextProxy_GetCookieManager_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::CookieManagerRequest param_cookie_manager) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_GetCookieManager_Message>(
          kFlags
          , std::move(param_cookie_manager)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_GetCookieManager_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_GetCookieManager_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::CookieManagerRequestDataView>(
      param_cookie_manager, &params->cookie_manager, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->cookie_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid cookie_manager in NetworkContext.GetCookieManager request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->GetCookieManager(
        std::move(param_cookie_manager_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_GetCookieManager_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::CookieManagerRequestDataView>(
      param_cookie_manager_, &params->cookie_manager, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->cookie_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid cookie_manager in NetworkContext.GetCookieManager request");
  }
  ::network::mojom::CookieManagerRequest param_cookie_manager_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_GetCookieManager_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_GetCookieManager_Message::kMessageTag = {};

void NetworkContextProxy::GetCookieManager(
    ::network::mojom::CookieManagerRequest in_cookie_manager) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::GetCookieManager");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_GetCookieManager_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_cookie_manager));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("GetCookieManager");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_GetRestrictedCookieManager_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_GetRestrictedCookieManager_Message(
      uint32_t message_flags
      , ::network::mojom::RestrictedCookieManagerRequest param_restricted_cookie_manager
      , const url::Origin& param_origin
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_GetRestrictedCookieManager_Name,
          message_flags)
      , param_restricted_cookie_manager_(std::move(param_restricted_cookie_manager))
      , param_origin_(std::move(param_origin)){}
  ~NetworkContextProxy_GetRestrictedCookieManager_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::RestrictedCookieManagerRequest param_restricted_cookie_manager, const url::Origin& param_origin) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_GetRestrictedCookieManager_Message>(
          kFlags
          , std::move(param_restricted_cookie_manager)
          , std::move(param_origin)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_GetRestrictedCookieManager_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_GetRestrictedCookieManager_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::RestrictedCookieManagerRequestDataView>(
      param_restricted_cookie_manager, &params->restricted_cookie_manager, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->restricted_cookie_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid restricted_cookie_manager in NetworkContext.GetRestrictedCookieManager request");
  typename decltype(params->origin)::BaseType::BufferWriter
      origin_writer;
  mojo::internal::Serialize<::url::mojom::OriginDataView>(
      param_origin, buffer, &origin_writer, &serialization_context);
  params->origin.Set(
      origin_writer.is_null() ? nullptr : origin_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in NetworkContext.GetRestrictedCookieManager request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->GetRestrictedCookieManager(
        std::move(param_restricted_cookie_manager_),
        std::move(param_origin_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_GetRestrictedCookieManager_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::RestrictedCookieManagerRequestDataView>(
      param_restricted_cookie_manager_, &params->restricted_cookie_manager, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->restricted_cookie_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid restricted_cookie_manager in NetworkContext.GetRestrictedCookieManager request");
  typename decltype(params->origin)::BaseType::BufferWriter
      origin_writer;
  mojo::internal::Serialize<::url::mojom::OriginDataView>(
      param_origin_, buffer, &origin_writer, serialization_context);
  params->origin.Set(
      origin_writer.is_null() ? nullptr : origin_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in NetworkContext.GetRestrictedCookieManager request");
  }
  ::network::mojom::RestrictedCookieManagerRequest param_restricted_cookie_manager_;
  url::Origin param_origin_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_GetRestrictedCookieManager_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_GetRestrictedCookieManager_Message::kMessageTag = {};

void NetworkContextProxy::GetRestrictedCookieManager(
    ::network::mojom::RestrictedCookieManagerRequest in_restricted_cookie_manager, const url::Origin& in_origin) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::GetRestrictedCookieManager");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_GetRestrictedCookieManager_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_restricted_cookie_manager), std::move(in_origin));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("GetRestrictedCookieManager");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_ClearNetworkingHistorySince_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ClearNetworkingHistorySince_Message(
      uint32_t message_flags
      , base::Time param_start_time
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearNetworkingHistorySince_Name,
          message_flags)
      , param_start_time_(std::move(param_start_time)){}
  ~NetworkContextProxy_ClearNetworkingHistorySince_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      base::Time param_start_time) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ClearNetworkingHistorySince_Message>(
          kFlags
          , std::move(param_start_time)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearNetworkingHistorySince_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearNetworkingHistorySince_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->start_time)::BaseType::BufferWriter
      start_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_start_time, buffer, &start_time_writer, &serialization_context);
  params->start_time.Set(
      start_time_writer.is_null() ? nullptr : start_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->start_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null start_time in NetworkContext.ClearNetworkingHistorySince request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ClearNetworkingHistorySinceCallback callback) {
    impl->ClearNetworkingHistorySince(
        std::move(param_start_time_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearNetworkingHistorySince_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->start_time)::BaseType::BufferWriter
      start_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_start_time_, buffer, &start_time_writer, serialization_context);
  params->start_time.Set(
      start_time_writer.is_null() ? nullptr : start_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->start_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null start_time in NetworkContext.ClearNetworkingHistorySince request");
  }
  base::Time param_start_time_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ClearNetworkingHistorySince_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ClearNetworkingHistorySince_Message::kMessageTag = {};

void NetworkContextProxy::ClearNetworkingHistorySince(
    base::Time in_start_time, ClearNetworkingHistorySinceCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ClearNetworkingHistorySince");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ClearNetworkingHistorySince_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_start_time));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearNetworkingHistorySince");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ClearNetworkingHistorySince_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_ClearHttpCache_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ClearHttpCache_Message(
      uint32_t message_flags
      , base::Time param_start_time
      , base::Time param_end_time
      , ClearDataFilterPtr param_filter
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearHttpCache_Name,
          message_flags)
      , param_start_time_(std::move(param_start_time))
      , param_end_time_(std::move(param_end_time))
      , param_filter_(std::move(param_filter)){}
  ~NetworkContextProxy_ClearHttpCache_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      base::Time param_start_time, base::Time param_end_time, ClearDataFilterPtr param_filter) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ClearHttpCache_Message>(
          kFlags
          , std::move(param_start_time)
          , std::move(param_end_time)
          , std::move(param_filter)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearHttpCache_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearHttpCache_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->start_time)::BaseType::BufferWriter
      start_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_start_time, buffer, &start_time_writer, &serialization_context);
  params->start_time.Set(
      start_time_writer.is_null() ? nullptr : start_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->start_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null start_time in NetworkContext.ClearHttpCache request");
  typename decltype(params->end_time)::BaseType::BufferWriter
      end_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_end_time, buffer, &end_time_writer, &serialization_context);
  params->end_time.Set(
      end_time_writer.is_null() ? nullptr : end_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->end_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null end_time in NetworkContext.ClearHttpCache request");
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearDataFilterDataView>(
      param_filter, buffer, &filter_writer, &serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ClearHttpCacheCallback callback) {
    impl->ClearHttpCache(
        std::move(param_start_time_),
        std::move(param_end_time_),
        std::move(param_filter_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearHttpCache_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->start_time)::BaseType::BufferWriter
      start_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_start_time_, buffer, &start_time_writer, serialization_context);
  params->start_time.Set(
      start_time_writer.is_null() ? nullptr : start_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->start_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null start_time in NetworkContext.ClearHttpCache request");
  typename decltype(params->end_time)::BaseType::BufferWriter
      end_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_end_time_, buffer, &end_time_writer, serialization_context);
  params->end_time.Set(
      end_time_writer.is_null() ? nullptr : end_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->end_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null end_time in NetworkContext.ClearHttpCache request");
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearDataFilterDataView>(
      param_filter_, buffer, &filter_writer, serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  }
  base::Time param_start_time_;
  base::Time param_end_time_;
  ClearDataFilterPtr param_filter_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ClearHttpCache_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ClearHttpCache_Message::kMessageTag = {};

void NetworkContextProxy::ClearHttpCache(
    base::Time in_start_time, base::Time in_end_time, ClearDataFilterPtr in_filter, ClearHttpCacheCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ClearHttpCache");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ClearHttpCache_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_start_time), std::move(in_end_time), std::move(in_filter));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearHttpCache");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ClearHttpCache_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_ComputeHttpCacheSize_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ComputeHttpCacheSize_Message(
      uint32_t message_flags
      , base::Time param_start_time
      , base::Time param_end_time
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ComputeHttpCacheSize_Name,
          message_flags)
      , param_start_time_(std::move(param_start_time))
      , param_end_time_(std::move(param_end_time)){}
  ~NetworkContextProxy_ComputeHttpCacheSize_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      base::Time param_start_time, base::Time param_end_time) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ComputeHttpCacheSize_Message>(
          kFlags
          , std::move(param_start_time)
          , std::move(param_end_time)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ComputeHttpCacheSize_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ComputeHttpCacheSize_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->start_time)::BaseType::BufferWriter
      start_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_start_time, buffer, &start_time_writer, &serialization_context);
  params->start_time.Set(
      start_time_writer.is_null() ? nullptr : start_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->start_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null start_time in NetworkContext.ComputeHttpCacheSize request");
  typename decltype(params->end_time)::BaseType::BufferWriter
      end_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_end_time, buffer, &end_time_writer, &serialization_context);
  params->end_time.Set(
      end_time_writer.is_null() ? nullptr : end_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->end_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null end_time in NetworkContext.ComputeHttpCacheSize request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ComputeHttpCacheSizeCallback callback) {
    impl->ComputeHttpCacheSize(
        std::move(param_start_time_),
        std::move(param_end_time_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ComputeHttpCacheSize_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->start_time)::BaseType::BufferWriter
      start_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_start_time_, buffer, &start_time_writer, serialization_context);
  params->start_time.Set(
      start_time_writer.is_null() ? nullptr : start_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->start_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null start_time in NetworkContext.ComputeHttpCacheSize request");
  typename decltype(params->end_time)::BaseType::BufferWriter
      end_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_end_time_, buffer, &end_time_writer, serialization_context);
  params->end_time.Set(
      end_time_writer.is_null() ? nullptr : end_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->end_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null end_time in NetworkContext.ComputeHttpCacheSize request");
  }
  base::Time param_start_time_;
  base::Time param_end_time_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ComputeHttpCacheSize_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ComputeHttpCacheSize_Message::kMessageTag = {};

void NetworkContextProxy::ComputeHttpCacheSize(
    base::Time in_start_time, base::Time in_end_time, ComputeHttpCacheSizeCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ComputeHttpCacheSize");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ComputeHttpCacheSize_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_start_time), std::move(in_end_time));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ComputeHttpCacheSize");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ComputeHttpCacheSize_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_WriteCacheMetadata_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_WriteCacheMetadata_Message(
      uint32_t message_flags
      , const GURL& param_url
      , net::RequestPriority param_priority
      , base::Time param_expected_response_time
      , const std::vector<uint8_t>& param_data
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_WriteCacheMetadata_Name,
          message_flags)
      , param_url_(std::move(param_url))
      , param_priority_(std::move(param_priority))
      , param_expected_response_time_(std::move(param_expected_response_time))
      , param_data_(std::move(param_data)){}
  ~NetworkContextProxy_WriteCacheMetadata_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const GURL& param_url, net::RequestPriority param_priority, base::Time param_expected_response_time, const std::vector<uint8_t>& param_data) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_WriteCacheMetadata_Message>(
          kFlags
          , std::move(param_url)
          , std::move(param_priority)
          , std::move(param_expected_response_time)
          , std::move(param_data)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_WriteCacheMetadata_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_WriteCacheMetadata_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url, buffer, &url_writer, &serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkContext.WriteCacheMetadata request");
  mojo::internal::Serialize<::network::mojom::RequestPriority>(
      param_priority, &params->priority);
  typename decltype(params->expected_response_time)::BaseType::BufferWriter
      expected_response_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_expected_response_time, buffer, &expected_response_time_writer, &serialization_context);
  params->expected_response_time.Set(
      expected_response_time_writer.is_null() ? nullptr : expected_response_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->expected_response_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null expected_response_time in NetworkContext.WriteCacheMetadata request");
  typename decltype(params->data)::BaseType::BufferWriter
      data_writer;
  const mojo::internal::ContainerValidateParams data_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      param_data, buffer, &data_writer, &data_validate_params,
      &serialization_context);
  params->data.Set(
      data_writer.is_null() ? nullptr : data_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->data.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null data in NetworkContext.WriteCacheMetadata request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->WriteCacheMetadata(
        std::move(param_url_),
        std::move(param_priority_),
        std::move(param_expected_response_time_),
        std::move(param_data_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_WriteCacheMetadata_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url_, buffer, &url_writer, serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkContext.WriteCacheMetadata request");
  mojo::internal::Serialize<::network::mojom::RequestPriority>(
      param_priority_, &params->priority);
  typename decltype(params->expected_response_time)::BaseType::BufferWriter
      expected_response_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_expected_response_time_, buffer, &expected_response_time_writer, serialization_context);
  params->expected_response_time.Set(
      expected_response_time_writer.is_null() ? nullptr : expected_response_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->expected_response_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null expected_response_time in NetworkContext.WriteCacheMetadata request");
  typename decltype(params->data)::BaseType::BufferWriter
      data_writer;
  const mojo::internal::ContainerValidateParams data_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      param_data_, buffer, &data_writer, &data_validate_params,
      serialization_context);
  params->data.Set(
      data_writer.is_null() ? nullptr : data_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->data.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null data in NetworkContext.WriteCacheMetadata request");
  }
  GURL param_url_;
  net::RequestPriority param_priority_;
  base::Time param_expected_response_time_;
  std::vector<uint8_t> param_data_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_WriteCacheMetadata_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_WriteCacheMetadata_Message::kMessageTag = {};

void NetworkContextProxy::WriteCacheMetadata(
    const GURL& in_url, net::RequestPriority in_priority, base::Time in_expected_response_time, const std::vector<uint8_t>& in_data) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::WriteCacheMetadata");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_WriteCacheMetadata_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_url), std::move(in_priority), std::move(in_expected_response_time), std::move(in_data));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("WriteCacheMetadata");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_ClearChannelIds_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ClearChannelIds_Message(
      uint32_t message_flags
      , base::Time param_start_time
      , base::Time param_end_time
      , ClearDataFilterPtr param_filter
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearChannelIds_Name,
          message_flags)
      , param_start_time_(std::move(param_start_time))
      , param_end_time_(std::move(param_end_time))
      , param_filter_(std::move(param_filter)){}
  ~NetworkContextProxy_ClearChannelIds_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      base::Time param_start_time, base::Time param_end_time, ClearDataFilterPtr param_filter) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ClearChannelIds_Message>(
          kFlags
          , std::move(param_start_time)
          , std::move(param_end_time)
          , std::move(param_filter)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearChannelIds_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearChannelIds_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->start_time)::BaseType::BufferWriter
      start_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_start_time, buffer, &start_time_writer, &serialization_context);
  params->start_time.Set(
      start_time_writer.is_null() ? nullptr : start_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->start_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null start_time in NetworkContext.ClearChannelIds request");
  typename decltype(params->end_time)::BaseType::BufferWriter
      end_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_end_time, buffer, &end_time_writer, &serialization_context);
  params->end_time.Set(
      end_time_writer.is_null() ? nullptr : end_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->end_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null end_time in NetworkContext.ClearChannelIds request");
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearDataFilterDataView>(
      param_filter, buffer, &filter_writer, &serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ClearChannelIdsCallback callback) {
    impl->ClearChannelIds(
        std::move(param_start_time_),
        std::move(param_end_time_),
        std::move(param_filter_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearChannelIds_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->start_time)::BaseType::BufferWriter
      start_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_start_time_, buffer, &start_time_writer, serialization_context);
  params->start_time.Set(
      start_time_writer.is_null() ? nullptr : start_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->start_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null start_time in NetworkContext.ClearChannelIds request");
  typename decltype(params->end_time)::BaseType::BufferWriter
      end_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_end_time_, buffer, &end_time_writer, serialization_context);
  params->end_time.Set(
      end_time_writer.is_null() ? nullptr : end_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->end_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null end_time in NetworkContext.ClearChannelIds request");
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearDataFilterDataView>(
      param_filter_, buffer, &filter_writer, serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  }
  base::Time param_start_time_;
  base::Time param_end_time_;
  ClearDataFilterPtr param_filter_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ClearChannelIds_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ClearChannelIds_Message::kMessageTag = {};

void NetworkContextProxy::ClearChannelIds(
    base::Time in_start_time, base::Time in_end_time, ClearDataFilterPtr in_filter, ClearChannelIdsCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ClearChannelIds");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ClearChannelIds_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_start_time), std::move(in_end_time), std::move(in_filter));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearChannelIds");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ClearChannelIds_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_ClearHostCache_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ClearHostCache_Message(
      uint32_t message_flags
      , ClearDataFilterPtr param_filter
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearHostCache_Name,
          message_flags)
      , param_filter_(std::move(param_filter)){}
  ~NetworkContextProxy_ClearHostCache_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ClearDataFilterPtr param_filter) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ClearHostCache_Message>(
          kFlags
          , std::move(param_filter)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearHostCache_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearHostCache_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearDataFilterDataView>(
      param_filter, buffer, &filter_writer, &serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ClearHostCacheCallback callback) {
    impl->ClearHostCache(
        std::move(param_filter_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearHostCache_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearDataFilterDataView>(
      param_filter_, buffer, &filter_writer, serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  }
  ClearDataFilterPtr param_filter_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ClearHostCache_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ClearHostCache_Message::kMessageTag = {};

void NetworkContextProxy::ClearHostCache(
    ClearDataFilterPtr in_filter, ClearHostCacheCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ClearHostCache");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ClearHostCache_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_filter));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearHostCache");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ClearHostCache_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_ClearHttpAuthCache_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ClearHttpAuthCache_Message(
      uint32_t message_flags
      , base::Time param_start_time
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearHttpAuthCache_Name,
          message_flags)
      , param_start_time_(std::move(param_start_time)){}
  ~NetworkContextProxy_ClearHttpAuthCache_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      base::Time param_start_time) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ClearHttpAuthCache_Message>(
          kFlags
          , std::move(param_start_time)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearHttpAuthCache_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearHttpAuthCache_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->start_time)::BaseType::BufferWriter
      start_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_start_time, buffer, &start_time_writer, &serialization_context);
  params->start_time.Set(
      start_time_writer.is_null() ? nullptr : start_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->start_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null start_time in NetworkContext.ClearHttpAuthCache request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ClearHttpAuthCacheCallback callback) {
    impl->ClearHttpAuthCache(
        std::move(param_start_time_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearHttpAuthCache_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->start_time)::BaseType::BufferWriter
      start_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_start_time_, buffer, &start_time_writer, serialization_context);
  params->start_time.Set(
      start_time_writer.is_null() ? nullptr : start_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->start_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null start_time in NetworkContext.ClearHttpAuthCache request");
  }
  base::Time param_start_time_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ClearHttpAuthCache_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ClearHttpAuthCache_Message::kMessageTag = {};

void NetworkContextProxy::ClearHttpAuthCache(
    base::Time in_start_time, ClearHttpAuthCacheCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ClearHttpAuthCache");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ClearHttpAuthCache_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_start_time));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearHttpAuthCache");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ClearHttpAuthCache_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_ClearReportingCacheReports_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ClearReportingCacheReports_Message(
      uint32_t message_flags
      , ClearDataFilterPtr param_filter
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearReportingCacheReports_Name,
          message_flags)
      , param_filter_(std::move(param_filter)){}
  ~NetworkContextProxy_ClearReportingCacheReports_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ClearDataFilterPtr param_filter) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ClearReportingCacheReports_Message>(
          kFlags
          , std::move(param_filter)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearReportingCacheReports_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearReportingCacheReports_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearDataFilterDataView>(
      param_filter, buffer, &filter_writer, &serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ClearReportingCacheReportsCallback callback) {
    impl->ClearReportingCacheReports(
        std::move(param_filter_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearReportingCacheReports_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearDataFilterDataView>(
      param_filter_, buffer, &filter_writer, serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  }
  ClearDataFilterPtr param_filter_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ClearReportingCacheReports_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ClearReportingCacheReports_Message::kMessageTag = {};

void NetworkContextProxy::ClearReportingCacheReports(
    ClearDataFilterPtr in_filter, ClearReportingCacheReportsCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ClearReportingCacheReports");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ClearReportingCacheReports_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_filter));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearReportingCacheReports");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ClearReportingCacheReports_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_ClearReportingCacheClients_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ClearReportingCacheClients_Message(
      uint32_t message_flags
      , ClearDataFilterPtr param_filter
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearReportingCacheClients_Name,
          message_flags)
      , param_filter_(std::move(param_filter)){}
  ~NetworkContextProxy_ClearReportingCacheClients_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ClearDataFilterPtr param_filter) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ClearReportingCacheClients_Message>(
          kFlags
          , std::move(param_filter)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearReportingCacheClients_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearReportingCacheClients_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearDataFilterDataView>(
      param_filter, buffer, &filter_writer, &serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ClearReportingCacheClientsCallback callback) {
    impl->ClearReportingCacheClients(
        std::move(param_filter_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearReportingCacheClients_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearDataFilterDataView>(
      param_filter_, buffer, &filter_writer, serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  }
  ClearDataFilterPtr param_filter_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ClearReportingCacheClients_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ClearReportingCacheClients_Message::kMessageTag = {};

void NetworkContextProxy::ClearReportingCacheClients(
    ClearDataFilterPtr in_filter, ClearReportingCacheClientsCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ClearReportingCacheClients");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ClearReportingCacheClients_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_filter));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearReportingCacheClients");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ClearReportingCacheClients_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_ClearNetworkErrorLogging_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ClearNetworkErrorLogging_Message(
      uint32_t message_flags
      , ClearDataFilterPtr param_filter
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearNetworkErrorLogging_Name,
          message_flags)
      , param_filter_(std::move(param_filter)){}
  ~NetworkContextProxy_ClearNetworkErrorLogging_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ClearDataFilterPtr param_filter) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ClearNetworkErrorLogging_Message>(
          kFlags
          , std::move(param_filter)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearNetworkErrorLogging_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearNetworkErrorLogging_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearDataFilterDataView>(
      param_filter, buffer, &filter_writer, &serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ClearNetworkErrorLoggingCallback callback) {
    impl->ClearNetworkErrorLogging(
        std::move(param_filter_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearNetworkErrorLogging_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearDataFilterDataView>(
      param_filter_, buffer, &filter_writer, serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  }
  ClearDataFilterPtr param_filter_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ClearNetworkErrorLogging_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ClearNetworkErrorLogging_Message::kMessageTag = {};

void NetworkContextProxy::ClearNetworkErrorLogging(
    ClearDataFilterPtr in_filter, ClearNetworkErrorLoggingCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ClearNetworkErrorLogging");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ClearNetworkErrorLogging_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_filter));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearNetworkErrorLogging");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ClearNetworkErrorLogging_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_ClearDomainReliability_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ClearDomainReliability_Message(
      uint32_t message_flags
      , ClearDataFilterPtr param_filter
      , NetworkContext::DomainReliabilityClearMode param_mode
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearDomainReliability_Name,
          message_flags)
      , param_filter_(std::move(param_filter))
      , param_mode_(std::move(param_mode)){}
  ~NetworkContextProxy_ClearDomainReliability_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ClearDataFilterPtr param_filter, NetworkContext::DomainReliabilityClearMode param_mode) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ClearDomainReliability_Message>(
          kFlags
          , std::move(param_filter)
          , std::move(param_mode)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearDomainReliability_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearDomainReliability_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearDataFilterDataView>(
      param_filter, buffer, &filter_writer, &serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  mojo::internal::Serialize<::network::mojom::NetworkContext_DomainReliabilityClearMode>(
      param_mode, &params->mode);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ClearDomainReliabilityCallback callback) {
    impl->ClearDomainReliability(
        std::move(param_filter_),
        std::move(param_mode_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearDomainReliability_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearDataFilterDataView>(
      param_filter_, buffer, &filter_writer, serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  mojo::internal::Serialize<::network::mojom::NetworkContext_DomainReliabilityClearMode>(
      param_mode_, &params->mode);
  }
  ClearDataFilterPtr param_filter_;
  NetworkContext::DomainReliabilityClearMode param_mode_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ClearDomainReliability_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ClearDomainReliability_Message::kMessageTag = {};

void NetworkContextProxy::ClearDomainReliability(
    ClearDataFilterPtr in_filter, NetworkContext::DomainReliabilityClearMode in_mode, ClearDomainReliabilityCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ClearDomainReliability");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ClearDomainReliability_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_filter), std::move(in_mode));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearDomainReliability");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ClearDomainReliability_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_GetDomainReliabilityJSON_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_GetDomainReliabilityJSON_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_GetDomainReliabilityJSON_Name,
          message_flags){}
  ~NetworkContextProxy_GetDomainReliabilityJSON_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_GetDomainReliabilityJSON_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_GetDomainReliabilityJSON_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_GetDomainReliabilityJSON_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::GetDomainReliabilityJSONCallback callback) {
    impl->GetDomainReliabilityJSON(std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_GetDomainReliabilityJSON_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_GetDomainReliabilityJSON_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_GetDomainReliabilityJSON_Message::kMessageTag = {};

void NetworkContextProxy::GetDomainReliabilityJSON(
    GetDomainReliabilityJSONCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::GetDomainReliabilityJSON");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_GetDomainReliabilityJSON_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("GetDomainReliabilityJSON");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_GetDomainReliabilityJSON_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_QueueReport_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_QueueReport_Message(
      uint32_t message_flags
      , const std::string& param_type
      , const std::string& param_group
      , const GURL& param_url
      , const base::Optional<std::string>& param_user_agent
      , base::Value param_body
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_QueueReport_Name,
          message_flags)
      , param_type_(std::move(param_type))
      , param_group_(std::move(param_group))
      , param_url_(std::move(param_url))
      , param_user_agent_(std::move(param_user_agent))
      , param_body_(std::move(param_body)){}
  ~NetworkContextProxy_QueueReport_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const std::string& param_type, const std::string& param_group, const GURL& param_url, const base::Optional<std::string>& param_user_agent, base::Value param_body) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_QueueReport_Message>(
          kFlags
          , std::move(param_type)
          , std::move(param_group)
          , std::move(param_url)
          , std::move(param_user_agent)
          , std::move(param_body)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_QueueReport_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_QueueReport_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->type)::BaseType::BufferWriter
      type_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_type, buffer, &type_writer, &serialization_context);
  params->type.Set(
      type_writer.is_null() ? nullptr : type_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->type.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null type in NetworkContext.QueueReport request");
  typename decltype(params->group)::BaseType::BufferWriter
      group_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_group, buffer, &group_writer, &serialization_context);
  params->group.Set(
      group_writer.is_null() ? nullptr : group_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->group.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null group in NetworkContext.QueueReport request");
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url, buffer, &url_writer, &serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkContext.QueueReport request");
  typename decltype(params->user_agent)::BaseType::BufferWriter
      user_agent_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_user_agent, buffer, &user_agent_writer, &serialization_context);
  params->user_agent.Set(
      user_agent_writer.is_null() ? nullptr : user_agent_writer.data());
  typename decltype(params->body)::BaseType::BufferWriter
      body_writer;
  mojo::internal::Serialize<::mojo_base::mojom::DictionaryValueDataView>(
      param_body, buffer, &body_writer, &serialization_context);
  params->body.Set(
      body_writer.is_null() ? nullptr : body_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->body.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null body in NetworkContext.QueueReport request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->QueueReport(
        std::move(param_type_),
        std::move(param_group_),
        std::move(param_url_),
        std::move(param_user_agent_),
        std::move(param_body_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_QueueReport_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->type)::BaseType::BufferWriter
      type_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_type_, buffer, &type_writer, serialization_context);
  params->type.Set(
      type_writer.is_null() ? nullptr : type_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->type.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null type in NetworkContext.QueueReport request");
  typename decltype(params->group)::BaseType::BufferWriter
      group_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_group_, buffer, &group_writer, serialization_context);
  params->group.Set(
      group_writer.is_null() ? nullptr : group_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->group.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null group in NetworkContext.QueueReport request");
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url_, buffer, &url_writer, serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkContext.QueueReport request");
  typename decltype(params->user_agent)::BaseType::BufferWriter
      user_agent_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_user_agent_, buffer, &user_agent_writer, serialization_context);
  params->user_agent.Set(
      user_agent_writer.is_null() ? nullptr : user_agent_writer.data());
  typename decltype(params->body)::BaseType::BufferWriter
      body_writer;
  mojo::internal::Serialize<::mojo_base::mojom::DictionaryValueDataView>(
      param_body_, buffer, &body_writer, serialization_context);
  params->body.Set(
      body_writer.is_null() ? nullptr : body_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->body.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null body in NetworkContext.QueueReport request");
  }
  std::string param_type_;
  std::string param_group_;
  GURL param_url_;
  base::Optional<std::string> param_user_agent_;
  base::Value param_body_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_QueueReport_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_QueueReport_Message::kMessageTag = {};

void NetworkContextProxy::QueueReport(
    const std::string& in_type, const std::string& in_group, const GURL& in_url, const base::Optional<std::string>& in_user_agent, base::Value in_body) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::QueueReport");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_QueueReport_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_type), std::move(in_group), std::move(in_url), std::move(in_user_agent), std::move(in_body));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("QueueReport");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_CloseAllConnections_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CloseAllConnections_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CloseAllConnections_Name,
          message_flags){}
  ~NetworkContextProxy_CloseAllConnections_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CloseAllConnections_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CloseAllConnections_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CloseAllConnections_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::CloseAllConnectionsCallback callback) {
    impl->CloseAllConnections(std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CloseAllConnections_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CloseAllConnections_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CloseAllConnections_Message::kMessageTag = {};

void NetworkContextProxy::CloseAllConnections(
    CloseAllConnectionsCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CloseAllConnections");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CloseAllConnections_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CloseAllConnections");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_CloseAllConnections_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_CloseIdleConnections_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CloseIdleConnections_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CloseIdleConnections_Name,
          message_flags){}
  ~NetworkContextProxy_CloseIdleConnections_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CloseIdleConnections_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CloseIdleConnections_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CloseIdleConnections_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::CloseIdleConnectionsCallback callback) {
    impl->CloseIdleConnections(std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CloseIdleConnections_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CloseIdleConnections_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CloseIdleConnections_Message::kMessageTag = {};

void NetworkContextProxy::CloseIdleConnections(
    CloseIdleConnectionsCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CloseIdleConnections");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CloseIdleConnections_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CloseIdleConnections");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_CloseIdleConnections_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_SetNetworkConditions_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_SetNetworkConditions_Message(
      uint32_t message_flags
      , const base::UnguessableToken& param_throttling_profile_id
      , NetworkConditionsPtr param_conditions
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_SetNetworkConditions_Name,
          message_flags)
      , param_throttling_profile_id_(std::move(param_throttling_profile_id))
      , param_conditions_(std::move(param_conditions)){}
  ~NetworkContextProxy_SetNetworkConditions_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const base::UnguessableToken& param_throttling_profile_id, NetworkConditionsPtr param_conditions) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_SetNetworkConditions_Message>(
          kFlags
          , std::move(param_throttling_profile_id)
          , std::move(param_conditions)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_SetNetworkConditions_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_SetNetworkConditions_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->throttling_profile_id)::BaseType::BufferWriter
      throttling_profile_id_writer;
  mojo::internal::Serialize<::mojo_base::mojom::UnguessableTokenDataView>(
      param_throttling_profile_id, buffer, &throttling_profile_id_writer, &serialization_context);
  params->throttling_profile_id.Set(
      throttling_profile_id_writer.is_null() ? nullptr : throttling_profile_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->throttling_profile_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null throttling_profile_id in NetworkContext.SetNetworkConditions request");
  typename decltype(params->conditions)::BaseType::BufferWriter
      conditions_writer;
  mojo::internal::Serialize<::network::mojom::NetworkConditionsDataView>(
      param_conditions, buffer, &conditions_writer, &serialization_context);
  params->conditions.Set(
      conditions_writer.is_null() ? nullptr : conditions_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->SetNetworkConditions(
        std::move(param_throttling_profile_id_),
        std::move(param_conditions_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_SetNetworkConditions_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->throttling_profile_id)::BaseType::BufferWriter
      throttling_profile_id_writer;
  mojo::internal::Serialize<::mojo_base::mojom::UnguessableTokenDataView>(
      param_throttling_profile_id_, buffer, &throttling_profile_id_writer, serialization_context);
  params->throttling_profile_id.Set(
      throttling_profile_id_writer.is_null() ? nullptr : throttling_profile_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->throttling_profile_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null throttling_profile_id in NetworkContext.SetNetworkConditions request");
  typename decltype(params->conditions)::BaseType::BufferWriter
      conditions_writer;
  mojo::internal::Serialize<::network::mojom::NetworkConditionsDataView>(
      param_conditions_, buffer, &conditions_writer, serialization_context);
  params->conditions.Set(
      conditions_writer.is_null() ? nullptr : conditions_writer.data());
  }
  base::UnguessableToken param_throttling_profile_id_;
  NetworkConditionsPtr param_conditions_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_SetNetworkConditions_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_SetNetworkConditions_Message::kMessageTag = {};

void NetworkContextProxy::SetNetworkConditions(
    const base::UnguessableToken& in_throttling_profile_id, NetworkConditionsPtr in_conditions) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::SetNetworkConditions");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_SetNetworkConditions_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_throttling_profile_id), std::move(in_conditions));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("SetNetworkConditions");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_SetAcceptLanguage_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_SetAcceptLanguage_Message(
      uint32_t message_flags
      , const std::string& param_new_accept_language
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_SetAcceptLanguage_Name,
          message_flags)
      , param_new_accept_language_(std::move(param_new_accept_language)){}
  ~NetworkContextProxy_SetAcceptLanguage_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const std::string& param_new_accept_language) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_SetAcceptLanguage_Message>(
          kFlags
          , std::move(param_new_accept_language)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_SetAcceptLanguage_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_SetAcceptLanguage_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->new_accept_language)::BaseType::BufferWriter
      new_accept_language_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_new_accept_language, buffer, &new_accept_language_writer, &serialization_context);
  params->new_accept_language.Set(
      new_accept_language_writer.is_null() ? nullptr : new_accept_language_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->new_accept_language.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null new_accept_language in NetworkContext.SetAcceptLanguage request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->SetAcceptLanguage(
        std::move(param_new_accept_language_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_SetAcceptLanguage_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->new_accept_language)::BaseType::BufferWriter
      new_accept_language_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_new_accept_language_, buffer, &new_accept_language_writer, serialization_context);
  params->new_accept_language.Set(
      new_accept_language_writer.is_null() ? nullptr : new_accept_language_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->new_accept_language.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null new_accept_language in NetworkContext.SetAcceptLanguage request");
  }
  std::string param_new_accept_language_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_SetAcceptLanguage_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_SetAcceptLanguage_Message::kMessageTag = {};

void NetworkContextProxy::SetAcceptLanguage(
    const std::string& in_new_accept_language) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::SetAcceptLanguage");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_SetAcceptLanguage_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_new_accept_language));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("SetAcceptLanguage");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_SetEnableReferrers_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_SetEnableReferrers_Message(
      uint32_t message_flags
      , bool param_enable_referrers
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_SetEnableReferrers_Name,
          message_flags)
      , param_enable_referrers_(std::move(param_enable_referrers)){}
  ~NetworkContextProxy_SetEnableReferrers_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      bool param_enable_referrers) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_SetEnableReferrers_Message>(
          kFlags
          , std::move(param_enable_referrers)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_SetEnableReferrers_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_SetEnableReferrers_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->enable_referrers = param_enable_referrers;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->SetEnableReferrers(
        std::move(param_enable_referrers_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_SetEnableReferrers_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->enable_referrers = param_enable_referrers_;
  }
  bool param_enable_referrers_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_SetEnableReferrers_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_SetEnableReferrers_Message::kMessageTag = {};

void NetworkContextProxy::SetEnableReferrers(
    bool in_enable_referrers) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::SetEnableReferrers");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_SetEnableReferrers_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_enable_referrers));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("SetEnableReferrers");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_SetCTPolicy_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_SetCTPolicy_Message(
      uint32_t message_flags
      , const std::vector<std::string>& param_required_hosts
      , const std::vector<std::string>& param_excluded_hosts
      , const std::vector<std::string>& param_excluded_spkis
      , const std::vector<std::string>& param_excluded_legacy_spkis
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_SetCTPolicy_Name,
          message_flags)
      , param_required_hosts_(std::move(param_required_hosts))
      , param_excluded_hosts_(std::move(param_excluded_hosts))
      , param_excluded_spkis_(std::move(param_excluded_spkis))
      , param_excluded_legacy_spkis_(std::move(param_excluded_legacy_spkis)){}
  ~NetworkContextProxy_SetCTPolicy_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const std::vector<std::string>& param_required_hosts, const std::vector<std::string>& param_excluded_hosts, const std::vector<std::string>& param_excluded_spkis, const std::vector<std::string>& param_excluded_legacy_spkis) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_SetCTPolicy_Message>(
          kFlags
          , std::move(param_required_hosts)
          , std::move(param_excluded_hosts)
          , std::move(param_excluded_spkis)
          , std::move(param_excluded_legacy_spkis)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_SetCTPolicy_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_SetCTPolicy_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->required_hosts)::BaseType::BufferWriter
      required_hosts_writer;
  const mojo::internal::ContainerValidateParams required_hosts_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_required_hosts, buffer, &required_hosts_writer, &required_hosts_validate_params,
      &serialization_context);
  params->required_hosts.Set(
      required_hosts_writer.is_null() ? nullptr : required_hosts_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->required_hosts.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null required_hosts in NetworkContext.SetCTPolicy request");
  typename decltype(params->excluded_hosts)::BaseType::BufferWriter
      excluded_hosts_writer;
  const mojo::internal::ContainerValidateParams excluded_hosts_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_excluded_hosts, buffer, &excluded_hosts_writer, &excluded_hosts_validate_params,
      &serialization_context);
  params->excluded_hosts.Set(
      excluded_hosts_writer.is_null() ? nullptr : excluded_hosts_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->excluded_hosts.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null excluded_hosts in NetworkContext.SetCTPolicy request");
  typename decltype(params->excluded_spkis)::BaseType::BufferWriter
      excluded_spkis_writer;
  const mojo::internal::ContainerValidateParams excluded_spkis_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_excluded_spkis, buffer, &excluded_spkis_writer, &excluded_spkis_validate_params,
      &serialization_context);
  params->excluded_spkis.Set(
      excluded_spkis_writer.is_null() ? nullptr : excluded_spkis_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->excluded_spkis.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null excluded_spkis in NetworkContext.SetCTPolicy request");
  typename decltype(params->excluded_legacy_spkis)::BaseType::BufferWriter
      excluded_legacy_spkis_writer;
  const mojo::internal::ContainerValidateParams excluded_legacy_spkis_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_excluded_legacy_spkis, buffer, &excluded_legacy_spkis_writer, &excluded_legacy_spkis_validate_params,
      &serialization_context);
  params->excluded_legacy_spkis.Set(
      excluded_legacy_spkis_writer.is_null() ? nullptr : excluded_legacy_spkis_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->excluded_legacy_spkis.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null excluded_legacy_spkis in NetworkContext.SetCTPolicy request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->SetCTPolicy(
        std::move(param_required_hosts_),
        std::move(param_excluded_hosts_),
        std::move(param_excluded_spkis_),
        std::move(param_excluded_legacy_spkis_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_SetCTPolicy_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->required_hosts)::BaseType::BufferWriter
      required_hosts_writer;
  const mojo::internal::ContainerValidateParams required_hosts_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_required_hosts_, buffer, &required_hosts_writer, &required_hosts_validate_params,
      serialization_context);
  params->required_hosts.Set(
      required_hosts_writer.is_null() ? nullptr : required_hosts_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->required_hosts.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null required_hosts in NetworkContext.SetCTPolicy request");
  typename decltype(params->excluded_hosts)::BaseType::BufferWriter
      excluded_hosts_writer;
  const mojo::internal::ContainerValidateParams excluded_hosts_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_excluded_hosts_, buffer, &excluded_hosts_writer, &excluded_hosts_validate_params,
      serialization_context);
  params->excluded_hosts.Set(
      excluded_hosts_writer.is_null() ? nullptr : excluded_hosts_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->excluded_hosts.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null excluded_hosts in NetworkContext.SetCTPolicy request");
  typename decltype(params->excluded_spkis)::BaseType::BufferWriter
      excluded_spkis_writer;
  const mojo::internal::ContainerValidateParams excluded_spkis_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_excluded_spkis_, buffer, &excluded_spkis_writer, &excluded_spkis_validate_params,
      serialization_context);
  params->excluded_spkis.Set(
      excluded_spkis_writer.is_null() ? nullptr : excluded_spkis_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->excluded_spkis.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null excluded_spkis in NetworkContext.SetCTPolicy request");
  typename decltype(params->excluded_legacy_spkis)::BaseType::BufferWriter
      excluded_legacy_spkis_writer;
  const mojo::internal::ContainerValidateParams excluded_legacy_spkis_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_excluded_legacy_spkis_, buffer, &excluded_legacy_spkis_writer, &excluded_legacy_spkis_validate_params,
      serialization_context);
  params->excluded_legacy_spkis.Set(
      excluded_legacy_spkis_writer.is_null() ? nullptr : excluded_legacy_spkis_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->excluded_legacy_spkis.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null excluded_legacy_spkis in NetworkContext.SetCTPolicy request");
  }
  std::vector<std::string> param_required_hosts_;
  std::vector<std::string> param_excluded_hosts_;
  std::vector<std::string> param_excluded_spkis_;
  std::vector<std::string> param_excluded_legacy_spkis_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_SetCTPolicy_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_SetCTPolicy_Message::kMessageTag = {};

void NetworkContextProxy::SetCTPolicy(
    const std::vector<std::string>& in_required_hosts, const std::vector<std::string>& in_excluded_hosts, const std::vector<std::string>& in_excluded_spkis, const std::vector<std::string>& in_excluded_legacy_spkis) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::SetCTPolicy");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_SetCTPolicy_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_required_hosts), std::move(in_excluded_hosts), std::move(in_excluded_spkis), std::move(in_excluded_legacy_spkis));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("SetCTPolicy");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_AddExpectCT_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_AddExpectCT_Message(
      uint32_t message_flags
      , const std::string& param_host
      , base::Time param_expiry
      , bool param_enforce
      , const GURL& param_report_uri
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_AddExpectCT_Name,
          message_flags)
      , param_host_(std::move(param_host))
      , param_expiry_(std::move(param_expiry))
      , param_enforce_(std::move(param_enforce))
      , param_report_uri_(std::move(param_report_uri)){}
  ~NetworkContextProxy_AddExpectCT_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const std::string& param_host, base::Time param_expiry, bool param_enforce, const GURL& param_report_uri) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_AddExpectCT_Message>(
          kFlags
          , std::move(param_host)
          , std::move(param_expiry)
          , std::move(param_enforce)
          , std::move(param_report_uri)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_AddExpectCT_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_AddExpectCT_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->host)::BaseType::BufferWriter
      host_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_host, buffer, &host_writer, &serialization_context);
  params->host.Set(
      host_writer.is_null() ? nullptr : host_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->host.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null host in NetworkContext.AddExpectCT request");
  typename decltype(params->expiry)::BaseType::BufferWriter
      expiry_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_expiry, buffer, &expiry_writer, &serialization_context);
  params->expiry.Set(
      expiry_writer.is_null() ? nullptr : expiry_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->expiry.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null expiry in NetworkContext.AddExpectCT request");
  params->enforce = param_enforce;
  typename decltype(params->report_uri)::BaseType::BufferWriter
      report_uri_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_report_uri, buffer, &report_uri_writer, &serialization_context);
  params->report_uri.Set(
      report_uri_writer.is_null() ? nullptr : report_uri_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->report_uri.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null report_uri in NetworkContext.AddExpectCT request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::AddExpectCTCallback callback) {
    impl->AddExpectCT(
        std::move(param_host_),
        std::move(param_expiry_),
        std::move(param_enforce_),
        std::move(param_report_uri_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_AddExpectCT_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->host)::BaseType::BufferWriter
      host_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_host_, buffer, &host_writer, serialization_context);
  params->host.Set(
      host_writer.is_null() ? nullptr : host_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->host.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null host in NetworkContext.AddExpectCT request");
  typename decltype(params->expiry)::BaseType::BufferWriter
      expiry_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_expiry_, buffer, &expiry_writer, serialization_context);
  params->expiry.Set(
      expiry_writer.is_null() ? nullptr : expiry_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->expiry.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null expiry in NetworkContext.AddExpectCT request");
  params->enforce = param_enforce_;
  typename decltype(params->report_uri)::BaseType::BufferWriter
      report_uri_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_report_uri_, buffer, &report_uri_writer, serialization_context);
  params->report_uri.Set(
      report_uri_writer.is_null() ? nullptr : report_uri_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->report_uri.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null report_uri in NetworkContext.AddExpectCT request");
  }
  std::string param_host_;
  base::Time param_expiry_;
  bool param_enforce_;
  GURL param_report_uri_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_AddExpectCT_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_AddExpectCT_Message::kMessageTag = {};

void NetworkContextProxy::AddExpectCT(
    const std::string& in_host, base::Time in_expiry, bool in_enforce, const GURL& in_report_uri, AddExpectCTCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::AddExpectCT");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_AddExpectCT_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_host), std::move(in_expiry), std::move(in_enforce), std::move(in_report_uri));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("AddExpectCT");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_AddExpectCT_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_SetExpectCTTestReport_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_SetExpectCTTestReport_Message(
      uint32_t message_flags
      , const GURL& param_report_uri
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_SetExpectCTTestReport_Name,
          message_flags)
      , param_report_uri_(std::move(param_report_uri)){}
  ~NetworkContextProxy_SetExpectCTTestReport_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const GURL& param_report_uri) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_SetExpectCTTestReport_Message>(
          kFlags
          , std::move(param_report_uri)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_SetExpectCTTestReport_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_SetExpectCTTestReport_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->report_uri)::BaseType::BufferWriter
      report_uri_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_report_uri, buffer, &report_uri_writer, &serialization_context);
  params->report_uri.Set(
      report_uri_writer.is_null() ? nullptr : report_uri_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->report_uri.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null report_uri in NetworkContext.SetExpectCTTestReport request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::SetExpectCTTestReportCallback callback) {
    impl->SetExpectCTTestReport(
        std::move(param_report_uri_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_SetExpectCTTestReport_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->report_uri)::BaseType::BufferWriter
      report_uri_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_report_uri_, buffer, &report_uri_writer, serialization_context);
  params->report_uri.Set(
      report_uri_writer.is_null() ? nullptr : report_uri_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->report_uri.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null report_uri in NetworkContext.SetExpectCTTestReport request");
  }
  GURL param_report_uri_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_SetExpectCTTestReport_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_SetExpectCTTestReport_Message::kMessageTag = {};

void NetworkContextProxy::SetExpectCTTestReport(
    const GURL& in_report_uri, SetExpectCTTestReportCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::SetExpectCTTestReport");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_SetExpectCTTestReport_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_report_uri));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("SetExpectCTTestReport");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_SetExpectCTTestReport_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_GetExpectCTState_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_GetExpectCTState_Message(
      uint32_t message_flags
      , const std::string& param_domain
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_GetExpectCTState_Name,
          message_flags)
      , param_domain_(std::move(param_domain)){}
  ~NetworkContextProxy_GetExpectCTState_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const std::string& param_domain) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_GetExpectCTState_Message>(
          kFlags
          , std::move(param_domain)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_GetExpectCTState_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_GetExpectCTState_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->domain)::BaseType::BufferWriter
      domain_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_domain, buffer, &domain_writer, &serialization_context);
  params->domain.Set(
      domain_writer.is_null() ? nullptr : domain_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->domain.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null domain in NetworkContext.GetExpectCTState request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::GetExpectCTStateCallback callback) {
    impl->GetExpectCTState(
        std::move(param_domain_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_GetExpectCTState_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->domain)::BaseType::BufferWriter
      domain_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_domain_, buffer, &domain_writer, serialization_context);
  params->domain.Set(
      domain_writer.is_null() ? nullptr : domain_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->domain.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null domain in NetworkContext.GetExpectCTState request");
  }
  std::string param_domain_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_GetExpectCTState_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_GetExpectCTState_Message::kMessageTag = {};

void NetworkContextProxy::GetExpectCTState(
    const std::string& in_domain, GetExpectCTStateCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::GetExpectCTState");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_GetExpectCTState_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_domain));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("GetExpectCTState");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_GetExpectCTState_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_CreateUDPSocket_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateUDPSocket_Message(
      uint32_t message_flags
      , ::network::mojom::UDPSocketRequest param_request
      , ::network::mojom::UDPSocketReceiverPtr param_receiver
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateUDPSocket_Name,
          message_flags)
      , param_request_(std::move(param_request))
      , param_receiver_(param_receiver.PassInterface()){}
  ~NetworkContextProxy_CreateUDPSocket_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::UDPSocketRequest param_request, ::network::mojom::UDPSocketReceiverPtr param_receiver) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateUDPSocket_Message>(
          kFlags
          , std::move(param_request)
          , std::move(param_receiver)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateUDPSocket_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateUDPSocket_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::UDPSocketRequestDataView>(
      param_request, &params->request, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid request in NetworkContext.CreateUDPSocket request");
  mojo::internal::Serialize<::network::mojom::UDPSocketReceiverPtrDataView>(
      param_receiver, &params->receiver, &serialization_context);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->CreateUDPSocket(
        std::move(param_request_),
        ::network::mojom::UDPSocketReceiverPtr(std::move(param_receiver_)));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateUDPSocket_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::UDPSocketRequestDataView>(
      param_request_, &params->request, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid request in NetworkContext.CreateUDPSocket request");
  mojo::internal::Serialize<::network::mojom::UDPSocketReceiverPtrDataView>(
      param_receiver_, &params->receiver, serialization_context);
  }
  ::network::mojom::UDPSocketRequest param_request_;
  ::network::mojom::UDPSocketReceiverPtrInfo param_receiver_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateUDPSocket_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateUDPSocket_Message::kMessageTag = {};

void NetworkContextProxy::CreateUDPSocket(
    ::network::mojom::UDPSocketRequest in_request, ::network::mojom::UDPSocketReceiverPtr in_receiver) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateUDPSocket");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateUDPSocket_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_request), std::move(in_receiver));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CreateUDPSocket");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_CreateTCPServerSocket_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateTCPServerSocket_Message(
      uint32_t message_flags
      , const net::IPEndPoint& param_local_addr
      , uint32_t param_backlog
      , const net::MutableNetworkTrafficAnnotationTag& param_traffic_annotation
      , ::network::mojom::TCPServerSocketRequest param_socket
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateTCPServerSocket_Name,
          message_flags)
      , param_local_addr_(std::move(param_local_addr))
      , param_backlog_(std::move(param_backlog))
      , param_traffic_annotation_(std::move(param_traffic_annotation))
      , param_socket_(std::move(param_socket)){}
  ~NetworkContextProxy_CreateTCPServerSocket_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const net::IPEndPoint& param_local_addr, uint32_t param_backlog, const net::MutableNetworkTrafficAnnotationTag& param_traffic_annotation, ::network::mojom::TCPServerSocketRequest param_socket) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateTCPServerSocket_Message>(
          kFlags
          , std::move(param_local_addr)
          , std::move(param_backlog)
          , std::move(param_traffic_annotation)
          , std::move(param_socket)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateTCPServerSocket_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateTCPServerSocket_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->local_addr)::BaseType::BufferWriter
      local_addr_writer;
  mojo::internal::Serialize<::network::mojom::IPEndPointDataView>(
      param_local_addr, buffer, &local_addr_writer, &serialization_context);
  params->local_addr.Set(
      local_addr_writer.is_null() ? nullptr : local_addr_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->local_addr.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null local_addr in NetworkContext.CreateTCPServerSocket request");
  params->backlog = param_backlog;
  typename decltype(params->traffic_annotation)::BaseType::BufferWriter
      traffic_annotation_writer;
  mojo::internal::Serialize<::network::mojom::MutableNetworkTrafficAnnotationTagDataView>(
      param_traffic_annotation, buffer, &traffic_annotation_writer, &serialization_context);
  params->traffic_annotation.Set(
      traffic_annotation_writer.is_null() ? nullptr : traffic_annotation_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->traffic_annotation.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null traffic_annotation in NetworkContext.CreateTCPServerSocket request");
  mojo::internal::Serialize<::network::mojom::TCPServerSocketRequestDataView>(
      param_socket, &params->socket, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->socket),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid socket in NetworkContext.CreateTCPServerSocket request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::CreateTCPServerSocketCallback callback) {
    impl->CreateTCPServerSocket(
        std::move(param_local_addr_),
        std::move(param_backlog_),
        std::move(param_traffic_annotation_),
        std::move(param_socket_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateTCPServerSocket_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->local_addr)::BaseType::BufferWriter
      local_addr_writer;
  mojo::internal::Serialize<::network::mojom::IPEndPointDataView>(
      param_local_addr_, buffer, &local_addr_writer, serialization_context);
  params->local_addr.Set(
      local_addr_writer.is_null() ? nullptr : local_addr_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->local_addr.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null local_addr in NetworkContext.CreateTCPServerSocket request");
  params->backlog = param_backlog_;
  typename decltype(params->traffic_annotation)::BaseType::BufferWriter
      traffic_annotation_writer;
  mojo::internal::Serialize<::network::mojom::MutableNetworkTrafficAnnotationTagDataView>(
      param_traffic_annotation_, buffer, &traffic_annotation_writer, serialization_context);
  params->traffic_annotation.Set(
      traffic_annotation_writer.is_null() ? nullptr : traffic_annotation_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->traffic_annotation.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null traffic_annotation in NetworkContext.CreateTCPServerSocket request");
  mojo::internal::Serialize<::network::mojom::TCPServerSocketRequestDataView>(
      param_socket_, &params->socket, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->socket),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid socket in NetworkContext.CreateTCPServerSocket request");
  }
  net::IPEndPoint param_local_addr_;
  uint32_t param_backlog_;
  net::MutableNetworkTrafficAnnotationTag param_traffic_annotation_;
  ::network::mojom::TCPServerSocketRequest param_socket_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateTCPServerSocket_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateTCPServerSocket_Message::kMessageTag = {};

void NetworkContextProxy::CreateTCPServerSocket(
    const net::IPEndPoint& in_local_addr, uint32_t in_backlog, const net::MutableNetworkTrafficAnnotationTag& in_traffic_annotation, ::network::mojom::TCPServerSocketRequest in_socket, CreateTCPServerSocketCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateTCPServerSocket");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateTCPServerSocket_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_local_addr), std::move(in_backlog), std::move(in_traffic_annotation), std::move(in_socket));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CreateTCPServerSocket");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_CreateTCPServerSocket_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_CreateTCPConnectedSocket_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateTCPConnectedSocket_Message(
      uint32_t message_flags
      , const base::Optional<net::IPEndPoint>& param_local_addr
      , const net::AddressList& param_remote_addr_list
      , ::network::mojom::TCPConnectedSocketOptionsPtr param_tcp_connected_socket_options
      , const net::MutableNetworkTrafficAnnotationTag& param_traffic_annotation
      , ::network::mojom::TCPConnectedSocketRequest param_socket
      , ::network::mojom::SocketObserverPtr param_observer
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateTCPConnectedSocket_Name,
          message_flags)
      , param_local_addr_(std::move(param_local_addr))
      , param_remote_addr_list_(std::move(param_remote_addr_list))
      , param_tcp_connected_socket_options_(std::move(param_tcp_connected_socket_options))
      , param_traffic_annotation_(std::move(param_traffic_annotation))
      , param_socket_(std::move(param_socket))
      , param_observer_(param_observer.PassInterface()){}
  ~NetworkContextProxy_CreateTCPConnectedSocket_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const base::Optional<net::IPEndPoint>& param_local_addr, const net::AddressList& param_remote_addr_list, ::network::mojom::TCPConnectedSocketOptionsPtr param_tcp_connected_socket_options, const net::MutableNetworkTrafficAnnotationTag& param_traffic_annotation, ::network::mojom::TCPConnectedSocketRequest param_socket, ::network::mojom::SocketObserverPtr param_observer) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateTCPConnectedSocket_Message>(
          kFlags
          , std::move(param_local_addr)
          , std::move(param_remote_addr_list)
          , std::move(param_tcp_connected_socket_options)
          , std::move(param_traffic_annotation)
          , std::move(param_socket)
          , std::move(param_observer)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateTCPConnectedSocket_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateTCPConnectedSocket_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->local_addr)::BaseType::BufferWriter
      local_addr_writer;
  mojo::internal::Serialize<::network::mojom::IPEndPointDataView>(
      param_local_addr, buffer, &local_addr_writer, &serialization_context);
  params->local_addr.Set(
      local_addr_writer.is_null() ? nullptr : local_addr_writer.data());
  typename decltype(params->remote_addr_list)::BaseType::BufferWriter
      remote_addr_list_writer;
  mojo::internal::Serialize<::network::mojom::AddressListDataView>(
      param_remote_addr_list, buffer, &remote_addr_list_writer, &serialization_context);
  params->remote_addr_list.Set(
      remote_addr_list_writer.is_null() ? nullptr : remote_addr_list_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->remote_addr_list.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null remote_addr_list in NetworkContext.CreateTCPConnectedSocket request");
  typename decltype(params->tcp_connected_socket_options)::BaseType::BufferWriter
      tcp_connected_socket_options_writer;
  mojo::internal::Serialize<::network::mojom::TCPConnectedSocketOptionsDataView>(
      param_tcp_connected_socket_options, buffer, &tcp_connected_socket_options_writer, &serialization_context);
  params->tcp_connected_socket_options.Set(
      tcp_connected_socket_options_writer.is_null() ? nullptr : tcp_connected_socket_options_writer.data());
  typename decltype(params->traffic_annotation)::BaseType::BufferWriter
      traffic_annotation_writer;
  mojo::internal::Serialize<::network::mojom::MutableNetworkTrafficAnnotationTagDataView>(
      param_traffic_annotation, buffer, &traffic_annotation_writer, &serialization_context);
  params->traffic_annotation.Set(
      traffic_annotation_writer.is_null() ? nullptr : traffic_annotation_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->traffic_annotation.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null traffic_annotation in NetworkContext.CreateTCPConnectedSocket request");
  mojo::internal::Serialize<::network::mojom::TCPConnectedSocketRequestDataView>(
      param_socket, &params->socket, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->socket),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid socket in NetworkContext.CreateTCPConnectedSocket request");
  mojo::internal::Serialize<::network::mojom::SocketObserverPtrDataView>(
      param_observer, &params->observer, &serialization_context);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::CreateTCPConnectedSocketCallback callback) {
    impl->CreateTCPConnectedSocket(
        std::move(param_local_addr_),
        std::move(param_remote_addr_list_),
        std::move(param_tcp_connected_socket_options_),
        std::move(param_traffic_annotation_),
        std::move(param_socket_),
        ::network::mojom::SocketObserverPtr(std::move(param_observer_)), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateTCPConnectedSocket_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->local_addr)::BaseType::BufferWriter
      local_addr_writer;
  mojo::internal::Serialize<::network::mojom::IPEndPointDataView>(
      param_local_addr_, buffer, &local_addr_writer, serialization_context);
  params->local_addr.Set(
      local_addr_writer.is_null() ? nullptr : local_addr_writer.data());
  typename decltype(params->remote_addr_list)::BaseType::BufferWriter
      remote_addr_list_writer;
  mojo::internal::Serialize<::network::mojom::AddressListDataView>(
      param_remote_addr_list_, buffer, &remote_addr_list_writer, serialization_context);
  params->remote_addr_list.Set(
      remote_addr_list_writer.is_null() ? nullptr : remote_addr_list_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->remote_addr_list.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null remote_addr_list in NetworkContext.CreateTCPConnectedSocket request");
  typename decltype(params->tcp_connected_socket_options)::BaseType::BufferWriter
      tcp_connected_socket_options_writer;
  mojo::internal::Serialize<::network::mojom::TCPConnectedSocketOptionsDataView>(
      param_tcp_connected_socket_options_, buffer, &tcp_connected_socket_options_writer, serialization_context);
  params->tcp_connected_socket_options.Set(
      tcp_connected_socket_options_writer.is_null() ? nullptr : tcp_connected_socket_options_writer.data());
  typename decltype(params->traffic_annotation)::BaseType::BufferWriter
      traffic_annotation_writer;
  mojo::internal::Serialize<::network::mojom::MutableNetworkTrafficAnnotationTagDataView>(
      param_traffic_annotation_, buffer, &traffic_annotation_writer, serialization_context);
  params->traffic_annotation.Set(
      traffic_annotation_writer.is_null() ? nullptr : traffic_annotation_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->traffic_annotation.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null traffic_annotation in NetworkContext.CreateTCPConnectedSocket request");
  mojo::internal::Serialize<::network::mojom::TCPConnectedSocketRequestDataView>(
      param_socket_, &params->socket, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->socket),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid socket in NetworkContext.CreateTCPConnectedSocket request");
  mojo::internal::Serialize<::network::mojom::SocketObserverPtrDataView>(
      param_observer_, &params->observer, serialization_context);
  }
  base::Optional<net::IPEndPoint> param_local_addr_;
  net::AddressList param_remote_addr_list_;
  ::network::mojom::TCPConnectedSocketOptionsPtr param_tcp_connected_socket_options_;
  net::MutableNetworkTrafficAnnotationTag param_traffic_annotation_;
  ::network::mojom::TCPConnectedSocketRequest param_socket_;
  ::network::mojom::SocketObserverPtrInfo param_observer_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateTCPConnectedSocket_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateTCPConnectedSocket_Message::kMessageTag = {};

void NetworkContextProxy::CreateTCPConnectedSocket(
    const base::Optional<net::IPEndPoint>& in_local_addr, const net::AddressList& in_remote_addr_list, ::network::mojom::TCPConnectedSocketOptionsPtr in_tcp_connected_socket_options, const net::MutableNetworkTrafficAnnotationTag& in_traffic_annotation, ::network::mojom::TCPConnectedSocketRequest in_socket, ::network::mojom::SocketObserverPtr in_observer, CreateTCPConnectedSocketCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateTCPConnectedSocket");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateTCPConnectedSocket_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_local_addr), std::move(in_remote_addr_list), std::move(in_tcp_connected_socket_options), std::move(in_traffic_annotation), std::move(in_socket), std::move(in_observer));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CreateTCPConnectedSocket");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_CreateTCPConnectedSocket_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_CreateTCPBoundSocket_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateTCPBoundSocket_Message(
      uint32_t message_flags
      , const net::IPEndPoint& param_local_addr
      , const net::MutableNetworkTrafficAnnotationTag& param_traffic_annotation
      , ::network::mojom::TCPBoundSocketRequest param_socket
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateTCPBoundSocket_Name,
          message_flags)
      , param_local_addr_(std::move(param_local_addr))
      , param_traffic_annotation_(std::move(param_traffic_annotation))
      , param_socket_(std::move(param_socket)){}
  ~NetworkContextProxy_CreateTCPBoundSocket_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const net::IPEndPoint& param_local_addr, const net::MutableNetworkTrafficAnnotationTag& param_traffic_annotation, ::network::mojom::TCPBoundSocketRequest param_socket) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateTCPBoundSocket_Message>(
          kFlags
          , std::move(param_local_addr)
          , std::move(param_traffic_annotation)
          , std::move(param_socket)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateTCPBoundSocket_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateTCPBoundSocket_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->local_addr)::BaseType::BufferWriter
      local_addr_writer;
  mojo::internal::Serialize<::network::mojom::IPEndPointDataView>(
      param_local_addr, buffer, &local_addr_writer, &serialization_context);
  params->local_addr.Set(
      local_addr_writer.is_null() ? nullptr : local_addr_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->local_addr.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null local_addr in NetworkContext.CreateTCPBoundSocket request");
  typename decltype(params->traffic_annotation)::BaseType::BufferWriter
      traffic_annotation_writer;
  mojo::internal::Serialize<::network::mojom::MutableNetworkTrafficAnnotationTagDataView>(
      param_traffic_annotation, buffer, &traffic_annotation_writer, &serialization_context);
  params->traffic_annotation.Set(
      traffic_annotation_writer.is_null() ? nullptr : traffic_annotation_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->traffic_annotation.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null traffic_annotation in NetworkContext.CreateTCPBoundSocket request");
  mojo::internal::Serialize<::network::mojom::TCPBoundSocketRequestDataView>(
      param_socket, &params->socket, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->socket),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid socket in NetworkContext.CreateTCPBoundSocket request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::CreateTCPBoundSocketCallback callback) {
    impl->CreateTCPBoundSocket(
        std::move(param_local_addr_),
        std::move(param_traffic_annotation_),
        std::move(param_socket_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateTCPBoundSocket_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->local_addr)::BaseType::BufferWriter
      local_addr_writer;
  mojo::internal::Serialize<::network::mojom::IPEndPointDataView>(
      param_local_addr_, buffer, &local_addr_writer, serialization_context);
  params->local_addr.Set(
      local_addr_writer.is_null() ? nullptr : local_addr_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->local_addr.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null local_addr in NetworkContext.CreateTCPBoundSocket request");
  typename decltype(params->traffic_annotation)::BaseType::BufferWriter
      traffic_annotation_writer;
  mojo::internal::Serialize<::network::mojom::MutableNetworkTrafficAnnotationTagDataView>(
      param_traffic_annotation_, buffer, &traffic_annotation_writer, serialization_context);
  params->traffic_annotation.Set(
      traffic_annotation_writer.is_null() ? nullptr : traffic_annotation_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->traffic_annotation.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null traffic_annotation in NetworkContext.CreateTCPBoundSocket request");
  mojo::internal::Serialize<::network::mojom::TCPBoundSocketRequestDataView>(
      param_socket_, &params->socket, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->socket),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid socket in NetworkContext.CreateTCPBoundSocket request");
  }
  net::IPEndPoint param_local_addr_;
  net::MutableNetworkTrafficAnnotationTag param_traffic_annotation_;
  ::network::mojom::TCPBoundSocketRequest param_socket_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateTCPBoundSocket_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateTCPBoundSocket_Message::kMessageTag = {};

void NetworkContextProxy::CreateTCPBoundSocket(
    const net::IPEndPoint& in_local_addr, const net::MutableNetworkTrafficAnnotationTag& in_traffic_annotation, ::network::mojom::TCPBoundSocketRequest in_socket, CreateTCPBoundSocketCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateTCPBoundSocket");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateTCPBoundSocket_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_local_addr), std::move(in_traffic_annotation), std::move(in_socket));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CreateTCPBoundSocket");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_CreateTCPBoundSocket_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_CreateProxyResolvingSocketFactory_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateProxyResolvingSocketFactory_Message(
      uint32_t message_flags
      , ::network::mojom::ProxyResolvingSocketFactoryRequest param_factory
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateProxyResolvingSocketFactory_Name,
          message_flags)
      , param_factory_(std::move(param_factory)){}
  ~NetworkContextProxy_CreateProxyResolvingSocketFactory_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::ProxyResolvingSocketFactoryRequest param_factory) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateProxyResolvingSocketFactory_Message>(
          kFlags
          , std::move(param_factory)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateProxyResolvingSocketFactory_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateProxyResolvingSocketFactory_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::ProxyResolvingSocketFactoryRequestDataView>(
      param_factory, &params->factory, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->factory),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid factory in NetworkContext.CreateProxyResolvingSocketFactory request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->CreateProxyResolvingSocketFactory(
        std::move(param_factory_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateProxyResolvingSocketFactory_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::ProxyResolvingSocketFactoryRequestDataView>(
      param_factory_, &params->factory, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->factory),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid factory in NetworkContext.CreateProxyResolvingSocketFactory request");
  }
  ::network::mojom::ProxyResolvingSocketFactoryRequest param_factory_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateProxyResolvingSocketFactory_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateProxyResolvingSocketFactory_Message::kMessageTag = {};

void NetworkContextProxy::CreateProxyResolvingSocketFactory(
    ::network::mojom::ProxyResolvingSocketFactoryRequest in_factory) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateProxyResolvingSocketFactory");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateProxyResolvingSocketFactory_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_factory));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CreateProxyResolvingSocketFactory");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_LookUpProxyForURL_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_LookUpProxyForURL_Message(
      uint32_t message_flags
      , const GURL& param_url
      , ::network::mojom::ProxyLookupClientPtr param_proxy_lookup_client
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_LookUpProxyForURL_Name,
          message_flags)
      , param_url_(std::move(param_url))
      , param_proxy_lookup_client_(param_proxy_lookup_client.PassInterface()){}
  ~NetworkContextProxy_LookUpProxyForURL_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const GURL& param_url, ::network::mojom::ProxyLookupClientPtr param_proxy_lookup_client) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_LookUpProxyForURL_Message>(
          kFlags
          , std::move(param_url)
          , std::move(param_proxy_lookup_client)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_LookUpProxyForURL_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_LookUpProxyForURL_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url, buffer, &url_writer, &serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkContext.LookUpProxyForURL request");
  mojo::internal::Serialize<::network::mojom::ProxyLookupClientPtrDataView>(
      param_proxy_lookup_client, &params->proxy_lookup_client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->proxy_lookup_client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid proxy_lookup_client in NetworkContext.LookUpProxyForURL request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->LookUpProxyForURL(
        std::move(param_url_),
        ::network::mojom::ProxyLookupClientPtr(std::move(param_proxy_lookup_client_)));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_LookUpProxyForURL_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url_, buffer, &url_writer, serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkContext.LookUpProxyForURL request");
  mojo::internal::Serialize<::network::mojom::ProxyLookupClientPtrDataView>(
      param_proxy_lookup_client_, &params->proxy_lookup_client, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->proxy_lookup_client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid proxy_lookup_client in NetworkContext.LookUpProxyForURL request");
  }
  GURL param_url_;
  ::network::mojom::ProxyLookupClientPtrInfo param_proxy_lookup_client_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_LookUpProxyForURL_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_LookUpProxyForURL_Message::kMessageTag = {};

void NetworkContextProxy::LookUpProxyForURL(
    const GURL& in_url, ::network::mojom::ProxyLookupClientPtr in_proxy_lookup_client) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::LookUpProxyForURL");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_LookUpProxyForURL_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_url), std::move(in_proxy_lookup_client));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("LookUpProxyForURL");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_ForceReloadProxyConfig_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ForceReloadProxyConfig_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ForceReloadProxyConfig_Name,
          message_flags){}
  ~NetworkContextProxy_ForceReloadProxyConfig_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ForceReloadProxyConfig_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ForceReloadProxyConfig_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ForceReloadProxyConfig_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ForceReloadProxyConfigCallback callback) {
    impl->ForceReloadProxyConfig(std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ForceReloadProxyConfig_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ForceReloadProxyConfig_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ForceReloadProxyConfig_Message::kMessageTag = {};

void NetworkContextProxy::ForceReloadProxyConfig(
    ForceReloadProxyConfigCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ForceReloadProxyConfig");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ForceReloadProxyConfig_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ForceReloadProxyConfig");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ForceReloadProxyConfig_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_ClearBadProxiesCache_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ClearBadProxiesCache_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearBadProxiesCache_Name,
          message_flags){}
  ~NetworkContextProxy_ClearBadProxiesCache_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ClearBadProxiesCache_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearBadProxiesCache_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearBadProxiesCache_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ClearBadProxiesCacheCallback callback) {
    impl->ClearBadProxiesCache(std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearBadProxiesCache_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ClearBadProxiesCache_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ClearBadProxiesCache_Message::kMessageTag = {};

void NetworkContextProxy::ClearBadProxiesCache(
    ClearBadProxiesCacheCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ClearBadProxiesCache");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ClearBadProxiesCache_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearBadProxiesCache");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ClearBadProxiesCache_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_CreateWebSocket_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateWebSocket_Message(
      uint32_t message_flags
      , ::network::mojom::WebSocketRequest param_request
      , int32_t param_process_id
      , int32_t param_render_frame_id
      , const url::Origin& param_origin
      , ::network::mojom::AuthenticationHandlerPtr param_auth_handler
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateWebSocket_Name,
          message_flags)
      , param_request_(std::move(param_request))
      , param_process_id_(std::move(param_process_id))
      , param_render_frame_id_(std::move(param_render_frame_id))
      , param_origin_(std::move(param_origin))
      , param_auth_handler_(param_auth_handler.PassInterface()){}
  ~NetworkContextProxy_CreateWebSocket_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::WebSocketRequest param_request, int32_t param_process_id, int32_t param_render_frame_id, const url::Origin& param_origin, ::network::mojom::AuthenticationHandlerPtr param_auth_handler) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateWebSocket_Message>(
          kFlags
          , std::move(param_request)
          , std::move(param_process_id)
          , std::move(param_render_frame_id)
          , std::move(param_origin)
          , std::move(param_auth_handler)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateWebSocket_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateWebSocket_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::WebSocketRequestDataView>(
      param_request, &params->request, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid request in NetworkContext.CreateWebSocket request");
  params->process_id = param_process_id;
  params->render_frame_id = param_render_frame_id;
  typename decltype(params->origin)::BaseType::BufferWriter
      origin_writer;
  mojo::internal::Serialize<::url::mojom::OriginDataView>(
      param_origin, buffer, &origin_writer, &serialization_context);
  params->origin.Set(
      origin_writer.is_null() ? nullptr : origin_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in NetworkContext.CreateWebSocket request");
  mojo::internal::Serialize<::network::mojom::AuthenticationHandlerPtrDataView>(
      param_auth_handler, &params->auth_handler, &serialization_context);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->CreateWebSocket(
        std::move(param_request_),
        std::move(param_process_id_),
        std::move(param_render_frame_id_),
        std::move(param_origin_),
        ::network::mojom::AuthenticationHandlerPtr(std::move(param_auth_handler_)));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateWebSocket_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::WebSocketRequestDataView>(
      param_request_, &params->request, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid request in NetworkContext.CreateWebSocket request");
  params->process_id = param_process_id_;
  params->render_frame_id = param_render_frame_id_;
  typename decltype(params->origin)::BaseType::BufferWriter
      origin_writer;
  mojo::internal::Serialize<::url::mojom::OriginDataView>(
      param_origin_, buffer, &origin_writer, serialization_context);
  params->origin.Set(
      origin_writer.is_null() ? nullptr : origin_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in NetworkContext.CreateWebSocket request");
  mojo::internal::Serialize<::network::mojom::AuthenticationHandlerPtrDataView>(
      param_auth_handler_, &params->auth_handler, serialization_context);
  }
  ::network::mojom::WebSocketRequest param_request_;
  int32_t param_process_id_;
  int32_t param_render_frame_id_;
  url::Origin param_origin_;
  ::network::mojom::AuthenticationHandlerPtrInfo param_auth_handler_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateWebSocket_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateWebSocket_Message::kMessageTag = {};

void NetworkContextProxy::CreateWebSocket(
    ::network::mojom::WebSocketRequest in_request, int32_t in_process_id, int32_t in_render_frame_id, const url::Origin& in_origin, ::network::mojom::AuthenticationHandlerPtr in_auth_handler) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateWebSocket");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateWebSocket_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_request), std::move(in_process_id), std::move(in_render_frame_id), std::move(in_origin), std::move(in_auth_handler));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CreateWebSocket");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_CreateNetLogExporter_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateNetLogExporter_Message(
      uint32_t message_flags
      , ::network::mojom::NetLogExporterRequest param_exporter
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateNetLogExporter_Name,
          message_flags)
      , param_exporter_(std::move(param_exporter)){}
  ~NetworkContextProxy_CreateNetLogExporter_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::NetLogExporterRequest param_exporter) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateNetLogExporter_Message>(
          kFlags
          , std::move(param_exporter)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateNetLogExporter_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateNetLogExporter_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetLogExporterRequestDataView>(
      param_exporter, &params->exporter, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->exporter),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid exporter in NetworkContext.CreateNetLogExporter request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->CreateNetLogExporter(
        std::move(param_exporter_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateNetLogExporter_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetLogExporterRequestDataView>(
      param_exporter_, &params->exporter, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->exporter),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid exporter in NetworkContext.CreateNetLogExporter request");
  }
  ::network::mojom::NetLogExporterRequest param_exporter_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateNetLogExporter_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateNetLogExporter_Message::kMessageTag = {};

void NetworkContextProxy::CreateNetLogExporter(
    ::network::mojom::NetLogExporterRequest in_exporter) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateNetLogExporter");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateNetLogExporter_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_exporter));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CreateNetLogExporter");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_PreconnectSockets_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_PreconnectSockets_Message(
      uint32_t message_flags
      , uint32_t param_num_streams
      , const GURL& param_url
      , int32_t param_load_flags
      , bool param_privacy_mode_enabled
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_PreconnectSockets_Name,
          message_flags)
      , param_num_streams_(std::move(param_num_streams))
      , param_url_(std::move(param_url))
      , param_load_flags_(std::move(param_load_flags))
      , param_privacy_mode_enabled_(std::move(param_privacy_mode_enabled)){}
  ~NetworkContextProxy_PreconnectSockets_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      uint32_t param_num_streams, const GURL& param_url, int32_t param_load_flags, bool param_privacy_mode_enabled) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_PreconnectSockets_Message>(
          kFlags
          , std::move(param_num_streams)
          , std::move(param_url)
          , std::move(param_load_flags)
          , std::move(param_privacy_mode_enabled)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_PreconnectSockets_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_PreconnectSockets_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->num_streams = param_num_streams;
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url, buffer, &url_writer, &serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkContext.PreconnectSockets request");
  params->load_flags = param_load_flags;
  params->privacy_mode_enabled = param_privacy_mode_enabled;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->PreconnectSockets(
        std::move(param_num_streams_),
        std::move(param_url_),
        std::move(param_load_flags_),
        std::move(param_privacy_mode_enabled_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_PreconnectSockets_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->num_streams = param_num_streams_;
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url_, buffer, &url_writer, serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkContext.PreconnectSockets request");
  params->load_flags = param_load_flags_;
  params->privacy_mode_enabled = param_privacy_mode_enabled_;
  }
  uint32_t param_num_streams_;
  GURL param_url_;
  int32_t param_load_flags_;
  bool param_privacy_mode_enabled_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_PreconnectSockets_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_PreconnectSockets_Message::kMessageTag = {};

void NetworkContextProxy::PreconnectSockets(
    uint32_t in_num_streams, const GURL& in_url, int32_t in_load_flags, bool in_privacy_mode_enabled) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::PreconnectSockets");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_PreconnectSockets_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_num_streams), std::move(in_url), std::move(in_load_flags), std::move(in_privacy_mode_enabled));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("PreconnectSockets");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_CreateP2PSocketManager_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateP2PSocketManager_Message(
      uint32_t message_flags
      , ::network::mojom::P2PTrustedSocketManagerClientPtr param_client
      , ::network::mojom::P2PTrustedSocketManagerRequest param_trusted_socket_manager
      , ::network::mojom::P2PSocketManagerRequest param_socket_manager
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateP2PSocketManager_Name,
          message_flags)
      , param_client_(param_client.PassInterface())
      , param_trusted_socket_manager_(std::move(param_trusted_socket_manager))
      , param_socket_manager_(std::move(param_socket_manager)){}
  ~NetworkContextProxy_CreateP2PSocketManager_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::P2PTrustedSocketManagerClientPtr param_client, ::network::mojom::P2PTrustedSocketManagerRequest param_trusted_socket_manager, ::network::mojom::P2PSocketManagerRequest param_socket_manager) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateP2PSocketManager_Message>(
          kFlags
          , std::move(param_client)
          , std::move(param_trusted_socket_manager)
          , std::move(param_socket_manager)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateP2PSocketManager_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateP2PSocketManager_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::P2PTrustedSocketManagerClientPtrDataView>(
      param_client, &params->client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in NetworkContext.CreateP2PSocketManager request");
  mojo::internal::Serialize<::network::mojom::P2PTrustedSocketManagerRequestDataView>(
      param_trusted_socket_manager, &params->trusted_socket_manager, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->trusted_socket_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid trusted_socket_manager in NetworkContext.CreateP2PSocketManager request");
  mojo::internal::Serialize<::network::mojom::P2PSocketManagerRequestDataView>(
      param_socket_manager, &params->socket_manager, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->socket_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid socket_manager in NetworkContext.CreateP2PSocketManager request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->CreateP2PSocketManager(
        ::network::mojom::P2PTrustedSocketManagerClientPtr(std::move(param_client_)),
        std::move(param_trusted_socket_manager_),
        std::move(param_socket_manager_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateP2PSocketManager_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::P2PTrustedSocketManagerClientPtrDataView>(
      param_client_, &params->client, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in NetworkContext.CreateP2PSocketManager request");
  mojo::internal::Serialize<::network::mojom::P2PTrustedSocketManagerRequestDataView>(
      param_trusted_socket_manager_, &params->trusted_socket_manager, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->trusted_socket_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid trusted_socket_manager in NetworkContext.CreateP2PSocketManager request");
  mojo::internal::Serialize<::network::mojom::P2PSocketManagerRequestDataView>(
      param_socket_manager_, &params->socket_manager, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->socket_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid socket_manager in NetworkContext.CreateP2PSocketManager request");
  }
  ::network::mojom::P2PTrustedSocketManagerClientPtrInfo param_client_;
  ::network::mojom::P2PTrustedSocketManagerRequest param_trusted_socket_manager_;
  ::network::mojom::P2PSocketManagerRequest param_socket_manager_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateP2PSocketManager_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateP2PSocketManager_Message::kMessageTag = {};

void NetworkContextProxy::CreateP2PSocketManager(
    ::network::mojom::P2PTrustedSocketManagerClientPtr in_client, ::network::mojom::P2PTrustedSocketManagerRequest in_trusted_socket_manager, ::network::mojom::P2PSocketManagerRequest in_socket_manager) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateP2PSocketManager");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateP2PSocketManager_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_client), std::move(in_trusted_socket_manager), std::move(in_socket_manager));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CreateP2PSocketManager");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_CreateMdnsResponder_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateMdnsResponder_Message(
      uint32_t message_flags
      , ::network::mojom::MdnsResponderRequest param_responder_request
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateMdnsResponder_Name,
          message_flags)
      , param_responder_request_(std::move(param_responder_request)){}
  ~NetworkContextProxy_CreateMdnsResponder_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::MdnsResponderRequest param_responder_request) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateMdnsResponder_Message>(
          kFlags
          , std::move(param_responder_request)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateMdnsResponder_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateMdnsResponder_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::MdnsResponderRequestDataView>(
      param_responder_request, &params->responder_request, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->responder_request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid responder_request in NetworkContext.CreateMdnsResponder request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->CreateMdnsResponder(
        std::move(param_responder_request_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateMdnsResponder_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::MdnsResponderRequestDataView>(
      param_responder_request_, &params->responder_request, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->responder_request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid responder_request in NetworkContext.CreateMdnsResponder request");
  }
  ::network::mojom::MdnsResponderRequest param_responder_request_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateMdnsResponder_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateMdnsResponder_Message::kMessageTag = {};

void NetworkContextProxy::CreateMdnsResponder(
    ::network::mojom::MdnsResponderRequest in_responder_request) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateMdnsResponder");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateMdnsResponder_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_responder_request));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CreateMdnsResponder");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_ResolveHost_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ResolveHost_Message(
      uint32_t message_flags
      , const net::HostPortPair& param_host
      , ::network::mojom::ResolveHostParametersPtr param_optional_parameters
      , ::network::mojom::ResolveHostClientPtr param_response_client
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ResolveHost_Name,
          message_flags)
      , param_host_(std::move(param_host))
      , param_optional_parameters_(std::move(param_optional_parameters))
      , param_response_client_(param_response_client.PassInterface()){}
  ~NetworkContextProxy_ResolveHost_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const net::HostPortPair& param_host, ::network::mojom::ResolveHostParametersPtr param_optional_parameters, ::network::mojom::ResolveHostClientPtr param_response_client) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ResolveHost_Message>(
          kFlags
          , std::move(param_host)
          , std::move(param_optional_parameters)
          , std::move(param_response_client)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ResolveHost_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ResolveHost_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->host)::BaseType::BufferWriter
      host_writer;
  mojo::internal::Serialize<::network::mojom::HostPortPairDataView>(
      param_host, buffer, &host_writer, &serialization_context);
  params->host.Set(
      host_writer.is_null() ? nullptr : host_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->host.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null host in NetworkContext.ResolveHost request");
  typename decltype(params->optional_parameters)::BaseType::BufferWriter
      optional_parameters_writer;
  mojo::internal::Serialize<::network::mojom::ResolveHostParametersDataView>(
      param_optional_parameters, buffer, &optional_parameters_writer, &serialization_context);
  params->optional_parameters.Set(
      optional_parameters_writer.is_null() ? nullptr : optional_parameters_writer.data());
  mojo::internal::Serialize<::network::mojom::ResolveHostClientPtrDataView>(
      param_response_client, &params->response_client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->response_client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid response_client in NetworkContext.ResolveHost request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->ResolveHost(
        std::move(param_host_),
        std::move(param_optional_parameters_),
        ::network::mojom::ResolveHostClientPtr(std::move(param_response_client_)));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ResolveHost_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->host)::BaseType::BufferWriter
      host_writer;
  mojo::internal::Serialize<::network::mojom::HostPortPairDataView>(
      param_host_, buffer, &host_writer, serialization_context);
  params->host.Set(
      host_writer.is_null() ? nullptr : host_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->host.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null host in NetworkContext.ResolveHost request");
  typename decltype(params->optional_parameters)::BaseType::BufferWriter
      optional_parameters_writer;
  mojo::internal::Serialize<::network::mojom::ResolveHostParametersDataView>(
      param_optional_parameters_, buffer, &optional_parameters_writer, serialization_context);
  params->optional_parameters.Set(
      optional_parameters_writer.is_null() ? nullptr : optional_parameters_writer.data());
  mojo::internal::Serialize<::network::mojom::ResolveHostClientPtrDataView>(
      param_response_client_, &params->response_client, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->response_client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid response_client in NetworkContext.ResolveHost request");
  }
  net::HostPortPair param_host_;
  ::network::mojom::ResolveHostParametersPtr param_optional_parameters_;
  ::network::mojom::ResolveHostClientPtrInfo param_response_client_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ResolveHost_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ResolveHost_Message::kMessageTag = {};

void NetworkContextProxy::ResolveHost(
    const net::HostPortPair& in_host, ::network::mojom::ResolveHostParametersPtr in_optional_parameters, ::network::mojom::ResolveHostClientPtr in_response_client) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ResolveHost");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ResolveHost_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_host), std::move(in_optional_parameters), std::move(in_response_client));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ResolveHost");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_CreateHostResolver_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateHostResolver_Message(
      uint32_t message_flags
      , const base::Optional<net::DnsConfigOverrides>& param_config_overrides
      , ::network::mojom::HostResolverRequest param_host_resolver
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateHostResolver_Name,
          message_flags)
      , param_config_overrides_(std::move(param_config_overrides))
      , param_host_resolver_(std::move(param_host_resolver)){}
  ~NetworkContextProxy_CreateHostResolver_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const base::Optional<net::DnsConfigOverrides>& param_config_overrides, ::network::mojom::HostResolverRequest param_host_resolver) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateHostResolver_Message>(
          kFlags
          , std::move(param_config_overrides)
          , std::move(param_host_resolver)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateHostResolver_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateHostResolver_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->config_overrides)::BaseType::BufferWriter
      config_overrides_writer;
  mojo::internal::Serialize<::network::mojom::DnsConfigOverridesDataView>(
      param_config_overrides, buffer, &config_overrides_writer, &serialization_context);
  params->config_overrides.Set(
      config_overrides_writer.is_null() ? nullptr : config_overrides_writer.data());
  mojo::internal::Serialize<::network::mojom::HostResolverRequestDataView>(
      param_host_resolver, &params->host_resolver, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->host_resolver),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid host_resolver in NetworkContext.CreateHostResolver request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->CreateHostResolver(
        std::move(param_config_overrides_),
        std::move(param_host_resolver_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateHostResolver_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->config_overrides)::BaseType::BufferWriter
      config_overrides_writer;
  mojo::internal::Serialize<::network::mojom::DnsConfigOverridesDataView>(
      param_config_overrides_, buffer, &config_overrides_writer, serialization_context);
  params->config_overrides.Set(
      config_overrides_writer.is_null() ? nullptr : config_overrides_writer.data());
  mojo::internal::Serialize<::network::mojom::HostResolverRequestDataView>(
      param_host_resolver_, &params->host_resolver, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->host_resolver),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid host_resolver in NetworkContext.CreateHostResolver request");
  }
  base::Optional<net::DnsConfigOverrides> param_config_overrides_;
  ::network::mojom::HostResolverRequest param_host_resolver_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateHostResolver_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateHostResolver_Message::kMessageTag = {};

void NetworkContextProxy::CreateHostResolver(
    const base::Optional<net::DnsConfigOverrides>& in_config_overrides, ::network::mojom::HostResolverRequest in_host_resolver) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateHostResolver");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateHostResolver_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_config_overrides), std::move(in_host_resolver));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CreateHostResolver");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_VerifyCertForSignedExchange_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_VerifyCertForSignedExchange_Message(
      uint32_t message_flags
      , const scoped_refptr<net::X509Certificate>& param_certificate
      , const GURL& param_url
      , const std::string& param_ocsp_response
      , const std::string& param_sct_list
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_VerifyCertForSignedExchange_Name,
          message_flags)
      , param_certificate_(std::move(param_certificate))
      , param_url_(std::move(param_url))
      , param_ocsp_response_(std::move(param_ocsp_response))
      , param_sct_list_(std::move(param_sct_list)){}
  ~NetworkContextProxy_VerifyCertForSignedExchange_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const scoped_refptr<net::X509Certificate>& param_certificate, const GURL& param_url, const std::string& param_ocsp_response, const std::string& param_sct_list) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_VerifyCertForSignedExchange_Message>(
          kFlags
          , std::move(param_certificate)
          , std::move(param_url)
          , std::move(param_ocsp_response)
          , std::move(param_sct_list)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_VerifyCertForSignedExchange_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_VerifyCertForSignedExchange_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->certificate)::BaseType::BufferWriter
      certificate_writer;
  mojo::internal::Serialize<::network::mojom::X509CertificateDataView>(
      param_certificate, buffer, &certificate_writer, &serialization_context);
  params->certificate.Set(
      certificate_writer.is_null() ? nullptr : certificate_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->certificate.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null certificate in NetworkContext.VerifyCertForSignedExchange request");
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url, buffer, &url_writer, &serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkContext.VerifyCertForSignedExchange request");
  typename decltype(params->ocsp_response)::BaseType::BufferWriter
      ocsp_response_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_ocsp_response, buffer, &ocsp_response_writer, &serialization_context);
  params->ocsp_response.Set(
      ocsp_response_writer.is_null() ? nullptr : ocsp_response_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->ocsp_response.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null ocsp_response in NetworkContext.VerifyCertForSignedExchange request");
  typename decltype(params->sct_list)::BaseType::BufferWriter
      sct_list_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_sct_list, buffer, &sct_list_writer, &serialization_context);
  params->sct_list.Set(
      sct_list_writer.is_null() ? nullptr : sct_list_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->sct_list.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null sct_list in NetworkContext.VerifyCertForSignedExchange request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::VerifyCertForSignedExchangeCallback callback) {
    impl->VerifyCertForSignedExchange(
        std::move(param_certificate_),
        std::move(param_url_),
        std::move(param_ocsp_response_),
        std::move(param_sct_list_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_VerifyCertForSignedExchange_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->certificate)::BaseType::BufferWriter
      certificate_writer;
  mojo::internal::Serialize<::network::mojom::X509CertificateDataView>(
      param_certificate_, buffer, &certificate_writer, serialization_context);
  params->certificate.Set(
      certificate_writer.is_null() ? nullptr : certificate_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->certificate.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null certificate in NetworkContext.VerifyCertForSignedExchange request");
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url_, buffer, &url_writer, serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkContext.VerifyCertForSignedExchange request");
  typename decltype(params->ocsp_response)::BaseType::BufferWriter
      ocsp_response_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_ocsp_response_, buffer, &ocsp_response_writer, serialization_context);
  params->ocsp_response.Set(
      ocsp_response_writer.is_null() ? nullptr : ocsp_response_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->ocsp_response.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null ocsp_response in NetworkContext.VerifyCertForSignedExchange request");
  typename decltype(params->sct_list)::BaseType::BufferWriter
      sct_list_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_sct_list_, buffer, &sct_list_writer, serialization_context);
  params->sct_list.Set(
      sct_list_writer.is_null() ? nullptr : sct_list_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->sct_list.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null sct_list in NetworkContext.VerifyCertForSignedExchange request");
  }
  scoped_refptr<net::X509Certificate> param_certificate_;
  GURL param_url_;
  std::string param_ocsp_response_;
  std::string param_sct_list_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_VerifyCertForSignedExchange_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_VerifyCertForSignedExchange_Message::kMessageTag = {};

void NetworkContextProxy::VerifyCertForSignedExchange(
    const scoped_refptr<net::X509Certificate>& in_certificate, const GURL& in_url, const std::string& in_ocsp_response, const std::string& in_sct_list, VerifyCertForSignedExchangeCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::VerifyCertForSignedExchange");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_VerifyCertForSignedExchange_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_certificate), std::move(in_url), std::move(in_ocsp_response), std::move(in_sct_list));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("VerifyCertForSignedExchange");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_VerifyCertForSignedExchange_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_AddHSTS_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_AddHSTS_Message(
      uint32_t message_flags
      , const std::string& param_host
      , base::Time param_expiry
      , bool param_include_subdomains
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_AddHSTS_Name,
          message_flags)
      , param_host_(std::move(param_host))
      , param_expiry_(std::move(param_expiry))
      , param_include_subdomains_(std::move(param_include_subdomains)){}
  ~NetworkContextProxy_AddHSTS_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const std::string& param_host, base::Time param_expiry, bool param_include_subdomains) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_AddHSTS_Message>(
          kFlags
          , std::move(param_host)
          , std::move(param_expiry)
          , std::move(param_include_subdomains)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_AddHSTS_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_AddHSTS_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->host)::BaseType::BufferWriter
      host_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_host, buffer, &host_writer, &serialization_context);
  params->host.Set(
      host_writer.is_null() ? nullptr : host_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->host.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null host in NetworkContext.AddHSTS request");
  typename decltype(params->expiry)::BaseType::BufferWriter
      expiry_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_expiry, buffer, &expiry_writer, &serialization_context);
  params->expiry.Set(
      expiry_writer.is_null() ? nullptr : expiry_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->expiry.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null expiry in NetworkContext.AddHSTS request");
  params->include_subdomains = param_include_subdomains;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::AddHSTSCallback callback) {
    impl->AddHSTS(
        std::move(param_host_),
        std::move(param_expiry_),
        std::move(param_include_subdomains_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_AddHSTS_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->host)::BaseType::BufferWriter
      host_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_host_, buffer, &host_writer, serialization_context);
  params->host.Set(
      host_writer.is_null() ? nullptr : host_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->host.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null host in NetworkContext.AddHSTS request");
  typename decltype(params->expiry)::BaseType::BufferWriter
      expiry_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_expiry_, buffer, &expiry_writer, serialization_context);
  params->expiry.Set(
      expiry_writer.is_null() ? nullptr : expiry_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->expiry.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null expiry in NetworkContext.AddHSTS request");
  params->include_subdomains = param_include_subdomains_;
  }
  std::string param_host_;
  base::Time param_expiry_;
  bool param_include_subdomains_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_AddHSTS_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_AddHSTS_Message::kMessageTag = {};

void NetworkContextProxy::AddHSTS(
    const std::string& in_host, base::Time in_expiry, bool in_include_subdomains, AddHSTSCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::AddHSTS");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_AddHSTS_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_host), std::move(in_expiry), std::move(in_include_subdomains));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("AddHSTS");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_AddHSTS_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_IsHSTSActiveForHost_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_IsHSTSActiveForHost_Message(
      uint32_t message_flags
      , const std::string& param_host
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_IsHSTSActiveForHost_Name,
          message_flags)
      , param_host_(std::move(param_host)){}
  ~NetworkContextProxy_IsHSTSActiveForHost_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const std::string& param_host) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_IsHSTSActiveForHost_Message>(
          kFlags
          , std::move(param_host)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_IsHSTSActiveForHost_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_IsHSTSActiveForHost_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->host)::BaseType::BufferWriter
      host_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_host, buffer, &host_writer, &serialization_context);
  params->host.Set(
      host_writer.is_null() ? nullptr : host_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->host.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null host in NetworkContext.IsHSTSActiveForHost request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::IsHSTSActiveForHostCallback callback) {
    impl->IsHSTSActiveForHost(
        std::move(param_host_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_IsHSTSActiveForHost_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->host)::BaseType::BufferWriter
      host_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_host_, buffer, &host_writer, serialization_context);
  params->host.Set(
      host_writer.is_null() ? nullptr : host_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->host.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null host in NetworkContext.IsHSTSActiveForHost request");
  }
  std::string param_host_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_IsHSTSActiveForHost_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_IsHSTSActiveForHost_Message::kMessageTag = {};

void NetworkContextProxy::IsHSTSActiveForHost(
    const std::string& in_host, IsHSTSActiveForHostCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::IsHSTSActiveForHost");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_IsHSTSActiveForHost_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_host));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("IsHSTSActiveForHost");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_IsHSTSActiveForHost_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_GetHSTSState_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_GetHSTSState_Message(
      uint32_t message_flags
      , const std::string& param_domain
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_GetHSTSState_Name,
          message_flags)
      , param_domain_(std::move(param_domain)){}
  ~NetworkContextProxy_GetHSTSState_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const std::string& param_domain) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_GetHSTSState_Message>(
          kFlags
          , std::move(param_domain)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_GetHSTSState_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_GetHSTSState_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->domain)::BaseType::BufferWriter
      domain_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_domain, buffer, &domain_writer, &serialization_context);
  params->domain.Set(
      domain_writer.is_null() ? nullptr : domain_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->domain.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null domain in NetworkContext.GetHSTSState request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::GetHSTSStateCallback callback) {
    impl->GetHSTSState(
        std::move(param_domain_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_GetHSTSState_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->domain)::BaseType::BufferWriter
      domain_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_domain_, buffer, &domain_writer, serialization_context);
  params->domain.Set(
      domain_writer.is_null() ? nullptr : domain_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->domain.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null domain in NetworkContext.GetHSTSState request");
  }
  std::string param_domain_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_GetHSTSState_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_GetHSTSState_Message::kMessageTag = {};

void NetworkContextProxy::GetHSTSState(
    const std::string& in_domain, GetHSTSStateCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::GetHSTSState");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_GetHSTSState_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_domain));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("GetHSTSState");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_GetHSTSState_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_SetCorsOriginAccessListsForOrigin_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_SetCorsOriginAccessListsForOrigin_Message(
      uint32_t message_flags
      , const url::Origin& param_source_origin
      , std::vector<::network::mojom::CorsOriginPatternPtr> param_allow_patterns
      , std::vector<::network::mojom::CorsOriginPatternPtr> param_block_patterns
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_SetCorsOriginAccessListsForOrigin_Name,
          message_flags)
      , param_source_origin_(std::move(param_source_origin))
      , param_allow_patterns_(std::move(param_allow_patterns))
      , param_block_patterns_(std::move(param_block_patterns)){}
  ~NetworkContextProxy_SetCorsOriginAccessListsForOrigin_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const url::Origin& param_source_origin, std::vector<::network::mojom::CorsOriginPatternPtr> param_allow_patterns, std::vector<::network::mojom::CorsOriginPatternPtr> param_block_patterns) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_SetCorsOriginAccessListsForOrigin_Message>(
          kFlags
          , std::move(param_source_origin)
          , std::move(param_allow_patterns)
          , std::move(param_block_patterns)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_SetCorsOriginAccessListsForOrigin_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_SetCorsOriginAccessListsForOrigin_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->source_origin)::BaseType::BufferWriter
      source_origin_writer;
  mojo::internal::Serialize<::url::mojom::OriginDataView>(
      param_source_origin, buffer, &source_origin_writer, &serialization_context);
  params->source_origin.Set(
      source_origin_writer.is_null() ? nullptr : source_origin_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->source_origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null source_origin in NetworkContext.SetCorsOriginAccessListsForOrigin request");
  typename decltype(params->allow_patterns)::BaseType::BufferWriter
      allow_patterns_writer;
  const mojo::internal::ContainerValidateParams allow_patterns_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::network::mojom::CorsOriginPatternDataView>>(
      param_allow_patterns, buffer, &allow_patterns_writer, &allow_patterns_validate_params,
      &serialization_context);
  params->allow_patterns.Set(
      allow_patterns_writer.is_null() ? nullptr : allow_patterns_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->allow_patterns.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null allow_patterns in NetworkContext.SetCorsOriginAccessListsForOrigin request");
  typename decltype(params->block_patterns)::BaseType::BufferWriter
      block_patterns_writer;
  const mojo::internal::ContainerValidateParams block_patterns_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::network::mojom::CorsOriginPatternDataView>>(
      param_block_patterns, buffer, &block_patterns_writer, &block_patterns_validate_params,
      &serialization_context);
  params->block_patterns.Set(
      block_patterns_writer.is_null() ? nullptr : block_patterns_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->block_patterns.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null block_patterns in NetworkContext.SetCorsOriginAccessListsForOrigin request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::SetCorsOriginAccessListsForOriginCallback callback) {
    impl->SetCorsOriginAccessListsForOrigin(
        std::move(param_source_origin_),
        std::move(param_allow_patterns_),
        std::move(param_block_patterns_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_SetCorsOriginAccessListsForOrigin_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->source_origin)::BaseType::BufferWriter
      source_origin_writer;
  mojo::internal::Serialize<::url::mojom::OriginDataView>(
      param_source_origin_, buffer, &source_origin_writer, serialization_context);
  params->source_origin.Set(
      source_origin_writer.is_null() ? nullptr : source_origin_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->source_origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null source_origin in NetworkContext.SetCorsOriginAccessListsForOrigin request");
  typename decltype(params->allow_patterns)::BaseType::BufferWriter
      allow_patterns_writer;
  const mojo::internal::ContainerValidateParams allow_patterns_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::network::mojom::CorsOriginPatternDataView>>(
      param_allow_patterns_, buffer, &allow_patterns_writer, &allow_patterns_validate_params,
      serialization_context);
  params->allow_patterns.Set(
      allow_patterns_writer.is_null() ? nullptr : allow_patterns_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->allow_patterns.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null allow_patterns in NetworkContext.SetCorsOriginAccessListsForOrigin request");
  typename decltype(params->block_patterns)::BaseType::BufferWriter
      block_patterns_writer;
  const mojo::internal::ContainerValidateParams block_patterns_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::network::mojom::CorsOriginPatternDataView>>(
      param_block_patterns_, buffer, &block_patterns_writer, &block_patterns_validate_params,
      serialization_context);
  params->block_patterns.Set(
      block_patterns_writer.is_null() ? nullptr : block_patterns_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->block_patterns.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null block_patterns in NetworkContext.SetCorsOriginAccessListsForOrigin request");
  }
  url::Origin param_source_origin_;
  std::vector<::network::mojom::CorsOriginPatternPtr> param_allow_patterns_;
  std::vector<::network::mojom::CorsOriginPatternPtr> param_block_patterns_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_SetCorsOriginAccessListsForOrigin_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_SetCorsOriginAccessListsForOrigin_Message::kMessageTag = {};

void NetworkContextProxy::SetCorsOriginAccessListsForOrigin(
    const url::Origin& in_source_origin, std::vector<::network::mojom::CorsOriginPatternPtr> in_allow_patterns, std::vector<::network::mojom::CorsOriginPatternPtr> in_block_patterns, SetCorsOriginAccessListsForOriginCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::SetCorsOriginAccessListsForOrigin");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_SetCorsOriginAccessListsForOrigin_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_source_origin), std::move(in_allow_patterns), std::move(in_block_patterns));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("SetCorsOriginAccessListsForOrigin");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_SetCorsOriginAccessListsForOrigin_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_DeleteDynamicDataForHost_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_DeleteDynamicDataForHost_Message(
      uint32_t message_flags
      , const std::string& param_host
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_DeleteDynamicDataForHost_Name,
          message_flags)
      , param_host_(std::move(param_host)){}
  ~NetworkContextProxy_DeleteDynamicDataForHost_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const std::string& param_host) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_DeleteDynamicDataForHost_Message>(
          kFlags
          , std::move(param_host)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_DeleteDynamicDataForHost_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_DeleteDynamicDataForHost_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->host)::BaseType::BufferWriter
      host_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_host, buffer, &host_writer, &serialization_context);
  params->host.Set(
      host_writer.is_null() ? nullptr : host_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->host.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null host in NetworkContext.DeleteDynamicDataForHost request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::DeleteDynamicDataForHostCallback callback) {
    impl->DeleteDynamicDataForHost(
        std::move(param_host_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_DeleteDynamicDataForHost_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->host)::BaseType::BufferWriter
      host_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_host_, buffer, &host_writer, serialization_context);
  params->host.Set(
      host_writer.is_null() ? nullptr : host_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->host.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null host in NetworkContext.DeleteDynamicDataForHost request");
  }
  std::string param_host_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_DeleteDynamicDataForHost_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_DeleteDynamicDataForHost_Message::kMessageTag = {};

void NetworkContextProxy::DeleteDynamicDataForHost(
    const std::string& in_host, DeleteDynamicDataForHostCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::DeleteDynamicDataForHost");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_DeleteDynamicDataForHost_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_host));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("DeleteDynamicDataForHost");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_DeleteDynamicDataForHost_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_LookupBasicAuthCredentials_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_LookupBasicAuthCredentials_Message(
      uint32_t message_flags
      , const GURL& param_url
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_LookupBasicAuthCredentials_Name,
          message_flags)
      , param_url_(std::move(param_url)){}
  ~NetworkContextProxy_LookupBasicAuthCredentials_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const GURL& param_url) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_LookupBasicAuthCredentials_Message>(
          kFlags
          , std::move(param_url)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_LookupBasicAuthCredentials_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_LookupBasicAuthCredentials_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url, buffer, &url_writer, &serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkContext.LookupBasicAuthCredentials request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::LookupBasicAuthCredentialsCallback callback) {
    impl->LookupBasicAuthCredentials(
        std::move(param_url_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_LookupBasicAuthCredentials_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url_, buffer, &url_writer, serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkContext.LookupBasicAuthCredentials request");
  }
  GURL param_url_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_LookupBasicAuthCredentials_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_LookupBasicAuthCredentials_Message::kMessageTag = {};

void NetworkContextProxy::LookupBasicAuthCredentials(
    const GURL& in_url, LookupBasicAuthCredentialsCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::LookupBasicAuthCredentials");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_LookupBasicAuthCredentials_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_url));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("LookupBasicAuthCredentials");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_LookupBasicAuthCredentials_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_EnableStaticKeyPinningForTesting_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_EnableStaticKeyPinningForTesting_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_EnableStaticKeyPinningForTesting_Name,
          message_flags){}
  ~NetworkContextProxy_EnableStaticKeyPinningForTesting_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_EnableStaticKeyPinningForTesting_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_EnableStaticKeyPinningForTesting_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_EnableStaticKeyPinningForTesting_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::EnableStaticKeyPinningForTestingCallback callback) {
    impl->EnableStaticKeyPinningForTesting(std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_EnableStaticKeyPinningForTesting_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_EnableStaticKeyPinningForTesting_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_EnableStaticKeyPinningForTesting_Message::kMessageTag = {};
bool NetworkContextProxy::EnableStaticKeyPinningForTesting(
    ) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::EnableStaticKeyPinningForTesting");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = true;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_EnableStaticKeyPinningForTesting_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("EnableStaticKeyPinningForTesting");
#endif

  bool result = false;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_EnableStaticKeyPinningForTesting_HandleSyncResponse(
          &result));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
  return result;
}

void NetworkContextProxy::EnableStaticKeyPinningForTesting(
    EnableStaticKeyPinningForTestingCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::EnableStaticKeyPinningForTesting");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_EnableStaticKeyPinningForTesting_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("EnableStaticKeyPinningForTesting");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_EnableStaticKeyPinningForTesting_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_SetFailingHttpTransactionForTesting_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_SetFailingHttpTransactionForTesting_Message(
      uint32_t message_flags
      , int32_t param_rv
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_SetFailingHttpTransactionForTesting_Name,
          message_flags)
      , param_rv_(std::move(param_rv)){}
  ~NetworkContextProxy_SetFailingHttpTransactionForTesting_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      int32_t param_rv) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_SetFailingHttpTransactionForTesting_Message>(
          kFlags
          , std::move(param_rv)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_SetFailingHttpTransactionForTesting_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_SetFailingHttpTransactionForTesting_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->rv = param_rv;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::SetFailingHttpTransactionForTestingCallback callback) {
    impl->SetFailingHttpTransactionForTesting(
        std::move(param_rv_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_SetFailingHttpTransactionForTesting_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->rv = param_rv_;
  }
  int32_t param_rv_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_SetFailingHttpTransactionForTesting_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_SetFailingHttpTransactionForTesting_Message::kMessageTag = {};
bool NetworkContextProxy::SetFailingHttpTransactionForTesting(
    int32_t param_rv) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::SetFailingHttpTransactionForTesting");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = true;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_SetFailingHttpTransactionForTesting_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(param_rv));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("SetFailingHttpTransactionForTesting");
#endif

  bool result = false;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_SetFailingHttpTransactionForTesting_HandleSyncResponse(
          &result));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
  return result;
}

void NetworkContextProxy::SetFailingHttpTransactionForTesting(
    int32_t in_rv, SetFailingHttpTransactionForTestingCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::SetFailingHttpTransactionForTesting");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_SetFailingHttpTransactionForTesting_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_rv));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("SetFailingHttpTransactionForTesting");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_SetFailingHttpTransactionForTesting_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_VerifyCertificateForTesting_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_VerifyCertificateForTesting_Message(
      uint32_t message_flags
      , const scoped_refptr<net::X509Certificate>& param_certificate
      , const std::string& param_hostname
      , const std::string& param_ocsp_response
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_VerifyCertificateForTesting_Name,
          message_flags)
      , param_certificate_(std::move(param_certificate))
      , param_hostname_(std::move(param_hostname))
      , param_ocsp_response_(std::move(param_ocsp_response)){}
  ~NetworkContextProxy_VerifyCertificateForTesting_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const scoped_refptr<net::X509Certificate>& param_certificate, const std::string& param_hostname, const std::string& param_ocsp_response) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_VerifyCertificateForTesting_Message>(
          kFlags
          , std::move(param_certificate)
          , std::move(param_hostname)
          , std::move(param_ocsp_response)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_VerifyCertificateForTesting_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_VerifyCertificateForTesting_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->certificate)::BaseType::BufferWriter
      certificate_writer;
  mojo::internal::Serialize<::network::mojom::X509CertificateDataView>(
      param_certificate, buffer, &certificate_writer, &serialization_context);
  params->certificate.Set(
      certificate_writer.is_null() ? nullptr : certificate_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->certificate.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null certificate in NetworkContext.VerifyCertificateForTesting request");
  typename decltype(params->hostname)::BaseType::BufferWriter
      hostname_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_hostname, buffer, &hostname_writer, &serialization_context);
  params->hostname.Set(
      hostname_writer.is_null() ? nullptr : hostname_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->hostname.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null hostname in NetworkContext.VerifyCertificateForTesting request");
  typename decltype(params->ocsp_response)::BaseType::BufferWriter
      ocsp_response_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_ocsp_response, buffer, &ocsp_response_writer, &serialization_context);
  params->ocsp_response.Set(
      ocsp_response_writer.is_null() ? nullptr : ocsp_response_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->ocsp_response.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null ocsp_response in NetworkContext.VerifyCertificateForTesting request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::VerifyCertificateForTestingCallback callback) {
    impl->VerifyCertificateForTesting(
        std::move(param_certificate_),
        std::move(param_hostname_),
        std::move(param_ocsp_response_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_VerifyCertificateForTesting_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->certificate)::BaseType::BufferWriter
      certificate_writer;
  mojo::internal::Serialize<::network::mojom::X509CertificateDataView>(
      param_certificate_, buffer, &certificate_writer, serialization_context);
  params->certificate.Set(
      certificate_writer.is_null() ? nullptr : certificate_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->certificate.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null certificate in NetworkContext.VerifyCertificateForTesting request");
  typename decltype(params->hostname)::BaseType::BufferWriter
      hostname_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_hostname_, buffer, &hostname_writer, serialization_context);
  params->hostname.Set(
      hostname_writer.is_null() ? nullptr : hostname_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->hostname.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null hostname in NetworkContext.VerifyCertificateForTesting request");
  typename decltype(params->ocsp_response)::BaseType::BufferWriter
      ocsp_response_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_ocsp_response_, buffer, &ocsp_response_writer, serialization_context);
  params->ocsp_response.Set(
      ocsp_response_writer.is_null() ? nullptr : ocsp_response_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->ocsp_response.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null ocsp_response in NetworkContext.VerifyCertificateForTesting request");
  }
  scoped_refptr<net::X509Certificate> param_certificate_;
  std::string param_hostname_;
  std::string param_ocsp_response_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_VerifyCertificateForTesting_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_VerifyCertificateForTesting_Message::kMessageTag = {};
bool NetworkContextProxy::VerifyCertificateForTesting(
    const scoped_refptr<net::X509Certificate>& param_certificate, const std::string& param_hostname, const std::string& param_ocsp_response, int32_t* out_param_error_code) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::VerifyCertificateForTesting");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = true;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_VerifyCertificateForTesting_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(param_certificate), std::move(param_hostname), std::move(param_ocsp_response));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("VerifyCertificateForTesting");
#endif

  bool result = false;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_VerifyCertificateForTesting_HandleSyncResponse(
          &result, out_param_error_code));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
  return result;
}

void NetworkContextProxy::VerifyCertificateForTesting(
    const scoped_refptr<net::X509Certificate>& in_certificate, const std::string& in_hostname, const std::string& in_ocsp_response, VerifyCertificateForTestingCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::VerifyCertificateForTesting");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_VerifyCertificateForTesting_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_certificate), std::move(in_hostname), std::move(in_ocsp_response));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("VerifyCertificateForTesting");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_VerifyCertificateForTesting_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_AddDomainReliabilityContextForTesting_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_AddDomainReliabilityContextForTesting_Message(
      uint32_t message_flags
      , const GURL& param_origin
      , const GURL& param_upload_url
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_AddDomainReliabilityContextForTesting_Name,
          message_flags)
      , param_origin_(std::move(param_origin))
      , param_upload_url_(std::move(param_upload_url)){}
  ~NetworkContextProxy_AddDomainReliabilityContextForTesting_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const GURL& param_origin, const GURL& param_upload_url) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_AddDomainReliabilityContextForTesting_Message>(
          kFlags
          , std::move(param_origin)
          , std::move(param_upload_url)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_AddDomainReliabilityContextForTesting_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_AddDomainReliabilityContextForTesting_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->origin)::BaseType::BufferWriter
      origin_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_origin, buffer, &origin_writer, &serialization_context);
  params->origin.Set(
      origin_writer.is_null() ? nullptr : origin_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in NetworkContext.AddDomainReliabilityContextForTesting request");
  typename decltype(params->upload_url)::BaseType::BufferWriter
      upload_url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_upload_url, buffer, &upload_url_writer, &serialization_context);
  params->upload_url.Set(
      upload_url_writer.is_null() ? nullptr : upload_url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->upload_url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null upload_url in NetworkContext.AddDomainReliabilityContextForTesting request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::AddDomainReliabilityContextForTestingCallback callback) {
    impl->AddDomainReliabilityContextForTesting(
        std::move(param_origin_),
        std::move(param_upload_url_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_AddDomainReliabilityContextForTesting_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->origin)::BaseType::BufferWriter
      origin_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_origin_, buffer, &origin_writer, serialization_context);
  params->origin.Set(
      origin_writer.is_null() ? nullptr : origin_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in NetworkContext.AddDomainReliabilityContextForTesting request");
  typename decltype(params->upload_url)::BaseType::BufferWriter
      upload_url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_upload_url_, buffer, &upload_url_writer, serialization_context);
  params->upload_url.Set(
      upload_url_writer.is_null() ? nullptr : upload_url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->upload_url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null upload_url in NetworkContext.AddDomainReliabilityContextForTesting request");
  }
  GURL param_origin_;
  GURL param_upload_url_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_AddDomainReliabilityContextForTesting_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_AddDomainReliabilityContextForTesting_Message::kMessageTag = {};
bool NetworkContextProxy::AddDomainReliabilityContextForTesting(
    const GURL& param_origin, const GURL& param_upload_url) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::AddDomainReliabilityContextForTesting");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = true;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_AddDomainReliabilityContextForTesting_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(param_origin), std::move(param_upload_url));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("AddDomainReliabilityContextForTesting");
#endif

  bool result = false;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_AddDomainReliabilityContextForTesting_HandleSyncResponse(
          &result));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
  return result;
}

void NetworkContextProxy::AddDomainReliabilityContextForTesting(
    const GURL& in_origin, const GURL& in_upload_url, AddDomainReliabilityContextForTestingCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::AddDomainReliabilityContextForTesting");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_AddDomainReliabilityContextForTesting_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_origin), std::move(in_upload_url));

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("AddDomainReliabilityContextForTesting");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_AddDomainReliabilityContextForTesting_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_ForceDomainReliabilityUploadsForTesting_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ForceDomainReliabilityUploadsForTesting_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ForceDomainReliabilityUploadsForTesting_Name,
          message_flags){}
  ~NetworkContextProxy_ForceDomainReliabilityUploadsForTesting_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ForceDomainReliabilityUploadsForTesting_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ForceDomainReliabilityUploadsForTesting_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ForceDomainReliabilityUploadsForTestingCallback callback) {
    impl->ForceDomainReliabilityUploadsForTesting(std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ForceDomainReliabilityUploadsForTesting_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ForceDomainReliabilityUploadsForTesting_Message::kMessageTag = {};
bool NetworkContextProxy::ForceDomainReliabilityUploadsForTesting(
    ) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ForceDomainReliabilityUploadsForTesting");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = true;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ForceDomainReliabilityUploadsForTesting_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ForceDomainReliabilityUploadsForTesting");
#endif

  bool result = false;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ForceDomainReliabilityUploadsForTesting_HandleSyncResponse(
          &result));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
  return result;
}

void NetworkContextProxy::ForceDomainReliabilityUploadsForTesting(
    ForceDomainReliabilityUploadsForTestingCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ForceDomainReliabilityUploadsForTesting");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ForceDomainReliabilityUploadsForTesting_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ForceDomainReliabilityUploadsForTesting");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ForceDomainReliabilityUploadsForTesting_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContext_ClearNetworkingHistorySince_ProxyToResponder {
 public:
  static NetworkContext::ClearNetworkingHistorySinceCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ClearNetworkingHistorySince_ProxyToResponder> proxy(
        new NetworkContext_ClearNetworkingHistorySince_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ClearNetworkingHistorySince_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ClearNetworkingHistorySince_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ClearNetworkingHistorySince_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ClearNetworkingHistorySinceCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearNetworkingHistorySince_ProxyToResponder);
};
class NetworkContext_ClearNetworkingHistorySince_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ClearNetworkingHistorySince_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearNetworkingHistorySince_Name,
          message_flags){}
  ~NetworkContext_ClearNetworkingHistorySince_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ClearNetworkingHistorySince_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearNetworkingHistorySince_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ClearNetworkingHistorySinceCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearNetworkingHistorySince_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ClearNetworkingHistorySince_Response_Message::kMessageTag = {};

bool NetworkContext_ClearNetworkingHistorySince_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ClearNetworkingHistorySinceCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ClearNetworkingHistorySince_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ClearNetworkingHistorySince_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ClearNetworkingHistorySince response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_ClearNetworkingHistorySince_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ClearNetworkingHistorySince_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearNetworkingHistorySinceCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearNetworkingHistorySince");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_ClearHttpCache_ProxyToResponder {
 public:
  static NetworkContext::ClearHttpCacheCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ClearHttpCache_ProxyToResponder> proxy(
        new NetworkContext_ClearHttpCache_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ClearHttpCache_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ClearHttpCache_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ClearHttpCache_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ClearHttpCacheCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearHttpCache_ProxyToResponder);
};
class NetworkContext_ClearHttpCache_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ClearHttpCache_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearHttpCache_Name,
          message_flags){}
  ~NetworkContext_ClearHttpCache_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ClearHttpCache_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearHttpCache_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearHttpCache_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ClearHttpCacheCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearHttpCache_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearHttpCache_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ClearHttpCache_Response_Message::kMessageTag = {};

bool NetworkContext_ClearHttpCache_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ClearHttpCacheCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ClearHttpCache_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ClearHttpCache_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ClearHttpCache_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ClearHttpCache_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ClearHttpCache response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_ClearHttpCache_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ClearHttpCache_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearHttpCacheCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearHttpCache");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_ComputeHttpCacheSize_ProxyToResponder {
 public:
  static NetworkContext::ComputeHttpCacheSizeCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ComputeHttpCacheSize_ProxyToResponder> proxy(
        new NetworkContext_ComputeHttpCacheSize_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ComputeHttpCacheSize_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ComputeHttpCacheSize_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ComputeHttpCacheSize_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ComputeHttpCacheSizeCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      bool in_is_upper_bound, int64_t in_size_or_error);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ComputeHttpCacheSize_ProxyToResponder);
};
class NetworkContext_ComputeHttpCacheSize_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ComputeHttpCacheSize_Response_Message(
      uint32_t message_flags
      , bool param_is_upper_bound
      , int64_t param_size_or_error
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ComputeHttpCacheSize_Name,
          message_flags)
      , param_is_upper_bound_(std::move(param_is_upper_bound))
      , param_size_or_error_(std::move(param_size_or_error)){}
  ~NetworkContext_ComputeHttpCacheSize_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      bool param_is_upper_bound, int64_t param_size_or_error) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ComputeHttpCacheSize_Response_Message>(
          kFlags
          , std::move(param_is_upper_bound)
          , std::move(param_size_or_error)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ComputeHttpCacheSize_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ComputeHttpCacheSize_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->is_upper_bound = param_is_upper_bound;
  params->size_or_error = param_size_or_error;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ComputeHttpCacheSizeCallback* callback) {
    std::move(*callback).Run(
        std::move(param_is_upper_bound_), 
        std::move(param_size_or_error_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ComputeHttpCacheSize_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->is_upper_bound = param_is_upper_bound_;
  params->size_or_error = param_size_or_error_;
  }
  bool param_is_upper_bound_;
  int64_t param_size_or_error_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ComputeHttpCacheSize_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ComputeHttpCacheSize_Response_Message::kMessageTag = {};

bool NetworkContext_ComputeHttpCacheSize_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ComputeHttpCacheSizeCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ComputeHttpCacheSize_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ComputeHttpCacheSize_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ComputeHttpCacheSize_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  bool p_is_upper_bound{};
  int64_t p_size_or_error{};
  NetworkContext_ComputeHttpCacheSize_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_is_upper_bound = input_data_view.is_upper_bound();
  p_size_or_error = input_data_view.size_or_error();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ComputeHttpCacheSize response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_is_upper_bound), 
std::move(p_size_or_error));
  return true;
}

void NetworkContext_ComputeHttpCacheSize_ProxyToResponder::Run(
    bool in_is_upper_bound, int64_t in_size_or_error) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ComputeHttpCacheSize_Response_Message::Build(kSerialize, is_sync_, std::move(in_is_upper_bound), std::move(in_size_or_error));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ComputeHttpCacheSizeCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ComputeHttpCacheSize");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_ClearChannelIds_ProxyToResponder {
 public:
  static NetworkContext::ClearChannelIdsCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ClearChannelIds_ProxyToResponder> proxy(
        new NetworkContext_ClearChannelIds_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ClearChannelIds_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ClearChannelIds_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ClearChannelIds_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ClearChannelIdsCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearChannelIds_ProxyToResponder);
};
class NetworkContext_ClearChannelIds_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ClearChannelIds_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearChannelIds_Name,
          message_flags){}
  ~NetworkContext_ClearChannelIds_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ClearChannelIds_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearChannelIds_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearChannelIds_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ClearChannelIdsCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearChannelIds_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearChannelIds_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ClearChannelIds_Response_Message::kMessageTag = {};

bool NetworkContext_ClearChannelIds_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ClearChannelIdsCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ClearChannelIds_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ClearChannelIds_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ClearChannelIds_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ClearChannelIds_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ClearChannelIds response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_ClearChannelIds_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ClearChannelIds_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearChannelIdsCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearChannelIds");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_ClearHostCache_ProxyToResponder {
 public:
  static NetworkContext::ClearHostCacheCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ClearHostCache_ProxyToResponder> proxy(
        new NetworkContext_ClearHostCache_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ClearHostCache_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ClearHostCache_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ClearHostCache_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ClearHostCacheCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearHostCache_ProxyToResponder);
};
class NetworkContext_ClearHostCache_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ClearHostCache_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearHostCache_Name,
          message_flags){}
  ~NetworkContext_ClearHostCache_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ClearHostCache_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearHostCache_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearHostCache_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ClearHostCacheCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearHostCache_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearHostCache_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ClearHostCache_Response_Message::kMessageTag = {};

bool NetworkContext_ClearHostCache_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ClearHostCacheCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ClearHostCache_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ClearHostCache_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ClearHostCache_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ClearHostCache_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ClearHostCache response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_ClearHostCache_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ClearHostCache_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearHostCacheCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearHostCache");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_ClearHttpAuthCache_ProxyToResponder {
 public:
  static NetworkContext::ClearHttpAuthCacheCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ClearHttpAuthCache_ProxyToResponder> proxy(
        new NetworkContext_ClearHttpAuthCache_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ClearHttpAuthCache_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ClearHttpAuthCache_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ClearHttpAuthCache_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ClearHttpAuthCacheCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearHttpAuthCache_ProxyToResponder);
};
class NetworkContext_ClearHttpAuthCache_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ClearHttpAuthCache_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearHttpAuthCache_Name,
          message_flags){}
  ~NetworkContext_ClearHttpAuthCache_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ClearHttpAuthCache_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearHttpAuthCache_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearHttpAuthCache_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ClearHttpAuthCacheCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearHttpAuthCache_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearHttpAuthCache_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ClearHttpAuthCache_Response_Message::kMessageTag = {};

bool NetworkContext_ClearHttpAuthCache_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ClearHttpAuthCacheCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ClearHttpAuthCache_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ClearHttpAuthCache_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ClearHttpAuthCache_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ClearHttpAuthCache_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ClearHttpAuthCache response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_ClearHttpAuthCache_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ClearHttpAuthCache_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearHttpAuthCacheCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearHttpAuthCache");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_ClearReportingCacheReports_ProxyToResponder {
 public:
  static NetworkContext::ClearReportingCacheReportsCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ClearReportingCacheReports_ProxyToResponder> proxy(
        new NetworkContext_ClearReportingCacheReports_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ClearReportingCacheReports_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ClearReportingCacheReports_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ClearReportingCacheReports_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ClearReportingCacheReportsCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearReportingCacheReports_ProxyToResponder);
};
class NetworkContext_ClearReportingCacheReports_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ClearReportingCacheReports_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearReportingCacheReports_Name,
          message_flags){}
  ~NetworkContext_ClearReportingCacheReports_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ClearReportingCacheReports_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearReportingCacheReports_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearReportingCacheReports_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ClearReportingCacheReportsCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearReportingCacheReports_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearReportingCacheReports_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ClearReportingCacheReports_Response_Message::kMessageTag = {};

bool NetworkContext_ClearReportingCacheReports_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ClearReportingCacheReportsCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ClearReportingCacheReports_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ClearReportingCacheReports_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ClearReportingCacheReports_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ClearReportingCacheReports_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ClearReportingCacheReports response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_ClearReportingCacheReports_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ClearReportingCacheReports_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearReportingCacheReportsCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearReportingCacheReports");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_ClearReportingCacheClients_ProxyToResponder {
 public:
  static NetworkContext::ClearReportingCacheClientsCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ClearReportingCacheClients_ProxyToResponder> proxy(
        new NetworkContext_ClearReportingCacheClients_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ClearReportingCacheClients_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ClearReportingCacheClients_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ClearReportingCacheClients_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ClearReportingCacheClientsCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearReportingCacheClients_ProxyToResponder);
};
class NetworkContext_ClearReportingCacheClients_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ClearReportingCacheClients_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearReportingCacheClients_Name,
          message_flags){}
  ~NetworkContext_ClearReportingCacheClients_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ClearReportingCacheClients_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearReportingCacheClients_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearReportingCacheClients_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ClearReportingCacheClientsCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearReportingCacheClients_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearReportingCacheClients_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ClearReportingCacheClients_Response_Message::kMessageTag = {};

bool NetworkContext_ClearReportingCacheClients_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ClearReportingCacheClientsCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ClearReportingCacheClients_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ClearReportingCacheClients_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ClearReportingCacheClients_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ClearReportingCacheClients_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ClearReportingCacheClients response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_ClearReportingCacheClients_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ClearReportingCacheClients_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearReportingCacheClientsCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearReportingCacheClients");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_ClearNetworkErrorLogging_ProxyToResponder {
 public:
  static NetworkContext::ClearNetworkErrorLoggingCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ClearNetworkErrorLogging_ProxyToResponder> proxy(
        new NetworkContext_ClearNetworkErrorLogging_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ClearNetworkErrorLogging_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ClearNetworkErrorLogging_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ClearNetworkErrorLogging_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ClearNetworkErrorLoggingCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearNetworkErrorLogging_ProxyToResponder);
};
class NetworkContext_ClearNetworkErrorLogging_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ClearNetworkErrorLogging_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearNetworkErrorLogging_Name,
          message_flags){}
  ~NetworkContext_ClearNetworkErrorLogging_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ClearNetworkErrorLogging_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearNetworkErrorLogging_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearNetworkErrorLogging_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ClearNetworkErrorLoggingCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearNetworkErrorLogging_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearNetworkErrorLogging_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ClearNetworkErrorLogging_Response_Message::kMessageTag = {};

bool NetworkContext_ClearNetworkErrorLogging_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ClearNetworkErrorLoggingCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ClearNetworkErrorLogging_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ClearNetworkErrorLogging_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ClearNetworkErrorLogging_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ClearNetworkErrorLogging_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ClearNetworkErrorLogging response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_ClearNetworkErrorLogging_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ClearNetworkErrorLogging_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearNetworkErrorLoggingCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearNetworkErrorLogging");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_ClearDomainReliability_ProxyToResponder {
 public:
  static NetworkContext::ClearDomainReliabilityCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ClearDomainReliability_ProxyToResponder> proxy(
        new NetworkContext_ClearDomainReliability_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ClearDomainReliability_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ClearDomainReliability_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ClearDomainReliability_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ClearDomainReliabilityCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearDomainReliability_ProxyToResponder);
};
class NetworkContext_ClearDomainReliability_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ClearDomainReliability_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearDomainReliability_Name,
          message_flags){}
  ~NetworkContext_ClearDomainReliability_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ClearDomainReliability_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearDomainReliability_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearDomainReliability_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ClearDomainReliabilityCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearDomainReliability_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearDomainReliability_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ClearDomainReliability_Response_Message::kMessageTag = {};

bool NetworkContext_ClearDomainReliability_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ClearDomainReliabilityCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ClearDomainReliability_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ClearDomainReliability_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ClearDomainReliability_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ClearDomainReliability_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ClearDomainReliability response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_ClearDomainReliability_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ClearDomainReliability_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearDomainReliabilityCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearDomainReliability");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_GetDomainReliabilityJSON_ProxyToResponder {
 public:
  static NetworkContext::GetDomainReliabilityJSONCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_GetDomainReliabilityJSON_ProxyToResponder> proxy(
        new NetworkContext_GetDomainReliabilityJSON_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_GetDomainReliabilityJSON_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_GetDomainReliabilityJSON_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_GetDomainReliabilityJSON_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::GetDomainReliabilityJSONCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      base::Value in_data);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_GetDomainReliabilityJSON_ProxyToResponder);
};
class NetworkContext_GetDomainReliabilityJSON_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_GetDomainReliabilityJSON_Response_Message(
      uint32_t message_flags
      , base::Value param_data
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_GetDomainReliabilityJSON_Name,
          message_flags)
      , param_data_(std::move(param_data)){}
  ~NetworkContext_GetDomainReliabilityJSON_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      base::Value param_data) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_GetDomainReliabilityJSON_Response_Message>(
          kFlags
          , std::move(param_data)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_GetDomainReliabilityJSON_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_GetDomainReliabilityJSON_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->data)::BufferWriter data_writer;
  data_writer.AllocateInline(buffer, &params->data);
  mojo::internal::Serialize<::mojo_base::mojom::ValueDataView>(
      param_data, buffer, &data_writer, true, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->data.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null data in NetworkContext.GetDomainReliabilityJSON response");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::GetDomainReliabilityJSONCallback* callback) {
    std::move(*callback).Run(
        std::move(param_data_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_GetDomainReliabilityJSON_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->data)::BufferWriter data_writer;
  data_writer.AllocateInline(buffer, &params->data);
  mojo::internal::Serialize<::mojo_base::mojom::ValueDataView>(
      param_data_, buffer, &data_writer, true, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->data.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null data in NetworkContext.GetDomainReliabilityJSON response");
  }
  base::Value param_data_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_GetDomainReliabilityJSON_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_GetDomainReliabilityJSON_Response_Message::kMessageTag = {};

bool NetworkContext_GetDomainReliabilityJSON_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::GetDomainReliabilityJSONCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_GetDomainReliabilityJSON_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_GetDomainReliabilityJSON_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_GetDomainReliabilityJSON_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  base::Value p_data{};
  NetworkContext_GetDomainReliabilityJSON_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadData(&p_data))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::GetDomainReliabilityJSON response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_data));
  return true;
}

void NetworkContext_GetDomainReliabilityJSON_ProxyToResponder::Run(
    base::Value in_data) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_GetDomainReliabilityJSON_Response_Message::Build(kSerialize, is_sync_, std::move(in_data));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetDomainReliabilityJSONCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("GetDomainReliabilityJSON");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_CloseAllConnections_ProxyToResponder {
 public:
  static NetworkContext::CloseAllConnectionsCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_CloseAllConnections_ProxyToResponder> proxy(
        new NetworkContext_CloseAllConnections_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_CloseAllConnections_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_CloseAllConnections_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_CloseAllConnections_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::CloseAllConnectionsCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CloseAllConnections_ProxyToResponder);
};
class NetworkContext_CloseAllConnections_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_CloseAllConnections_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CloseAllConnections_Name,
          message_flags){}
  ~NetworkContext_CloseAllConnections_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_CloseAllConnections_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CloseAllConnections_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CloseAllConnections_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::CloseAllConnectionsCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CloseAllConnections_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CloseAllConnections_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_CloseAllConnections_Response_Message::kMessageTag = {};

bool NetworkContext_CloseAllConnections_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::CloseAllConnectionsCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_CloseAllConnections_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_CloseAllConnections_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_CloseAllConnections_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_CloseAllConnections_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::CloseAllConnections response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_CloseAllConnections_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_CloseAllConnections_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CloseAllConnectionsCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CloseAllConnections");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_CloseIdleConnections_ProxyToResponder {
 public:
  static NetworkContext::CloseIdleConnectionsCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_CloseIdleConnections_ProxyToResponder> proxy(
        new NetworkContext_CloseIdleConnections_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_CloseIdleConnections_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_CloseIdleConnections_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_CloseIdleConnections_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::CloseIdleConnectionsCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CloseIdleConnections_ProxyToResponder);
};
class NetworkContext_CloseIdleConnections_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_CloseIdleConnections_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CloseIdleConnections_Name,
          message_flags){}
  ~NetworkContext_CloseIdleConnections_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_CloseIdleConnections_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CloseIdleConnections_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CloseIdleConnections_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::CloseIdleConnectionsCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CloseIdleConnections_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CloseIdleConnections_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_CloseIdleConnections_Response_Message::kMessageTag = {};

bool NetworkContext_CloseIdleConnections_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::CloseIdleConnectionsCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_CloseIdleConnections_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_CloseIdleConnections_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_CloseIdleConnections_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_CloseIdleConnections_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::CloseIdleConnections response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_CloseIdleConnections_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_CloseIdleConnections_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CloseIdleConnectionsCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CloseIdleConnections");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_AddExpectCT_ProxyToResponder {
 public:
  static NetworkContext::AddExpectCTCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_AddExpectCT_ProxyToResponder> proxy(
        new NetworkContext_AddExpectCT_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_AddExpectCT_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_AddExpectCT_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_AddExpectCT_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::AddExpectCTCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      bool in_success);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_AddExpectCT_ProxyToResponder);
};
class NetworkContext_AddExpectCT_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_AddExpectCT_Response_Message(
      uint32_t message_flags
      , bool param_success
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_AddExpectCT_Name,
          message_flags)
      , param_success_(std::move(param_success)){}
  ~NetworkContext_AddExpectCT_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      bool param_success) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_AddExpectCT_Response_Message>(
          kFlags
          , std::move(param_success)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_AddExpectCT_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_AddExpectCT_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->success = param_success;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::AddExpectCTCallback* callback) {
    std::move(*callback).Run(
        std::move(param_success_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_AddExpectCT_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->success = param_success_;
  }
  bool param_success_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_AddExpectCT_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_AddExpectCT_Response_Message::kMessageTag = {};

bool NetworkContext_AddExpectCT_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::AddExpectCTCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_AddExpectCT_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_AddExpectCT_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_AddExpectCT_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  bool p_success{};
  NetworkContext_AddExpectCT_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_success = input_data_view.success();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::AddExpectCT response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_success));
  return true;
}

void NetworkContext_AddExpectCT_ProxyToResponder::Run(
    bool in_success) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_AddExpectCT_Response_Message::Build(kSerialize, is_sync_, std::move(in_success));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::AddExpectCTCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("AddExpectCT");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_SetExpectCTTestReport_ProxyToResponder {
 public:
  static NetworkContext::SetExpectCTTestReportCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_SetExpectCTTestReport_ProxyToResponder> proxy(
        new NetworkContext_SetExpectCTTestReport_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_SetExpectCTTestReport_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_SetExpectCTTestReport_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_SetExpectCTTestReport_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::SetExpectCTTestReportCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      bool in_success);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_SetExpectCTTestReport_ProxyToResponder);
};
class NetworkContext_SetExpectCTTestReport_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_SetExpectCTTestReport_Response_Message(
      uint32_t message_flags
      , bool param_success
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_SetExpectCTTestReport_Name,
          message_flags)
      , param_success_(std::move(param_success)){}
  ~NetworkContext_SetExpectCTTestReport_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      bool param_success) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_SetExpectCTTestReport_Response_Message>(
          kFlags
          , std::move(param_success)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_SetExpectCTTestReport_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_SetExpectCTTestReport_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->success = param_success;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::SetExpectCTTestReportCallback* callback) {
    std::move(*callback).Run(
        std::move(param_success_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_SetExpectCTTestReport_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->success = param_success_;
  }
  bool param_success_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_SetExpectCTTestReport_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_SetExpectCTTestReport_Response_Message::kMessageTag = {};

bool NetworkContext_SetExpectCTTestReport_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::SetExpectCTTestReportCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_SetExpectCTTestReport_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_SetExpectCTTestReport_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_SetExpectCTTestReport_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  bool p_success{};
  NetworkContext_SetExpectCTTestReport_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_success = input_data_view.success();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::SetExpectCTTestReport response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_success));
  return true;
}

void NetworkContext_SetExpectCTTestReport_ProxyToResponder::Run(
    bool in_success) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_SetExpectCTTestReport_Response_Message::Build(kSerialize, is_sync_, std::move(in_success));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetExpectCTTestReportCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("SetExpectCTTestReport");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_GetExpectCTState_ProxyToResponder {
 public:
  static NetworkContext::GetExpectCTStateCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_GetExpectCTState_ProxyToResponder> proxy(
        new NetworkContext_GetExpectCTState_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_GetExpectCTState_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_GetExpectCTState_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_GetExpectCTState_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::GetExpectCTStateCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      base::Value in_state);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_GetExpectCTState_ProxyToResponder);
};
class NetworkContext_GetExpectCTState_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_GetExpectCTState_Response_Message(
      uint32_t message_flags
      , base::Value param_state
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_GetExpectCTState_Name,
          message_flags)
      , param_state_(std::move(param_state)){}
  ~NetworkContext_GetExpectCTState_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      base::Value param_state) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_GetExpectCTState_Response_Message>(
          kFlags
          , std::move(param_state)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_GetExpectCTState_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_GetExpectCTState_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->state)::BaseType::BufferWriter
      state_writer;
  mojo::internal::Serialize<::mojo_base::mojom::DictionaryValueDataView>(
      param_state, buffer, &state_writer, &serialization_context);
  params->state.Set(
      state_writer.is_null() ? nullptr : state_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->state.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null state in NetworkContext.GetExpectCTState response");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::GetExpectCTStateCallback* callback) {
    std::move(*callback).Run(
        std::move(param_state_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_GetExpectCTState_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->state)::BaseType::BufferWriter
      state_writer;
  mojo::internal::Serialize<::mojo_base::mojom::DictionaryValueDataView>(
      param_state_, buffer, &state_writer, serialization_context);
  params->state.Set(
      state_writer.is_null() ? nullptr : state_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->state.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null state in NetworkContext.GetExpectCTState response");
  }
  base::Value param_state_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_GetExpectCTState_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_GetExpectCTState_Response_Message::kMessageTag = {};

bool NetworkContext_GetExpectCTState_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::GetExpectCTStateCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_GetExpectCTState_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_GetExpectCTState_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_GetExpectCTState_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  base::Value p_state{};
  NetworkContext_GetExpectCTState_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadState(&p_state))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::GetExpectCTState response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_state));
  return true;
}

void NetworkContext_GetExpectCTState_ProxyToResponder::Run(
    base::Value in_state) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_GetExpectCTState_Response_Message::Build(kSerialize, is_sync_, std::move(in_state));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetExpectCTStateCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("GetExpectCTState");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_CreateTCPServerSocket_ProxyToResponder {
 public:
  static NetworkContext::CreateTCPServerSocketCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_CreateTCPServerSocket_ProxyToResponder> proxy(
        new NetworkContext_CreateTCPServerSocket_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_CreateTCPServerSocket_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_CreateTCPServerSocket_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_CreateTCPServerSocket_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::CreateTCPServerSocketCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      int32_t in_result, const base::Optional<net::IPEndPoint>& in_local_addr_out);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPServerSocket_ProxyToResponder);
};
class NetworkContext_CreateTCPServerSocket_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_CreateTCPServerSocket_Response_Message(
      uint32_t message_flags
      , int32_t param_result
      , const base::Optional<net::IPEndPoint>& param_local_addr_out
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateTCPServerSocket_Name,
          message_flags)
      , param_result_(std::move(param_result))
      , param_local_addr_out_(std::move(param_local_addr_out)){}
  ~NetworkContext_CreateTCPServerSocket_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      int32_t param_result, const base::Optional<net::IPEndPoint>& param_local_addr_out) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_CreateTCPServerSocket_Response_Message>(
          kFlags
          , std::move(param_result)
          , std::move(param_local_addr_out)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateTCPServerSocket_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateTCPServerSocket_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->result = param_result;
  typename decltype(params->local_addr_out)::BaseType::BufferWriter
      local_addr_out_writer;
  mojo::internal::Serialize<::network::mojom::IPEndPointDataView>(
      param_local_addr_out, buffer, &local_addr_out_writer, &serialization_context);
  params->local_addr_out.Set(
      local_addr_out_writer.is_null() ? nullptr : local_addr_out_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::CreateTCPServerSocketCallback* callback) {
    std::move(*callback).Run(
        std::move(param_result_), 
        std::move(param_local_addr_out_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateTCPServerSocket_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->result = param_result_;
  typename decltype(params->local_addr_out)::BaseType::BufferWriter
      local_addr_out_writer;
  mojo::internal::Serialize<::network::mojom::IPEndPointDataView>(
      param_local_addr_out_, buffer, &local_addr_out_writer, serialization_context);
  params->local_addr_out.Set(
      local_addr_out_writer.is_null() ? nullptr : local_addr_out_writer.data());
  }
  int32_t param_result_;
  base::Optional<net::IPEndPoint> param_local_addr_out_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPServerSocket_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_CreateTCPServerSocket_Response_Message::kMessageTag = {};

bool NetworkContext_CreateTCPServerSocket_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::CreateTCPServerSocketCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_CreateTCPServerSocket_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_CreateTCPServerSocket_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_CreateTCPServerSocket_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_result{};
  base::Optional<net::IPEndPoint> p_local_addr_out{};
  NetworkContext_CreateTCPServerSocket_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_result = input_data_view.result();
  if (!input_data_view.ReadLocalAddrOut(&p_local_addr_out))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::CreateTCPServerSocket response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_result), 
std::move(p_local_addr_out));
  return true;
}

void NetworkContext_CreateTCPServerSocket_ProxyToResponder::Run(
    int32_t in_result, const base::Optional<net::IPEndPoint>& in_local_addr_out) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_CreateTCPServerSocket_Response_Message::Build(kSerialize, is_sync_, std::move(in_result), std::move(in_local_addr_out));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPServerSocketCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CreateTCPServerSocket");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_CreateTCPConnectedSocket_ProxyToResponder {
 public:
  static NetworkContext::CreateTCPConnectedSocketCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_CreateTCPConnectedSocket_ProxyToResponder> proxy(
        new NetworkContext_CreateTCPConnectedSocket_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_CreateTCPConnectedSocket_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_CreateTCPConnectedSocket_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_CreateTCPConnectedSocket_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::CreateTCPConnectedSocketCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      int32_t in_result, const base::Optional<net::IPEndPoint>& in_local_addr, const base::Optional<net::IPEndPoint>& in_peer_addr, mojo::ScopedDataPipeConsumerHandle in_receive_stream, mojo::ScopedDataPipeProducerHandle in_send_stream);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPConnectedSocket_ProxyToResponder);
};
class NetworkContext_CreateTCPConnectedSocket_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_CreateTCPConnectedSocket_Response_Message(
      uint32_t message_flags
      , int32_t param_result
      , const base::Optional<net::IPEndPoint>& param_local_addr
      , const base::Optional<net::IPEndPoint>& param_peer_addr
      , mojo::ScopedDataPipeConsumerHandle param_receive_stream
      , mojo::ScopedDataPipeProducerHandle param_send_stream
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateTCPConnectedSocket_Name,
          message_flags)
      , param_result_(std::move(param_result))
      , param_local_addr_(std::move(param_local_addr))
      , param_peer_addr_(std::move(param_peer_addr))
      , param_receive_stream_(std::move(param_receive_stream))
      , param_send_stream_(std::move(param_send_stream)){}
  ~NetworkContext_CreateTCPConnectedSocket_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      int32_t param_result, const base::Optional<net::IPEndPoint>& param_local_addr, const base::Optional<net::IPEndPoint>& param_peer_addr, mojo::ScopedDataPipeConsumerHandle param_receive_stream, mojo::ScopedDataPipeProducerHandle param_send_stream) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_CreateTCPConnectedSocket_Response_Message>(
          kFlags
          , std::move(param_result)
          , std::move(param_local_addr)
          , std::move(param_peer_addr)
          , std::move(param_receive_stream)
          , std::move(param_send_stream)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateTCPConnectedSocket_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateTCPConnectedSocket_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->result = param_result;
  typename decltype(params->local_addr)::BaseType::BufferWriter
      local_addr_writer;
  mojo::internal::Serialize<::network::mojom::IPEndPointDataView>(
      param_local_addr, buffer, &local_addr_writer, &serialization_context);
  params->local_addr.Set(
      local_addr_writer.is_null() ? nullptr : local_addr_writer.data());
  typename decltype(params->peer_addr)::BaseType::BufferWriter
      peer_addr_writer;
  mojo::internal::Serialize<::network::mojom::IPEndPointDataView>(
      param_peer_addr, buffer, &peer_addr_writer, &serialization_context);
  params->peer_addr.Set(
      peer_addr_writer.is_null() ? nullptr : peer_addr_writer.data());
  mojo::internal::Serialize<mojo::ScopedDataPipeConsumerHandle>(
      param_receive_stream, &params->receive_stream, &serialization_context);
  mojo::internal::Serialize<mojo::ScopedDataPipeProducerHandle>(
      param_send_stream, &params->send_stream, &serialization_context);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::CreateTCPConnectedSocketCallback* callback) {
    std::move(*callback).Run(
        std::move(param_result_), 
        std::move(param_local_addr_), 
        std::move(param_peer_addr_), 
        std::move(param_receive_stream_), 
        std::move(param_send_stream_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateTCPConnectedSocket_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->result = param_result_;
  typename decltype(params->local_addr)::BaseType::BufferWriter
      local_addr_writer;
  mojo::internal::Serialize<::network::mojom::IPEndPointDataView>(
      param_local_addr_, buffer, &local_addr_writer, serialization_context);
  params->local_addr.Set(
      local_addr_writer.is_null() ? nullptr : local_addr_writer.data());
  typename decltype(params->peer_addr)::BaseType::BufferWriter
      peer_addr_writer;
  mojo::internal::Serialize<::network::mojom::IPEndPointDataView>(
      param_peer_addr_, buffer, &peer_addr_writer, serialization_context);
  params->peer_addr.Set(
      peer_addr_writer.is_null() ? nullptr : peer_addr_writer.data());
  mojo::internal::Serialize<mojo::ScopedDataPipeConsumerHandle>(
      param_receive_stream_, &params->receive_stream, serialization_context);
  mojo::internal::Serialize<mojo::ScopedDataPipeProducerHandle>(
      param_send_stream_, &params->send_stream, serialization_context);
  }
  int32_t param_result_;
  base::Optional<net::IPEndPoint> param_local_addr_;
  base::Optional<net::IPEndPoint> param_peer_addr_;
  mojo::ScopedDataPipeConsumerHandle param_receive_stream_;
  mojo::ScopedDataPipeProducerHandle param_send_stream_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPConnectedSocket_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_CreateTCPConnectedSocket_Response_Message::kMessageTag = {};

bool NetworkContext_CreateTCPConnectedSocket_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::CreateTCPConnectedSocketCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_CreateTCPConnectedSocket_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_CreateTCPConnectedSocket_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_CreateTCPConnectedSocket_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_result{};
  base::Optional<net::IPEndPoint> p_local_addr{};
  base::Optional<net::IPEndPoint> p_peer_addr{};
  mojo::ScopedDataPipeConsumerHandle p_receive_stream{};
  mojo::ScopedDataPipeProducerHandle p_send_stream{};
  NetworkContext_CreateTCPConnectedSocket_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_result = input_data_view.result();
  if (!input_data_view.ReadLocalAddr(&p_local_addr))
    success = false;
  if (!input_data_view.ReadPeerAddr(&p_peer_addr))
    success = false;
  p_receive_stream = input_data_view.TakeReceiveStream();
  p_send_stream = input_data_view.TakeSendStream();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::CreateTCPConnectedSocket response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_result), 
std::move(p_local_addr), 
std::move(p_peer_addr), 
std::move(p_receive_stream), 
std::move(p_send_stream));
  return true;
}

void NetworkContext_CreateTCPConnectedSocket_ProxyToResponder::Run(
    int32_t in_result, const base::Optional<net::IPEndPoint>& in_local_addr, const base::Optional<net::IPEndPoint>& in_peer_addr, mojo::ScopedDataPipeConsumerHandle in_receive_stream, mojo::ScopedDataPipeProducerHandle in_send_stream) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_CreateTCPConnectedSocket_Response_Message::Build(kSerialize, is_sync_, std::move(in_result), std::move(in_local_addr), std::move(in_peer_addr), std::move(in_receive_stream), std::move(in_send_stream));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPConnectedSocketCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CreateTCPConnectedSocket");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_CreateTCPBoundSocket_ProxyToResponder {
 public:
  static NetworkContext::CreateTCPBoundSocketCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_CreateTCPBoundSocket_ProxyToResponder> proxy(
        new NetworkContext_CreateTCPBoundSocket_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_CreateTCPBoundSocket_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_CreateTCPBoundSocket_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_CreateTCPBoundSocket_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::CreateTCPBoundSocketCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      int32_t in_result, const base::Optional<net::IPEndPoint>& in_local_addr);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPBoundSocket_ProxyToResponder);
};
class NetworkContext_CreateTCPBoundSocket_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_CreateTCPBoundSocket_Response_Message(
      uint32_t message_flags
      , int32_t param_result
      , const base::Optional<net::IPEndPoint>& param_local_addr
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateTCPBoundSocket_Name,
          message_flags)
      , param_result_(std::move(param_result))
      , param_local_addr_(std::move(param_local_addr)){}
  ~NetworkContext_CreateTCPBoundSocket_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      int32_t param_result, const base::Optional<net::IPEndPoint>& param_local_addr) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_CreateTCPBoundSocket_Response_Message>(
          kFlags
          , std::move(param_result)
          , std::move(param_local_addr)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateTCPBoundSocket_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateTCPBoundSocket_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->result = param_result;
  typename decltype(params->local_addr)::BaseType::BufferWriter
      local_addr_writer;
  mojo::internal::Serialize<::network::mojom::IPEndPointDataView>(
      param_local_addr, buffer, &local_addr_writer, &serialization_context);
  params->local_addr.Set(
      local_addr_writer.is_null() ? nullptr : local_addr_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::CreateTCPBoundSocketCallback* callback) {
    std::move(*callback).Run(
        std::move(param_result_), 
        std::move(param_local_addr_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateTCPBoundSocket_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->result = param_result_;
  typename decltype(params->local_addr)::BaseType::BufferWriter
      local_addr_writer;
  mojo::internal::Serialize<::network::mojom::IPEndPointDataView>(
      param_local_addr_, buffer, &local_addr_writer, serialization_context);
  params->local_addr.Set(
      local_addr_writer.is_null() ? nullptr : local_addr_writer.data());
  }
  int32_t param_result_;
  base::Optional<net::IPEndPoint> param_local_addr_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPBoundSocket_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_CreateTCPBoundSocket_Response_Message::kMessageTag = {};

bool NetworkContext_CreateTCPBoundSocket_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::CreateTCPBoundSocketCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_CreateTCPBoundSocket_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_CreateTCPBoundSocket_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_CreateTCPBoundSocket_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_result{};
  base::Optional<net::IPEndPoint> p_local_addr{};
  NetworkContext_CreateTCPBoundSocket_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_result = input_data_view.result();
  if (!input_data_view.ReadLocalAddr(&p_local_addr))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::CreateTCPBoundSocket response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_result), 
std::move(p_local_addr));
  return true;
}

void NetworkContext_CreateTCPBoundSocket_ProxyToResponder::Run(
    int32_t in_result, const base::Optional<net::IPEndPoint>& in_local_addr) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_CreateTCPBoundSocket_Response_Message::Build(kSerialize, is_sync_, std::move(in_result), std::move(in_local_addr));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPBoundSocketCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("CreateTCPBoundSocket");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_ForceReloadProxyConfig_ProxyToResponder {
 public:
  static NetworkContext::ForceReloadProxyConfigCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ForceReloadProxyConfig_ProxyToResponder> proxy(
        new NetworkContext_ForceReloadProxyConfig_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ForceReloadProxyConfig_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ForceReloadProxyConfig_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ForceReloadProxyConfig_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ForceReloadProxyConfigCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ForceReloadProxyConfig_ProxyToResponder);
};
class NetworkContext_ForceReloadProxyConfig_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ForceReloadProxyConfig_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ForceReloadProxyConfig_Name,
          message_flags){}
  ~NetworkContext_ForceReloadProxyConfig_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ForceReloadProxyConfig_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ForceReloadProxyConfig_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ForceReloadProxyConfig_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ForceReloadProxyConfigCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ForceReloadProxyConfig_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ForceReloadProxyConfig_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ForceReloadProxyConfig_Response_Message::kMessageTag = {};

bool NetworkContext_ForceReloadProxyConfig_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ForceReloadProxyConfigCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ForceReloadProxyConfig_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ForceReloadProxyConfig_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ForceReloadProxyConfig_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ForceReloadProxyConfig_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ForceReloadProxyConfig response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_ForceReloadProxyConfig_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ForceReloadProxyConfig_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ForceReloadProxyConfigCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ForceReloadProxyConfig");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_ClearBadProxiesCache_ProxyToResponder {
 public:
  static NetworkContext::ClearBadProxiesCacheCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ClearBadProxiesCache_ProxyToResponder> proxy(
        new NetworkContext_ClearBadProxiesCache_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ClearBadProxiesCache_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ClearBadProxiesCache_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ClearBadProxiesCache_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ClearBadProxiesCacheCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearBadProxiesCache_ProxyToResponder);
};
class NetworkContext_ClearBadProxiesCache_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ClearBadProxiesCache_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearBadProxiesCache_Name,
          message_flags){}
  ~NetworkContext_ClearBadProxiesCache_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ClearBadProxiesCache_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearBadProxiesCache_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearBadProxiesCache_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ClearBadProxiesCacheCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearBadProxiesCache_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearBadProxiesCache_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ClearBadProxiesCache_Response_Message::kMessageTag = {};

bool NetworkContext_ClearBadProxiesCache_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ClearBadProxiesCacheCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ClearBadProxiesCache_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ClearBadProxiesCache_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ClearBadProxiesCache_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ClearBadProxiesCache_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ClearBadProxiesCache response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_ClearBadProxiesCache_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ClearBadProxiesCache_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearBadProxiesCacheCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ClearBadProxiesCache");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_VerifyCertForSignedExchange_ProxyToResponder {
 public:
  static NetworkContext::VerifyCertForSignedExchangeCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_VerifyCertForSignedExchange_ProxyToResponder> proxy(
        new NetworkContext_VerifyCertForSignedExchange_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_VerifyCertForSignedExchange_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_VerifyCertForSignedExchange_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_VerifyCertForSignedExchange_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::VerifyCertForSignedExchangeCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      int32_t in_error_code, const net::CertVerifyResult& in_cv_result, const net::ct::CTVerifyResult& in_ct_result);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_VerifyCertForSignedExchange_ProxyToResponder);
};
class NetworkContext_VerifyCertForSignedExchange_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_VerifyCertForSignedExchange_Response_Message(
      uint32_t message_flags
      , int32_t param_error_code
      , const net::CertVerifyResult& param_cv_result
      , const net::ct::CTVerifyResult& param_ct_result
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_VerifyCertForSignedExchange_Name,
          message_flags)
      , param_error_code_(std::move(param_error_code))
      , param_cv_result_(std::move(param_cv_result))
      , param_ct_result_(std::move(param_ct_result)){}
  ~NetworkContext_VerifyCertForSignedExchange_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      int32_t param_error_code, const net::CertVerifyResult& param_cv_result, const net::ct::CTVerifyResult& param_ct_result) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_VerifyCertForSignedExchange_Response_Message>(
          kFlags
          , std::move(param_error_code)
          , std::move(param_cv_result)
          , std::move(param_ct_result)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_VerifyCertForSignedExchange_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_VerifyCertForSignedExchange_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->error_code = param_error_code;
  typename decltype(params->cv_result)::BaseType::BufferWriter
      cv_result_writer;
  mojo::internal::Serialize<::network::mojom::CertVerifyResultDataView>(
      param_cv_result, buffer, &cv_result_writer, &serialization_context);
  params->cv_result.Set(
      cv_result_writer.is_null() ? nullptr : cv_result_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->cv_result.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null cv_result in NetworkContext.VerifyCertForSignedExchange response");
  typename decltype(params->ct_result)::BaseType::BufferWriter
      ct_result_writer;
  mojo::internal::Serialize<::network::mojom::CTVerifyResultDataView>(
      param_ct_result, buffer, &ct_result_writer, &serialization_context);
  params->ct_result.Set(
      ct_result_writer.is_null() ? nullptr : ct_result_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->ct_result.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null ct_result in NetworkContext.VerifyCertForSignedExchange response");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::VerifyCertForSignedExchangeCallback* callback) {
    std::move(*callback).Run(
        std::move(param_error_code_), 
        std::move(param_cv_result_), 
        std::move(param_ct_result_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_VerifyCertForSignedExchange_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->error_code = param_error_code_;
  typename decltype(params->cv_result)::BaseType::BufferWriter
      cv_result_writer;
  mojo::internal::Serialize<::network::mojom::CertVerifyResultDataView>(
      param_cv_result_, buffer, &cv_result_writer, serialization_context);
  params->cv_result.Set(
      cv_result_writer.is_null() ? nullptr : cv_result_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->cv_result.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null cv_result in NetworkContext.VerifyCertForSignedExchange response");
  typename decltype(params->ct_result)::BaseType::BufferWriter
      ct_result_writer;
  mojo::internal::Serialize<::network::mojom::CTVerifyResultDataView>(
      param_ct_result_, buffer, &ct_result_writer, serialization_context);
  params->ct_result.Set(
      ct_result_writer.is_null() ? nullptr : ct_result_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->ct_result.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null ct_result in NetworkContext.VerifyCertForSignedExchange response");
  }
  int32_t param_error_code_;
  net::CertVerifyResult param_cv_result_;
  net::ct::CTVerifyResult param_ct_result_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_VerifyCertForSignedExchange_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_VerifyCertForSignedExchange_Response_Message::kMessageTag = {};

bool NetworkContext_VerifyCertForSignedExchange_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::VerifyCertForSignedExchangeCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_VerifyCertForSignedExchange_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_VerifyCertForSignedExchange_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_VerifyCertForSignedExchange_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_error_code{};
  net::CertVerifyResult p_cv_result{};
  net::ct::CTVerifyResult p_ct_result{};
  NetworkContext_VerifyCertForSignedExchange_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_error_code = input_data_view.error_code();
  if (!input_data_view.ReadCvResult(&p_cv_result))
    success = false;
  if (!input_data_view.ReadCtResult(&p_ct_result))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::VerifyCertForSignedExchange response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_error_code), 
std::move(p_cv_result), 
std::move(p_ct_result));
  return true;
}

void NetworkContext_VerifyCertForSignedExchange_ProxyToResponder::Run(
    int32_t in_error_code, const net::CertVerifyResult& in_cv_result, const net::ct::CTVerifyResult& in_ct_result) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_VerifyCertForSignedExchange_Response_Message::Build(kSerialize, is_sync_, std::move(in_error_code), std::move(in_cv_result), std::move(in_ct_result));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::VerifyCertForSignedExchangeCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("VerifyCertForSignedExchange");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_AddHSTS_ProxyToResponder {
 public:
  static NetworkContext::AddHSTSCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_AddHSTS_ProxyToResponder> proxy(
        new NetworkContext_AddHSTS_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_AddHSTS_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_AddHSTS_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_AddHSTS_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::AddHSTSCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_AddHSTS_ProxyToResponder);
};
class NetworkContext_AddHSTS_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_AddHSTS_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_AddHSTS_Name,
          message_flags){}
  ~NetworkContext_AddHSTS_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_AddHSTS_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_AddHSTS_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_AddHSTS_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::AddHSTSCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_AddHSTS_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_AddHSTS_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_AddHSTS_Response_Message::kMessageTag = {};

bool NetworkContext_AddHSTS_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::AddHSTSCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_AddHSTS_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_AddHSTS_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_AddHSTS_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_AddHSTS_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::AddHSTS response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_AddHSTS_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_AddHSTS_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::AddHSTSCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("AddHSTS");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_IsHSTSActiveForHost_ProxyToResponder {
 public:
  static NetworkContext::IsHSTSActiveForHostCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_IsHSTSActiveForHost_ProxyToResponder> proxy(
        new NetworkContext_IsHSTSActiveForHost_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_IsHSTSActiveForHost_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_IsHSTSActiveForHost_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_IsHSTSActiveForHost_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::IsHSTSActiveForHostCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      bool in_result);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_IsHSTSActiveForHost_ProxyToResponder);
};
class NetworkContext_IsHSTSActiveForHost_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_IsHSTSActiveForHost_Response_Message(
      uint32_t message_flags
      , bool param_result
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_IsHSTSActiveForHost_Name,
          message_flags)
      , param_result_(std::move(param_result)){}
  ~NetworkContext_IsHSTSActiveForHost_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      bool param_result) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_IsHSTSActiveForHost_Response_Message>(
          kFlags
          , std::move(param_result)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_IsHSTSActiveForHost_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_IsHSTSActiveForHost_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->result = param_result;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::IsHSTSActiveForHostCallback* callback) {
    std::move(*callback).Run(
        std::move(param_result_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_IsHSTSActiveForHost_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->result = param_result_;
  }
  bool param_result_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_IsHSTSActiveForHost_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_IsHSTSActiveForHost_Response_Message::kMessageTag = {};

bool NetworkContext_IsHSTSActiveForHost_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::IsHSTSActiveForHostCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_IsHSTSActiveForHost_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_IsHSTSActiveForHost_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_IsHSTSActiveForHost_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  bool p_result{};
  NetworkContext_IsHSTSActiveForHost_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_result = input_data_view.result();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::IsHSTSActiveForHost response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_result));
  return true;
}

void NetworkContext_IsHSTSActiveForHost_ProxyToResponder::Run(
    bool in_result) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_IsHSTSActiveForHost_Response_Message::Build(kSerialize, is_sync_, std::move(in_result));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::IsHSTSActiveForHostCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("IsHSTSActiveForHost");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_GetHSTSState_ProxyToResponder {
 public:
  static NetworkContext::GetHSTSStateCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_GetHSTSState_ProxyToResponder> proxy(
        new NetworkContext_GetHSTSState_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_GetHSTSState_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_GetHSTSState_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_GetHSTSState_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::GetHSTSStateCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      base::Value in_state);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_GetHSTSState_ProxyToResponder);
};
class NetworkContext_GetHSTSState_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_GetHSTSState_Response_Message(
      uint32_t message_flags
      , base::Value param_state
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_GetHSTSState_Name,
          message_flags)
      , param_state_(std::move(param_state)){}
  ~NetworkContext_GetHSTSState_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      base::Value param_state) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_GetHSTSState_Response_Message>(
          kFlags
          , std::move(param_state)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_GetHSTSState_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_GetHSTSState_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->state)::BaseType::BufferWriter
      state_writer;
  mojo::internal::Serialize<::mojo_base::mojom::DictionaryValueDataView>(
      param_state, buffer, &state_writer, &serialization_context);
  params->state.Set(
      state_writer.is_null() ? nullptr : state_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->state.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null state in NetworkContext.GetHSTSState response");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::GetHSTSStateCallback* callback) {
    std::move(*callback).Run(
        std::move(param_state_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_GetHSTSState_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->state)::BaseType::BufferWriter
      state_writer;
  mojo::internal::Serialize<::mojo_base::mojom::DictionaryValueDataView>(
      param_state_, buffer, &state_writer, serialization_context);
  params->state.Set(
      state_writer.is_null() ? nullptr : state_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->state.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null state in NetworkContext.GetHSTSState response");
  }
  base::Value param_state_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_GetHSTSState_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_GetHSTSState_Response_Message::kMessageTag = {};

bool NetworkContext_GetHSTSState_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::GetHSTSStateCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_GetHSTSState_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_GetHSTSState_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_GetHSTSState_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  base::Value p_state{};
  NetworkContext_GetHSTSState_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadState(&p_state))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::GetHSTSState response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_state));
  return true;
}

void NetworkContext_GetHSTSState_ProxyToResponder::Run(
    base::Value in_state) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_GetHSTSState_Response_Message::Build(kSerialize, is_sync_, std::move(in_state));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetHSTSStateCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("GetHSTSState");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_SetCorsOriginAccessListsForOrigin_ProxyToResponder {
 public:
  static NetworkContext::SetCorsOriginAccessListsForOriginCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_SetCorsOriginAccessListsForOrigin_ProxyToResponder> proxy(
        new NetworkContext_SetCorsOriginAccessListsForOrigin_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_SetCorsOriginAccessListsForOrigin_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_SetCorsOriginAccessListsForOrigin_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_SetCorsOriginAccessListsForOrigin_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::SetCorsOriginAccessListsForOriginCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_SetCorsOriginAccessListsForOrigin_ProxyToResponder);
};
class NetworkContext_SetCorsOriginAccessListsForOrigin_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_SetCorsOriginAccessListsForOrigin_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_SetCorsOriginAccessListsForOrigin_Name,
          message_flags){}
  ~NetworkContext_SetCorsOriginAccessListsForOrigin_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_SetCorsOriginAccessListsForOrigin_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_SetCorsOriginAccessListsForOrigin_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_SetCorsOriginAccessListsForOrigin_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::SetCorsOriginAccessListsForOriginCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_SetCorsOriginAccessListsForOrigin_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_SetCorsOriginAccessListsForOrigin_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_SetCorsOriginAccessListsForOrigin_Response_Message::kMessageTag = {};

bool NetworkContext_SetCorsOriginAccessListsForOrigin_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::SetCorsOriginAccessListsForOriginCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_SetCorsOriginAccessListsForOrigin_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_SetCorsOriginAccessListsForOrigin_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_SetCorsOriginAccessListsForOrigin_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_SetCorsOriginAccessListsForOrigin_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::SetCorsOriginAccessListsForOrigin response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_SetCorsOriginAccessListsForOrigin_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_SetCorsOriginAccessListsForOrigin_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetCorsOriginAccessListsForOriginCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("SetCorsOriginAccessListsForOrigin");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_DeleteDynamicDataForHost_ProxyToResponder {
 public:
  static NetworkContext::DeleteDynamicDataForHostCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_DeleteDynamicDataForHost_ProxyToResponder> proxy(
        new NetworkContext_DeleteDynamicDataForHost_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_DeleteDynamicDataForHost_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_DeleteDynamicDataForHost_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_DeleteDynamicDataForHost_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::DeleteDynamicDataForHostCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      bool in_result);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_DeleteDynamicDataForHost_ProxyToResponder);
};
class NetworkContext_DeleteDynamicDataForHost_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_DeleteDynamicDataForHost_Response_Message(
      uint32_t message_flags
      , bool param_result
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_DeleteDynamicDataForHost_Name,
          message_flags)
      , param_result_(std::move(param_result)){}
  ~NetworkContext_DeleteDynamicDataForHost_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      bool param_result) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_DeleteDynamicDataForHost_Response_Message>(
          kFlags
          , std::move(param_result)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_DeleteDynamicDataForHost_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_DeleteDynamicDataForHost_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->result = param_result;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::DeleteDynamicDataForHostCallback* callback) {
    std::move(*callback).Run(
        std::move(param_result_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_DeleteDynamicDataForHost_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->result = param_result_;
  }
  bool param_result_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_DeleteDynamicDataForHost_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_DeleteDynamicDataForHost_Response_Message::kMessageTag = {};

bool NetworkContext_DeleteDynamicDataForHost_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::DeleteDynamicDataForHostCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_DeleteDynamicDataForHost_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_DeleteDynamicDataForHost_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_DeleteDynamicDataForHost_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  bool p_result{};
  NetworkContext_DeleteDynamicDataForHost_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_result = input_data_view.result();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::DeleteDynamicDataForHost response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_result));
  return true;
}

void NetworkContext_DeleteDynamicDataForHost_ProxyToResponder::Run(
    bool in_result) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_DeleteDynamicDataForHost_Response_Message::Build(kSerialize, is_sync_, std::move(in_result));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::DeleteDynamicDataForHostCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("DeleteDynamicDataForHost");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_LookupBasicAuthCredentials_ProxyToResponder {
 public:
  static NetworkContext::LookupBasicAuthCredentialsCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_LookupBasicAuthCredentials_ProxyToResponder> proxy(
        new NetworkContext_LookupBasicAuthCredentials_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_LookupBasicAuthCredentials_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_LookupBasicAuthCredentials_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_LookupBasicAuthCredentials_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::LookupBasicAuthCredentialsCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      const base::Optional<net::AuthCredentials>& in_credentials);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_LookupBasicAuthCredentials_ProxyToResponder);
};
class NetworkContext_LookupBasicAuthCredentials_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_LookupBasicAuthCredentials_Response_Message(
      uint32_t message_flags
      , const base::Optional<net::AuthCredentials>& param_credentials
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_LookupBasicAuthCredentials_Name,
          message_flags)
      , param_credentials_(std::move(param_credentials)){}
  ~NetworkContext_LookupBasicAuthCredentials_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      const base::Optional<net::AuthCredentials>& param_credentials) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_LookupBasicAuthCredentials_Response_Message>(
          kFlags
          , std::move(param_credentials)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_LookupBasicAuthCredentials_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_LookupBasicAuthCredentials_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->credentials)::BaseType::BufferWriter
      credentials_writer;
  mojo::internal::Serialize<::network::mojom::AuthCredentialsDataView>(
      param_credentials, buffer, &credentials_writer, &serialization_context);
  params->credentials.Set(
      credentials_writer.is_null() ? nullptr : credentials_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::LookupBasicAuthCredentialsCallback* callback) {
    std::move(*callback).Run(
        std::move(param_credentials_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_LookupBasicAuthCredentials_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->credentials)::BaseType::BufferWriter
      credentials_writer;
  mojo::internal::Serialize<::network::mojom::AuthCredentialsDataView>(
      param_credentials_, buffer, &credentials_writer, serialization_context);
  params->credentials.Set(
      credentials_writer.is_null() ? nullptr : credentials_writer.data());
  }
  base::Optional<net::AuthCredentials> param_credentials_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_LookupBasicAuthCredentials_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_LookupBasicAuthCredentials_Response_Message::kMessageTag = {};

bool NetworkContext_LookupBasicAuthCredentials_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::LookupBasicAuthCredentialsCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_LookupBasicAuthCredentials_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_LookupBasicAuthCredentials_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_LookupBasicAuthCredentials_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  base::Optional<net::AuthCredentials> p_credentials{};
  NetworkContext_LookupBasicAuthCredentials_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadCredentials(&p_credentials))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::LookupBasicAuthCredentials response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_credentials));
  return true;
}

void NetworkContext_LookupBasicAuthCredentials_ProxyToResponder::Run(
    const base::Optional<net::AuthCredentials>& in_credentials) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_LookupBasicAuthCredentials_Response_Message::Build(kSerialize, is_sync_, std::move(in_credentials));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::LookupBasicAuthCredentialsCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("LookupBasicAuthCredentials");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_EnableStaticKeyPinningForTesting_ProxyToResponder {
 public:
  static NetworkContext::EnableStaticKeyPinningForTestingCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_EnableStaticKeyPinningForTesting_ProxyToResponder> proxy(
        new NetworkContext_EnableStaticKeyPinningForTesting_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_EnableStaticKeyPinningForTesting_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_EnableStaticKeyPinningForTesting_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_EnableStaticKeyPinningForTesting_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::EnableStaticKeyPinningForTestingCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_EnableStaticKeyPinningForTesting_ProxyToResponder);
};
class NetworkContext_EnableStaticKeyPinningForTesting_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_EnableStaticKeyPinningForTesting_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_EnableStaticKeyPinningForTesting_Name,
          message_flags){}
  ~NetworkContext_EnableStaticKeyPinningForTesting_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_EnableStaticKeyPinningForTesting_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_EnableStaticKeyPinningForTesting_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_EnableStaticKeyPinningForTesting_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::EnableStaticKeyPinningForTestingCallback* callback) {
    std::move(*callback).Run();
  }


  void HandleSyncResponse(
) {

  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_EnableStaticKeyPinningForTesting_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_EnableStaticKeyPinningForTesting_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_EnableStaticKeyPinningForTesting_Response_Message::kMessageTag = {};

bool NetworkContext_EnableStaticKeyPinningForTesting_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::EnableStaticKeyPinningForTestingCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_EnableStaticKeyPinningForTesting_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_EnableStaticKeyPinningForTesting_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_EnableStaticKeyPinningForTesting_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_EnableStaticKeyPinningForTesting_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::EnableStaticKeyPinningForTesting response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_EnableStaticKeyPinningForTesting_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_EnableStaticKeyPinningForTesting_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::EnableStaticKeyPinningForTestingCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("EnableStaticKeyPinningForTesting");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
bool NetworkContext_EnableStaticKeyPinningForTesting_HandleSyncResponse::Accept(
    mojo::Message* message) {
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_EnableStaticKeyPinningForTesting_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was built using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      context->HandleSyncResponse();
      *result_ = true;
      mojo::internal::SyncMessageResponseSetup::SetCurrentSyncResponseMessage(
          message);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_EnableStaticKeyPinningForTesting_ResponseParams_Data* params =
      reinterpret_cast<internal::NetworkContext_EnableStaticKeyPinningForTesting_ResponseParams_Data*>(
          message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_EnableStaticKeyPinningForTesting_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::EnableStaticKeyPinningForTesting response deserializer");
    return false;
  }
  mojo::internal::SyncMessageResponseSetup::SetCurrentSyncResponseMessage(
      message);
  *result_ = true;
  return true;
}
class NetworkContext_SetFailingHttpTransactionForTesting_ProxyToResponder {
 public:
  static NetworkContext::SetFailingHttpTransactionForTestingCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_SetFailingHttpTransactionForTesting_ProxyToResponder> proxy(
        new NetworkContext_SetFailingHttpTransactionForTesting_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_SetFailingHttpTransactionForTesting_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_SetFailingHttpTransactionForTesting_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_SetFailingHttpTransactionForTesting_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::SetFailingHttpTransactionForTestingCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_SetFailingHttpTransactionForTesting_ProxyToResponder);
};
class NetworkContext_SetFailingHttpTransactionForTesting_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_SetFailingHttpTransactionForTesting_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_SetFailingHttpTransactionForTesting_Name,
          message_flags){}
  ~NetworkContext_SetFailingHttpTransactionForTesting_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_SetFailingHttpTransactionForTesting_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_SetFailingHttpTransactionForTesting_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_SetFailingHttpTransactionForTesting_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::SetFailingHttpTransactionForTestingCallback* callback) {
    std::move(*callback).Run();
  }


  void HandleSyncResponse(
) {

  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_SetFailingHttpTransactionForTesting_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_SetFailingHttpTransactionForTesting_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_SetFailingHttpTransactionForTesting_Response_Message::kMessageTag = {};

bool NetworkContext_SetFailingHttpTransactionForTesting_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::SetFailingHttpTransactionForTestingCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_SetFailingHttpTransactionForTesting_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_SetFailingHttpTransactionForTesting_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_SetFailingHttpTransactionForTesting_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_SetFailingHttpTransactionForTesting_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::SetFailingHttpTransactionForTesting response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_SetFailingHttpTransactionForTesting_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_SetFailingHttpTransactionForTesting_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetFailingHttpTransactionForTestingCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("SetFailingHttpTransactionForTesting");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
bool NetworkContext_SetFailingHttpTransactionForTesting_HandleSyncResponse::Accept(
    mojo::Message* message) {
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_SetFailingHttpTransactionForTesting_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was built using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      context->HandleSyncResponse();
      *result_ = true;
      mojo::internal::SyncMessageResponseSetup::SetCurrentSyncResponseMessage(
          message);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_SetFailingHttpTransactionForTesting_ResponseParams_Data* params =
      reinterpret_cast<internal::NetworkContext_SetFailingHttpTransactionForTesting_ResponseParams_Data*>(
          message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_SetFailingHttpTransactionForTesting_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::SetFailingHttpTransactionForTesting response deserializer");
    return false;
  }
  mojo::internal::SyncMessageResponseSetup::SetCurrentSyncResponseMessage(
      message);
  *result_ = true;
  return true;
}
class NetworkContext_VerifyCertificateForTesting_ProxyToResponder {
 public:
  static NetworkContext::VerifyCertificateForTestingCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_VerifyCertificateForTesting_ProxyToResponder> proxy(
        new NetworkContext_VerifyCertificateForTesting_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_VerifyCertificateForTesting_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_VerifyCertificateForTesting_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_VerifyCertificateForTesting_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::VerifyCertificateForTestingCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      int32_t in_error_code);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_VerifyCertificateForTesting_ProxyToResponder);
};
class NetworkContext_VerifyCertificateForTesting_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_VerifyCertificateForTesting_Response_Message(
      uint32_t message_flags
      , int32_t param_error_code
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_VerifyCertificateForTesting_Name,
          message_flags)
      , param_error_code_(std::move(param_error_code)){}
  ~NetworkContext_VerifyCertificateForTesting_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      int32_t param_error_code) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_VerifyCertificateForTesting_Response_Message>(
          kFlags
          , std::move(param_error_code)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_VerifyCertificateForTesting_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_VerifyCertificateForTesting_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->error_code = param_error_code;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::VerifyCertificateForTestingCallback* callback) {
    std::move(*callback).Run(
        std::move(param_error_code_));
  }


  void HandleSyncResponse(

      int32_t* out_error_code) {

    *out_error_code = std::move(param_error_code_);

  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_VerifyCertificateForTesting_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->error_code = param_error_code_;
  }
  int32_t param_error_code_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_VerifyCertificateForTesting_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_VerifyCertificateForTesting_Response_Message::kMessageTag = {};

bool NetworkContext_VerifyCertificateForTesting_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::VerifyCertificateForTestingCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_VerifyCertificateForTesting_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_VerifyCertificateForTesting_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_VerifyCertificateForTesting_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_error_code{};
  NetworkContext_VerifyCertificateForTesting_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_error_code = input_data_view.error_code();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::VerifyCertificateForTesting response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_error_code));
  return true;
}

void NetworkContext_VerifyCertificateForTesting_ProxyToResponder::Run(
    int32_t in_error_code) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_VerifyCertificateForTesting_Response_Message::Build(kSerialize, is_sync_, std::move(in_error_code));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::VerifyCertificateForTestingCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("VerifyCertificateForTesting");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
bool NetworkContext_VerifyCertificateForTesting_HandleSyncResponse::Accept(
    mojo::Message* message) {
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_VerifyCertificateForTesting_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was built using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      context->HandleSyncResponse(
          out_error_code_);
      *result_ = true;
      mojo::internal::SyncMessageResponseSetup::SetCurrentSyncResponseMessage(
          message);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_VerifyCertificateForTesting_ResponseParams_Data* params =
      reinterpret_cast<internal::NetworkContext_VerifyCertificateForTesting_ResponseParams_Data*>(
          message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_error_code{};
  NetworkContext_VerifyCertificateForTesting_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_error_code = input_data_view.error_code();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::VerifyCertificateForTesting response deserializer");
    return false;
  }
  *out_error_code_ = std::move(p_error_code);
  mojo::internal::SyncMessageResponseSetup::SetCurrentSyncResponseMessage(
      message);
  *result_ = true;
  return true;
}
class NetworkContext_AddDomainReliabilityContextForTesting_ProxyToResponder {
 public:
  static NetworkContext::AddDomainReliabilityContextForTestingCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_AddDomainReliabilityContextForTesting_ProxyToResponder> proxy(
        new NetworkContext_AddDomainReliabilityContextForTesting_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_AddDomainReliabilityContextForTesting_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_AddDomainReliabilityContextForTesting_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_AddDomainReliabilityContextForTesting_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::AddDomainReliabilityContextForTestingCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_AddDomainReliabilityContextForTesting_ProxyToResponder);
};
class NetworkContext_AddDomainReliabilityContextForTesting_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_AddDomainReliabilityContextForTesting_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_AddDomainReliabilityContextForTesting_Name,
          message_flags){}
  ~NetworkContext_AddDomainReliabilityContextForTesting_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_AddDomainReliabilityContextForTesting_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_AddDomainReliabilityContextForTesting_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_AddDomainReliabilityContextForTesting_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::AddDomainReliabilityContextForTestingCallback* callback) {
    std::move(*callback).Run();
  }


  void HandleSyncResponse(
) {

  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_AddDomainReliabilityContextForTesting_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_AddDomainReliabilityContextForTesting_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_AddDomainReliabilityContextForTesting_Response_Message::kMessageTag = {};

bool NetworkContext_AddDomainReliabilityContextForTesting_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::AddDomainReliabilityContextForTestingCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_AddDomainReliabilityContextForTesting_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_AddDomainReliabilityContextForTesting_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_AddDomainReliabilityContextForTesting_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_AddDomainReliabilityContextForTesting_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::AddDomainReliabilityContextForTesting response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_AddDomainReliabilityContextForTesting_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_AddDomainReliabilityContextForTesting_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::AddDomainReliabilityContextForTestingCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("AddDomainReliabilityContextForTesting");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
bool NetworkContext_AddDomainReliabilityContextForTesting_HandleSyncResponse::Accept(
    mojo::Message* message) {
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_AddDomainReliabilityContextForTesting_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was built using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      context->HandleSyncResponse();
      *result_ = true;
      mojo::internal::SyncMessageResponseSetup::SetCurrentSyncResponseMessage(
          message);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_AddDomainReliabilityContextForTesting_ResponseParams_Data* params =
      reinterpret_cast<internal::NetworkContext_AddDomainReliabilityContextForTesting_ResponseParams_Data*>(
          message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_AddDomainReliabilityContextForTesting_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::AddDomainReliabilityContextForTesting response deserializer");
    return false;
  }
  mojo::internal::SyncMessageResponseSetup::SetCurrentSyncResponseMessage(
      message);
  *result_ = true;
  return true;
}
class NetworkContext_ForceDomainReliabilityUploadsForTesting_ProxyToResponder {
 public:
  static NetworkContext::ForceDomainReliabilityUploadsForTestingCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ForceDomainReliabilityUploadsForTesting_ProxyToResponder> proxy(
        new NetworkContext_ForceDomainReliabilityUploadsForTesting_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ForceDomainReliabilityUploadsForTesting_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ForceDomainReliabilityUploadsForTesting_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ForceDomainReliabilityUploadsForTesting_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ForceDomainReliabilityUploadsForTestingCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ForceDomainReliabilityUploadsForTesting_ProxyToResponder);
};
class NetworkContext_ForceDomainReliabilityUploadsForTesting_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ForceDomainReliabilityUploadsForTesting_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ForceDomainReliabilityUploadsForTesting_Name,
          message_flags){}
  ~NetworkContext_ForceDomainReliabilityUploadsForTesting_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ForceDomainReliabilityUploadsForTesting_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ForceDomainReliabilityUploadsForTesting_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ForceDomainReliabilityUploadsForTestingCallback* callback) {
    std::move(*callback).Run();
  }


  void HandleSyncResponse(
) {

  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ForceDomainReliabilityUploadsForTesting_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ForceDomainReliabilityUploadsForTesting_Response_Message::kMessageTag = {};

bool NetworkContext_ForceDomainReliabilityUploadsForTesting_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ForceDomainReliabilityUploadsForTestingCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ForceDomainReliabilityUploadsForTesting_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ForceDomainReliabilityUploadsForTesting_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ForceDomainReliabilityUploadsForTesting response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_ForceDomainReliabilityUploadsForTesting_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ForceDomainReliabilityUploadsForTesting_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ForceDomainReliabilityUploadsForTestingCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(NetworkContext::Name_);
  message.set_method_name("ForceDomainReliabilityUploadsForTesting");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
bool NetworkContext_ForceDomainReliabilityUploadsForTesting_HandleSyncResponse::Accept(
    mojo::Message* message) {
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ForceDomainReliabilityUploadsForTesting_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was built using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      context->HandleSyncResponse();
      *result_ = true;
      mojo::internal::SyncMessageResponseSetup::SetCurrentSyncResponseMessage(
          message);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_ResponseParams_Data* params =
      reinterpret_cast<internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_ResponseParams_Data*>(
          message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ForceDomainReliabilityUploadsForTesting_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ForceDomainReliabilityUploadsForTesting response deserializer");
    return false;
  }
  mojo::internal::SyncMessageResponseSetup::SetCurrentSyncResponseMessage(
      message);
  *result_ = true;
  return true;
}

// static
bool NetworkContextStubDispatch::Accept(
    NetworkContext* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kNetworkContext_SetClient_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetClient",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_SetClient_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_SetClient_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_SetClient_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkContextClientPtr p_client{};
      NetworkContext_SetClient_ParamsDataView input_data_view(params, &serialization_context);
      
      p_client =
          input_data_view.TakeClient<decltype(p_client)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::SetClient deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetClient(
std::move(p_client));
      return true;
    }
    case internal::kNetworkContext_CreateURLLoaderFactory_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateURLLoaderFactory",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateURLLoaderFactory_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_CreateURLLoaderFactory_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_CreateURLLoaderFactory_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::URLLoaderFactoryRequest p_url_loader_factory{};
      URLLoaderFactoryParamsPtr p_params{};
      NetworkContext_CreateURLLoaderFactory_ParamsDataView input_data_view(params, &serialization_context);
      
      p_url_loader_factory =
          input_data_view.TakeUrlLoaderFactory<decltype(p_url_loader_factory)>();
      if (!input_data_view.ReadParams(&p_params))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateURLLoaderFactory deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateURLLoaderFactory(
std::move(p_url_loader_factory), 
std::move(p_params));
      return true;
    }
    case internal::kNetworkContext_ResetURLLoaderFactories_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ResetURLLoaderFactories",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ResetURLLoaderFactories_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_ResetURLLoaderFactories_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_ResetURLLoaderFactories_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkContext_ResetURLLoaderFactories_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ResetURLLoaderFactories deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ResetURLLoaderFactories();
      return true;
    }
    case internal::kNetworkContext_GetCookieManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetCookieManager",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_GetCookieManager_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_GetCookieManager_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_GetCookieManager_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::CookieManagerRequest p_cookie_manager{};
      NetworkContext_GetCookieManager_ParamsDataView input_data_view(params, &serialization_context);
      
      p_cookie_manager =
          input_data_view.TakeCookieManager<decltype(p_cookie_manager)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::GetCookieManager deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetCookieManager(
std::move(p_cookie_manager));
      return true;
    }
    case internal::kNetworkContext_GetRestrictedCookieManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetRestrictedCookieManager",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_GetRestrictedCookieManager_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_GetRestrictedCookieManager_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_GetRestrictedCookieManager_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::RestrictedCookieManagerRequest p_restricted_cookie_manager{};
      url::Origin p_origin{};
      NetworkContext_GetRestrictedCookieManager_ParamsDataView input_data_view(params, &serialization_context);
      
      p_restricted_cookie_manager =
          input_data_view.TakeRestrictedCookieManager<decltype(p_restricted_cookie_manager)>();
      if (!input_data_view.ReadOrigin(&p_origin))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::GetRestrictedCookieManager deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetRestrictedCookieManager(
std::move(p_restricted_cookie_manager), 
std::move(p_origin));
      return true;
    }
    case internal::kNetworkContext_ClearNetworkingHistorySince_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearNetworkingHistorySince",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ClearHttpCache_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearHttpCache",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ComputeHttpCacheSize_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ComputeHttpCacheSize",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_WriteCacheMetadata_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::WriteCacheMetadata",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_WriteCacheMetadata_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_WriteCacheMetadata_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_WriteCacheMetadata_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      GURL p_url{};
      net::RequestPriority p_priority{};
      base::Time p_expected_response_time{};
      std::vector<uint8_t> p_data{};
      NetworkContext_WriteCacheMetadata_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadUrl(&p_url))
        success = false;
      if (!input_data_view.ReadPriority(&p_priority))
        success = false;
      if (!input_data_view.ReadExpectedResponseTime(&p_expected_response_time))
        success = false;
      if (!input_data_view.ReadData(&p_data))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::WriteCacheMetadata deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->WriteCacheMetadata(
std::move(p_url), 
std::move(p_priority), 
std::move(p_expected_response_time), 
std::move(p_data));
      return true;
    }
    case internal::kNetworkContext_ClearChannelIds_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearChannelIds",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ClearHostCache_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearHostCache",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ClearHttpAuthCache_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearHttpAuthCache",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ClearReportingCacheReports_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearReportingCacheReports",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ClearReportingCacheClients_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearReportingCacheClients",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ClearNetworkErrorLogging_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearNetworkErrorLogging",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ClearDomainReliability_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearDomainReliability",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_GetDomainReliabilityJSON_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetDomainReliabilityJSON",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_QueueReport_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::QueueReport",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_QueueReport_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_QueueReport_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_QueueReport_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_type{};
      std::string p_group{};
      GURL p_url{};
      base::Optional<std::string> p_user_agent{};
      base::Value p_body{};
      NetworkContext_QueueReport_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadType(&p_type))
        success = false;
      if (!input_data_view.ReadGroup(&p_group))
        success = false;
      if (!input_data_view.ReadUrl(&p_url))
        success = false;
      if (!input_data_view.ReadUserAgent(&p_user_agent))
        success = false;
      if (!input_data_view.ReadBody(&p_body))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::QueueReport deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->QueueReport(
std::move(p_type), 
std::move(p_group), 
std::move(p_url), 
std::move(p_user_agent), 
std::move(p_body));
      return true;
    }
    case internal::kNetworkContext_CloseAllConnections_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CloseAllConnections",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CloseIdleConnections_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CloseIdleConnections",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_SetNetworkConditions_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetNetworkConditions",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_SetNetworkConditions_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_SetNetworkConditions_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_SetNetworkConditions_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      base::UnguessableToken p_throttling_profile_id{};
      NetworkConditionsPtr p_conditions{};
      NetworkContext_SetNetworkConditions_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadThrottlingProfileId(&p_throttling_profile_id))
        success = false;
      if (!input_data_view.ReadConditions(&p_conditions))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::SetNetworkConditions deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetNetworkConditions(
std::move(p_throttling_profile_id), 
std::move(p_conditions));
      return true;
    }
    case internal::kNetworkContext_SetAcceptLanguage_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetAcceptLanguage",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_SetAcceptLanguage_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_SetAcceptLanguage_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_SetAcceptLanguage_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_new_accept_language{};
      NetworkContext_SetAcceptLanguage_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadNewAcceptLanguage(&p_new_accept_language))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::SetAcceptLanguage deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetAcceptLanguage(
std::move(p_new_accept_language));
      return true;
    }
    case internal::kNetworkContext_SetEnableReferrers_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetEnableReferrers",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_SetEnableReferrers_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_SetEnableReferrers_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_SetEnableReferrers_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      bool p_enable_referrers{};
      NetworkContext_SetEnableReferrers_ParamsDataView input_data_view(params, &serialization_context);
      
      p_enable_referrers = input_data_view.enable_referrers();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::SetEnableReferrers deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetEnableReferrers(
std::move(p_enable_referrers));
      return true;
    }
    case internal::kNetworkContext_SetCTPolicy_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetCTPolicy",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_SetCTPolicy_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_SetCTPolicy_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_SetCTPolicy_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::vector<std::string> p_required_hosts{};
      std::vector<std::string> p_excluded_hosts{};
      std::vector<std::string> p_excluded_spkis{};
      std::vector<std::string> p_excluded_legacy_spkis{};
      NetworkContext_SetCTPolicy_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadRequiredHosts(&p_required_hosts))
        success = false;
      if (!input_data_view.ReadExcludedHosts(&p_excluded_hosts))
        success = false;
      if (!input_data_view.ReadExcludedSpkis(&p_excluded_spkis))
        success = false;
      if (!input_data_view.ReadExcludedLegacySpkis(&p_excluded_legacy_spkis))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::SetCTPolicy deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetCTPolicy(
std::move(p_required_hosts), 
std::move(p_excluded_hosts), 
std::move(p_excluded_spkis), 
std::move(p_excluded_legacy_spkis));
      return true;
    }
    case internal::kNetworkContext_AddExpectCT_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::AddExpectCT",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_SetExpectCTTestReport_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetExpectCTTestReport",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_GetExpectCTState_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetExpectCTState",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateUDPSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateUDPSocket",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateUDPSocket_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_CreateUDPSocket_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_CreateUDPSocket_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::UDPSocketRequest p_request{};
      ::network::mojom::UDPSocketReceiverPtr p_receiver{};
      NetworkContext_CreateUDPSocket_ParamsDataView input_data_view(params, &serialization_context);
      
      p_request =
          input_data_view.TakeRequest<decltype(p_request)>();
      p_receiver =
          input_data_view.TakeReceiver<decltype(p_receiver)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateUDPSocket deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateUDPSocket(
std::move(p_request), 
std::move(p_receiver));
      return true;
    }
    case internal::kNetworkContext_CreateTCPServerSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPServerSocket",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateTCPConnectedSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPConnectedSocket",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateTCPBoundSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPBoundSocket",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateProxyResolvingSocketFactory_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateProxyResolvingSocketFactory",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateProxyResolvingSocketFactory_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_CreateProxyResolvingSocketFactory_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_CreateProxyResolvingSocketFactory_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::ProxyResolvingSocketFactoryRequest p_factory{};
      NetworkContext_CreateProxyResolvingSocketFactory_ParamsDataView input_data_view(params, &serialization_context);
      
      p_factory =
          input_data_view.TakeFactory<decltype(p_factory)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateProxyResolvingSocketFactory deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateProxyResolvingSocketFactory(
std::move(p_factory));
      return true;
    }
    case internal::kNetworkContext_LookUpProxyForURL_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::LookUpProxyForURL",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_LookUpProxyForURL_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_LookUpProxyForURL_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_LookUpProxyForURL_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      GURL p_url{};
      ::network::mojom::ProxyLookupClientPtr p_proxy_lookup_client{};
      NetworkContext_LookUpProxyForURL_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadUrl(&p_url))
        success = false;
      p_proxy_lookup_client =
          input_data_view.TakeProxyLookupClient<decltype(p_proxy_lookup_client)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::LookUpProxyForURL deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->LookUpProxyForURL(
std::move(p_url), 
std::move(p_proxy_lookup_client));
      return true;
    }
    case internal::kNetworkContext_ForceReloadProxyConfig_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ForceReloadProxyConfig",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ClearBadProxiesCache_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearBadProxiesCache",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateWebSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateWebSocket",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateWebSocket_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_CreateWebSocket_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_CreateWebSocket_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::WebSocketRequest p_request{};
      int32_t p_process_id{};
      int32_t p_render_frame_id{};
      url::Origin p_origin{};
      ::network::mojom::AuthenticationHandlerPtr p_auth_handler{};
      NetworkContext_CreateWebSocket_ParamsDataView input_data_view(params, &serialization_context);
      
      p_request =
          input_data_view.TakeRequest<decltype(p_request)>();
      p_process_id = input_data_view.process_id();
      p_render_frame_id = input_data_view.render_frame_id();
      if (!input_data_view.ReadOrigin(&p_origin))
        success = false;
      p_auth_handler =
          input_data_view.TakeAuthHandler<decltype(p_auth_handler)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateWebSocket deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateWebSocket(
std::move(p_request), 
std::move(p_process_id), 
std::move(p_render_frame_id), 
std::move(p_origin), 
std::move(p_auth_handler));
      return true;
    }
    case internal::kNetworkContext_CreateNetLogExporter_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateNetLogExporter",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateNetLogExporter_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_CreateNetLogExporter_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_CreateNetLogExporter_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::NetLogExporterRequest p_exporter{};
      NetworkContext_CreateNetLogExporter_ParamsDataView input_data_view(params, &serialization_context);
      
      p_exporter =
          input_data_view.TakeExporter<decltype(p_exporter)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateNetLogExporter deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateNetLogExporter(
std::move(p_exporter));
      return true;
    }
    case internal::kNetworkContext_PreconnectSockets_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::PreconnectSockets",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_PreconnectSockets_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_PreconnectSockets_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_PreconnectSockets_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_num_streams{};
      GURL p_url{};
      int32_t p_load_flags{};
      bool p_privacy_mode_enabled{};
      NetworkContext_PreconnectSockets_ParamsDataView input_data_view(params, &serialization_context);
      
      p_num_streams = input_data_view.num_streams();
      if (!input_data_view.ReadUrl(&p_url))
        success = false;
      p_load_flags = input_data_view.load_flags();
      p_privacy_mode_enabled = input_data_view.privacy_mode_enabled();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::PreconnectSockets deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->PreconnectSockets(
std::move(p_num_streams), 
std::move(p_url), 
std::move(p_load_flags), 
std::move(p_privacy_mode_enabled));
      return true;
    }
    case internal::kNetworkContext_CreateP2PSocketManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateP2PSocketManager",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateP2PSocketManager_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_CreateP2PSocketManager_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_CreateP2PSocketManager_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::P2PTrustedSocketManagerClientPtr p_client{};
      ::network::mojom::P2PTrustedSocketManagerRequest p_trusted_socket_manager{};
      ::network::mojom::P2PSocketManagerRequest p_socket_manager{};
      NetworkContext_CreateP2PSocketManager_ParamsDataView input_data_view(params, &serialization_context);
      
      p_client =
          input_data_view.TakeClient<decltype(p_client)>();
      p_trusted_socket_manager =
          input_data_view.TakeTrustedSocketManager<decltype(p_trusted_socket_manager)>();
      p_socket_manager =
          input_data_view.TakeSocketManager<decltype(p_socket_manager)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateP2PSocketManager deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateP2PSocketManager(
std::move(p_client), 
std::move(p_trusted_socket_manager), 
std::move(p_socket_manager));
      return true;
    }
    case internal::kNetworkContext_CreateMdnsResponder_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateMdnsResponder",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateMdnsResponder_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_CreateMdnsResponder_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_CreateMdnsResponder_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::MdnsResponderRequest p_responder_request{};
      NetworkContext_CreateMdnsResponder_ParamsDataView input_data_view(params, &serialization_context);
      
      p_responder_request =
          input_data_view.TakeResponderRequest<decltype(p_responder_request)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateMdnsResponder deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateMdnsResponder(
std::move(p_responder_request));
      return true;
    }
    case internal::kNetworkContext_ResolveHost_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ResolveHost",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ResolveHost_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_ResolveHost_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_ResolveHost_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      net::HostPortPair p_host{};
      ::network::mojom::ResolveHostParametersPtr p_optional_parameters{};
      ::network::mojom::ResolveHostClientPtr p_response_client{};
      NetworkContext_ResolveHost_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadHost(&p_host))
        success = false;
      if (!input_data_view.ReadOptionalParameters(&p_optional_parameters))
        success = false;
      p_response_client =
          input_data_view.TakeResponseClient<decltype(p_response_client)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ResolveHost deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ResolveHost(
std::move(p_host), 
std::move(p_optional_parameters), 
std::move(p_response_client));
      return true;
    }
    case internal::kNetworkContext_CreateHostResolver_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateHostResolver",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateHostResolver_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_CreateHostResolver_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_CreateHostResolver_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      base::Optional<net::DnsConfigOverrides> p_config_overrides{};
      ::network::mojom::HostResolverRequest p_host_resolver{};
      NetworkContext_CreateHostResolver_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadConfigOverrides(&p_config_overrides))
        success = false;
      p_host_resolver =
          input_data_view.TakeHostResolver<decltype(p_host_resolver)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateHostResolver deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateHostResolver(
std::move(p_config_overrides), 
std::move(p_host_resolver));
      return true;
    }
    case internal::kNetworkContext_VerifyCertForSignedExchange_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::VerifyCertForSignedExchange",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_AddHSTS_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::AddHSTS",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_IsHSTSActiveForHost_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::IsHSTSActiveForHost",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_GetHSTSState_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetHSTSState",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_SetCorsOriginAccessListsForOrigin_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetCorsOriginAccessListsForOrigin",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_DeleteDynamicDataForHost_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::DeleteDynamicDataForHost",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_LookupBasicAuthCredentials_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::LookupBasicAuthCredentials",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_EnableStaticKeyPinningForTesting_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::EnableStaticKeyPinningForTesting",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_SetFailingHttpTransactionForTesting_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetFailingHttpTransactionForTesting",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_VerifyCertificateForTesting_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::VerifyCertificateForTesting",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_AddDomainReliabilityContextForTesting_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::AddDomainReliabilityContextForTesting",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ForceDomainReliabilityUploadsForTesting_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ForceDomainReliabilityUploadsForTesting",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

// static
bool NetworkContextStubDispatch::AcceptWithResponder(
    NetworkContext* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kNetworkContext_SetClient_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetClient",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateURLLoaderFactory_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateURLLoaderFactory",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ResetURLLoaderFactories_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ResetURLLoaderFactories",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_GetCookieManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetCookieManager",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_GetRestrictedCookieManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetRestrictedCookieManager",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ClearNetworkingHistorySince_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearNetworkingHistorySince",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ClearNetworkingHistorySince_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ClearNetworkingHistorySinceCallback callback =
              NetworkContext_ClearNetworkingHistorySince_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ClearNetworkingHistorySince_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ClearNetworkingHistorySince_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      base::Time p_start_time{};
      NetworkContext_ClearNetworkingHistorySince_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadStartTime(&p_start_time))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ClearNetworkingHistorySince deserializer");
        return false;
      }
      NetworkContext::ClearNetworkingHistorySinceCallback callback =
          NetworkContext_ClearNetworkingHistorySince_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ClearNetworkingHistorySince(
std::move(p_start_time), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_ClearHttpCache_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearHttpCache",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ClearHttpCache_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ClearHttpCacheCallback callback =
              NetworkContext_ClearHttpCache_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ClearHttpCache_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ClearHttpCache_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      base::Time p_start_time{};
      base::Time p_end_time{};
      ClearDataFilterPtr p_filter{};
      NetworkContext_ClearHttpCache_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadStartTime(&p_start_time))
        success = false;
      if (!input_data_view.ReadEndTime(&p_end_time))
        success = false;
      if (!input_data_view.ReadFilter(&p_filter))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ClearHttpCache deserializer");
        return false;
      }
      NetworkContext::ClearHttpCacheCallback callback =
          NetworkContext_ClearHttpCache_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ClearHttpCache(
std::move(p_start_time), 
std::move(p_end_time), 
std::move(p_filter), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_ComputeHttpCacheSize_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ComputeHttpCacheSize",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ComputeHttpCacheSize_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ComputeHttpCacheSizeCallback callback =
              NetworkContext_ComputeHttpCacheSize_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ComputeHttpCacheSize_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ComputeHttpCacheSize_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      base::Time p_start_time{};
      base::Time p_end_time{};
      NetworkContext_ComputeHttpCacheSize_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadStartTime(&p_start_time))
        success = false;
      if (!input_data_view.ReadEndTime(&p_end_time))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ComputeHttpCacheSize deserializer");
        return false;
      }
      NetworkContext::ComputeHttpCacheSizeCallback callback =
          NetworkContext_ComputeHttpCacheSize_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ComputeHttpCacheSize(
std::move(p_start_time), 
std::move(p_end_time), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_WriteCacheMetadata_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::WriteCacheMetadata",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ClearChannelIds_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearChannelIds",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ClearChannelIds_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ClearChannelIdsCallback callback =
              NetworkContext_ClearChannelIds_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ClearChannelIds_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ClearChannelIds_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      base::Time p_start_time{};
      base::Time p_end_time{};
      ClearDataFilterPtr p_filter{};
      NetworkContext_ClearChannelIds_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadStartTime(&p_start_time))
        success = false;
      if (!input_data_view.ReadEndTime(&p_end_time))
        success = false;
      if (!input_data_view.ReadFilter(&p_filter))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ClearChannelIds deserializer");
        return false;
      }
      NetworkContext::ClearChannelIdsCallback callback =
          NetworkContext_ClearChannelIds_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ClearChannelIds(
std::move(p_start_time), 
std::move(p_end_time), 
std::move(p_filter), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_ClearHostCache_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearHostCache",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ClearHostCache_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ClearHostCacheCallback callback =
              NetworkContext_ClearHostCache_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ClearHostCache_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ClearHostCache_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ClearDataFilterPtr p_filter{};
      NetworkContext_ClearHostCache_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadFilter(&p_filter))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ClearHostCache deserializer");
        return false;
      }
      NetworkContext::ClearHostCacheCallback callback =
          NetworkContext_ClearHostCache_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ClearHostCache(
std::move(p_filter), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_ClearHttpAuthCache_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearHttpAuthCache",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ClearHttpAuthCache_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ClearHttpAuthCacheCallback callback =
              NetworkContext_ClearHttpAuthCache_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ClearHttpAuthCache_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ClearHttpAuthCache_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      base::Time p_start_time{};
      NetworkContext_ClearHttpAuthCache_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadStartTime(&p_start_time))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ClearHttpAuthCache deserializer");
        return false;
      }
      NetworkContext::ClearHttpAuthCacheCallback callback =
          NetworkContext_ClearHttpAuthCache_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ClearHttpAuthCache(
std::move(p_start_time), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_ClearReportingCacheReports_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearReportingCacheReports",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ClearReportingCacheReports_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ClearReportingCacheReportsCallback callback =
              NetworkContext_ClearReportingCacheReports_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ClearReportingCacheReports_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ClearReportingCacheReports_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ClearDataFilterPtr p_filter{};
      NetworkContext_ClearReportingCacheReports_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadFilter(&p_filter))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ClearReportingCacheReports deserializer");
        return false;
      }
      NetworkContext::ClearReportingCacheReportsCallback callback =
          NetworkContext_ClearReportingCacheReports_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ClearReportingCacheReports(
std::move(p_filter), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_ClearReportingCacheClients_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearReportingCacheClients",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ClearReportingCacheClients_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ClearReportingCacheClientsCallback callback =
              NetworkContext_ClearReportingCacheClients_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ClearReportingCacheClients_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ClearReportingCacheClients_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ClearDataFilterPtr p_filter{};
      NetworkContext_ClearReportingCacheClients_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadFilter(&p_filter))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ClearReportingCacheClients deserializer");
        return false;
      }
      NetworkContext::ClearReportingCacheClientsCallback callback =
          NetworkContext_ClearReportingCacheClients_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ClearReportingCacheClients(
std::move(p_filter), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_ClearNetworkErrorLogging_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearNetworkErrorLogging",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ClearNetworkErrorLogging_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ClearNetworkErrorLoggingCallback callback =
              NetworkContext_ClearNetworkErrorLogging_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ClearNetworkErrorLogging_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ClearNetworkErrorLogging_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ClearDataFilterPtr p_filter{};
      NetworkContext_ClearNetworkErrorLogging_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadFilter(&p_filter))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ClearNetworkErrorLogging deserializer");
        return false;
      }
      NetworkContext::ClearNetworkErrorLoggingCallback callback =
          NetworkContext_ClearNetworkErrorLogging_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ClearNetworkErrorLogging(
std::move(p_filter), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_ClearDomainReliability_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearDomainReliability",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ClearDomainReliability_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ClearDomainReliabilityCallback callback =
              NetworkContext_ClearDomainReliability_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ClearDomainReliability_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ClearDomainReliability_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ClearDataFilterPtr p_filter{};
      NetworkContext::DomainReliabilityClearMode p_mode{};
      NetworkContext_ClearDomainReliability_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadFilter(&p_filter))
        success = false;
      if (!input_data_view.ReadMode(&p_mode))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ClearDomainReliability deserializer");
        return false;
      }
      NetworkContext::ClearDomainReliabilityCallback callback =
          NetworkContext_ClearDomainReliability_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ClearDomainReliability(
std::move(p_filter), 
std::move(p_mode), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_GetDomainReliabilityJSON_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetDomainReliabilityJSON",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_GetDomainReliabilityJSON_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::GetDomainReliabilityJSONCallback callback =
              NetworkContext_GetDomainReliabilityJSON_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_GetDomainReliabilityJSON_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_GetDomainReliabilityJSON_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkContext_GetDomainReliabilityJSON_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::GetDomainReliabilityJSON deserializer");
        return false;
      }
      NetworkContext::GetDomainReliabilityJSONCallback callback =
          NetworkContext_GetDomainReliabilityJSON_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetDomainReliabilityJSON(std::move(callback));
      return true;
    }
    case internal::kNetworkContext_QueueReport_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::QueueReport",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CloseAllConnections_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CloseAllConnections",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CloseAllConnections_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::CloseAllConnectionsCallback callback =
              NetworkContext_CloseAllConnections_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_CloseAllConnections_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_CloseAllConnections_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkContext_CloseAllConnections_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CloseAllConnections deserializer");
        return false;
      }
      NetworkContext::CloseAllConnectionsCallback callback =
          NetworkContext_CloseAllConnections_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CloseAllConnections(std::move(callback));
      return true;
    }
    case internal::kNetworkContext_CloseIdleConnections_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CloseIdleConnections",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CloseIdleConnections_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::CloseIdleConnectionsCallback callback =
              NetworkContext_CloseIdleConnections_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_CloseIdleConnections_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_CloseIdleConnections_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkContext_CloseIdleConnections_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CloseIdleConnections deserializer");
        return false;
      }
      NetworkContext::CloseIdleConnectionsCallback callback =
          NetworkContext_CloseIdleConnections_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CloseIdleConnections(std::move(callback));
      return true;
    }
    case internal::kNetworkContext_SetNetworkConditions_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetNetworkConditions",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_SetAcceptLanguage_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetAcceptLanguage",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_SetEnableReferrers_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetEnableReferrers",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_SetCTPolicy_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetCTPolicy",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_AddExpectCT_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::AddExpectCT",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_AddExpectCT_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::AddExpectCTCallback callback =
              NetworkContext_AddExpectCT_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_AddExpectCT_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_AddExpectCT_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_host{};
      base::Time p_expiry{};
      bool p_enforce{};
      GURL p_report_uri{};
      NetworkContext_AddExpectCT_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadHost(&p_host))
        success = false;
      if (!input_data_view.ReadExpiry(&p_expiry))
        success = false;
      p_enforce = input_data_view.enforce();
      if (!input_data_view.ReadReportUri(&p_report_uri))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::AddExpectCT deserializer");
        return false;
      }
      NetworkContext::AddExpectCTCallback callback =
          NetworkContext_AddExpectCT_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->AddExpectCT(
std::move(p_host), 
std::move(p_expiry), 
std::move(p_enforce), 
std::move(p_report_uri), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_SetExpectCTTestReport_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetExpectCTTestReport",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_SetExpectCTTestReport_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::SetExpectCTTestReportCallback callback =
              NetworkContext_SetExpectCTTestReport_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_SetExpectCTTestReport_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_SetExpectCTTestReport_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      GURL p_report_uri{};
      NetworkContext_SetExpectCTTestReport_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadReportUri(&p_report_uri))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::SetExpectCTTestReport deserializer");
        return false;
      }
      NetworkContext::SetExpectCTTestReportCallback callback =
          NetworkContext_SetExpectCTTestReport_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetExpectCTTestReport(
std::move(p_report_uri), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_GetExpectCTState_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetExpectCTState",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_GetExpectCTState_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::GetExpectCTStateCallback callback =
              NetworkContext_GetExpectCTState_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_GetExpectCTState_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_GetExpectCTState_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_domain{};
      NetworkContext_GetExpectCTState_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadDomain(&p_domain))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::GetExpectCTState deserializer");
        return false;
      }
      NetworkContext::GetExpectCTStateCallback callback =
          NetworkContext_GetExpectCTState_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetExpectCTState(
std::move(p_domain), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_CreateUDPSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateUDPSocket",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateTCPServerSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPServerSocket",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateTCPServerSocket_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::CreateTCPServerSocketCallback callback =
              NetworkContext_CreateTCPServerSocket_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_CreateTCPServerSocket_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_CreateTCPServerSocket_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      net::IPEndPoint p_local_addr{};
      uint32_t p_backlog{};
      net::MutableNetworkTrafficAnnotationTag p_traffic_annotation{};
      ::network::mojom::TCPServerSocketRequest p_socket{};
      NetworkContext_CreateTCPServerSocket_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadLocalAddr(&p_local_addr))
        success = false;
      p_backlog = input_data_view.backlog();
      if (!input_data_view.ReadTrafficAnnotation(&p_traffic_annotation))
        success = false;
      p_socket =
          input_data_view.TakeSocket<decltype(p_socket)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateTCPServerSocket deserializer");
        return false;
      }
      NetworkContext::CreateTCPServerSocketCallback callback =
          NetworkContext_CreateTCPServerSocket_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateTCPServerSocket(
std::move(p_local_addr), 
std::move(p_backlog), 
std::move(p_traffic_annotation), 
std::move(p_socket), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_CreateTCPConnectedSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPConnectedSocket",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateTCPConnectedSocket_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::CreateTCPConnectedSocketCallback callback =
              NetworkContext_CreateTCPConnectedSocket_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_CreateTCPConnectedSocket_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_CreateTCPConnectedSocket_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      base::Optional<net::IPEndPoint> p_local_addr{};
      net::AddressList p_remote_addr_list{};
      ::network::mojom::TCPConnectedSocketOptionsPtr p_tcp_connected_socket_options{};
      net::MutableNetworkTrafficAnnotationTag p_traffic_annotation{};
      ::network::mojom::TCPConnectedSocketRequest p_socket{};
      ::network::mojom::SocketObserverPtr p_observer{};
      NetworkContext_CreateTCPConnectedSocket_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadLocalAddr(&p_local_addr))
        success = false;
      if (!input_data_view.ReadRemoteAddrList(&p_remote_addr_list))
        success = false;
      if (!input_data_view.ReadTcpConnectedSocketOptions(&p_tcp_connected_socket_options))
        success = false;
      if (!input_data_view.ReadTrafficAnnotation(&p_traffic_annotation))
        success = false;
      p_socket =
          input_data_view.TakeSocket<decltype(p_socket)>();
      p_observer =
          input_data_view.TakeObserver<decltype(p_observer)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateTCPConnectedSocket deserializer");
        return false;
      }
      NetworkContext::CreateTCPConnectedSocketCallback callback =
          NetworkContext_CreateTCPConnectedSocket_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateTCPConnectedSocket(
std::move(p_local_addr), 
std::move(p_remote_addr_list), 
std::move(p_tcp_connected_socket_options), 
std::move(p_traffic_annotation), 
std::move(p_socket), 
std::move(p_observer), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_CreateTCPBoundSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPBoundSocket",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateTCPBoundSocket_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::CreateTCPBoundSocketCallback callback =
              NetworkContext_CreateTCPBoundSocket_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_CreateTCPBoundSocket_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_CreateTCPBoundSocket_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      net::IPEndPoint p_local_addr{};
      net::MutableNetworkTrafficAnnotationTag p_traffic_annotation{};
      ::network::mojom::TCPBoundSocketRequest p_socket{};
      NetworkContext_CreateTCPBoundSocket_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadLocalAddr(&p_local_addr))
        success = false;
      if (!input_data_view.ReadTrafficAnnotation(&p_traffic_annotation))
        success = false;
      p_socket =
          input_data_view.TakeSocket<decltype(p_socket)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateTCPBoundSocket deserializer");
        return false;
      }
      NetworkContext::CreateTCPBoundSocketCallback callback =
          NetworkContext_CreateTCPBoundSocket_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateTCPBoundSocket(
std::move(p_local_addr), 
std::move(p_traffic_annotation), 
std::move(p_socket), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_CreateProxyResolvingSocketFactory_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateProxyResolvingSocketFactory",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_LookUpProxyForURL_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::LookUpProxyForURL",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ForceReloadProxyConfig_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ForceReloadProxyConfig",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ForceReloadProxyConfig_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ForceReloadProxyConfigCallback callback =
              NetworkContext_ForceReloadProxyConfig_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ForceReloadProxyConfig_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ForceReloadProxyConfig_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkContext_ForceReloadProxyConfig_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ForceReloadProxyConfig deserializer");
        return false;
      }
      NetworkContext::ForceReloadProxyConfigCallback callback =
          NetworkContext_ForceReloadProxyConfig_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ForceReloadProxyConfig(std::move(callback));
      return true;
    }
    case internal::kNetworkContext_ClearBadProxiesCache_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearBadProxiesCache",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ClearBadProxiesCache_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ClearBadProxiesCacheCallback callback =
              NetworkContext_ClearBadProxiesCache_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ClearBadProxiesCache_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ClearBadProxiesCache_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkContext_ClearBadProxiesCache_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ClearBadProxiesCache deserializer");
        return false;
      }
      NetworkContext::ClearBadProxiesCacheCallback callback =
          NetworkContext_ClearBadProxiesCache_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ClearBadProxiesCache(std::move(callback));
      return true;
    }
    case internal::kNetworkContext_CreateWebSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateWebSocket",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateNetLogExporter_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateNetLogExporter",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_PreconnectSockets_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::PreconnectSockets",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateP2PSocketManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateP2PSocketManager",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateMdnsResponder_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateMdnsResponder",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ResolveHost_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ResolveHost",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateHostResolver_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateHostResolver",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_VerifyCertForSignedExchange_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::VerifyCertForSignedExchange",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_VerifyCertForSignedExchange_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::VerifyCertForSignedExchangeCallback callback =
              NetworkContext_VerifyCertForSignedExchange_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_VerifyCertForSignedExchange_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_VerifyCertForSignedExchange_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      scoped_refptr<net::X509Certificate> p_certificate{};
      GURL p_url{};
      std::string p_ocsp_response{};
      std::string p_sct_list{};
      NetworkContext_VerifyCertForSignedExchange_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadCertificate(&p_certificate))
        success = false;
      if (!input_data_view.ReadUrl(&p_url))
        success = false;
      if (!input_data_view.ReadOcspResponse(&p_ocsp_response))
        success = false;
      if (!input_data_view.ReadSctList(&p_sct_list))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::VerifyCertForSignedExchange deserializer");
        return false;
      }
      NetworkContext::VerifyCertForSignedExchangeCallback callback =
          NetworkContext_VerifyCertForSignedExchange_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->VerifyCertForSignedExchange(
std::move(p_certificate), 
std::move(p_url), 
std::move(p_ocsp_response), 
std::move(p_sct_list), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_AddHSTS_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::AddHSTS",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_AddHSTS_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::AddHSTSCallback callback =
              NetworkContext_AddHSTS_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_AddHSTS_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_AddHSTS_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_host{};
      base::Time p_expiry{};
      bool p_include_subdomains{};
      NetworkContext_AddHSTS_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadHost(&p_host))
        success = false;
      if (!input_data_view.ReadExpiry(&p_expiry))
        success = false;
      p_include_subdomains = input_data_view.include_subdomains();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::AddHSTS deserializer");
        return false;
      }
      NetworkContext::AddHSTSCallback callback =
          NetworkContext_AddHSTS_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->AddHSTS(
std::move(p_host), 
std::move(p_expiry), 
std::move(p_include_subdomains), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_IsHSTSActiveForHost_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::IsHSTSActiveForHost",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_IsHSTSActiveForHost_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::IsHSTSActiveForHostCallback callback =
              NetworkContext_IsHSTSActiveForHost_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_IsHSTSActiveForHost_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_IsHSTSActiveForHost_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_host{};
      NetworkContext_IsHSTSActiveForHost_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadHost(&p_host))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::IsHSTSActiveForHost deserializer");
        return false;
      }
      NetworkContext::IsHSTSActiveForHostCallback callback =
          NetworkContext_IsHSTSActiveForHost_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->IsHSTSActiveForHost(
std::move(p_host), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_GetHSTSState_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetHSTSState",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_GetHSTSState_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::GetHSTSStateCallback callback =
              NetworkContext_GetHSTSState_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_GetHSTSState_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_GetHSTSState_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_domain{};
      NetworkContext_GetHSTSState_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadDomain(&p_domain))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::GetHSTSState deserializer");
        return false;
      }
      NetworkContext::GetHSTSStateCallback callback =
          NetworkContext_GetHSTSState_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetHSTSState(
std::move(p_domain), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_SetCorsOriginAccessListsForOrigin_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetCorsOriginAccessListsForOrigin",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_SetCorsOriginAccessListsForOrigin_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::SetCorsOriginAccessListsForOriginCallback callback =
              NetworkContext_SetCorsOriginAccessListsForOrigin_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_SetCorsOriginAccessListsForOrigin_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_SetCorsOriginAccessListsForOrigin_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      url::Origin p_source_origin{};
      std::vector<::network::mojom::CorsOriginPatternPtr> p_allow_patterns{};
      std::vector<::network::mojom::CorsOriginPatternPtr> p_block_patterns{};
      NetworkContext_SetCorsOriginAccessListsForOrigin_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadSourceOrigin(&p_source_origin))
        success = false;
      if (!input_data_view.ReadAllowPatterns(&p_allow_patterns))
        success = false;
      if (!input_data_view.ReadBlockPatterns(&p_block_patterns))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::SetCorsOriginAccessListsForOrigin deserializer");
        return false;
      }
      NetworkContext::SetCorsOriginAccessListsForOriginCallback callback =
          NetworkContext_SetCorsOriginAccessListsForOrigin_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetCorsOriginAccessListsForOrigin(
std::move(p_source_origin), 
std::move(p_allow_patterns), 
std::move(p_block_patterns), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_DeleteDynamicDataForHost_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::DeleteDynamicDataForHost",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_DeleteDynamicDataForHost_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::DeleteDynamicDataForHostCallback callback =
              NetworkContext_DeleteDynamicDataForHost_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_DeleteDynamicDataForHost_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_DeleteDynamicDataForHost_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_host{};
      NetworkContext_DeleteDynamicDataForHost_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadHost(&p_host))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::DeleteDynamicDataForHost deserializer");
        return false;
      }
      NetworkContext::DeleteDynamicDataForHostCallback callback =
          NetworkContext_DeleteDynamicDataForHost_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->DeleteDynamicDataForHost(
std::move(p_host), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_LookupBasicAuthCredentials_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::LookupBasicAuthCredentials",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_LookupBasicAuthCredentials_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::LookupBasicAuthCredentialsCallback callback =
              NetworkContext_LookupBasicAuthCredentials_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_LookupBasicAuthCredentials_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_LookupBasicAuthCredentials_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      GURL p_url{};
      NetworkContext_LookupBasicAuthCredentials_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadUrl(&p_url))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::LookupBasicAuthCredentials deserializer");
        return false;
      }
      NetworkContext::LookupBasicAuthCredentialsCallback callback =
          NetworkContext_LookupBasicAuthCredentials_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->LookupBasicAuthCredentials(
std::move(p_url), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_EnableStaticKeyPinningForTesting_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::EnableStaticKeyPinningForTesting",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_EnableStaticKeyPinningForTesting_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::EnableStaticKeyPinningForTestingCallback callback =
              NetworkContext_EnableStaticKeyPinningForTesting_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_EnableStaticKeyPinningForTesting_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_EnableStaticKeyPinningForTesting_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkContext_EnableStaticKeyPinningForTesting_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::EnableStaticKeyPinningForTesting deserializer");
        return false;
      }
      NetworkContext::EnableStaticKeyPinningForTestingCallback callback =
          NetworkContext_EnableStaticKeyPinningForTesting_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->EnableStaticKeyPinningForTesting(std::move(callback));
      return true;
    }
    case internal::kNetworkContext_SetFailingHttpTransactionForTesting_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetFailingHttpTransactionForTesting",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_SetFailingHttpTransactionForTesting_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::SetFailingHttpTransactionForTestingCallback callback =
              NetworkContext_SetFailingHttpTransactionForTesting_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_SetFailingHttpTransactionForTesting_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_SetFailingHttpTransactionForTesting_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_rv{};
      NetworkContext_SetFailingHttpTransactionForTesting_ParamsDataView input_data_view(params, &serialization_context);
      
      p_rv = input_data_view.rv();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::SetFailingHttpTransactionForTesting deserializer");
        return false;
      }
      NetworkContext::SetFailingHttpTransactionForTestingCallback callback =
          NetworkContext_SetFailingHttpTransactionForTesting_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetFailingHttpTransactionForTesting(
std::move(p_rv), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_VerifyCertificateForTesting_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::VerifyCertificateForTesting",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_VerifyCertificateForTesting_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::VerifyCertificateForTestingCallback callback =
              NetworkContext_VerifyCertificateForTesting_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_VerifyCertificateForTesting_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_VerifyCertificateForTesting_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      scoped_refptr<net::X509Certificate> p_certificate{};
      std::string p_hostname{};
      std::string p_ocsp_response{};
      NetworkContext_VerifyCertificateForTesting_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadCertificate(&p_certificate))
        success = false;
      if (!input_data_view.ReadHostname(&p_hostname))
        success = false;
      if (!input_data_view.ReadOcspResponse(&p_ocsp_response))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::VerifyCertificateForTesting deserializer");
        return false;
      }
      NetworkContext::VerifyCertificateForTestingCallback callback =
          NetworkContext_VerifyCertificateForTesting_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->VerifyCertificateForTesting(
std::move(p_certificate), 
std::move(p_hostname), 
std::move(p_ocsp_response), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_AddDomainReliabilityContextForTesting_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::AddDomainReliabilityContextForTesting",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_AddDomainReliabilityContextForTesting_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::AddDomainReliabilityContextForTestingCallback callback =
              NetworkContext_AddDomainReliabilityContextForTesting_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_AddDomainReliabilityContextForTesting_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_AddDomainReliabilityContextForTesting_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      GURL p_origin{};
      GURL p_upload_url{};
      NetworkContext_AddDomainReliabilityContextForTesting_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadOrigin(&p_origin))
        success = false;
      if (!input_data_view.ReadUploadUrl(&p_upload_url))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::AddDomainReliabilityContextForTesting deserializer");
        return false;
      }
      NetworkContext::AddDomainReliabilityContextForTestingCallback callback =
          NetworkContext_AddDomainReliabilityContextForTesting_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->AddDomainReliabilityContextForTesting(
std::move(p_origin), 
std::move(p_upload_url), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_ForceDomainReliabilityUploadsForTesting_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ForceDomainReliabilityUploadsForTesting",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ForceDomainReliabilityUploadsForTesting_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ForceDomainReliabilityUploadsForTestingCallback callback =
              NetworkContext_ForceDomainReliabilityUploadsForTesting_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkContext_ForceDomainReliabilityUploadsForTesting_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ForceDomainReliabilityUploadsForTesting deserializer");
        return false;
      }
      NetworkContext::ForceDomainReliabilityUploadsForTestingCallback callback =
          NetworkContext_ForceDomainReliabilityUploadsForTesting_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ForceDomainReliabilityUploadsForTesting(std::move(callback));
      return true;
    }
  }
  return false;
}

bool NetworkContextRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "NetworkContext RequestValidator");

  switch (message->header()->name) {
    case internal::kNetworkContext_SetClient_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_SetClient_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateURLLoaderFactory_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateURLLoaderFactory_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ResetURLLoaderFactories_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ResetURLLoaderFactories_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_GetCookieManager_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_GetCookieManager_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_GetRestrictedCookieManager_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_GetRestrictedCookieManager_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearNetworkingHistorySince_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearNetworkingHistorySince_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearHttpCache_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearHttpCache_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ComputeHttpCacheSize_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ComputeHttpCacheSize_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_WriteCacheMetadata_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_WriteCacheMetadata_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearChannelIds_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearChannelIds_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearHostCache_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearHostCache_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearHttpAuthCache_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearHttpAuthCache_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearReportingCacheReports_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearReportingCacheReports_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearReportingCacheClients_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearReportingCacheClients_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearNetworkErrorLogging_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearNetworkErrorLogging_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearDomainReliability_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearDomainReliability_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_GetDomainReliabilityJSON_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_GetDomainReliabilityJSON_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_QueueReport_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_QueueReport_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CloseAllConnections_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CloseAllConnections_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CloseIdleConnections_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CloseIdleConnections_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_SetNetworkConditions_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_SetNetworkConditions_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_SetAcceptLanguage_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_SetAcceptLanguage_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_SetEnableReferrers_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_SetEnableReferrers_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_SetCTPolicy_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_SetCTPolicy_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_AddExpectCT_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_AddExpectCT_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_SetExpectCTTestReport_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_SetExpectCTTestReport_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_GetExpectCTState_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_GetExpectCTState_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateUDPSocket_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateUDPSocket_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateTCPServerSocket_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateTCPServerSocket_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateTCPConnectedSocket_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateTCPConnectedSocket_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateTCPBoundSocket_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateTCPBoundSocket_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateProxyResolvingSocketFactory_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateProxyResolvingSocketFactory_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_LookUpProxyForURL_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_LookUpProxyForURL_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ForceReloadProxyConfig_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ForceReloadProxyConfig_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearBadProxiesCache_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearBadProxiesCache_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateWebSocket_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateWebSocket_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateNetLogExporter_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateNetLogExporter_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_PreconnectSockets_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_PreconnectSockets_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateP2PSocketManager_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateP2PSocketManager_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateMdnsResponder_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateMdnsResponder_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ResolveHost_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ResolveHost_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateHostResolver_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateHostResolver_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_VerifyCertForSignedExchange_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_VerifyCertForSignedExchange_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_AddHSTS_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_AddHSTS_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_IsHSTSActiveForHost_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_IsHSTSActiveForHost_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_GetHSTSState_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_GetHSTSState_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_SetCorsOriginAccessListsForOrigin_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_SetCorsOriginAccessListsForOrigin_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_DeleteDynamicDataForHost_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_DeleteDynamicDataForHost_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_LookupBasicAuthCredentials_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_LookupBasicAuthCredentials_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_EnableStaticKeyPinningForTesting_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_EnableStaticKeyPinningForTesting_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_SetFailingHttpTransactionForTesting_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_SetFailingHttpTransactionForTesting_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_VerifyCertificateForTesting_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_VerifyCertificateForTesting_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_AddDomainReliabilityContextForTesting_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_AddDomainReliabilityContextForTesting_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ForceDomainReliabilityUploadsForTesting_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool NetworkContextResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "NetworkContext ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kNetworkContext_ClearNetworkingHistorySince_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearHttpCache_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearHttpCache_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ComputeHttpCacheSize_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ComputeHttpCacheSize_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearChannelIds_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearChannelIds_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearHostCache_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearHostCache_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearHttpAuthCache_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearHttpAuthCache_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearReportingCacheReports_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearReportingCacheReports_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearReportingCacheClients_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearReportingCacheClients_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearNetworkErrorLogging_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearNetworkErrorLogging_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearDomainReliability_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearDomainReliability_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_GetDomainReliabilityJSON_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_GetDomainReliabilityJSON_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CloseAllConnections_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CloseAllConnections_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CloseIdleConnections_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CloseIdleConnections_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_AddExpectCT_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_AddExpectCT_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_SetExpectCTTestReport_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_SetExpectCTTestReport_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_GetExpectCTState_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_GetExpectCTState_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateTCPServerSocket_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateTCPServerSocket_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateTCPConnectedSocket_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateTCPConnectedSocket_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateTCPBoundSocket_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateTCPBoundSocket_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ForceReloadProxyConfig_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ForceReloadProxyConfig_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearBadProxiesCache_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearBadProxiesCache_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_VerifyCertForSignedExchange_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_VerifyCertForSignedExchange_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_AddHSTS_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_AddHSTS_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_IsHSTSActiveForHost_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_IsHSTSActiveForHost_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_GetHSTSState_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_GetHSTSState_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_SetCorsOriginAccessListsForOrigin_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_SetCorsOriginAccessListsForOrigin_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_DeleteDynamicDataForHost_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_DeleteDynamicDataForHost_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_LookupBasicAuthCredentials_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_LookupBasicAuthCredentials_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_EnableStaticKeyPinningForTesting_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_EnableStaticKeyPinningForTesting_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_SetFailingHttpTransactionForTesting_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_SetFailingHttpTransactionForTesting_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_VerifyCertificateForTesting_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_VerifyCertificateForTesting_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_AddDomainReliabilityContextForTesting_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_AddDomainReliabilityContextForTesting_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ForceDomainReliabilityUploadsForTesting_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
void NetworkContextInterceptorForTesting::SetClient(NetworkContextClientPtr client) {
  GetForwardingInterface()->SetClient(std::move(client));
}
void NetworkContextInterceptorForTesting::CreateURLLoaderFactory(::network::mojom::URLLoaderFactoryRequest url_loader_factory, URLLoaderFactoryParamsPtr params) {
  GetForwardingInterface()->CreateURLLoaderFactory(std::move(url_loader_factory), std::move(params));
}
void NetworkContextInterceptorForTesting::ResetURLLoaderFactories() {
  GetForwardingInterface()->ResetURLLoaderFactories();
}
void NetworkContextInterceptorForTesting::GetCookieManager(::network::mojom::CookieManagerRequest cookie_manager) {
  GetForwardingInterface()->GetCookieManager(std::move(cookie_manager));
}
void NetworkContextInterceptorForTesting::GetRestrictedCookieManager(::network::mojom::RestrictedCookieManagerRequest restricted_cookie_manager, const url::Origin& origin) {
  GetForwardingInterface()->GetRestrictedCookieManager(std::move(restricted_cookie_manager), std::move(origin));
}
void NetworkContextInterceptorForTesting::ClearNetworkingHistorySince(base::Time start_time, ClearNetworkingHistorySinceCallback callback) {
  GetForwardingInterface()->ClearNetworkingHistorySince(std::move(start_time), std::move(callback));
}
void NetworkContextInterceptorForTesting::ClearHttpCache(base::Time start_time, base::Time end_time, ClearDataFilterPtr filter, ClearHttpCacheCallback callback) {
  GetForwardingInterface()->ClearHttpCache(std::move(start_time), std::move(end_time), std::move(filter), std::move(callback));
}
void NetworkContextInterceptorForTesting::ComputeHttpCacheSize(base::Time start_time, base::Time end_time, ComputeHttpCacheSizeCallback callback) {
  GetForwardingInterface()->ComputeHttpCacheSize(std::move(start_time), std::move(end_time), std::move(callback));
}
void NetworkContextInterceptorForTesting::WriteCacheMetadata(const GURL& url, net::RequestPriority priority, base::Time expected_response_time, const std::vector<uint8_t>& data) {
  GetForwardingInterface()->WriteCacheMetadata(std::move(url), std::move(priority), std::move(expected_response_time), std::move(data));
}
void NetworkContextInterceptorForTesting::ClearChannelIds(base::Time start_time, base::Time end_time, ClearDataFilterPtr filter, ClearChannelIdsCallback callback) {
  GetForwardingInterface()->ClearChannelIds(std::move(start_time), std::move(end_time), std::move(filter), std::move(callback));
}
void NetworkContextInterceptorForTesting::ClearHostCache(ClearDataFilterPtr filter, ClearHostCacheCallback callback) {
  GetForwardingInterface()->ClearHostCache(std::move(filter), std::move(callback));
}
void NetworkContextInterceptorForTesting::ClearHttpAuthCache(base::Time start_time, ClearHttpAuthCacheCallback callback) {
  GetForwardingInterface()->ClearHttpAuthCache(std::move(start_time), std::move(callback));
}
void NetworkContextInterceptorForTesting::ClearReportingCacheReports(ClearDataFilterPtr filter, ClearReportingCacheReportsCallback callback) {
  GetForwardingInterface()->ClearReportingCacheReports(std::move(filter), std::move(callback));
}
void NetworkContextInterceptorForTesting::ClearReportingCacheClients(ClearDataFilterPtr filter, ClearReportingCacheClientsCallback callback) {
  GetForwardingInterface()->ClearReportingCacheClients(std::move(filter), std::move(callback));
}
void NetworkContextInterceptorForTesting::ClearNetworkErrorLogging(ClearDataFilterPtr filter, ClearNetworkErrorLoggingCallback callback) {
  GetForwardingInterface()->ClearNetworkErrorLogging(std::move(filter), std::move(callback));
}
void NetworkContextInterceptorForTesting::ClearDomainReliability(ClearDataFilterPtr filter, NetworkContext::DomainReliabilityClearMode mode, ClearDomainReliabilityCallback callback) {
  GetForwardingInterface()->ClearDomainReliability(std::move(filter), std::move(mode), std::move(callback));
}
void NetworkContextInterceptorForTesting::GetDomainReliabilityJSON(GetDomainReliabilityJSONCallback callback) {
  GetForwardingInterface()->GetDomainReliabilityJSON(std::move(callback));
}
void NetworkContextInterceptorForTesting::QueueReport(const std::string& type, const std::string& group, const GURL& url, const base::Optional<std::string>& user_agent, base::Value body) {
  GetForwardingInterface()->QueueReport(std::move(type), std::move(group), std::move(url), std::move(user_agent), std::move(body));
}
void NetworkContextInterceptorForTesting::CloseAllConnections(CloseAllConnectionsCallback callback) {
  GetForwardingInterface()->CloseAllConnections(std::move(callback));
}
void NetworkContextInterceptorForTesting::CloseIdleConnections(CloseIdleConnectionsCallback callback) {
  GetForwardingInterface()->CloseIdleConnections(std::move(callback));
}
void NetworkContextInterceptorForTesting::SetNetworkConditions(const base::UnguessableToken& throttling_profile_id, NetworkConditionsPtr conditions) {
  GetForwardingInterface()->SetNetworkConditions(std::move(throttling_profile_id), std::move(conditions));
}
void NetworkContextInterceptorForTesting::SetAcceptLanguage(const std::string& new_accept_language) {
  GetForwardingInterface()->SetAcceptLanguage(std::move(new_accept_language));
}
void NetworkContextInterceptorForTesting::SetEnableReferrers(bool enable_referrers) {
  GetForwardingInterface()->SetEnableReferrers(std::move(enable_referrers));
}
void NetworkContextInterceptorForTesting::SetCTPolicy(const std::vector<std::string>& required_hosts, const std::vector<std::string>& excluded_hosts, const std::vector<std::string>& excluded_spkis, const std::vector<std::string>& excluded_legacy_spkis) {
  GetForwardingInterface()->SetCTPolicy(std::move(required_hosts), std::move(excluded_hosts), std::move(excluded_spkis), std::move(excluded_legacy_spkis));
}
void NetworkContextInterceptorForTesting::AddExpectCT(const std::string& host, base::Time expiry, bool enforce, const GURL& report_uri, AddExpectCTCallback callback) {
  GetForwardingInterface()->AddExpectCT(std::move(host), std::move(expiry), std::move(enforce), std::move(report_uri), std::move(callback));
}
void NetworkContextInterceptorForTesting::SetExpectCTTestReport(const GURL& report_uri, SetExpectCTTestReportCallback callback) {
  GetForwardingInterface()->SetExpectCTTestReport(std::move(report_uri), std::move(callback));
}
void NetworkContextInterceptorForTesting::GetExpectCTState(const std::string& domain, GetExpectCTStateCallback callback) {
  GetForwardingInterface()->GetExpectCTState(std::move(domain), std::move(callback));
}
void NetworkContextInterceptorForTesting::CreateUDPSocket(::network::mojom::UDPSocketRequest request, ::network::mojom::UDPSocketReceiverPtr receiver) {
  GetForwardingInterface()->CreateUDPSocket(std::move(request), std::move(receiver));
}
void NetworkContextInterceptorForTesting::CreateTCPServerSocket(const net::IPEndPoint& local_addr, uint32_t backlog, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation, ::network::mojom::TCPServerSocketRequest socket, CreateTCPServerSocketCallback callback) {
  GetForwardingInterface()->CreateTCPServerSocket(std::move(local_addr), std::move(backlog), std::move(traffic_annotation), std::move(socket), std::move(callback));
}
void NetworkContextInterceptorForTesting::CreateTCPConnectedSocket(const base::Optional<net::IPEndPoint>& local_addr, const net::AddressList& remote_addr_list, ::network::mojom::TCPConnectedSocketOptionsPtr tcp_connected_socket_options, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation, ::network::mojom::TCPConnectedSocketRequest socket, ::network::mojom::SocketObserverPtr observer, CreateTCPConnectedSocketCallback callback) {
  GetForwardingInterface()->CreateTCPConnectedSocket(std::move(local_addr), std::move(remote_addr_list), std::move(tcp_connected_socket_options), std::move(traffic_annotation), std::move(socket), std::move(observer), std::move(callback));
}
void NetworkContextInterceptorForTesting::CreateTCPBoundSocket(const net::IPEndPoint& local_addr, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation, ::network::mojom::TCPBoundSocketRequest socket, CreateTCPBoundSocketCallback callback) {
  GetForwardingInterface()->CreateTCPBoundSocket(std::move(local_addr), std::move(traffic_annotation), std::move(socket), std::move(callback));
}
void NetworkContextInterceptorForTesting::CreateProxyResolvingSocketFactory(::network::mojom::ProxyResolvingSocketFactoryRequest factory) {
  GetForwardingInterface()->CreateProxyResolvingSocketFactory(std::move(factory));
}
void NetworkContextInterceptorForTesting::LookUpProxyForURL(const GURL& url, ::network::mojom::ProxyLookupClientPtr proxy_lookup_client) {
  GetForwardingInterface()->LookUpProxyForURL(std::move(url), std::move(proxy_lookup_client));
}
void NetworkContextInterceptorForTesting::ForceReloadProxyConfig(ForceReloadProxyConfigCallback callback) {
  GetForwardingInterface()->ForceReloadProxyConfig(std::move(callback));
}
void NetworkContextInterceptorForTesting::ClearBadProxiesCache(ClearBadProxiesCacheCallback callback) {
  GetForwardingInterface()->ClearBadProxiesCache(std::move(callback));
}
void NetworkContextInterceptorForTesting::CreateWebSocket(::network::mojom::WebSocketRequest request, int32_t process_id, int32_t render_frame_id, const url::Origin& origin, ::network::mojom::AuthenticationHandlerPtr auth_handler) {
  GetForwardingInterface()->CreateWebSocket(std::move(request), std::move(process_id), std::move(render_frame_id), std::move(origin), std::move(auth_handler));
}
void NetworkContextInterceptorForTesting::CreateNetLogExporter(::network::mojom::NetLogExporterRequest exporter) {
  GetForwardingInterface()->CreateNetLogExporter(std::move(exporter));
}
void NetworkContextInterceptorForTesting::PreconnectSockets(uint32_t num_streams, const GURL& url, int32_t load_flags, bool privacy_mode_enabled) {
  GetForwardingInterface()->PreconnectSockets(std::move(num_streams), std::move(url), std::move(load_flags), std::move(privacy_mode_enabled));
}
void NetworkContextInterceptorForTesting::CreateP2PSocketManager(::network::mojom::P2PTrustedSocketManagerClientPtr client, ::network::mojom::P2PTrustedSocketManagerRequest trusted_socket_manager, ::network::mojom::P2PSocketManagerRequest socket_manager) {
  GetForwardingInterface()->CreateP2PSocketManager(std::move(client), std::move(trusted_socket_manager), std::move(socket_manager));
}
void NetworkContextInterceptorForTesting::CreateMdnsResponder(::network::mojom::MdnsResponderRequest responder_request) {
  GetForwardingInterface()->CreateMdnsResponder(std::move(responder_request));
}
void NetworkContextInterceptorForTesting::ResolveHost(const net::HostPortPair& host, ::network::mojom::ResolveHostParametersPtr optional_parameters, ::network::mojom::ResolveHostClientPtr response_client) {
  GetForwardingInterface()->ResolveHost(std::move(host), std::move(optional_parameters), std::move(response_client));
}
void NetworkContextInterceptorForTesting::CreateHostResolver(const base::Optional<net::DnsConfigOverrides>& config_overrides, ::network::mojom::HostResolverRequest host_resolver) {
  GetForwardingInterface()->CreateHostResolver(std::move(config_overrides), std::move(host_resolver));
}
void NetworkContextInterceptorForTesting::VerifyCertForSignedExchange(const scoped_refptr<net::X509Certificate>& certificate, const GURL& url, const std::string& ocsp_response, const std::string& sct_list, VerifyCertForSignedExchangeCallback callback) {
  GetForwardingInterface()->VerifyCertForSignedExchange(std::move(certificate), std::move(url), std::move(ocsp_response), std::move(sct_list), std::move(callback));
}
void NetworkContextInterceptorForTesting::AddHSTS(const std::string& host, base::Time expiry, bool include_subdomains, AddHSTSCallback callback) {
  GetForwardingInterface()->AddHSTS(std::move(host), std::move(expiry), std::move(include_subdomains), std::move(callback));
}
void NetworkContextInterceptorForTesting::IsHSTSActiveForHost(const std::string& host, IsHSTSActiveForHostCallback callback) {
  GetForwardingInterface()->IsHSTSActiveForHost(std::move(host), std::move(callback));
}
void NetworkContextInterceptorForTesting::GetHSTSState(const std::string& domain, GetHSTSStateCallback callback) {
  GetForwardingInterface()->GetHSTSState(std::move(domain), std::move(callback));
}
void NetworkContextInterceptorForTesting::SetCorsOriginAccessListsForOrigin(const url::Origin& source_origin, std::vector<::network::mojom::CorsOriginPatternPtr> allow_patterns, std::vector<::network::mojom::CorsOriginPatternPtr> block_patterns, SetCorsOriginAccessListsForOriginCallback callback) {
  GetForwardingInterface()->SetCorsOriginAccessListsForOrigin(std::move(source_origin), std::move(allow_patterns), std::move(block_patterns), std::move(callback));
}
void NetworkContextInterceptorForTesting::DeleteDynamicDataForHost(const std::string& host, DeleteDynamicDataForHostCallback callback) {
  GetForwardingInterface()->DeleteDynamicDataForHost(std::move(host), std::move(callback));
}
void NetworkContextInterceptorForTesting::LookupBasicAuthCredentials(const GURL& url, LookupBasicAuthCredentialsCallback callback) {
  GetForwardingInterface()->LookupBasicAuthCredentials(std::move(url), std::move(callback));
}
void NetworkContextInterceptorForTesting::EnableStaticKeyPinningForTesting(EnableStaticKeyPinningForTestingCallback callback) {
  GetForwardingInterface()->EnableStaticKeyPinningForTesting(std::move(callback));
}
void NetworkContextInterceptorForTesting::SetFailingHttpTransactionForTesting(int32_t rv, SetFailingHttpTransactionForTestingCallback callback) {
  GetForwardingInterface()->SetFailingHttpTransactionForTesting(std::move(rv), std::move(callback));
}
void NetworkContextInterceptorForTesting::VerifyCertificateForTesting(const scoped_refptr<net::X509Certificate>& certificate, const std::string& hostname, const std::string& ocsp_response, VerifyCertificateForTestingCallback callback) {
  GetForwardingInterface()->VerifyCertificateForTesting(std::move(certificate), std::move(hostname), std::move(ocsp_response), std::move(callback));
}
void NetworkContextInterceptorForTesting::AddDomainReliabilityContextForTesting(const GURL& origin, const GURL& upload_url, AddDomainReliabilityContextForTestingCallback callback) {
  GetForwardingInterface()->AddDomainReliabilityContextForTesting(std::move(origin), std::move(upload_url), std::move(callback));
}
void NetworkContextInterceptorForTesting::ForceDomainReliabilityUploadsForTesting(ForceDomainReliabilityUploadsForTestingCallback callback) {
  GetForwardingInterface()->ForceDomainReliabilityUploadsForTesting(std::move(callback));
}
NetworkContextAsyncWaiter::NetworkContextAsyncWaiter(
    NetworkContext* proxy) : proxy_(proxy) {}

NetworkContextAsyncWaiter::~NetworkContextAsyncWaiter() = default;

void NetworkContextAsyncWaiter::ClearNetworkingHistorySince(
    base::Time start_time) {
  base::RunLoop loop;
  proxy_->ClearNetworkingHistorySince(std::move(start_time),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::ClearHttpCache(
    base::Time start_time, base::Time end_time, ClearDataFilterPtr filter) {
  base::RunLoop loop;
  proxy_->ClearHttpCache(std::move(start_time),std::move(end_time),std::move(filter),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::ComputeHttpCacheSize(
    base::Time start_time, base::Time end_time, bool* out_is_upper_bound, int64_t* out_size_or_error) {
  base::RunLoop loop;
  proxy_->ComputeHttpCacheSize(std::move(start_time),std::move(end_time),
      base::BindOnce(
          [](base::RunLoop* loop,
             bool* out_is_upper_bound
,
             int64_t* out_size_or_error
,
             bool is_upper_bound,
             int64_t size_or_error) {*out_is_upper_bound = std::move(is_upper_bound);*out_size_or_error = std::move(size_or_error);
            loop->Quit();
          },
          &loop,
          out_is_upper_bound,
          out_size_or_error));
  loop.Run();
}
void NetworkContextAsyncWaiter::ClearChannelIds(
    base::Time start_time, base::Time end_time, ClearDataFilterPtr filter) {
  base::RunLoop loop;
  proxy_->ClearChannelIds(std::move(start_time),std::move(end_time),std::move(filter),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::ClearHostCache(
    ClearDataFilterPtr filter) {
  base::RunLoop loop;
  proxy_->ClearHostCache(std::move(filter),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::ClearHttpAuthCache(
    base::Time start_time) {
  base::RunLoop loop;
  proxy_->ClearHttpAuthCache(std::move(start_time),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::ClearReportingCacheReports(
    ClearDataFilterPtr filter) {
  base::RunLoop loop;
  proxy_->ClearReportingCacheReports(std::move(filter),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::ClearReportingCacheClients(
    ClearDataFilterPtr filter) {
  base::RunLoop loop;
  proxy_->ClearReportingCacheClients(std::move(filter),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::ClearNetworkErrorLogging(
    ClearDataFilterPtr filter) {
  base::RunLoop loop;
  proxy_->ClearNetworkErrorLogging(std::move(filter),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::ClearDomainReliability(
    ClearDataFilterPtr filter, NetworkContext::DomainReliabilityClearMode mode) {
  base::RunLoop loop;
  proxy_->ClearDomainReliability(std::move(filter),std::move(mode),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::GetDomainReliabilityJSON(
    base::Value* out_data) {
  base::RunLoop loop;
  proxy_->GetDomainReliabilityJSON(
      base::BindOnce(
          [](base::RunLoop* loop,
             base::Value* out_data
,
             base::Value data) {*out_data = std::move(data);
            loop->Quit();
          },
          &loop,
          out_data));
  loop.Run();
}
void NetworkContextAsyncWaiter::CloseAllConnections(
    ) {
  base::RunLoop loop;
  proxy_->CloseAllConnections(
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::CloseIdleConnections(
    ) {
  base::RunLoop loop;
  proxy_->CloseIdleConnections(
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::AddExpectCT(
    const std::string& host, base::Time expiry, bool enforce, const GURL& report_uri, bool* out_success) {
  base::RunLoop loop;
  proxy_->AddExpectCT(std::move(host),std::move(expiry),std::move(enforce),std::move(report_uri),
      base::BindOnce(
          [](base::RunLoop* loop,
             bool* out_success
,
             bool success) {*out_success = std::move(success);
            loop->Quit();
          },
          &loop,
          out_success));
  loop.Run();
}
void NetworkContextAsyncWaiter::SetExpectCTTestReport(
    const GURL& report_uri, bool* out_success) {
  base::RunLoop loop;
  proxy_->SetExpectCTTestReport(std::move(report_uri),
      base::BindOnce(
          [](base::RunLoop* loop,
             bool* out_success
,
             bool success) {*out_success = std::move(success);
            loop->Quit();
          },
          &loop,
          out_success));
  loop.Run();
}
void NetworkContextAsyncWaiter::GetExpectCTState(
    const std::string& domain, base::Value* out_state) {
  base::RunLoop loop;
  proxy_->GetExpectCTState(std::move(domain),
      base::BindOnce(
          [](base::RunLoop* loop,
             base::Value* out_state
,
             base::Value state) {*out_state = std::move(state);
            loop->Quit();
          },
          &loop,
          out_state));
  loop.Run();
}
void NetworkContextAsyncWaiter::CreateTCPServerSocket(
    const net::IPEndPoint& local_addr, uint32_t backlog, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation, ::network::mojom::TCPServerSocketRequest socket, int32_t* out_result, base::Optional<net::IPEndPoint>* out_local_addr_out) {
  base::RunLoop loop;
  proxy_->CreateTCPServerSocket(std::move(local_addr),std::move(backlog),std::move(traffic_annotation),std::move(socket),
      base::BindOnce(
          [](base::RunLoop* loop,
             int32_t* out_result
,
             base::Optional<net::IPEndPoint>* out_local_addr_out
,
             int32_t result,
             const base::Optional<net::IPEndPoint>& local_addr_out) {*out_result = std::move(result);*out_local_addr_out = std::move(local_addr_out);
            loop->Quit();
          },
          &loop,
          out_result,
          out_local_addr_out));
  loop.Run();
}
void NetworkContextAsyncWaiter::CreateTCPConnectedSocket(
    const base::Optional<net::IPEndPoint>& local_addr, const net::AddressList& remote_addr_list, ::network::mojom::TCPConnectedSocketOptionsPtr tcp_connected_socket_options, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation, ::network::mojom::TCPConnectedSocketRequest socket, ::network::mojom::SocketObserverPtr observer, int32_t* out_result, base::Optional<net::IPEndPoint>* out_local_addr, base::Optional<net::IPEndPoint>* out_peer_addr, mojo::ScopedDataPipeConsumerHandle* out_receive_stream, mojo::ScopedDataPipeProducerHandle* out_send_stream) {
  base::RunLoop loop;
  proxy_->CreateTCPConnectedSocket(std::move(local_addr),std::move(remote_addr_list),std::move(tcp_connected_socket_options),std::move(traffic_annotation),std::move(socket),std::move(observer),
      base::BindOnce(
          [](base::RunLoop* loop,
             int32_t* out_result
,
             base::Optional<net::IPEndPoint>* out_local_addr
,
             base::Optional<net::IPEndPoint>* out_peer_addr
,
             mojo::ScopedDataPipeConsumerHandle* out_receive_stream
,
             mojo::ScopedDataPipeProducerHandle* out_send_stream
,
             int32_t result,
             const base::Optional<net::IPEndPoint>& local_addr,
             const base::Optional<net::IPEndPoint>& peer_addr,
             mojo::ScopedDataPipeConsumerHandle receive_stream,
             mojo::ScopedDataPipeProducerHandle send_stream) {*out_result = std::move(result);*out_local_addr = std::move(local_addr);*out_peer_addr = std::move(peer_addr);*out_receive_stream = std::move(receive_stream);*out_send_stream = std::move(send_stream);
            loop->Quit();
          },
          &loop,
          out_result,
          out_local_addr,
          out_peer_addr,
          out_receive_stream,
          out_send_stream));
  loop.Run();
}
void NetworkContextAsyncWaiter::CreateTCPBoundSocket(
    const net::IPEndPoint& local_addr, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation, ::network::mojom::TCPBoundSocketRequest socket, int32_t* out_result, base::Optional<net::IPEndPoint>* out_local_addr) {
  base::RunLoop loop;
  proxy_->CreateTCPBoundSocket(std::move(local_addr),std::move(traffic_annotation),std::move(socket),
      base::BindOnce(
          [](base::RunLoop* loop,
             int32_t* out_result
,
             base::Optional<net::IPEndPoint>* out_local_addr
,
             int32_t result,
             const base::Optional<net::IPEndPoint>& local_addr) {*out_result = std::move(result);*out_local_addr = std::move(local_addr);
            loop->Quit();
          },
          &loop,
          out_result,
          out_local_addr));
  loop.Run();
}
void NetworkContextAsyncWaiter::ForceReloadProxyConfig(
    ) {
  base::RunLoop loop;
  proxy_->ForceReloadProxyConfig(
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::ClearBadProxiesCache(
    ) {
  base::RunLoop loop;
  proxy_->ClearBadProxiesCache(
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::VerifyCertForSignedExchange(
    const scoped_refptr<net::X509Certificate>& certificate, const GURL& url, const std::string& ocsp_response, const std::string& sct_list, int32_t* out_error_code, net::CertVerifyResult* out_cv_result, net::ct::CTVerifyResult* out_ct_result) {
  base::RunLoop loop;
  proxy_->VerifyCertForSignedExchange(std::move(certificate),std::move(url),std::move(ocsp_response),std::move(sct_list),
      base::BindOnce(
          [](base::RunLoop* loop,
             int32_t* out_error_code
,
             net::CertVerifyResult* out_cv_result
,
             net::ct::CTVerifyResult* out_ct_result
,
             int32_t error_code,
             const net::CertVerifyResult& cv_result,
             const net::ct::CTVerifyResult& ct_result) {*out_error_code = std::move(error_code);*out_cv_result = std::move(cv_result);*out_ct_result = std::move(ct_result);
            loop->Quit();
          },
          &loop,
          out_error_code,
          out_cv_result,
          out_ct_result));
  loop.Run();
}
void NetworkContextAsyncWaiter::AddHSTS(
    const std::string& host, base::Time expiry, bool include_subdomains) {
  base::RunLoop loop;
  proxy_->AddHSTS(std::move(host),std::move(expiry),std::move(include_subdomains),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::IsHSTSActiveForHost(
    const std::string& host, bool* out_result) {
  base::RunLoop loop;
  proxy_->IsHSTSActiveForHost(std::move(host),
      base::BindOnce(
          [](base::RunLoop* loop,
             bool* out_result
,
             bool result) {*out_result = std::move(result);
            loop->Quit();
          },
          &loop,
          out_result));
  loop.Run();
}
void NetworkContextAsyncWaiter::GetHSTSState(
    const std::string& domain, base::Value* out_state) {
  base::RunLoop loop;
  proxy_->GetHSTSState(std::move(domain),
      base::BindOnce(
          [](base::RunLoop* loop,
             base::Value* out_state
,
             base::Value state) {*out_state = std::move(state);
            loop->Quit();
          },
          &loop,
          out_state));
  loop.Run();
}
void NetworkContextAsyncWaiter::SetCorsOriginAccessListsForOrigin(
    const url::Origin& source_origin, std::vector<::network::mojom::CorsOriginPatternPtr> allow_patterns, std::vector<::network::mojom::CorsOriginPatternPtr> block_patterns) {
  base::RunLoop loop;
  proxy_->SetCorsOriginAccessListsForOrigin(std::move(source_origin),std::move(allow_patterns),std::move(block_patterns),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::DeleteDynamicDataForHost(
    const std::string& host, bool* out_result) {
  base::RunLoop loop;
  proxy_->DeleteDynamicDataForHost(std::move(host),
      base::BindOnce(
          [](base::RunLoop* loop,
             bool* out_result
,
             bool result) {*out_result = std::move(result);
            loop->Quit();
          },
          &loop,
          out_result));
  loop.Run();
}
void NetworkContextAsyncWaiter::LookupBasicAuthCredentials(
    const GURL& url, base::Optional<net::AuthCredentials>* out_credentials) {
  base::RunLoop loop;
  proxy_->LookupBasicAuthCredentials(std::move(url),
      base::BindOnce(
          [](base::RunLoop* loop,
             base::Optional<net::AuthCredentials>* out_credentials
,
             const base::Optional<net::AuthCredentials>& credentials) {*out_credentials = std::move(credentials);
            loop->Quit();
          },
          &loop,
          out_credentials));
  loop.Run();
}
void NetworkContextAsyncWaiter::EnableStaticKeyPinningForTesting(
    ) {
  base::RunLoop loop;
  proxy_->EnableStaticKeyPinningForTesting(
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::SetFailingHttpTransactionForTesting(
    int32_t rv) {
  base::RunLoop loop;
  proxy_->SetFailingHttpTransactionForTesting(std::move(rv),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::VerifyCertificateForTesting(
    const scoped_refptr<net::X509Certificate>& certificate, const std::string& hostname, const std::string& ocsp_response, int32_t* out_error_code) {
  base::RunLoop loop;
  proxy_->VerifyCertificateForTesting(std::move(certificate),std::move(hostname),std::move(ocsp_response),
      base::BindOnce(
          [](base::RunLoop* loop,
             int32_t* out_error_code
,
             int32_t error_code) {*out_error_code = std::move(error_code);
            loop->Quit();
          },
          &loop,
          out_error_code));
  loop.Run();
}
void NetworkContextAsyncWaiter::AddDomainReliabilityContextForTesting(
    const GURL& origin, const GURL& upload_url) {
  base::RunLoop loop;
  proxy_->AddDomainReliabilityContextForTesting(std::move(origin),std::move(upload_url),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::ForceDomainReliabilityUploadsForTesting(
    ) {
  base::RunLoop loop;
  proxy_->ForceDomainReliabilityUploadsForTesting(
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}

}  // namespace mojom
}  // namespace network

namespace mojo {


// static
bool StructTraits<::network::mojom::CustomProxyConfig::DataView, ::network::mojom::CustomProxyConfigPtr>::Read(
    ::network::mojom::CustomProxyConfig::DataView input,
    ::network::mojom::CustomProxyConfigPtr* output) {
  bool success = true;
  ::network::mojom::CustomProxyConfigPtr result(::network::mojom::CustomProxyConfig::New());
  
      if (!input.ReadRules(&result->rules))
        success = false;
      if (!input.ReadAlternateProxyList(&result->alternate_proxy_list))
        success = false;
      if (!input.ReadPreCacheHeaders(&result->pre_cache_headers))
        success = false;
      if (!input.ReadPostCacheHeaders(&result->post_cache_headers))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::network::mojom::NetworkContextParams::DataView, ::network::mojom::NetworkContextParamsPtr>::Read(
    ::network::mojom::NetworkContextParams::DataView input,
    ::network::mojom::NetworkContextParamsPtr* output) {
  bool success = true;
  ::network::mojom::NetworkContextParamsPtr result(::network::mojom::NetworkContextParams::New());
  
      if (!input.ReadContextName(&result->context_name))
        success = false;
      if (!input.ReadUserAgent(&result->user_agent))
        success = false;
      if (!input.ReadAcceptLanguage(&result->accept_language))
        success = false;
      result->enable_brotli = input.enable_brotli();
      if (!input.ReadQuicUserAgentId(&result->quic_user_agent_id))
        success = false;
      result->enable_referrers = input.enable_referrers();
      result->validate_referrer_policy_on_initial_request = input.validate_referrer_policy_on_initial_request();
      result->proxy_resolver_factory =
          input.TakeProxyResolverFactory<decltype(result->proxy_resolver_factory)>();
      if (!input.ReadCookiePath(&result->cookie_path))
        success = false;
      result->enable_encrypted_cookies = input.enable_encrypted_cookies();
      result->restore_old_session_cookies = input.restore_old_session_cookies();
      result->persist_session_cookies = input.persist_session_cookies();
      if (!input.ReadChannelIdPath(&result->channel_id_path))
        success = false;
      result->http_cache_enabled = input.http_cache_enabled();
      result->http_cache_max_size = input.http_cache_max_size();
      if (!input.ReadHttpCachePath(&result->http_cache_path))
        success = false;
      if (!input.ReadHttpServerPropertiesPath(&result->http_server_properties_path))
        success = false;
      if (!input.ReadTransportSecurityPersisterPath(&result->transport_security_persister_path))
        success = false;
      result->enable_data_url_support = input.enable_data_url_support();
      result->enable_file_url_support = input.enable_file_url_support();
      result->enable_ftp_url_support = input.enable_ftp_url_support();
      result->enforce_chrome_ct_policy = input.enforce_chrome_ct_policy();
      result->http_09_on_non_default_ports_enabled = input.http_09_on_non_default_ports_enabled();
      if (!input.ReadInitialSslConfig(&result->initial_ssl_config))
        success = false;
      result->ssl_config_client_request =
          input.TakeSslConfigClientRequest<decltype(result->ssl_config_client_request)>();
      if (!input.ReadInitialProxyConfig(&result->initial_proxy_config))
        success = false;
      result->proxy_config_client_request =
          input.TakeProxyConfigClientRequest<decltype(result->proxy_config_client_request)>();
      if (!input.ReadInitialCustomProxyConfig(&result->initial_custom_proxy_config))
        success = false;
      result->custom_proxy_config_client_request =
          input.TakeCustomProxyConfigClientRequest<decltype(result->custom_proxy_config_client_request)>();
      result->proxy_config_poller_client =
          input.TakeProxyConfigPollerClient<decltype(result->proxy_config_poller_client)>();
      result->proxy_error_client =
          input.TakeProxyErrorClient<decltype(result->proxy_error_client)>();
      result->pac_quick_check_enabled = input.pac_quick_check_enabled();
      result->dangerously_allow_pac_access_to_secure_urls = input.dangerously_allow_pac_access_to_secure_urls();
      result->enable_certificate_reporting = input.enable_certificate_reporting();
      result->enable_expect_ct_reporting = input.enable_expect_ct_reporting();
      if (!input.ReadCtLogs(&result->ct_logs))
        success = false;
      if (!input.ReadCookieManagerParams(&result->cookie_manager_params))
        success = false;
      result->enable_domain_reliability = input.enable_domain_reliability();
      if (!input.ReadDomainReliabilityUploadReporter(&result->domain_reliability_upload_reporter))
        success = false;
      result->discard_domain_reliablity_uploads = input.discard_domain_reliablity_uploads();
      result->primary_network_context = input.primary_network_context();
      if (!input.ReadCorsOriginAccessList(&result->cors_origin_access_list))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::network::mojom::NetworkConditions::DataView, ::network::mojom::NetworkConditionsPtr>::Read(
    ::network::mojom::NetworkConditions::DataView input,
    ::network::mojom::NetworkConditionsPtr* output) {
  bool success = true;
  ::network::mojom::NetworkConditionsPtr result(::network::mojom::NetworkConditions::New());
  
      result->offline = input.offline();
      if (!input.ReadLatency(&result->latency))
        success = false;
      result->download_throughput = input.download_throughput();
      result->upload_throughput = input.upload_throughput();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::network::mojom::ClearDataFilter::DataView, ::network::mojom::ClearDataFilterPtr>::Read(
    ::network::mojom::ClearDataFilter::DataView input,
    ::network::mojom::ClearDataFilterPtr* output) {
  bool success = true;
  ::network::mojom::ClearDataFilterPtr result(::network::mojom::ClearDataFilter::New());
  
      if (!input.ReadType(&result->type))
        success = false;
      if (!input.ReadDomains(&result->domains))
        success = false;
      if (!input.ReadOrigins(&result->origins))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::network::mojom::NetworkUsage::DataView, ::network::mojom::NetworkUsagePtr>::Read(
    ::network::mojom::NetworkUsage::DataView input,
    ::network::mojom::NetworkUsagePtr* output) {
  bool success = true;
  ::network::mojom::NetworkUsagePtr result(::network::mojom::NetworkUsage::New());
  
      result->process_id = input.process_id();
      result->routing_id = input.routing_id();
      result->total_bytes_received = input.total_bytes_received();
      result->total_bytes_sent = input.total_bytes_sent();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::network::mojom::URLLoaderFactoryParams::DataView, ::network::mojom::URLLoaderFactoryParamsPtr>::Read(
    ::network::mojom::URLLoaderFactoryParams::DataView input,
    ::network::mojom::URLLoaderFactoryParamsPtr* output) {
  bool success = true;
  ::network::mojom::URLLoaderFactoryParamsPtr result(::network::mojom::URLLoaderFactoryParams::New());
  
      result->process_id = input.process_id();
      result->is_corb_enabled = input.is_corb_enabled();
      result->corb_detachable_resource_type = input.corb_detachable_resource_type();
      result->corb_excluded_resource_type = input.corb_excluded_resource_type();
      result->disable_web_security = input.disable_web_security();
      result->header_client =
          input.TakeHeaderClient<decltype(result->header_client)>();
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif